// cl-translate-file.cc

/*--------------------------- BEGIN LICENSE BLOCK ---------------------------+
|                                                                            |
| Version: MPL 1.1/GPL 2.0/LGPL 2.1                                          |
|                                                                            |
| The contents of this file are subject to the Mozilla Public License        |
| Version 1.1 (the "License"); you may not use this file except in           |
| compliance with the License. You may obtain a copy of the License at       |
| http://www.mozilla.org/MPL/                                                |
|                                                                            |
| Software distributed under the License is distributed on an "AS IS" basis, |
| WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License   |
| for the specific language governing rights and limitations under the       |
| License.                                                                   |
|                                                                            |
| The Original Code is the STELLA Programming Language.                      |
|                                                                            |
| The Initial Developer of the Original Code is                              |
| UNIVERSITY OF SOUTHERN CALIFORNIA, INFORMATION SCIENCES INSTITUTE          |
| 4676 Admiralty Way, Marina Del Rey, California 90292, U.S.A.               |
|                                                                            |
| Portions created by the Initial Developer are Copyright (C) 1996-2001      |
| the Initial Developer. All Rights Reserved.                                |
|                                                                            |
| Contributor(s):                                                            |
|                                                                            |
| Alternatively, the contents of this file may be used under the terms of    |
| either the GNU General Public License Version 2 or later (the "GPL"), or   |
| the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),   |
| in which case the provisions of the GPL or the LGPL are applicable instead |
| of those above. If you wish to allow use of your version of this file only |
| under the terms of either the GPL or the LGPL, and not to allow others to  |
| use your version of this file under the terms of the MPL, indicate your    |
| decision by deleting the provisions above and replace them with the notice |
| and other provisions required by the GPL or the LGPL. If you do not delete |
| the provisions above, a recipient may use your version of this file under  |
| the terms of any one of the MPL, the GPL or the LGPL.                      |
|                                                                            |
+---------------------------- END LICENSE BLOCK ----------------------------*/

#include "stella/stella-system.hh"

namespace stella {

boolean auxiliary_variable_unitP(Translation_Unit* unit) {
  return (unit->auxiliaryP &&
      (unit->category == SYM_CL_TRANSLATE_FILE_STELLA_GLOBAL_VARIABLE));
}

void cl_output_one_unit(Translation_Unit* unit, ostream* stream) {
  if (((boolean)(unit->category))) {
    if (!(auxiliary_variable_unitP(unit))) {
      *(stream) << endl;
    }
    if (unit->annotation != NULL) {
      *(stream) << ";;; " << unit->annotation << endl << endl;
    }
    print_stella_definition(unit->translation, stream);
    unit->category = NULL;
  }
}

void cl_output_all_units_to_file(char* sourcefile) {
  { char* outputfile = make_file_name(sourcefile, KWD_CL_TRANSLATE_FILE_LISP, FALSE);
    Output_File_Stream* outputstream = new_output_file_stream(outputfile);
    ostream* nativeoutputstream = outputstream->native_stream;

    if (oTRANSLATIONVERBOSITYLEVELo.get() >= 1) {
      cout << "Writing " << "`" << outputfile << "'" << "..." << endl;
    }
    output_file_header(outputstream, sourcefile);
    *(nativeoutputstream) << ";;; Auxiliary variables:" << endl << endl;
    { Translation_Unit* unit = NULL;
      Cons* iter_000 = oTRANSLATIONUNITSo.get()->the_cons_list;

      while (!(iter_000 == NIL)) {
        unit = ((Translation_Unit*)(iter_000->value));
        iter_000 = iter_000->rest;
        if (auxiliary_variable_unitP(unit)) {
          cl_output_one_unit(unit, nativeoutputstream);
        }
      }
    }
    cl_output_forward_declarations(nativeoutputstream);
    if (oCURRENT_ENVIRONMENT_FEATURESo->membP(KWD_CL_TRANSLATE_FILE_USE_COMMON_LISP_STRUCTS) &&
        (!oCURRENT_ENVIRONMENT_FEATURESo->membP(KWD_CL_TRANSLATE_FILE_USE_COMMON_LISP_VECTOR_STRUCTS))) {
      { Translation_Unit* unit = NULL;
        Cons* iter_001 = oTRANSLATIONUNITSo.get()->the_cons_list;

        while (!(iter_001 == NIL)) {
          unit = ((Translation_Unit*)(iter_001->value));
          iter_001 = iter_001->rest;
          if (unit->category == SYM_CL_TRANSLATE_FILE_STELLA_CLASS) {
            cl_output_one_unit(unit, nativeoutputstream);
          }
        }
      }
    }
    { Translation_Unit* unit = NULL;
      Cons* iter_002 = oTRANSLATIONUNITSo.get()->the_cons_list;

      while (!(iter_002 == NIL)) {
        unit = ((Translation_Unit*)(iter_002->value));
        iter_002 = iter_002->rest;
        if (!auxiliary_variable_unitP(unit)) {
          cl_output_one_unit(unit, nativeoutputstream);
        }
      }
    }
    outputstream->free();
  }
}

void cl_output_file_header(Output_Stream* stream, char* file) {
  file = file;
  { char* package = oMODULEo.get()->lisp_package();
    boolean ensurepackageP = TRUE;

    *(stream->native_stream) << ";;; -*- Mode: Lisp; Package: " << package << "; Syntax: COMMON-LISP; Base: 10 -*-" << endl << endl;
    { 
      BIND_STELLA_SPECIAL(oTRANSLATOROUTPUTLANGUAGEo, Keyword*, KWD_CL_TRANSLATE_FILE_COMMON_LISP);
      output_copyright_header(stream);
    }
    if (!(string_eqlP(package, "STELLA") ||
        (!ensurepackageP))) {
      cl_output_ensure_package_definition(stream, package);
      *(stream->native_stream) << endl;
    }
    cl_output_in_package_declaration(stream, package);
    *(stream->native_stream) << endl;
  }
}

void cl_output_ensure_package_definition(Output_Stream* stream, char* package) {
  *(stream->native_stream) << "(CL:EVAL-WHEN (CL:COMPILE CL:LOAD CL:EVAL)" << endl << "  (CL:UNLESS (CL:FIND-PACKAGE \"" << package << "\")" << endl << "     (CL:DEFPACKAGE \"" << package << "\" (:USE))))" << endl;
}

void cl_output_in_package_declaration(Output_Stream* stream, char* package) {
  *(stream->native_stream) << "(CL:IN-PACKAGE \"" << package << "\")" << endl;
}

void cl_output_forward_declarations(ostream* stream) {
  { List* definedvariables = new_list();
    List* forwardreferencedvariables = new_list();

    { Translation_Unit* unit = NULL;
      Cons* iter_000 = oTRANSLATIONUNITSo.get()->the_cons_list;

      while (!(iter_000 == NIL)) {
        unit = ((Translation_Unit*)(iter_000->value));
        iter_000 = iter_000->rest;
        if (auxiliary_variable_unitP(unit)) {
          definedvariables->push(unit->the_object);
        }
      }
    }
    { Translation_Unit* unit = NULL;
      Cons* iter_001 = oTRANSLATIONUNITSo.get()->the_cons_list;

      while (!(iter_001 == NIL)) {
        unit = ((Translation_Unit*)(iter_001->value));
        iter_001 = iter_001->rest;
        if (!auxiliary_variable_unitP(unit)) {
          if (unit->category == SYM_CL_TRANSLATE_FILE_STELLA_GLOBAL_VARIABLE) {
            definedvariables->push(unit->the_object);
          }
          { Global_Variable* variable = NULL;
            Cons* iter_002 = unit->referenced_globals->the_cons_list;

            while (!(iter_002 == NIL)) {
              variable = ((Global_Variable*)(iter_002->value));
              iter_002 = iter_002->rest;
              if ((!definedvariables->memberP(variable)) &&
                  (!forwardreferencedvariables->memberP(variable))) {
                forwardreferencedvariables->push(variable);
              }
            }
          }
        }
      }
    }
    { List_Iterator* it = ((List_Iterator*)(forwardreferencedvariables->allocate_iterator()));

      while (it->nextP()) {
        it->value_setter(((Global_Variable*)(it->value))->variable_name);
      }
    }
    if (forwardreferencedvariables->non_emptyP()) {
      *(stream) << endl << ";;; Forward declarations:" << endl << endl;
      print_stella_definition(listO(3, intern_common_lisp_symbol("DECLAIM"), cons(intern_common_lisp_symbol("SPECIAL"), cl_translate_list_of_trees(forwardreferencedvariables->the_cons_list)->concatenate(NIL, 0)), NIL), stream);
    }
    definedvariables->free();
    forwardreferencedvariables->free();
  }
}

void cl_translate_file(char* file, boolean relativeP) {
  // Translate a Stella `file' to Common-Lisp.  If
  // `relative?', concatenate root directory to `file'.
  if (relativeP) {
    file = make_file_name(file, KWD_CL_TRANSLATE_FILE_STELLA, TRUE);
  }
  translate_file(file, KWD_CL_TRANSLATE_FILE_COMMON_LISP, FALSE);
}

void cl_translate_walked_system_units(Key_Value_List* systemunits) {
  { 
    BIND_STELLA_SPECIAL(oCURRENTFILEo, char*, NULL);
    BIND_STELLA_SPECIAL(oTRANSLATIONUNITSo, List*, NULL);
    { Cons* systemfiles = files_plus_system_startup(system_definition_source_files(oCURRENTSYSTEMDEFINITIONo.get()));
      List* classunits = new_list();
      boolean useclstructsP = oCURRENT_ENVIRONMENT_FEATURESo->membP(KWD_CL_TRANSLATE_FILE_USE_COMMON_LISP_STRUCTS);
      char* filename = NULL;

      { String_Wrapper* file = NULL;
        List* units = NULL;
        Kv_Cons* iter_000 = systemunits->the_kv_list;

        while (((boolean)(iter_000))) {
          file = ((String_Wrapper*)(iter_000->key));
          units = ((List*)(iter_000->value));
          iter_000 = iter_000->rest;
          oCURRENTFILEo.set(file_base_name(file->wrapper_value));
          if (useclstructsP) {
            oTRANSLATIONUNITSo.set(new_list());
            { Translation_Unit* unit = NULL;
              Cons* iter_001 = units->the_cons_list;

              while (!(iter_001 == NIL)) {
                unit = ((Translation_Unit*)(iter_001->value));
                iter_001 = iter_001->rest;
                if (unit->category == SYM_CL_TRANSLATE_FILE_STELLA_CLASS) {
                  classunits->push(unit);
                }
                else {
                  oTRANSLATIONUNITSo.get()->push(unit);
                }
              }
            }
            oTRANSLATIONUNITSo.get()->reverse();
          }
          else {
            oTRANSLATIONUNITSo.set(units);
          }
          { 
            BIND_STELLA_SPECIAL(oMODULEo, Module*, compute_file_units_module(units));
            BIND_STELLA_SPECIAL(oCONTEXTo, Context*, oMODULEo.get());
            translate_and_output_units_to_file(file->wrapper_value, "Common Lisp");
          }
          clean_up_translation_units_special();
        }
      }
      if (useclstructsP) {
        if (oTRANSLATIONVERBOSITYLEVELo.get() >= 1) {
          cout << "Collecting CL structure class definitions..." << endl;
        }
        { String_Wrapper* file = NULL;
          Cons* iter_002 = files_plus_system_startup(system_definition_source_files(oCURRENTSYSTEMDEFINITIONo.get()));

          while (!(iter_002 == NIL)) {
            file = ((String_Wrapper*)(iter_002->value));
            iter_002 = iter_002->rest;
            if (!((boolean)(((List*)(systemunits->lookup(wrap_string(filename = make_file_name(file->wrapper_value, KWD_CL_TRANSLATE_FILE_STELLA, TRUE)))))))) {
              oTRANSLATIONUNITSo.set(new_list());
              oCURRENTFILEo.set(file_base_name(filename));
              oTRANSLATIONUNITSo.set(new_list());
              operate_on_file(filename, KWD_CL_TRANSLATE_FILE_FINALIZE);
              { Translation_Unit* unit = NULL;
                Cons* iter_003 = oTRANSLATIONUNITSo.get()->the_cons_list;
                Cons* collect_000 = NULL;

                while (!(iter_003 == NIL)) {
                  unit = ((Translation_Unit*)(iter_003->value));
                  iter_003 = iter_003->rest;
                  if ((unit->category == SYM_CL_TRANSLATE_FILE_STELLA_CLASS) &&
                      (create_native_classP(((Class*)(unit->the_object))) &&
                       (!((Class*)(unit->the_object))->mixinP))) {
                    if (!((boolean)(collect_000))) {
                      {
                        collect_000 = cons(unit, NIL);
                        if (classunits->the_cons_list == NIL) {
                          classunits->the_cons_list = collect_000;
                        }
                        else {
                          add_cons_to_end_of_cons_list(classunits->the_cons_list, collect_000);
                        }
                      }
                    }
                    else {
                      {
                        collect_000->rest = cons(unit, NIL);
                        collect_000 = collect_000->rest;
                      }
                    }
                  }
                }
              }
              clean_up_translation_units_special();
            }
          }
        }
        cl_output_system_cl_structs_file(classunits);
      }
    }
  }
}

List* sort_cl_struct_classes(List* unsortedclasses) {
  { Hash_Table* classtable = new_hash_table();
    List* roots = list(0);
    List* sortedclasses = list(0);

    { Class* renamed_Class = NULL;
      Cons* iter_000 = unsortedclasses->the_cons_list;

      while (!(iter_000 == NIL)) {
        renamed_Class = ((Class*)(iter_000->value));
        iter_000 = iter_000->rest;
        classtable->insert_at(renamed_Class->class_type, TRUE_WRAPPER);
      }
    }
    { Class* renamed_Class = NULL;
      Cons* iter_001 = unsortedclasses->the_cons_list;
      Cons* collect_000 = NULL;

      while (!(iter_001 == NIL)) {
        renamed_Class = ((Class*)(iter_001->value));
        iter_001 = iter_001->rest;
        { boolean test_value_000 = FALSE;

          { boolean foundP_000 = FALSE;

            { Surrogate* super = NULL;
              Cons* iter_002 = renamed_Class->class_direct_supers->the_cons_list;

              while (!(iter_002 == NIL)) {
                super = ((Surrogate*)(iter_002->value));
                iter_002 = iter_002->rest;
                if (((boolean)(((Boolean_Wrapper*)(classtable->lookup(super)))))) {
                  foundP_000 = TRUE;
                  break;
                }
              }
            }
            test_value_000 = foundP_000;
          }
          test_value_000 = !test_value_000;
          if (test_value_000) {
            if (!((boolean)(collect_000))) {
              {
                collect_000 = cons(renamed_Class, NIL);
                if (roots->the_cons_list == NIL) {
                  roots->the_cons_list = collect_000;
                }
                else {
                  add_cons_to_end_of_cons_list(roots->the_cons_list, collect_000);
                }
              }
            }
            else {
              {
                collect_000->rest = cons(renamed_Class, NIL);
                collect_000 = collect_000->rest;
              }
            }
          }
        }
      }
    }
    while (roots->non_emptyP()) {
      sortedclasses->push(((Class*)(roots->first())));
      { Surrogate* sub = NULL;
        Cons* iter_003 = ((Class*)(roots->pop()))->class_direct_subs->the_cons_list;

        while (!(iter_003 == NIL)) {
          sub = ((Surrogate*)(iter_003->value));
          iter_003 = iter_003->rest;
          if (((boolean)(((Boolean_Wrapper*)(classtable->lookup(sub)))))) {
            roots->push(((Class*)(sub->surrogate_value)));
          }
        }
      }
    }
    return (sortedclasses->reverse());
  }
}

char* cl_yield_struct_class_file_name(char* systemname) {
  return (string_concatenate(string_downcase(systemname), "-system-structs", 0));
}

void cl_output_system_cl_structs_file(List* classunits) {
  { char* outputfile = make_file_name(cl_yield_struct_class_file_name(oCURRENTSYSTEMDEFINITIONo.get()->name), KWD_CL_TRANSLATE_FILE_LISP, TRUE);
    Output_File_Stream* outputstream = new_output_file_stream(outputfile);
    List* classes = new_list();
    Module* currentmodule = oMODULEo.get();
    char* package = NULL;
    List* ensuredpackages = list(0);

    { Translation_Unit* unit = NULL;
      Cons* iter_000 = classunits->the_cons_list;
      Cons* collect_000 = NULL;

      while (!(iter_000 == NIL)) {
        unit = ((Translation_Unit*)(iter_000->value));
        iter_000 = iter_000->rest;
        if (!((boolean)(collect_000))) {
          {
            collect_000 = cons(unit->the_object, NIL);
            if (classes->the_cons_list == NIL) {
              classes->the_cons_list = collect_000;
            }
            else {
              add_cons_to_end_of_cons_list(classes->the_cons_list, collect_000);
            }
          }
        }
        else {
          {
            collect_000->rest = cons(unit->the_object, NIL);
            collect_000 = collect_000->rest;
          }
        }
      }
    }
    classes = sort_cl_struct_classes(classes);
    if (oTRANSLATIONVERBOSITYLEVELo.get() >= 1) {
      cout << "Writing " << "`" << outputfile << "'" << "..." << endl;
    }
    { 
      BIND_STELLA_SPECIAL(oMODULEo, Module*, oMODULEo.get());
      BIND_STELLA_SPECIAL(oCONTEXTo, Context*, oMODULEo.get());
      if (classes->non_emptyP()) {
        oMODULEo.set(((Class*)(classes->first()))->home_module());
      }
      else {
        oMODULEo.set(get_stella_module(oCURRENTSYSTEMDEFINITIONo.get()->cardinal_module, TRUE));
      }
      currentmodule = oMODULEo.get();
      cl_output_file_header(outputstream, NULL);
      { Translation_Unit* unit = NULL;
        Class* renamed_Class = NULL;
        Cons* iter_001 = classunits->the_cons_list;
        Cons* iter_002 = classes->the_cons_list;

        while ((!(iter_001 == NIL)) &&
            (!(iter_002 == NIL))) {
          unit = ((Translation_Unit*)(iter_001->value));
          iter_001 = iter_001->rest;
          renamed_Class = ((Class*)(iter_002->value));
          iter_002 = iter_002->rest;
          unit->the_object = renamed_Class;
          unit->annotation = NULL;
          oMODULEo.set(renamed_Class->home_module());
          if (!(oMODULEo.get() == currentmodule)) {
            currentmodule = oMODULEo.get();
            package = oMODULEo.get()->lisp_package();
            if (!(ensuredpackages->memberP(wrap_string(package)))) {
              ensuredpackages->push(wrap_string(package));
              *(outputstream->native_stream) << endl;
              cl_output_ensure_package_definition(outputstream, package);
            }
            *(outputstream->native_stream) << endl;
            cl_output_in_package_declaration(outputstream, package);
          }
          unit->translation = cl_translate_define_native_class_unit(unit);
          cl_output_one_unit(unit, outputstream->native_stream);
        }
      }
    }
    outputstream->free();
  }
}

void cl_translate_system(char* system_name) {
  // Translate a Stella system named `system-name' to Common Lisp.
  translate_system(system_name, KWD_CL_TRANSLATE_FILE_COMMON_LISP, 6, KWD_CL_TRANSLATE_FILE_TWO_PASSp, TRUE_WRAPPER, KWD_CL_TRANSLATE_FILE_FORCE_TRANSLATIONp, TRUE_WRAPPER, KWD_CL_TRANSLATE_FILE_PRODUCTION_SETTINGSp, TRUE_WRAPPER);
}

void cl_compile_and_load_file(char* file, boolean relativeP, boolean forcerecompilationP) {
  { char* translatedfile = make_file_name(file, KWD_CL_TRANSLATE_FILE_LISP, relativeP);
    char* binaryfile = make_file_name(file, KWD_CL_TRANSLATE_FILE_LISP_BINARY, relativeP);

  }
}

void cl_translate_stella(boolean productionsettingsP) {
  translate_system("stella", KWD_CL_TRANSLATE_FILE_COMMON_LISP, 4, KWD_CL_TRANSLATE_FILE_FORCE_TRANSLATIONp, TRUE_WRAPPER, KWD_CL_TRANSLATE_FILE_PRODUCTION_SETTINGSp, (productionsettingsP ? TRUE_WRAPPER : FALSE_WRAPPER));
}

void cl_retranslate_stella(boolean productionsettingsP) {
  translate_system("stella", KWD_CL_TRANSLATE_FILE_COMMON_LISP, 6, KWD_CL_TRANSLATE_FILE_TWO_PASSp, TRUE_WRAPPER, KWD_CL_TRANSLATE_FILE_FORCE_TRANSLATIONp, TRUE_WRAPPER, KWD_CL_TRANSLATE_FILE_PRODUCTION_SETTINGSp, (productionsettingsP ? TRUE_WRAPPER : FALSE_WRAPPER));
}

void startup_cl_translate_file() {
  { 
    BIND_STELLA_SPECIAL(oMODULEo, Module*, oSTELLA_MODULEo);
    BIND_STELLA_SPECIAL(oCONTEXTo, Context*, oMODULEo.get());
    if (current_startup_time_phaseP(2)) {
      SYM_CL_TRANSLATE_FILE_STELLA_GLOBAL_VARIABLE = ((Symbol*)(intern_rigid_symbol_wrt_module("GLOBAL-VARIABLE", NULL, 0)));
      KWD_CL_TRANSLATE_FILE_LISP = ((Keyword*)(intern_rigid_symbol_wrt_module("LISP", NULL, 2)));
      KWD_CL_TRANSLATE_FILE_USE_COMMON_LISP_STRUCTS = ((Keyword*)(intern_rigid_symbol_wrt_module("USE-COMMON-LISP-STRUCTS", NULL, 2)));
      KWD_CL_TRANSLATE_FILE_USE_COMMON_LISP_VECTOR_STRUCTS = ((Keyword*)(intern_rigid_symbol_wrt_module("USE-COMMON-LISP-VECTOR-STRUCTS", NULL, 2)));
      SYM_CL_TRANSLATE_FILE_STELLA_CLASS = ((Symbol*)(intern_rigid_symbol_wrt_module("CLASS", NULL, 0)));
      KWD_CL_TRANSLATE_FILE_COMMON_LISP = ((Keyword*)(intern_rigid_symbol_wrt_module("COMMON-LISP", NULL, 2)));
      KWD_CL_TRANSLATE_FILE_STELLA = ((Keyword*)(intern_rigid_symbol_wrt_module("STELLA", NULL, 2)));
      KWD_CL_TRANSLATE_FILE_FINALIZE = ((Keyword*)(intern_rigid_symbol_wrt_module("FINALIZE", NULL, 2)));
      KWD_CL_TRANSLATE_FILE_TWO_PASSp = ((Keyword*)(intern_rigid_symbol_wrt_module("TWO-PASS?", NULL, 2)));
      KWD_CL_TRANSLATE_FILE_FORCE_TRANSLATIONp = ((Keyword*)(intern_rigid_symbol_wrt_module("FORCE-TRANSLATION?", NULL, 2)));
      KWD_CL_TRANSLATE_FILE_PRODUCTION_SETTINGSp = ((Keyword*)(intern_rigid_symbol_wrt_module("PRODUCTION-SETTINGS?", NULL, 2)));
      KWD_CL_TRANSLATE_FILE_LISP_BINARY = ((Keyword*)(intern_rigid_symbol_wrt_module("LISP-BINARY", NULL, 2)));
      SYM_CL_TRANSLATE_FILE_STELLA_STARTUP_CL_TRANSLATE_FILE = ((Symbol*)(intern_rigid_symbol_wrt_module("STARTUP-CL-TRANSLATE-FILE", NULL, 0)));
      SYM_CL_TRANSLATE_FILE_STELLA_METHOD_STARTUP_CLASSNAME = ((Symbol*)(intern_rigid_symbol_wrt_module("METHOD-STARTUP-CLASSNAME", NULL, 0)));
    }
    if (current_startup_time_phaseP(6)) {
      finalize_classes();
    }
    if (current_startup_time_phaseP(7)) {
      define_function_object("AUXILIARY-VARIABLE-UNIT?", "(DEFUN (AUXILIARY-VARIABLE-UNIT? BOOLEAN) ((UNIT TRANSLATION-UNIT)))", ((cpp_function_code)(&auxiliary_variable_unitP)), NULL);
      define_function_object("CL-OUTPUT-ONE-UNIT", "(DEFUN CL-OUTPUT-ONE-UNIT ((UNIT TRANSLATION-UNIT) (STREAM NATIVE-OUTPUT-STREAM)))", ((cpp_function_code)(&cl_output_one_unit)), NULL);
      define_function_object("CL-OUTPUT-ALL-UNITS-TO-FILE", "(DEFUN CL-OUTPUT-ALL-UNITS-TO-FILE ((SOURCEFILE FILE-NAME)))", ((cpp_function_code)(&cl_output_all_units_to_file)), NULL);
      define_function_object("CL-OUTPUT-FILE-HEADER", "(DEFUN CL-OUTPUT-FILE-HEADER ((STREAM OUTPUT-STREAM) (FILE FILE-NAME)))", ((cpp_function_code)(&cl_output_file_header)), NULL);
      define_function_object("CL-OUTPUT-ENSURE-PACKAGE-DEFINITION", "(DEFUN CL-OUTPUT-ENSURE-PACKAGE-DEFINITION ((STREAM OUTPUT-STREAM) (PACKAGE STRING)))", ((cpp_function_code)(&cl_output_ensure_package_definition)), NULL);
      define_function_object("CL-OUTPUT-IN-PACKAGE-DECLARATION", "(DEFUN CL-OUTPUT-IN-PACKAGE-DECLARATION ((STREAM OUTPUT-STREAM) (PACKAGE STRING)))", ((cpp_function_code)(&cl_output_in_package_declaration)), NULL);
      define_function_object("CL-OUTPUT-FORWARD-DECLARATIONS", "(DEFUN CL-OUTPUT-FORWARD-DECLARATIONS ((STREAM NATIVE-OUTPUT-STREAM)))", ((cpp_function_code)(&cl_output_forward_declarations)), NULL);
      define_function_object("CL-TRANSLATE-FILE", "(DEFUN CL-TRANSLATE-FILE ((FILE FILE-NAME) (RELATIVE? BOOLEAN)) :DOCUMENTATION \"Translate a Stella `file' to Common-Lisp.  If\n`relative?', concatenate root directory to `file'.\" :PUBLIC? TRUE)", ((cpp_function_code)(&cl_translate_file)), NULL);
      define_function_object("CL-TRANSLATE-WALKED-SYSTEM-UNITS", "(DEFUN CL-TRANSLATE-WALKED-SYSTEM-UNITS ((SYSTEMUNITS SYSTEM-UNITS-ALIST)))", ((cpp_function_code)(&cl_translate_walked_system_units)), NULL);
      define_function_object("SORT-CL-STRUCT-CLASSES", "(DEFUN (SORT-CL-STRUCT-CLASSES (LIST OF CLASS)) ((UNSORTEDCLASSES (LIST OF CLASS))))", ((cpp_function_code)(&sort_cl_struct_classes)), NULL);
      define_function_object("CL-YIELD-STRUCT-CLASS-FILE-NAME", "(DEFUN (CL-YIELD-STRUCT-CLASS-FILE-NAME FILE-NAME) ((SYSTEMNAME STRING)))", ((cpp_function_code)(&cl_yield_struct_class_file_name)), NULL);
      define_function_object("CL-OUTPUT-SYSTEM-CL-STRUCTS-FILE", "(DEFUN CL-OUTPUT-SYSTEM-CL-STRUCTS-FILE ((CLASSUNITS (LIST OF TRANSLATION-UNIT))))", ((cpp_function_code)(&cl_output_system_cl_structs_file)), NULL);
      define_function_object("CL-TRANSLATE-SYSTEM", "(DEFUN CL-TRANSLATE-SYSTEM ((SYSTEM-NAME STRING)) :DOCUMENTATION \"Translate a Stella system named `system-name' to Common Lisp.\" :PUBLIC? TRUE)", ((cpp_function_code)(&cl_translate_system)), NULL);
      define_function_object("CL-COMPILE-AND-LOAD-FILE", "(DEFUN CL-COMPILE-AND-LOAD-FILE ((FILE STRING) (RELATIVE? BOOLEAN) (FORCERECOMPILATION? BOOLEAN)))", ((cpp_function_code)(&cl_compile_and_load_file)), NULL);
      define_function_object("CL-TRANSLATE-STELLA", "(DEFUN CL-TRANSLATE-STELLA ((PRODUCTIONSETTINGS? BOOLEAN)))", ((cpp_function_code)(&cl_translate_stella)), NULL);
      define_function_object("CL-RETRANSLATE-STELLA", "(DEFUN CL-RETRANSLATE-STELLA ((PRODUCTIONSETTINGS? BOOLEAN)))", ((cpp_function_code)(&cl_retranslate_stella)), NULL);
      define_function_object("STARTUP-CL-TRANSLATE-FILE", "(DEFUN STARTUP-CL-TRANSLATE-FILE () :PUBLIC? TRUE)", ((cpp_function_code)(&startup_cl_translate_file)), NULL);
      { Method_Slot* function = lookup_function(SYM_CL_TRANSLATE_FILE_STELLA_STARTUP_CL_TRANSLATE_FILE);

        set_dynamic_slot_value(function->dynamic_slots, SYM_CL_TRANSLATE_FILE_STELLA_METHOD_STARTUP_CLASSNAME, wrap_string("Startup-Cl-Translate-File"), NULL_STRING_WRAPPER);
      }
    }
    if (current_startup_time_phaseP(8)) {
      finalize_slots();
      cleanup_unfinalized_classes();
    }
  }
}

Symbol* SYM_CL_TRANSLATE_FILE_STELLA_GLOBAL_VARIABLE = NULL;

Keyword* KWD_CL_TRANSLATE_FILE_LISP = NULL;

Keyword* KWD_CL_TRANSLATE_FILE_USE_COMMON_LISP_STRUCTS = NULL;

Keyword* KWD_CL_TRANSLATE_FILE_USE_COMMON_LISP_VECTOR_STRUCTS = NULL;

Symbol* SYM_CL_TRANSLATE_FILE_STELLA_CLASS = NULL;

Keyword* KWD_CL_TRANSLATE_FILE_COMMON_LISP = NULL;

Keyword* KWD_CL_TRANSLATE_FILE_STELLA = NULL;

Keyword* KWD_CL_TRANSLATE_FILE_FINALIZE = NULL;

Keyword* KWD_CL_TRANSLATE_FILE_TWO_PASSp = NULL;

Keyword* KWD_CL_TRANSLATE_FILE_FORCE_TRANSLATIONp = NULL;

Keyword* KWD_CL_TRANSLATE_FILE_PRODUCTION_SETTINGSp = NULL;

Keyword* KWD_CL_TRANSLATE_FILE_LISP_BINARY = NULL;

Symbol* SYM_CL_TRANSLATE_FILE_STELLA_STARTUP_CL_TRANSLATE_FILE = NULL;

Symbol* SYM_CL_TRANSLATE_FILE_STELLA_METHOD_STARTUP_CLASSNAME = NULL;

} // end of namespace stella
