// /tmp/build-stella/stella-3.0.0.beta/native/cpp/stella/date-time.hh

/*--------------------------- BEGIN LICENSE BLOCK ---------------------------+
|                                                                            |
| Version: MPL 1.1/GPL 2.0/LGPL 2.1                                          |
|                                                                            |
| The contents of this file are subject to the Mozilla Public License        |
| Version 1.1 (the "License"); you may not use this file except in           |
| compliance with the License. You may obtain a copy of the License at       |
| http://www.mozilla.org/MPL/                                                |
|                                                                            |
| Software distributed under the License is distributed on an "AS IS" basis, |
| WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License   |
| for the specific language governing rights and limitations under the       |
| License.                                                                   |
|                                                                            |
| The Original Code is the STELLA Programming Language.                      |
|                                                                            |
| The Initial Developer of the Original Code is                              |
| UNIVERSITY OF SOUTHERN CALIFORNIA, INFORMATION SCIENCES INSTITUTE          |
| 4676 Admiralty Way, Marina Del Rey, California 90292, U.S.A.               |
|                                                                            |
| Portions created by the Initial Developer are Copyright (C) 1996-2001      |
| the Initial Developer. All Rights Reserved.                                |
|                                                                            |
| Contributor(s):                                                            |
|                                                                            |
| Alternatively, the contents of this file may be used under the terms of    |
| either the GNU General Public License Version 2 or later (the "GPL"), or   |
| the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),   |
| in which case the provisions of the GPL or the LGPL are applicable instead |
| of those above. If you wish to allow use of your version of this file only |
| under the terms of either the GPL or the LGPL, and not to allow others to  |
| use your version of this file under the terms of the MPL, indicate your    |
| decision by deleting the provisions above and replace them with the notice |
| and other provisions required by the GPL or the LGPL. If you do not delete |
| the provisions above, a recipient may use your version of this file under  |
| the terms of any one of the MPL, the GPL or the LGPL.                      |
|                                                                            |
+---------------------------- END LICENSE BLOCK ----------------------------*/


namespace stella {

// Class definitions:
class Decoded_Date_Time : public Standard_Object {
// A data structure for holding decoded time values with fields
// for easy access of the components.
public:
  int date_time_millis;
  int date_time_second;
  int date_time_minute;
  int date_time_hour;
  int date_time_day;
  int date_time_month;
  int date_time_year;
  double date_time_zone;
  Keyword* date_time_dow;
public:
  virtual Calendar_Date* encode_calendar_date();
  virtual Surrogate* primary_type();
};

class Date_Time_Object : public Standard_Object {
};

class Calendar_Date : public Date_Time_Object {
// Representation of Date and Time with the date represented
// as a modified Julian day (starting at midnight instead of noon) and the
// time as the offset in milliseconds from midnight, UTC
public:
  int modified_julian_day;
  int time_millis;
public:
  virtual void print_object(ostream* stream);
  virtual Decoded_Date_Time* decode_calendar_date(double timezone);
  virtual int get_time(double timezone, int& _return1, int& _return2, int& _return3);
  virtual int get_calendar_date(double timezone, int& _return1, int& _return2, Keyword*& _return3);
  virtual Surrogate* primary_type();
};

class Relative_Date : public Date_Time_Object {
// Representation of Relative Days and Time with the date represented
// as a number of days and the time in milliseconds.
public:
  int days;
  int millis;
public:
  virtual void print_object(ostream* stream);
  virtual Surrogate* primary_type();
};


// Global declarations:
extern int MILLIS_PER_DAY;
extern int MILLIS_PER_HOUR;
extern Vector* oMONTH_ABBREVIATION_VECTORo;

// Function signatures:
int compute_julian_day(int yyyy, int mm, int dd);
int julian_day_to_modified_julian_day(int julian_day);
int modified_julian_day_to_julian_day(int modified_julian_day);
Keyword* compute_day_of_week_julian(int julian_day);
Keyword* compute_day_of_week(int yyyy, int mm, int dd);
int compute_next_moon_phase(int n, Keyword* phase, double& _return1);
int compute_calendar_date(int julian_day, int& _return1, int& _return2, Keyword*& _return3);
int get_current_date_time(int& _return1, int& _return2, Keyword*& _return3, int& _return4, int& _return5, int& _return6, int& _return7);
double get_local_time_zone();
Decoded_Date_Time* new_decoded_date_time();
Object* access_decoded_date_time_slot_value(Decoded_Date_Time* self, Symbol* slotname, Object* value, boolean setvalueP);
void set_current_time(Decoded_Date_Time* values_structure);
void set_current_date(Decoded_Date_Time* values_structure);
void set_current_date_time(Decoded_Date_Time* values_structure);
Calendar_Date* new_calendar_date();
Object* access_calendar_date_slot_value(Calendar_Date* self, Symbol* slotname, Object* value, boolean setvalueP);
char* calendar_date_to_string(Calendar_Date* date, double timezone);
time_t calendar_date_to_native_date_time(Calendar_Date* date);
Calendar_Date* native_date_time_to_calendar_date(time_t date);
void print_calendar_date(Calendar_Date* date, ostream* stream);
Relative_Date* new_relative_date();
Object* access_relative_date_slot_value(Relative_Date* self, Symbol* slotname, Object* value, boolean setvalueP);
char* relative_date_to_string(Relative_Date* date);
void print_relative_date(Relative_Date* date, ostream* stream);
Calendar_Date* make_date_time(int year, int month, int day, int hour, int minute, int second, int millis, double timezone);
Calendar_Date* make_current_date_time();
Calendar_Date* make_calendar_date(int day, int time);
Relative_Date* make_relative_date(int days, int millis);
int decode_time_in_millis(int time, int& _return1, int& _return2, int& _return3);
Date_Time_Object* time_add(Date_Time_Object* t1, Date_Time_Object* t2);
Date_Time_Object* time_subtract(Date_Time_Object* t1, Date_Time_Object* t2);
Relative_Date* time_multiply(Object* t1, Object* t2);
Object* time_divide(Relative_Date* t1, Object* t2);
Object* define_time_comparison_expander(Symbol* name, Symbol* renamed_Operator);
boolean time_eqlP(Date_Time_Object* t1, Date_Time_Object* t2);
boolean time_lessP(Date_Time_Object* t1, Date_Time_Object* t2);
boolean time_greaterP(Date_Time_Object* t1, Date_Time_Object* t2);
boolean time_less_equalP(Date_Time_Object* t1, Date_Time_Object* t2);
boolean time_greater_equalP(Date_Time_Object* t1, Date_Time_Object* t2);
void help_startup_date_time1();
void startup_date_time();

// Auxiliary global declarations:
extern Keyword* KWD_DATE_TIME_MONDAY;
extern Keyword* KWD_DATE_TIME_TUESDAY;
extern Keyword* KWD_DATE_TIME_WEDNESDAY;
extern Keyword* KWD_DATE_TIME_THURSDAY;
extern Keyword* KWD_DATE_TIME_FRIDAY;
extern Keyword* KWD_DATE_TIME_SATURDAY;
extern Keyword* KWD_DATE_TIME_SUNDAY;
extern Keyword* KWD_DATE_TIME_NEW_MOON;
extern Keyword* KWD_DATE_TIME_FIRST_QUARTER;
extern Keyword* KWD_DATE_TIME_FULL_MOON;
extern Keyword* KWD_DATE_TIME_LAST_QUARTER;
extern Surrogate* SGT_DATE_TIME_STELLA_DECODED_DATE_TIME;
extern Symbol* SYM_DATE_TIME_STELLA_DATE_TIME_MILLIS;
extern Symbol* SYM_DATE_TIME_STELLA_DATE_TIME_SECOND;
extern Symbol* SYM_DATE_TIME_STELLA_DATE_TIME_MINUTE;
extern Symbol* SYM_DATE_TIME_STELLA_DATE_TIME_HOUR;
extern Symbol* SYM_DATE_TIME_STELLA_DATE_TIME_DAY;
extern Symbol* SYM_DATE_TIME_STELLA_DATE_TIME_MONTH;
extern Symbol* SYM_DATE_TIME_STELLA_DATE_TIME_YEAR;
extern Symbol* SYM_DATE_TIME_STELLA_DATE_TIME_ZONE;
extern Symbol* SYM_DATE_TIME_STELLA_DATE_TIME_DOW;
extern Surrogate* SGT_DATE_TIME_STELLA_CALENDAR_DATE;
extern Symbol* SYM_DATE_TIME_STELLA_MODIFIED_JULIAN_DAY;
extern Symbol* SYM_DATE_TIME_STELLA_TIME_MILLIS;
extern Keyword* KWD_DATE_TIME_RIGHT;
extern Surrogate* SGT_DATE_TIME_STELLA_RELATIVE_DATE;
extern Symbol* SYM_DATE_TIME_STELLA_DAYS;
extern Symbol* SYM_DATE_TIME_STELLA_MILLIS;
extern Surrogate* SGT_DATE_TIME_STELLA_NUMBER_WRAPPER;
extern Symbol* SYM_DATE_TIME_STELLA_DEFUN;
extern Symbol* SYM_DATE_TIME_STELLA_BOOLEAN;
extern Symbol* SYM_DATE_TIME_STELLA_T1;
extern Symbol* SYM_DATE_TIME_STELLA_DATE_TIME_OBJECT;
extern Symbol* SYM_DATE_TIME_STELLA_T2;
extern Symbol* SYM_DATE_TIME_STELLA_TYPECASE;
extern Symbol* SYM_DATE_TIME_STELLA_CALENDAR_DATE;
extern Symbol* SYM_DATE_TIME_STELLA_IF;
extern Symbol* SYM_DATE_TIME_STELLA_e;
extern Symbol* SYM_DATE_TIME_STELLA_RETURN;
extern Symbol* SYM_DATE_TIME_STELLA_RELATIVE_DATE;
extern Symbol* SYM_DATE_TIME_STELLA_ERROR;
extern Symbol* SYM_DATE_TIME_STELLA_DEFINE_TIME_COMPARISON;
extern Symbol* SYM_DATE_TIME_STELLA_DEFINE_TIME_COMPARISON_EXPANDER;
extern Symbol* SYM_DATE_TIME_STELLA_STARTUP_DATE_TIME;
extern Symbol* SYM_DATE_TIME_STELLA_METHOD_STARTUP_CLASSNAME;


} // end of namespace stella
