// dynamic-slots.cc

/*--------------------------- BEGIN LICENSE BLOCK ---------------------------+
|                                                                            |
| Version: MPL 1.1/GPL 2.0/LGPL 2.1                                          |
|                                                                            |
| The contents of this file are subject to the Mozilla Public License        |
| Version 1.1 (the "License"); you may not use this file except in           |
| compliance with the License. You may obtain a copy of the License at       |
| http://www.mozilla.org/MPL/                                                |
|                                                                            |
| Software distributed under the License is distributed on an "AS IS" basis, |
| WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License   |
| for the specific language governing rights and limitations under the       |
| License.                                                                   |
|                                                                            |
| The Original Code is the STELLA Programming Language.                      |
|                                                                            |
| The Initial Developer of the Original Code is                              |
| UNIVERSITY OF SOUTHERN CALIFORNIA, INFORMATION SCIENCES INSTITUTE          |
| 4676 Admiralty Way, Marina Del Rey, California 90292, U.S.A.               |
|                                                                            |
| Portions created by the Initial Developer are Copyright (C) 1996-2001      |
| the Initial Developer. All Rights Reserved.                                |
|                                                                            |
| Contributor(s):                                                            |
|                                                                            |
| Alternatively, the contents of this file may be used under the terms of    |
| either the GNU General Public License Version 2 or later (the "GPL"), or   |
| the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),   |
| in which case the provisions of the GPL or the LGPL are applicable instead |
| of those above. If you wish to allow use of your version of this file only |
| under the terms of either the GPL or the LGPL, and not to allow others to  |
| use your version of this file under the terms of the MPL, indicate your    |
| decision by deleting the provisions above and replace them with the notice |
| and other provisions required by the GPL or the LGPL. If you do not delete |
| the provisions above, a recipient may use your version of this file under  |
| the terms of any one of the MPL, the GPL or the LGPL.                      |
|                                                                            |
+---------------------------- END LICENSE BLOCK ----------------------------*/

#include "stella/stella-system.hh"

namespace stella {

Object* dynamic_slot_value(Key_Value_List* dynamicslots, Symbol* slotname, Object* nullvalue) {
  { Object* key = NULL;
    Object* value = NULL;
    Kv_Cons* iter_000 = dynamicslots->the_kv_list;

    while (((boolean)(iter_000))) {
      key = iter_000->key;
      value = iter_000->value;
      iter_000 = iter_000->rest;
      if (key == slotname) {
        if (!((boolean)(value))) {
          return (nullvalue);
        }
        else {
          return (value);
        }
      }
    }
  }
  return (nullvalue);
}

Object* set_dynamic_slot_value(Key_Value_List* dynamicslots, Symbol* slotname, Object* value, Wrapper* nullwrapper) {
  { boolean deletevalueP = (value == nullwrapper) &&
        (!(nullwrapper == FALSE_WRAPPER));
    Kv_Cons* cursor = dynamicslots->the_kv_list;

    while (((boolean)(cursor))) {
      if (cursor->key == slotname) {
        if (deletevalueP) {
          dynamicslots->remove_at(slotname);
        }
        else {
          cursor->value = value;
        }
        return (value);
      }
      cursor = cursor->rest;
    }
    if (!deletevalueP) {
      dynamicslots->the_kv_list = kv_cons(slotname, value, dynamicslots->the_kv_list);
    }
    return (value);
  }
}

boolean standard_dynamic_slot_accessP(Storage_Slot* slot) {
  return ((!slot->activeP()) &&
      ((slot->allocation() == KWD_DYNAMIC_SLOTS_DYNAMIC) &&
       ((!slot_value_is_bare_literalP(slot)) ||
        ((boolean)(lookup_literal_type_info(slot->slot_base_type, KWD_DYNAMIC_SLOTS_NULL_WRAPPER))))));
}

Cons* yield_down_cast_tree(Cons* tree, Standard_Object* returntype) {
  { Standard_Object* dummy1;
    Standard_Object* dummy2;

    { Cons* otree = sys_tree(tree, returntype, dummy1);

      if (!(oTRANSLATOROUTPUTLANGUAGEo.get() == KWD_DYNAMIC_SLOTS_COMMON_LISP)) {
        set_target_language_type(otree, SGT_DYNAMIC_SLOTS_STELLA_OBJECT, dummy2);
      }
      return (otree);
    }
  }
}

Cons* yield_standard_dynamic_slot_value_tree(Storage_Slot* slot, Object* objectref, Standard_Object* returntype) {
  { Standard_Object* dummy1;

    { Symbol* slotname = canonical_slot(slot)->slot_name;

      if (slot_value_is_bare_literalP(slot)) {
        return (listO(3, SYM_DYNAMIC_SLOTS_STELLA_WRAPPER_VALUE, listO(4, SYM_DYNAMIC_SLOTS_STELLA_SAFE_CAST, listO(5, SYM_DYNAMIC_SLOTS_STELLA_DYNAMIC_SLOT_VALUE, listO(3, SYM_DYNAMIC_SLOTS_STELLA_DYNAMIC_SLOTS, objectref, NIL), listO(3, SYM_DYNAMIC_SLOTS_STELLA_QUOTE, slotname, NIL), lookup_literal_type_info(slot->type(), KWD_DYNAMIC_SLOTS_NULL_WRAPPER), NIL), type_to_symbol(slot->type()->type_to_wrapped_type()), NIL), NIL));
      }
      else {
        return (yield_down_cast_tree(walk_a_cons_tree(listO(5, SYM_DYNAMIC_SLOTS_STELLA_DYNAMIC_SLOT_VALUE, listO(3, SYM_DYNAMIC_SLOTS_STELLA_DYNAMIC_SLOTS, objectref, NIL), listO(3, SYM_DYNAMIC_SLOTS_STELLA_QUOTE, slotname, NIL), SYM_DYNAMIC_SLOTS_STELLA_NULL, NIL), dummy1), returntype));
      }
    }
  }
}

Cons* yield_standard_dynamic_slot_value_setter_tree(Storage_Slot* slot, Object* objectref, Object* valueref, Standard_Object* returntype) {
  { Standard_Object* dummy1;

    { Symbol* slotname = canonical_slot(slot)->slot_name;
      boolean literalslotP = slot_value_is_bare_literalP(slot);
      Cons* settertree = NULL;

      if (literalslotP) {
        settertree = listO(6, SYM_DYNAMIC_SLOTS_STELLA_SET_DYNAMIC_SLOT_VALUE, listO(3, SYM_DYNAMIC_SLOTS_STELLA_DYNAMIC_SLOTS, objectref, NIL), listO(3, SYM_DYNAMIC_SLOTS_STELLA_BQUOTE, slotname, NIL), cons(lookup_literal_type_info(slot->type(), KWD_DYNAMIC_SLOTS_WRAP_FUNCTION), cons(valueref, NIL)), lookup_literal_type_info(slot->type(), KWD_DYNAMIC_SLOTS_NULL_WRAPPER), NIL);
      }
      else {
        settertree = listO(5, SYM_DYNAMIC_SLOTS_STELLA_SET_DYNAMIC_SLOT_VALUE, listO(3, SYM_DYNAMIC_SLOTS_STELLA_DYNAMIC_SLOTS, objectref, NIL), listO(3, SYM_DYNAMIC_SLOTS_STELLA_BQUOTE, slotname, NIL), valueref, cons(SYM_DYNAMIC_SLOTS_STELLA_NULL, NIL));
      }
      if (walking_expressionP()) {
        if (literalslotP) {
          settertree = listO(3, SYM_DYNAMIC_SLOTS_STELLA_WRAPPER_VALUE, listO(3, SYM_DYNAMIC_SLOTS_STELLA_SAFE_CAST, settertree, cons(type_to_symbol(slot->type()->type_to_wrapped_type()), NIL)), NIL);
        }
        else {
          settertree = yield_down_cast_tree(walk_a_cons_tree(settertree, dummy1), returntype);
        }
      }
      return (settertree);
    }
  }
}

void startup_dynamic_slots() {
  { 
    BIND_STELLA_SPECIAL(oMODULEo, Module*, oSTELLA_MODULEo);
    BIND_STELLA_SPECIAL(oCONTEXTo, Context*, oMODULEo.get());
    if (current_startup_time_phaseP(2)) {
      KWD_DYNAMIC_SLOTS_DYNAMIC = ((Keyword*)(intern_rigid_symbol_wrt_module("DYNAMIC", NULL, 2)));
      KWD_DYNAMIC_SLOTS_NULL_WRAPPER = ((Keyword*)(intern_rigid_symbol_wrt_module("NULL-WRAPPER", NULL, 2)));
      KWD_DYNAMIC_SLOTS_COMMON_LISP = ((Keyword*)(intern_rigid_symbol_wrt_module("COMMON-LISP", NULL, 2)));
      SGT_DYNAMIC_SLOTS_STELLA_OBJECT = ((Surrogate*)(intern_rigid_symbol_wrt_module("OBJECT", NULL, 1)));
      SYM_DYNAMIC_SLOTS_STELLA_WRAPPER_VALUE = ((Symbol*)(intern_rigid_symbol_wrt_module("WRAPPER-VALUE", NULL, 0)));
      SYM_DYNAMIC_SLOTS_STELLA_SAFE_CAST = ((Symbol*)(intern_rigid_symbol_wrt_module("SAFE-CAST", NULL, 0)));
      SYM_DYNAMIC_SLOTS_STELLA_DYNAMIC_SLOT_VALUE = ((Symbol*)(intern_rigid_symbol_wrt_module("DYNAMIC-SLOT-VALUE", NULL, 0)));
      SYM_DYNAMIC_SLOTS_STELLA_DYNAMIC_SLOTS = ((Symbol*)(intern_rigid_symbol_wrt_module("DYNAMIC-SLOTS", NULL, 0)));
      SYM_DYNAMIC_SLOTS_STELLA_QUOTE = ((Symbol*)(intern_rigid_symbol_wrt_module("QUOTE", NULL, 0)));
      SYM_DYNAMIC_SLOTS_STELLA_NULL = ((Symbol*)(intern_rigid_symbol_wrt_module("NULL", NULL, 0)));
      SYM_DYNAMIC_SLOTS_STELLA_SET_DYNAMIC_SLOT_VALUE = ((Symbol*)(intern_rigid_symbol_wrt_module("SET-DYNAMIC-SLOT-VALUE", NULL, 0)));
      SYM_DYNAMIC_SLOTS_STELLA_BQUOTE = ((Symbol*)(intern_rigid_symbol_wrt_module("BQUOTE", NULL, 0)));
      KWD_DYNAMIC_SLOTS_WRAP_FUNCTION = ((Keyword*)(intern_rigid_symbol_wrt_module("WRAP-FUNCTION", NULL, 2)));
      SYM_DYNAMIC_SLOTS_STELLA_STARTUP_DYNAMIC_SLOTS = ((Symbol*)(intern_rigid_symbol_wrt_module("STARTUP-DYNAMIC-SLOTS", NULL, 0)));
      SYM_DYNAMIC_SLOTS_STELLA_METHOD_STARTUP_CLASSNAME = ((Symbol*)(intern_rigid_symbol_wrt_module("METHOD-STARTUP-CLASSNAME", NULL, 0)));
    }
    if (current_startup_time_phaseP(6)) {
      finalize_classes();
    }
    if (current_startup_time_phaseP(7)) {
      define_function_object("DYNAMIC-SLOT-VALUE", "(DEFUN (DYNAMIC-SLOT-VALUE OBJECT) ((DYNAMICSLOTS KEY-VALUE-LIST) (SLOTNAME SYMBOL) (NULLVALUE OBJECT)) :PUBLIC? TRUE)", ((cpp_function_code)(&dynamic_slot_value)), NULL);
      define_function_object("SET-DYNAMIC-SLOT-VALUE", "(DEFUN (SET-DYNAMIC-SLOT-VALUE OBJECT) ((DYNAMICSLOTS KEY-VALUE-LIST) (SLOTNAME SYMBOL) (VALUE OBJECT) (NULLWRAPPER WRAPPER)) :PUBLIC? TRUE)", ((cpp_function_code)(&set_dynamic_slot_value)), NULL);
      define_function_object("STANDARD-DYNAMIC-SLOT-ACCESS?", "(DEFUN (STANDARD-DYNAMIC-SLOT-ACCESS? BOOLEAN) ((SLOT STORAGE-SLOT)))", ((cpp_function_code)(&standard_dynamic_slot_accessP)), NULL);
      define_function_object("YIELD-DOWN-CAST-TREE", "(DEFUN (YIELD-DOWN-CAST-TREE CONS) ((TREE CONS) (RETURNTYPE TYPE-SPEC)))", ((cpp_function_code)(&yield_down_cast_tree)), NULL);
      define_function_object("YIELD-STANDARD-DYNAMIC-SLOT-VALUE-TREE", "(DEFUN (YIELD-STANDARD-DYNAMIC-SLOT-VALUE-TREE CONS) ((SLOT STORAGE-SLOT) (OBJECTREF OBJECT) (RETURNTYPE TYPE-SPEC)))", ((cpp_function_code)(&yield_standard_dynamic_slot_value_tree)), NULL);
      define_function_object("YIELD-STANDARD-DYNAMIC-SLOT-VALUE-SETTER-TREE", "(DEFUN (YIELD-STANDARD-DYNAMIC-SLOT-VALUE-SETTER-TREE CONS) ((SLOT STORAGE-SLOT) (OBJECTREF OBJECT) (VALUEREF OBJECT) (RETURNTYPE TYPE-SPEC)))", ((cpp_function_code)(&yield_standard_dynamic_slot_value_setter_tree)), NULL);
      define_function_object("STARTUP-DYNAMIC-SLOTS", "(DEFUN STARTUP-DYNAMIC-SLOTS () :PUBLIC? TRUE)", ((cpp_function_code)(&startup_dynamic_slots)), NULL);
      { Method_Slot* function = lookup_function(SYM_DYNAMIC_SLOTS_STELLA_STARTUP_DYNAMIC_SLOTS);

        set_dynamic_slot_value(function->dynamic_slots, SYM_DYNAMIC_SLOTS_STELLA_METHOD_STARTUP_CLASSNAME, wrap_string("Startup-Dynamic-Slots"), NULL_STRING_WRAPPER);
      }
    }
    if (current_startup_time_phaseP(8)) {
      finalize_slots();
      cleanup_unfinalized_classes();
    }
  }
}

Keyword* KWD_DYNAMIC_SLOTS_DYNAMIC = NULL;

Keyword* KWD_DYNAMIC_SLOTS_NULL_WRAPPER = NULL;

Keyword* KWD_DYNAMIC_SLOTS_COMMON_LISP = NULL;

Surrogate* SGT_DYNAMIC_SLOTS_STELLA_OBJECT = NULL;

Symbol* SYM_DYNAMIC_SLOTS_STELLA_WRAPPER_VALUE = NULL;

Symbol* SYM_DYNAMIC_SLOTS_STELLA_SAFE_CAST = NULL;

Symbol* SYM_DYNAMIC_SLOTS_STELLA_DYNAMIC_SLOT_VALUE = NULL;

Symbol* SYM_DYNAMIC_SLOTS_STELLA_DYNAMIC_SLOTS = NULL;

Symbol* SYM_DYNAMIC_SLOTS_STELLA_QUOTE = NULL;

Symbol* SYM_DYNAMIC_SLOTS_STELLA_NULL = NULL;

Symbol* SYM_DYNAMIC_SLOTS_STELLA_SET_DYNAMIC_SLOT_VALUE = NULL;

Symbol* SYM_DYNAMIC_SLOTS_STELLA_BQUOTE = NULL;

Keyword* KWD_DYNAMIC_SLOTS_WRAP_FUNCTION = NULL;

Symbol* SYM_DYNAMIC_SLOTS_STELLA_STARTUP_DYNAMIC_SLOTS = NULL;

Symbol* SYM_DYNAMIC_SLOTS_STELLA_METHOD_STARTUP_CLASSNAME = NULL;

} // end of namespace stella
