// literals.cc

/*--------------------------- BEGIN LICENSE BLOCK ---------------------------+
|                                                                            |
| Version: MPL 1.1/GPL 2.0/LGPL 2.1                                          |
|                                                                            |
| The contents of this file are subject to the Mozilla Public License        |
| Version 1.1 (the "License"); you may not use this file except in           |
| compliance with the License. You may obtain a copy of the License at       |
| http://www.mozilla.org/MPL/                                                |
|                                                                            |
| Software distributed under the License is distributed on an "AS IS" basis, |
| WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License   |
| for the specific language governing rights and limitations under the       |
| License.                                                                   |
|                                                                            |
| The Original Code is the STELLA Programming Language.                      |
|                                                                            |
| The Initial Developer of the Original Code is                              |
| UNIVERSITY OF SOUTHERN CALIFORNIA, INFORMATION SCIENCES INSTITUTE          |
| 4676 Admiralty Way, Marina Del Rey, California 90292, U.S.A.               |
|                                                                            |
| Portions created by the Initial Developer are Copyright (C) 1996-2001      |
| the Initial Developer. All Rights Reserved.                                |
|                                                                            |
| Contributor(s):                                                            |
|                                                                            |
| Alternatively, the contents of this file may be used under the terms of    |
| either the GNU General Public License Version 2 or later (the "GPL"), or   |
| the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),   |
| in which case the provisions of the GPL or the LGPL are applicable instead |
| of those above. If you wish to allow use of your version of this file only |
| under the terms of either the GPL or the LGPL, and not to allow others to  |
| use your version of this file under the terms of the MPL, indicate your    |
| decision by deleting the provisions above and replace them with the notice |
| and other provisions required by the GPL or the LGPL. If you do not delete |
| the provisions above, a recipient may use your version of this file under  |
| the terms of any one of the MPL, the GPL or the LGPL.                      |
|                                                                            |
+---------------------------- END LICENSE BLOCK ----------------------------*/

#include "stella/stella-system.hh"

namespace stella {

Integer_Wrapper* ZERO_WRAPPER = NULL;

Integer_Wrapper* ONE_WRAPPER = NULL;

Boolean_Wrapper* TRUE_WRAPPER = NULL;

Boolean_Wrapper* FALSE_WRAPPER = NULL;

Integer_Wrapper* NULL_INTEGER_WRAPPER = NULL;

Float_Wrapper* NULL_FLOAT_WRAPPER = NULL;

String_Wrapper* NULL_STRING_WRAPPER = NULL;

Mutable_String_Wrapper* NULL_MUTABLE_STRING_WRAPPER = NULL;

Character_Wrapper* NULL_CHARACTER_WRAPPER = NULL;

Function_Code_Wrapper* NULL_FUNCTION_CODE_WRAPPER = NULL;

Method_Code_Wrapper* NULL_METHOD_CODE_WRAPPER = NULL;

boolean Wrapper::terminate_wrapperP() {
  { Wrapper* self = this;

    if ((self == NULL_FLOAT_WRAPPER) ||
        ((self == NULL_STRING_WRAPPER) ||
         ((self == NULL_MUTABLE_STRING_WRAPPER) ||
          ((self == NULL_CHARACTER_WRAPPER) ||
           ((self == NULL_FUNCTION_CODE_WRAPPER) ||
            (self == NULL_METHOD_CODE_WRAPPER)))))) {
      return (FALSE);
    }
    else {
      return (TRUE);
    }
  }
}

boolean Integer_Wrapper::terminate_wrapperP() {
  { Integer_Wrapper* self = this;

    if ((self == ZERO_WRAPPER) ||
        ((self == ONE_WRAPPER) ||
         (self == NULL_INTEGER_WRAPPER))) {
      return (FALSE);
    }
    else {
      return (TRUE);
    }
  }
}

// Table that holds a variety of information about literal
// types, e.g., the name of their null-wrapper, wrap-function, etc.
Hash_Table* oLITERAL_TYPE_INFO_TABLEo = NULL;

// List of literal types stored in '*literal-type-info-table*'.
// Maintained for iteration purposes.
List* oLITERAL_TYPESo = NULL;

Object* lookup_literal_type_info(Surrogate* type, Keyword* key) {
  { Key_Value_List* entry = ((Key_Value_List*)(oLITERAL_TYPE_INFO_TABLEo->lookup(type)));

    if (((boolean)(entry))) {
      return (entry->lookup(key));
    }
    else {
      return (NULL);
    }
  }
}

void set_literal_type_info(Surrogate* type, Keyword* key, Object* value) {
  { Key_Value_List* entry = ((Key_Value_List*)(oLITERAL_TYPE_INFO_TABLEo->lookup(type)));

    if (!((boolean)(entry))) {
      entry = new_key_value_list();
      oLITERAL_TYPE_INFO_TABLEo->insert_at(type, entry);
      oLITERAL_TYPESo->insert_last(type);
    }
    entry->insert_at(key, value);
  }
}

Integer_Wrapper* wrap_integer(int value) {
  // Return a literal object whose value is the INTEGER 'value'.
  if (value == NULL_INTEGER) {
    return (NULL_INTEGER_WRAPPER);
  }
  else {
    switch (value) {
      case 0: 
        return (ZERO_WRAPPER);
      case 1: 
        return (ONE_WRAPPER);
      default:
        return (new_integer_wrapper(value));
    }
  }
}

int unwrap_integer(Integer_Wrapper* wrapper) {
  // Unwrap `wrapper' and return the result.
  // Return NULL if `wrapper' is NULL.
  if (!((boolean)(wrapper))) {
    return (NULL_INTEGER);
  }
  else {
    return (wrapper->wrapper_value);
  }
}

Float_Wrapper* wrap_float(double value) {
  // Return a literal object whose value is the FLOAT 'value'.
  if (value == NULL_FLOAT) {
    return (NULL_FLOAT_WRAPPER);
  }
  else {
    return (new_float_wrapper(value));
  }
}

double unwrap_float(Float_Wrapper* wrapper) {
  // Unwrap `wrapper' and return the result.
  // Return NULL if `wrapper' is NULL.
  if (!((boolean)(wrapper))) {
    return (NULL_FLOAT);
  }
  else {
    return (wrapper->wrapper_value);
  }
}

String_Wrapper* wrap_string(char* value) {
  // Return a literal object whose value is the STRING 'value'.
  if (value == NULL) {
    return (NULL_STRING_WRAPPER);
  }
  else {
    return (new_string_wrapper(value));
  }
}

char* unwrap_string(String_Wrapper* wrapper) {
  // Unwrap `wrapper' and return the result.
  // Return NULL if `wrapper' is NULL.
  if (!((boolean)(wrapper))) {
    return (NULL);
  }
  else {
    return (wrapper->wrapper_value);
  }
}

Mutable_String_Wrapper* wrap_mutable_string(char* value) {
  // Return a literal object whose value is the MUTABLE-STRING 'value'.
  if (value == NULL) {
    return (NULL_MUTABLE_STRING_WRAPPER);
  }
  else {
    return (new_mutable_string_wrapper(value));
  }
}

char* unwrap_mutable_string(Mutable_String_Wrapper* wrapper) {
  // Unwrap `wrapper' and return the result.
  // Return NULL if `wrapper' is NULL.
  if (!((boolean)(wrapper))) {
    return (NULL);
  }
  else {
    return (wrapper->wrapper_value);
  }
}

Character_Wrapper* wrap_character(char value) {
  // Return a literal object whose value is the CHARACTER 'value'.
  if (value == NULL_CHARACTER) {
    return (NULL_CHARACTER_WRAPPER);
  }
  else {
    return (new_character_wrapper(value));
  }
}

char unwrap_character(Character_Wrapper* wrapper) {
  // Unwrap `wrapper' and return the result.
  // Return NULL if `wrapper' is NULL.
  if (!((boolean)(wrapper))) {
    return (NULL_CHARACTER);
  }
  else {
    return (wrapper->wrapper_value);
  }
}

Function_Code_Wrapper* wrap_function_code(cpp_function_code value) {
  // Return a literal object whose value is the FUNCTION-CODE 'value'.
  if (value == NULL) {
    return (NULL_FUNCTION_CODE_WRAPPER);
  }
  else {
    return (new_function_code_wrapper(value));
  }
}

cpp_function_code unwrap_function_code(Function_Code_Wrapper* wrapper) {
  // Unwrap `wrapper' and return the result.
  // Return NULL if `wrapper' is NULL.
  if (!((boolean)(wrapper))) {
    return (NULL);
  }
  else {
    return (wrapper->wrapper_value);
  }
}

Method_Code_Wrapper* wrap_method_code(cpp_method_code value) {
  // Return a literal object whose value is the METHOD-CODE 'value'.
  if (value == NULL) {
    return (NULL_METHOD_CODE_WRAPPER);
  }
  else {
    return (new_method_code_wrapper(value));
  }
}

cpp_method_code unwrap_method_code(Method_Code_Wrapper* wrapper) {
  // Unwrap `wrapper' and return the result.
  // Return NULL if `wrapper' is NULL.
  if (!((boolean)(wrapper))) {
    return (((cpp_method_code)(NULL)));
  }
  else {
    return (wrapper->wrapper_value);
  }
}

Boolean_Wrapper* wrap_boolean(boolean value) {
  // Return a literal object whose value is the BOOLEAN 'value'.
  return ((value ? TRUE_WRAPPER : FALSE_WRAPPER));
}

boolean unwrap_boolean(Boolean_Wrapper* wrapper) {
  // Unwrap `wrapper' and return its values as a regular BOOLEAN.
  // Map NULL onto FALSE.
  return (wrapper == TRUE_WRAPPER);
}

boolean coerce_wrapped_boolean_to_boolean(Boolean_Wrapper* wrapper) {
  if (!((boolean)(wrapper))) {
    *(STANDARD_WARNING->native_stream) << "Warning: " << "Coercing an undefined BOOLEAN-WRAPPER to FALSE.  Explicitly guard" << endl << " with `defined?' or use `unwrap-boolean' to avoid this warning." << endl;
  }
  return (wrapper == TRUE_WRAPPER);
}

Object* inline_wrap_boolean_expander(Object* expression) {
  if (expression == SYM_LITERALS_STELLA_TRUE) {
    return (SYM_LITERALS_STELLA_TRUE_WRAPPER);
  }
  else if (expression == SYM_LITERALS_STELLA_FALSE) {
    return (SYM_LITERALS_STELLA_FALSE_WRAPPER);
  }
  else {
    return (listO(3, SYM_LITERALS_STELLA_WRAP_BOOLEAN, expression, NIL));
  }
}

Object* inline_unwrap_boolean_expander(Object* expression) {
  if (expression == SYM_LITERALS_STELLA_TRUE_WRAPPER) {
    return (SYM_LITERALS_STELLA_TRUE);
  }
  else if (expression == SYM_LITERALS_STELLA_FALSE_WRAPPER) {
    return (SYM_LITERALS_STELLA_FALSE);
  }
  else {
    return (listO(3, SYM_LITERALS_STELLA_COERCE_WRAPPED_BOOLEAN_TO_BOOLEAN, expression, NIL));
  }
}

Integer_Wrapper* integer_wrap_literal(int value) {
  return (wrap_integer(value));
}

Float_Wrapper* float_wrap_literal(double value) {
  return (wrap_float(value));
}

Mutable_String_Wrapper* mutable_string_wrap_literal(char* value) {
  return (wrap_mutable_string(value));
}

String_Wrapper* string_wrap_literal(char* value) {
  return (wrap_string(value));
}

Character_Wrapper* character_wrap_literal(char value) {
  return (wrap_character(value));
}

Function_Code_Wrapper* function_code_wrap_literal(cpp_function_code value) {
  return (wrap_function_code(value));
}

Method_Code_Wrapper* method_code_wrap_literal(cpp_method_code value) {
  return (wrap_method_code(value));
}

Object* Standard_Object::copy_wrapped_literal() {
  { Standard_Object* self = this;

    return (self);
  }
}

Object* Literal_Wrapper::copy_wrapped_literal() {
  { Literal_Wrapper* self = this;

    return (self);
  }
}

Object* Integer_Wrapper::copy_wrapped_literal() {
  { Integer_Wrapper* self = this;

    return (wrap_integer(self->wrapper_value));
  }
}

Object* Float_Wrapper::copy_wrapped_literal() {
  { Float_Wrapper* self = this;

    return (wrap_float(self->wrapper_value));
  }
}

Object* String_Wrapper::copy_wrapped_literal() {
  { String_Wrapper* self = this;

    return (wrap_string(self->wrapper_value));
  }
}

Object* Mutable_String_Wrapper::copy_wrapped_literal() {
  { Mutable_String_Wrapper* self = this;

    return (wrap_mutable_string(self->wrapper_value));
  }
}

Object* Character_Wrapper::copy_wrapped_literal() {
  { Character_Wrapper* self = this;

    return (wrap_character(self->wrapper_value));
  }
}

Object* Function_Code_Wrapper::copy_wrapped_literal() {
  { Function_Code_Wrapper* self = this;

    return (wrap_function_code(self->wrapper_value));
  }
}

Object* Method_Code_Wrapper::copy_wrapped_literal() {
  { Method_Code_Wrapper* self = this;

    return (wrap_method_code(self->wrapper_value));
  }
}

boolean Object::object_eqlP(Object* y) {
  { Object* x = this;

    { Output_String_Stream* stream_000 = new_output_string_stream();

      *(stream_000->native_stream) << "object-eql?: Don't know how to compare " << "`" << x << "'" << " with " << "`" << y << "'";
      throw *new_stella_exception(stream_000->the_string_reader());
    }
  }
}

boolean Standard_Object::object_eqlP(Object* y) {
  { Standard_Object* x = this;

    return (x == y);
  }
}

boolean Integer_Wrapper::object_eqlP(Object* y) {
  { Integer_Wrapper* x = this;

    return (((boolean)(y)) &&
        ((y->primary_type() == SGT_LITERALS_STELLA_INTEGER_WRAPPER) &&
         (x->wrapper_value == ((Integer_Wrapper*)(y))->wrapper_value)));
  }
}

boolean Float_Wrapper::object_eqlP(Object* y) {
  { Float_Wrapper* x = this;

    return (((boolean)(y)) &&
        ((y->primary_type() == SGT_LITERALS_STELLA_FLOAT_WRAPPER) &&
         (x->wrapper_value == ((Float_Wrapper*)(y))->wrapper_value)));
  }
}

boolean Boolean_Wrapper::object_eqlP(Object* y) {
  { Boolean_Wrapper* x = this;

    return (x == y);
  }
}

boolean String_Wrapper::object_eqlP(Object* y) {
  { String_Wrapper* x = this;

    if (((boolean)(y))) {
      { Surrogate* ytype = y->primary_type();

        if ((ytype == SGT_LITERALS_STELLA_STRING_WRAPPER) ||
            ((ytype == SGT_LITERALS_STELLA_VERBATIM_STRING_WRAPPER) ||
             ((!oCLASS_HIERARCHY_BOOTEDpo) &&
              ((!((boolean)(ytype))) ||
               ((!((boolean)(SGT_LITERALS_STELLA_STRING_WRAPPER))) ||
                (!((boolean)(SGT_LITERALS_STELLA_VERBATIM_STRING_WRAPPER)))))))) {
          return (string_eqlP(x->wrapper_value, ((String_Wrapper*)(y))->wrapper_value));
        }
      }
    }
    return (FALSE);
  }
}

boolean Mutable_String_Wrapper::object_eqlP(Object* y) {
  { Mutable_String_Wrapper* x = this;

    return (x == y);
  }
}

boolean Character_Wrapper::object_eqlP(Object* y) {
  { Character_Wrapper* x = this;

    return (((boolean)(y)) &&
        ((y->primary_type() == SGT_LITERALS_STELLA_CHARACTER_WRAPPER) &&
         (x->wrapper_value == ((Character_Wrapper*)(y))->wrapper_value)));
  }
}

boolean eql_to_booleanP(Object* y, boolean x) {
  return (booleanP(y) &&
      (((Boolean_Wrapper*)(y))->wrapper_value == x));
}

boolean eql_to_integerP(Object* y, int x) {
  return (integerP(y) &&
      (((Integer_Wrapper*)(y))->wrapper_value == x));
}

boolean eql_to_floatP(Object* y, double x) {
  return (floatP(y) &&
      (((Float_Wrapper*)(y))->wrapper_value == x));
}

boolean eql_to_stringP(Object* y, char* x) {
  return (stringP(y) &&
      string_eqlP(((String_Wrapper*)(y))->wrapper_value, x));
}

boolean eql_to_characterP(Object* y, char x) {
  return (characterP(y) &&
      (((Character_Wrapper*)(y))->wrapper_value == x));
}

Object* help_bquotify(Object* tree) {
  if (!((boolean)(tree))) {
    return (SYM_LITERALS_STELLA_NULL);
  }
  { Surrogate* test_value_000 = safe_primary_type(tree);

    if (test_value_000 == SGT_LITERALS_STELLA_CONS) {
      { Object* tree_000 = tree;
        Cons* tree = ((Cons*)(tree_000));

        if (tree == NIL) {
          return (SYM_LITERALS_STELLA_NIL);
        }
        else {
          return (cons_list(3, SYM_LITERALS_STELLA_CONS, help_bquotify(tree->value), help_bquotify(tree->rest)));
        }
      }
    }
    else if (subtype_of_integerP(test_value_000)) {
      { Object* tree_001 = tree;
        Integer_Wrapper* tree = ((Integer_Wrapper*)(tree_001));

        return (cons_list(2, SYM_LITERALS_STELLA_WRAP_LITERAL, tree));
      }
    }
    else if (subtype_of_stringP(test_value_000)) {
      { Object* tree_002 = tree;
        String_Wrapper* tree = ((String_Wrapper*)(tree_002));

        return (cons_list(2, SYM_LITERALS_STELLA_WRAP_LITERAL, tree));
      }
    }
    else if (subtype_of_floatP(test_value_000)) {
      { Object* tree_003 = tree;
        Float_Wrapper* tree = ((Float_Wrapper*)(tree_003));

        return (cons_list(2, SYM_LITERALS_STELLA_WRAP_LITERAL, tree));
      }
    }
    else if (subtype_of_characterP(test_value_000)) {
      { Object* tree_004 = tree;
        Character_Wrapper* tree = ((Character_Wrapper*)(tree_004));

        return (cons_list(2, SYM_LITERALS_STELLA_WRAP_LITERAL, tree));
      }
    }
    else if (subtype_of_keywordP(test_value_000)) {
      { Object* tree_005 = tree;
        Keyword* tree = ((Keyword*)(tree_005));

        if (use_hardcoded_symbolsP()) {
          return (cons_list(2, SYM_LITERALS_STELLA_GET_KWD, wrap_integer(tree->symbol_id)));
        }
        else {
          return (cons_list(3, SYM_LITERALS_STELLA_TYPED_SYS, register_symbol(tree), SGT_LITERALS_STELLA_SYMBOL));
        }
      }
    }
    else if (subtype_of_surrogateP(test_value_000)) {
      { Object* tree_006 = tree;
        Surrogate* tree = ((Surrogate*)(tree_006));

        if (use_hardcoded_symbolsP()) {
          return (cons_list(2, SYM_LITERALS_STELLA_GET_SGT, wrap_integer(tree->symbol_id)));
        }
        else {
          return (cons_list(3, SYM_LITERALS_STELLA_TYPED_SYS, register_symbol(tree), SGT_LITERALS_STELLA_SYMBOL));
        }
      }
    }
    else if (subtype_of_symbolP(test_value_000)) {
      { Object* tree_007 = tree;
        Symbol* tree = ((Symbol*)(tree_007));

        if (symbol_common_lispP(tree)) {
          return (cons_list(2, SYM_LITERALS_STELLA_INTERN_COMMON_LISP_SYMBOL, wrap_string(tree->symbol_name)));
        }
        else {
          if (use_hardcoded_symbolsP()) {
            return (cons_list(2, SYM_LITERALS_STELLA_GET_SYM, wrap_integer(((Symbol*)(tree->permanentify()))->symbol_id)));
          }
          else {
            return (cons_list(3, SYM_LITERALS_STELLA_TYPED_SYS, register_symbol(tree), SGT_LITERALS_STELLA_SYMBOL));
          }
        }
      }
    }
    else {
      { 
        BIND_STELLA_SPECIAL(oPRINTREADABLYpo, boolean, TRUE);
        signal_translation_error();
        if (!(suppress_warningsP())) {
          print_error_context(">> ERROR: ", STANDARD_ERROR);
          *(STANDARD_ERROR->native_stream) << endl << " " << "Illegal argument to quote: " << "`" << de_uglify_parse_tree(tree) << "'" << "." << endl;
        }
      }
      return (NULL);
    }
  }
}

Object* bquotify(Object* tree) {
  return (help_bquotify(tree));
}

Object* expand_bquote_tree(Object* tree) {
  if (safe_primary_type(tree) == SGT_LITERALS_STELLA_CONS) {
    { Object* tree_000 = tree;
      Cons* tree = ((Cons*)(tree_000));

      { Cons* expandedtree = cons(SYM_LITERALS_STELLA_LISTo, NIL);
        Cons* cursor = tree;
        Object* term = cursor->value;

        while ((!(cursor == NIL)) &&
            (!((term == SYM_LITERALS_STELLA_a) ||
            (term == SYM_LITERALS_STELLA_aa)))) {
          expandedtree = cons(expand_bquote_tree(term), expandedtree);
          cursor = cursor->rest;
          term = cursor->value;
        }
        if (cursor == NIL) {
          return (cons(SYM_LITERALS_STELLA_NIL, expandedtree)->reverse());
        }
        else if (term == SYM_LITERALS_STELLA_a) {
          cursor = cursor->rest;
          if (cursor == NIL) {
            { 
              BIND_STELLA_SPECIAL(oPRINTREADABLYpo, boolean, TRUE);
              signal_translation_error();
              if (!(suppress_warningsP())) {
                print_error_context(">> ERROR: ", STANDARD_ERROR);
                *(STANDARD_ERROR->native_stream) << endl << " " << "Illegal bquote tree -- missing term after '&'." << "." << endl;
              }
            }
          }
          return (cons(expand_bquote_tree(cursor->rest), cons(cursor->value, expandedtree))->reverse());
        }
        else {
          cursor = cursor->rest;
          if (cursor == NIL) {
            { 
              BIND_STELLA_SPECIAL(oPRINTREADABLYpo, boolean, TRUE);
              signal_translation_error();
              if (!(suppress_warningsP())) {
                print_error_context(">> ERROR: ", STANDARD_ERROR);
                *(STANDARD_ERROR->native_stream) << endl << " " << "Illegal bquote tree -- missing term after '&&'" << "." << endl;
              }
            }
          }
          return (cons(cons_list(3, SYM_LITERALS_STELLA_CONCATENATE, cursor->value, expand_bquote_tree(cursor->rest)), expandedtree)->reverse());
        }
      }
    }
  }
  else {
    return (bquotify(tree));
  }
}

Object* simplify_bquote_tree(Object* tree) {
  if (safe_primary_type(tree) == SGT_LITERALS_STELLA_CONS) {
    { Object* tree_000 = tree;
      Cons* tree = ((Cons*)(tree_000));

      { Cons* cursor = tree->rest;

        while (!(cursor == NIL)) {
          cursor->value = simplify_bquote_tree(cursor->value);
          cursor = cursor->rest;
        }
        if (!(tree->value == SYM_LITERALS_STELLA_LISTo)) {
          return (tree);
        }
        switch (tree->rest->length()) {
          case 1: 
            return (tree->rest->value);
          case 2: 
            tree->value = SYM_LITERALS_STELLA_CONS;
            return (tree);
          default:
            return (tree);
        }
      }
    }
  }
  else {
    return (tree);
  }
}

Object* Object::permanentify() {
  { Object* self = this;

    return (self);
  }
}

Object* Symbol::permanentify() {
  { Symbol* self = this;

    return (self);
  }
}

Object* Transient_Symbol::permanentify() {
  { Transient_Symbol* self = this;

    return (intern_permanent_symbol(self->symbol_name));
  }
}

Object* Literal_Wrapper::permanentify() {
  { Literal_Wrapper* self = this;

    return (((Literal_Wrapper*)(self->copy_wrapped_literal())));
  }
}

Object* permanent_copy(Object* tree) {
  if (safe_primary_type(tree) == SGT_LITERALS_STELLA_CONS) {
    { Object* tree_000 = tree;
      Cons* tree = ((Cons*)(tree_000));

      if (tree == NIL) {
        return (tree);
      }
      else {
        return (cons(permanent_copy(tree->value), ((Cons*)(permanent_copy(tree->rest)))));
      }
    }
  }
  else {
    return (tree->permanentify());
  }
}

Symbol* Symbol::soft_permanentify() {
  { Symbol* symbol = this;

    return (symbol);
  }
}

Symbol* Transient_Symbol::soft_permanentify() {
  { Transient_Symbol* symbol = this;

    { Generalized_Symbol* permanentsymbol = lookup_rigid_symbol_wrt_module(symbol->symbol_name, (((boolean)(((Module*)(symbol->home_context)))) ? ((Module*)(symbol->home_context)) : oMODULEo.get()), SYMBOL_SYM);

      if (((boolean)(permanentsymbol))) {
        return (((Symbol*)(permanentsymbol)));
      }
      else {
        return (symbol);
      }
    }
  }
}

void print_character(char renamed_Char, ostream* stream) {
  *(stream) << "#\\";
  switch (renamed_Char) {
    case '\n': 
      *(stream) << "Linefeed";
    break;
    case '\b': 
      *(stream) << "Backspace";
    break;
    case '\t': 
      *(stream) << "Tab";
    break;
    case '\r': 
      *(stream) << "Return";
    break;
    case '\f': 
      *(stream) << "Page";
    break;
    default:
      *(stream) << renamed_Char;
    break;
  }
}

Keyword** create_character_type_table() {
  { Keyword** table = new (GC)(Keyword*[256]);

    { int code = NULL_INTEGER;
      int iter_000 = 0;
      int upper_bound_000 = 255;

      while (iter_000 <= upper_bound_000) {
        code = iter_000;
        iter_000 = iter_000 + 1;
        table[code] = KWD_LITERALS_OTHER;
      }
    }
    { int code = NULL_INTEGER;
      int iter_001 = (int)(unsigned char) '0';
      int upper_bound_001 = (int)(unsigned char) '9';
      boolean unboundedP_000 = upper_bound_001 == NULL_INTEGER;

      while (unboundedP_000 ||
          (iter_001 <= upper_bound_001)) {
        code = iter_001;
        iter_001 = iter_001 + 1;
        table[code] = KWD_LITERALS_DIGIT;
      }
    }
    { int code = NULL_INTEGER;
      int iter_002 = (int)(unsigned char) 'A';
      int upper_bound_002 = (int)(unsigned char) 'Z';
      boolean unboundedP_001 = upper_bound_002 == NULL_INTEGER;

      while (unboundedP_001 ||
          (iter_002 <= upper_bound_002)) {
        code = iter_002;
        iter_002 = iter_002 + 1;
        table[code] = KWD_LITERALS_LETTER;
      }
    }
    { int code = NULL_INTEGER;
      int iter_003 = (int)(unsigned char) 'a';
      int upper_bound_003 = (int)(unsigned char) 'z';
      boolean unboundedP_002 = upper_bound_003 == NULL_INTEGER;

      while (unboundedP_002 ||
          (iter_003 <= upper_bound_003)) {
        code = iter_003;
        iter_003 = iter_003 + 1;
        table[code] = KWD_LITERALS_LETTER;
      }
    }
    table[(int)(unsigned char) '!'] = KWD_LITERALS_SYMBOL_CONSTITUENT;
    table[(int)(unsigned char) '$'] = KWD_LITERALS_SYMBOL_CONSTITUENT;
    table[(int)(unsigned char) '%'] = KWD_LITERALS_SYMBOL_CONSTITUENT;
    table[(int)(unsigned char) '<'] = KWD_LITERALS_SYMBOL_CONSTITUENT;
    table[(int)(unsigned char) '>'] = KWD_LITERALS_SYMBOL_CONSTITUENT;
    table[(int)(unsigned char) '='] = KWD_LITERALS_SYMBOL_CONSTITUENT;
    table[(int)(unsigned char) '?'] = KWD_LITERALS_SYMBOL_CONSTITUENT;
    table[(int)(unsigned char) '['] = KWD_LITERALS_SYMBOL_CONSTITUENT;
    table[(int)(unsigned char) ']'] = KWD_LITERALS_SYMBOL_CONSTITUENT;
    table[(int)(unsigned char) '^'] = KWD_LITERALS_SYMBOL_CONSTITUENT;
    table[(int)(unsigned char) '_'] = KWD_LITERALS_SYMBOL_CONSTITUENT;
    table[(int)(unsigned char) '{'] = KWD_LITERALS_SYMBOL_CONSTITUENT;
    table[(int)(unsigned char) '}'] = KWD_LITERALS_SYMBOL_CONSTITUENT;
    table[(int)(unsigned char) '~'] = KWD_LITERALS_SYMBOL_CONSTITUENT;
    table[(int)(unsigned char) '*'] = KWD_LITERALS_SYMBOL_CONSTITUENT;
    table[(int)(unsigned char) '.'] = KWD_LITERALS_SYMBOL_CONSTITUENT;
    table[(int)(unsigned char) '+'] = KWD_LITERALS_SYMBOL_CONSTITUENT;
    table[(int)(unsigned char) '-'] = KWD_LITERALS_SYMBOL_CONSTITUENT;
    table[(int)(unsigned char) ':'] = KWD_LITERALS_SYMBOL_QUALIFIER;
    table[(int)(unsigned char) '/'] = KWD_LITERALS_SYMBOL_QUALIFIER;
    table[(int)(unsigned char) '@'] = KWD_LITERALS_SYMBOL_QUALIFIER;
    table[(int)(unsigned char) '\\'] = KWD_LITERALS_ESCAPE;
    table[(int)(unsigned char) '('] = KWD_LITERALS_DELIMITER;
    table[(int)(unsigned char) ')'] = KWD_LITERALS_DELIMITER;
    table[(int)(unsigned char) '"'] = KWD_LITERALS_DELIMITER;
    table[(int)(unsigned char) '|'] = KWD_LITERALS_DELIMITER;
    table[(int)(unsigned char) '\''] = KWD_LITERALS_DELIMITER;
    table[(int)(unsigned char) '`'] = KWD_LITERALS_DELIMITER;
    table[(int)(unsigned char) ','] = KWD_LITERALS_DELIMITER;
    table[(int)(unsigned char) ' '] = KWD_LITERALS_WHITE_SPACE;
    table[(int)(unsigned char) '\t'] = KWD_LITERALS_WHITE_SPACE;
    table[(int)(unsigned char) '\n'] = KWD_LITERALS_WHITE_SPACE;
    table[(int)(unsigned char) '\r'] = KWD_LITERALS_WHITE_SPACE;
    table[(int)(unsigned char) '\n'] = KWD_LITERALS_WHITE_SPACE;
    table[(int)(unsigned char) '\f'] = KWD_LITERALS_WHITE_SPACE;
    return (table);
  }
}

// Table of character types.  Entry 'i' represents the type
// of the character whose 'char-code' equals 'i'.  Each character is classified 
// by one of the following keywords: :DIGIT, :LETTER, :SYMBOL-CONSTITUENT, 
// :SYMBOL-QUALIFIER, :ESCAPE, :DELIMITER, :WHITE-SPACE, or :OTHER.
Keyword** oCHARACTER_TYPE_TABLEo = NULL;

boolean digit_characterP(char ch) {
  // Return TRUE if `ch' represents a digit.
  return (oCHARACTER_TYPE_TABLEo[(int)(unsigned char) ch] == KWD_LITERALS_DIGIT);
}

boolean letter_characterP(char ch) {
  // Return TRUE if `ch' represents a letter.
  return (oCHARACTER_TYPE_TABLEo[(int)(unsigned char) ch] == KWD_LITERALS_LETTER);
}

boolean upper_case_characterP(char ch) {
  // Return TRUE if `ch' represents an upper-case character.
  { int charcode = (int)(unsigned char) ch;

    return ((charcode >= (int)(unsigned char) 'A') &&
        (charcode <= (int)(unsigned char) 'Z'));
  }
}

boolean lower_case_characterP(char ch) {
  // Return TRUE if `ch' represents a lower-case character.
  { int charcode = (int)(unsigned char) ch;

    return ((charcode >= (int)(unsigned char) 'a') &&
        (charcode <= (int)(unsigned char) 'z'));
  }
}

boolean white_space_characterP(char ch) {
  // Return TRUE if `ch' is a white space character.
  return (oCHARACTER_TYPE_TABLEo[(int)(unsigned char) ch] == KWD_LITERALS_WHITE_SPACE);
}

boolean all_upper_case_stringP(char* s) {
  // Return TRUE if all letters in `s' are upper case.
  { int code = 0;
    int codea = (int)(unsigned char) 'a';
    int codez = (int)(unsigned char) 'z';

    { char ch = NULL_CHARACTER;
      char* vector_000 = s;
      int index_000 = 0;
      int length_000 = strlen(vector_000);

      while (index_000 < length_000) {
        ch = vector_000[index_000];
        index_000 = index_000 + 1;
        code = (int)(unsigned char) ch;
        if ((code >= codea) &&
            (code <= codez)) {
          return (FALSE);
        }
      }
    }
    return (TRUE);
  }
}

boolean all_lower_case_stringP(char* s) {
  // Return TRUE if all letters in `s' are lower case.
  { int code = 0;
    int codea = (int)(unsigned char) 'A';
    int codez = (int)(unsigned char) 'Z';

    { char ch = NULL_CHARACTER;
      char* vector_000 = s;
      int index_000 = 0;
      int length_000 = strlen(vector_000);

      while (index_000 < length_000) {
        ch = vector_000[index_000];
        index_000 = index_000 + 1;
        code = (int)(unsigned char) ch;
        if ((code <= codez) &&
            (code >= codea)) {
          return (FALSE);
        }
      }
    }
    return (TRUE);
  }
}

char* oCHARACTER_UPCASE_TABLEo = NULL;

char* initialize_character_upcase_table() {
  { char* buffer = make_string(256, NULL_CHARACTER);

    { int i = NULL_INTEGER;
      int iter_000 = 0;
      int upper_bound_000 = 255;

      while (iter_000 <= upper_bound_000) {
        i = iter_000;
        iter_000 = iter_000 + 1;
        buffer[i] = ((char) i);
        if ((i >= (int)(unsigned char) 'a') &&
            (i <= (int)(unsigned char) 'z')) {
          buffer[i] = ((char) ((int)(unsigned char) 'A' + (i - (int)(unsigned char) 'a')));
        }
      }
    }
    return (buffer);
  }
}

char* oCHARACTER_DOWNCASE_TABLEo = NULL;

char* initialize_character_downcase_table() {
  { char* buffer = make_string(256, NULL_CHARACTER);

    { int i = NULL_INTEGER;
      int iter_000 = 0;
      int upper_bound_000 = 255;

      while (iter_000 <= upper_bound_000) {
        i = iter_000;
        iter_000 = iter_000 + 1;
        buffer[i] = ((char) i);
        if ((i >= (int)(unsigned char) 'A') &&
            (i <= (int)(unsigned char) 'Z')) {
          buffer[i] = ((char) ((int)(unsigned char) 'a' + (i - (int)(unsigned char) 'A')));
        }
      }
    }
    return (buffer);
  }
}

char upcase_character(char renamed_Char) {
  // If `char' is lowercase, return its uppercase version,
  // otherwise, return 'char' unmodified.
  return (oCHARACTER_UPCASE_TABLEo[((int)(unsigned char) renamed_Char)]);
}

char downcase_character(char renamed_Char) {
  // If `char' is uppercase, return its lowercase version,
  // otherwise, return 'char' unmodified.
  return (oCHARACTER_DOWNCASE_TABLEo[((int)(unsigned char) renamed_Char)]);
}

void print_string_readably(char* string, ostream* stream) {
  *(stream) << "\"";
  { int i = NULL_INTEGER;
    int iter_000 = 0;
    int upper_bound_000 = strlen(string) - 1;

    while (iter_000 <= upper_bound_000) {
      i = iter_000;
      iter_000 = iter_000 + 1;
      { char renamed_Char = string[i];

        if ((renamed_Char == '"') ||
            (renamed_Char == '\\')) {
          *(stream) << "\\" << renamed_Char;
        }
        else {
          *(stream) << renamed_Char;
        }
      }
    }
  }
  *(stream) << "\"";
}

char* string_to_mutable_string(char* s) {
  return (strcpy(new (GC) char[strlen(s)+1], s));
}

char* mutable_string_to_string(char* s) {
  return (s);
}

double Number_Wrapper::number_wrapper_to_float() {
  { Number_Wrapper* self = this;

    return (NULL_FLOAT);
  }
}

double Integer_Wrapper::number_wrapper_to_float() {
  { Integer_Wrapper* self = this;

    return (self->wrapper_value * 1.0);
  }
}

double Float_Wrapper::number_wrapper_to_float() {
  { Float_Wrapper* self = this;

    return (self->wrapper_value);
  }
}

void help_startup_literals1() {
  {
    SGT_LITERALS_STELLA_BOOLEAN = ((Surrogate*)(intern_rigid_symbol_wrt_module("BOOLEAN", NULL, 1)));
    KWD_LITERALS_NULL_WRAPPER = ((Keyword*)(intern_rigid_symbol_wrt_module("NULL-WRAPPER", NULL, 2)));
    SYM_LITERALS_STELLA_FALSE_WRAPPER = ((Symbol*)(intern_rigid_symbol_wrt_module("FALSE-WRAPPER", NULL, 0)));
    SGT_LITERALS_STELLA_INTEGER = ((Surrogate*)(intern_rigid_symbol_wrt_module("INTEGER", NULL, 1)));
    SYM_LITERALS_STELLA_NULL_INTEGER_WRAPPER = ((Symbol*)(intern_rigid_symbol_wrt_module("NULL-INTEGER-WRAPPER", NULL, 0)));
    SGT_LITERALS_STELLA_FLOAT = ((Surrogate*)(intern_rigid_symbol_wrt_module("FLOAT", NULL, 1)));
    SYM_LITERALS_STELLA_NULL_FLOAT_WRAPPER = ((Symbol*)(intern_rigid_symbol_wrt_module("NULL-FLOAT-WRAPPER", NULL, 0)));
    SGT_LITERALS_STELLA_STRING = ((Surrogate*)(intern_rigid_symbol_wrt_module("STRING", NULL, 1)));
    SYM_LITERALS_STELLA_NULL_STRING_WRAPPER = ((Symbol*)(intern_rigid_symbol_wrt_module("NULL-STRING-WRAPPER", NULL, 0)));
    SGT_LITERALS_STELLA_MUTABLE_STRING = ((Surrogate*)(intern_rigid_symbol_wrt_module("MUTABLE-STRING", NULL, 1)));
    SYM_LITERALS_STELLA_NULL_MUTABLE_STRING_WRAPPER = ((Symbol*)(intern_rigid_symbol_wrt_module("NULL-MUTABLE-STRING-WRAPPER", NULL, 0)));
    SGT_LITERALS_STELLA_CHARACTER = ((Surrogate*)(intern_rigid_symbol_wrt_module("CHARACTER", NULL, 1)));
    SYM_LITERALS_STELLA_NULL_CHARACTER_WRAPPER = ((Symbol*)(intern_rigid_symbol_wrt_module("NULL-CHARACTER-WRAPPER", NULL, 0)));
    SGT_LITERALS_STELLA_FUNCTION_CODE = ((Surrogate*)(intern_rigid_symbol_wrt_module("FUNCTION-CODE", NULL, 1)));
    SYM_LITERALS_STELLA_NULL_FUNCTION_CODE_WRAPPER = ((Symbol*)(intern_rigid_symbol_wrt_module("NULL-FUNCTION-CODE-WRAPPER", NULL, 0)));
    SGT_LITERALS_STELLA_METHOD_CODE = ((Surrogate*)(intern_rigid_symbol_wrt_module("METHOD-CODE", NULL, 1)));
    SYM_LITERALS_STELLA_NULL_METHOD_CODE_WRAPPER = ((Symbol*)(intern_rigid_symbol_wrt_module("NULL-METHOD-CODE-WRAPPER", NULL, 0)));
    SYM_LITERALS_STELLA_TRUE = ((Symbol*)(intern_rigid_symbol_wrt_module("TRUE", NULL, 0)));
    SYM_LITERALS_STELLA_TRUE_WRAPPER = ((Symbol*)(intern_rigid_symbol_wrt_module("TRUE-WRAPPER", NULL, 0)));
    SYM_LITERALS_STELLA_FALSE = ((Symbol*)(intern_rigid_symbol_wrt_module("FALSE", NULL, 0)));
    SYM_LITERALS_STELLA_WRAP_BOOLEAN = ((Symbol*)(intern_rigid_symbol_wrt_module("WRAP-BOOLEAN", NULL, 0)));
    SYM_LITERALS_STELLA_INLINE_WRAP_BOOLEAN = ((Symbol*)(intern_rigid_symbol_wrt_module("INLINE-WRAP-BOOLEAN", NULL, 0)));
    SYM_LITERALS_STELLA_INLINE_WRAP_BOOLEAN_EXPANDER = ((Symbol*)(intern_rigid_symbol_wrt_module("INLINE-WRAP-BOOLEAN-EXPANDER", NULL, 0)));
    SYM_LITERALS_STELLA_COERCE_WRAPPED_BOOLEAN_TO_BOOLEAN = ((Symbol*)(intern_rigid_symbol_wrt_module("COERCE-WRAPPED-BOOLEAN-TO-BOOLEAN", NULL, 0)));
    SYM_LITERALS_STELLA_INLINE_UNWRAP_BOOLEAN = ((Symbol*)(intern_rigid_symbol_wrt_module("INLINE-UNWRAP-BOOLEAN", NULL, 0)));
    SYM_LITERALS_STELLA_INLINE_UNWRAP_BOOLEAN_EXPANDER = ((Symbol*)(intern_rigid_symbol_wrt_module("INLINE-UNWRAP-BOOLEAN-EXPANDER", NULL, 0)));
    KWD_LITERALS_WRAP_FUNCTION = ((Keyword*)(intern_rigid_symbol_wrt_module("WRAP-FUNCTION", NULL, 2)));
    SYM_LITERALS_STELLA_WRAP_INTEGER = ((Symbol*)(intern_rigid_symbol_wrt_module("WRAP-INTEGER", NULL, 0)));
    SYM_LITERALS_STELLA_WRAP_FLOAT = ((Symbol*)(intern_rigid_symbol_wrt_module("WRAP-FLOAT", NULL, 0)));
    SYM_LITERALS_STELLA_WRAP_STRING = ((Symbol*)(intern_rigid_symbol_wrt_module("WRAP-STRING", NULL, 0)));
    SYM_LITERALS_STELLA_WRAP_CHARACTER = ((Symbol*)(intern_rigid_symbol_wrt_module("WRAP-CHARACTER", NULL, 0)));
    SYM_LITERALS_STELLA_WRAP_FUNCTION_CODE = ((Symbol*)(intern_rigid_symbol_wrt_module("WRAP-FUNCTION-CODE", NULL, 0)));
    SYM_LITERALS_STELLA_WRAP_METHOD_CODE = ((Symbol*)(intern_rigid_symbol_wrt_module("WRAP-METHOD-CODE", NULL, 0)));
    SGT_LITERALS_STELLA_INTEGER_WRAPPER = ((Surrogate*)(intern_rigid_symbol_wrt_module("INTEGER-WRAPPER", NULL, 1)));
    SGT_LITERALS_STELLA_FLOAT_WRAPPER = ((Surrogate*)(intern_rigid_symbol_wrt_module("FLOAT-WRAPPER", NULL, 1)));
    SGT_LITERALS_STELLA_STRING_WRAPPER = ((Surrogate*)(intern_rigid_symbol_wrt_module("STRING-WRAPPER", NULL, 1)));
    SGT_LITERALS_STELLA_VERBATIM_STRING_WRAPPER = ((Surrogate*)(intern_rigid_symbol_wrt_module("VERBATIM-STRING-WRAPPER", NULL, 1)));
    SGT_LITERALS_STELLA_CHARACTER_WRAPPER = ((Surrogate*)(intern_rigid_symbol_wrt_module("CHARACTER-WRAPPER", NULL, 1)));
    SYM_LITERALS_STELLA_NULL = ((Symbol*)(intern_rigid_symbol_wrt_module("NULL", NULL, 0)));
    SGT_LITERALS_STELLA_CONS = ((Surrogate*)(intern_rigid_symbol_wrt_module("CONS", NULL, 1)));
    SYM_LITERALS_STELLA_NIL = ((Symbol*)(intern_rigid_symbol_wrt_module("NIL", NULL, 0)));
    SYM_LITERALS_STELLA_CONS = ((Symbol*)(intern_rigid_symbol_wrt_module("CONS", NULL, 0)));
    SYM_LITERALS_STELLA_WRAP_LITERAL = ((Symbol*)(intern_rigid_symbol_wrt_module("WRAP-LITERAL", NULL, 0)));
    SYM_LITERALS_STELLA_GET_KWD = ((Symbol*)(intern_rigid_symbol_wrt_module("GET-KWD", NULL, 0)));
    SYM_LITERALS_STELLA_TYPED_SYS = ((Symbol*)(intern_rigid_symbol_wrt_module("TYPED-SYS", NULL, 0)));
    SGT_LITERALS_STELLA_SYMBOL = ((Surrogate*)(intern_rigid_symbol_wrt_module("SYMBOL", NULL, 1)));
    SYM_LITERALS_STELLA_GET_SGT = ((Symbol*)(intern_rigid_symbol_wrt_module("GET-SGT", NULL, 0)));
    SYM_LITERALS_STELLA_INTERN_COMMON_LISP_SYMBOL = ((Symbol*)(intern_rigid_symbol_wrt_module("INTERN-COMMON-LISP-SYMBOL", NULL, 0)));
    SYM_LITERALS_STELLA_GET_SYM = ((Symbol*)(intern_rigid_symbol_wrt_module("GET-SYM", NULL, 0)));
    SYM_LITERALS_STELLA_LISTo = ((Symbol*)(intern_rigid_symbol_wrt_module("LIST*", NULL, 0)));
    SYM_LITERALS_STELLA_a = ((Symbol*)(intern_rigid_symbol_wrt_module("&", NULL, 0)));
    SYM_LITERALS_STELLA_aa = ((Symbol*)(intern_rigid_symbol_wrt_module("&&", NULL, 0)));
    SYM_LITERALS_STELLA_CONCATENATE = ((Symbol*)(intern_rigid_symbol_wrt_module("CONCATENATE", NULL, 0)));
    KWD_LITERALS_OTHER = ((Keyword*)(intern_rigid_symbol_wrt_module("OTHER", NULL, 2)));
    KWD_LITERALS_DIGIT = ((Keyword*)(intern_rigid_symbol_wrt_module("DIGIT", NULL, 2)));
    KWD_LITERALS_LETTER = ((Keyword*)(intern_rigid_symbol_wrt_module("LETTER", NULL, 2)));
    KWD_LITERALS_SYMBOL_CONSTITUENT = ((Keyword*)(intern_rigid_symbol_wrt_module("SYMBOL-CONSTITUENT", NULL, 2)));
    KWD_LITERALS_SYMBOL_QUALIFIER = ((Keyword*)(intern_rigid_symbol_wrt_module("SYMBOL-QUALIFIER", NULL, 2)));
    KWD_LITERALS_ESCAPE = ((Keyword*)(intern_rigid_symbol_wrt_module("ESCAPE", NULL, 2)));
    KWD_LITERALS_DELIMITER = ((Keyword*)(intern_rigid_symbol_wrt_module("DELIMITER", NULL, 2)));
  }
}

void help_startup_literals2() {
  {
    oLITERAL_TYPE_INFO_TABLEo = new_hash_table();
    oLITERAL_TYPESo = new_list();
    set_literal_type_info(SGT_LITERALS_STELLA_BOOLEAN, KWD_LITERALS_NULL_WRAPPER, SYM_LITERALS_STELLA_FALSE_WRAPPER);
    set_literal_type_info(SGT_LITERALS_STELLA_INTEGER, KWD_LITERALS_NULL_WRAPPER, SYM_LITERALS_STELLA_NULL_INTEGER_WRAPPER);
    set_literal_type_info(SGT_LITERALS_STELLA_FLOAT, KWD_LITERALS_NULL_WRAPPER, SYM_LITERALS_STELLA_NULL_FLOAT_WRAPPER);
    set_literal_type_info(SGT_LITERALS_STELLA_STRING, KWD_LITERALS_NULL_WRAPPER, SYM_LITERALS_STELLA_NULL_STRING_WRAPPER);
    set_literal_type_info(SGT_LITERALS_STELLA_MUTABLE_STRING, KWD_LITERALS_NULL_WRAPPER, SYM_LITERALS_STELLA_NULL_MUTABLE_STRING_WRAPPER);
    set_literal_type_info(SGT_LITERALS_STELLA_CHARACTER, KWD_LITERALS_NULL_WRAPPER, SYM_LITERALS_STELLA_NULL_CHARACTER_WRAPPER);
    set_literal_type_info(SGT_LITERALS_STELLA_FUNCTION_CODE, KWD_LITERALS_NULL_WRAPPER, SYM_LITERALS_STELLA_NULL_FUNCTION_CODE_WRAPPER);
    set_literal_type_info(SGT_LITERALS_STELLA_METHOD_CODE, KWD_LITERALS_NULL_WRAPPER, SYM_LITERALS_STELLA_NULL_METHOD_CODE_WRAPPER);
    set_literal_type_info(SGT_LITERALS_STELLA_BOOLEAN, KWD_LITERALS_WRAP_FUNCTION, SYM_LITERALS_STELLA_WRAP_BOOLEAN);
    set_literal_type_info(SGT_LITERALS_STELLA_INTEGER, KWD_LITERALS_WRAP_FUNCTION, SYM_LITERALS_STELLA_WRAP_INTEGER);
    set_literal_type_info(SGT_LITERALS_STELLA_FLOAT, KWD_LITERALS_WRAP_FUNCTION, SYM_LITERALS_STELLA_WRAP_FLOAT);
    set_literal_type_info(SGT_LITERALS_STELLA_STRING, KWD_LITERALS_WRAP_FUNCTION, SYM_LITERALS_STELLA_WRAP_STRING);
    set_literal_type_info(SGT_LITERALS_STELLA_CHARACTER, KWD_LITERALS_WRAP_FUNCTION, SYM_LITERALS_STELLA_WRAP_CHARACTER);
    set_literal_type_info(SGT_LITERALS_STELLA_FUNCTION_CODE, KWD_LITERALS_WRAP_FUNCTION, SYM_LITERALS_STELLA_WRAP_FUNCTION_CODE);
    set_literal_type_info(SGT_LITERALS_STELLA_METHOD_CODE, KWD_LITERALS_WRAP_FUNCTION, SYM_LITERALS_STELLA_WRAP_METHOD_CODE);
    oCHARACTER_TYPE_TABLEo = create_character_type_table();
    oCHARACTER_UPCASE_TABLEo = initialize_character_upcase_table();
    oCHARACTER_DOWNCASE_TABLEo = initialize_character_downcase_table();
  }
}

void help_startup_literals3() {
  {
    define_method_object("(DEFMETHOD (TERMINATE-WRAPPER? BOOLEAN) ((SELF WRAPPER)) :PUBLIC? TRUE)", ((cpp_method_code)(&Wrapper::terminate_wrapperP)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (TERMINATE-WRAPPER? BOOLEAN) ((SELF INTEGER-WRAPPER)) :PUBLIC? TRUE)", ((cpp_method_code)(&Integer_Wrapper::terminate_wrapperP)), ((cpp_method_code)(NULL)));
    define_function_object("LOOKUP-LITERAL-TYPE-INFO", "(DEFUN (LOOKUP-LITERAL-TYPE-INFO OBJECT) ((TYPE TYPE) (KEY KEYWORD)))", ((cpp_function_code)(&lookup_literal_type_info)), NULL);
    define_function_object("SET-LITERAL-TYPE-INFO", "(DEFUN SET-LITERAL-TYPE-INFO ((TYPE TYPE) (KEY KEYWORD) (VALUE OBJECT)))", ((cpp_function_code)(&set_literal_type_info)), NULL);
    define_function_object("WRAP-INTEGER", "(DEFUN (WRAP-INTEGER INTEGER-WRAPPER) ((VALUE INTEGER)) :DOCUMENTATION \"Return a literal object whose value is the INTEGER 'value'.\" :PUBLIC? TRUE)", ((cpp_function_code)(&wrap_integer)), NULL);
    define_function_object("UNWRAP-INTEGER", "(DEFUN (UNWRAP-INTEGER INTEGER) ((WRAPPER INTEGER-WRAPPER)) :DOCUMENTATION \"Unwrap `wrapper' and return the result.\nReturn NULL if `wrapper' is NULL.\" :PUBLIC? TRUE)", ((cpp_function_code)(&unwrap_integer)), NULL);
    define_function_object("WRAP-FLOAT", "(DEFUN (WRAP-FLOAT FLOAT-WRAPPER) ((VALUE FLOAT)) :DOCUMENTATION \"Return a literal object whose value is the FLOAT 'value'.\" :PUBLIC? TRUE)", ((cpp_function_code)(&wrap_float)), NULL);
    define_function_object("UNWRAP-FLOAT", "(DEFUN (UNWRAP-FLOAT FLOAT) ((WRAPPER FLOAT-WRAPPER)) :DOCUMENTATION \"Unwrap `wrapper' and return the result.\nReturn NULL if `wrapper' is NULL.\" :PUBLIC? TRUE)", ((cpp_function_code)(&unwrap_float)), NULL);
    define_function_object("WRAP-STRING", "(DEFUN (WRAP-STRING STRING-WRAPPER) ((VALUE STRING)) :DOCUMENTATION \"Return a literal object whose value is the STRING 'value'.\" :PUBLIC? TRUE)", ((cpp_function_code)(&wrap_string)), NULL);
    define_function_object("UNWRAP-STRING", "(DEFUN (UNWRAP-STRING STRING) ((WRAPPER STRING-WRAPPER)) :DOCUMENTATION \"Unwrap `wrapper' and return the result.\nReturn NULL if `wrapper' is NULL.\" :PUBLIC? TRUE)", ((cpp_function_code)(&unwrap_string)), NULL);
    define_function_object("WRAP-MUTABLE-STRING", "(DEFUN (WRAP-MUTABLE-STRING MUTABLE-STRING-WRAPPER) ((VALUE MUTABLE-STRING)) :DOCUMENTATION \"Return a literal object whose value is the MUTABLE-STRING 'value'.\" :PUBLIC? TRUE)", ((cpp_function_code)(&wrap_mutable_string)), NULL);
    define_function_object("UNWRAP-MUTABLE-STRING", "(DEFUN (UNWRAP-MUTABLE-STRING MUTABLE-STRING) ((WRAPPER MUTABLE-STRING-WRAPPER)) :DOCUMENTATION \"Unwrap `wrapper' and return the result.\nReturn NULL if `wrapper' is NULL.\" :PUBLIC? TRUE)", ((cpp_function_code)(&unwrap_mutable_string)), NULL);
    define_function_object("WRAP-CHARACTER", "(DEFUN (WRAP-CHARACTER CHARACTER-WRAPPER) ((VALUE CHARACTER)) :DOCUMENTATION \"Return a literal object whose value is the CHARACTER 'value'.\" :PUBLIC? TRUE)", ((cpp_function_code)(&wrap_character)), NULL);
    define_function_object("UNWRAP-CHARACTER", "(DEFUN (UNWRAP-CHARACTER CHARACTER) ((WRAPPER CHARACTER-WRAPPER)) :DOCUMENTATION \"Unwrap `wrapper' and return the result.\nReturn NULL if `wrapper' is NULL.\" :PUBLIC? TRUE)", ((cpp_function_code)(&unwrap_character)), NULL);
    define_function_object("WRAP-FUNCTION-CODE", "(DEFUN (WRAP-FUNCTION-CODE FUNCTION-CODE-WRAPPER) ((VALUE FUNCTION-CODE)) :DOCUMENTATION \"Return a literal object whose value is the FUNCTION-CODE 'value'.\" :PUBLIC? TRUE)", ((cpp_function_code)(&wrap_function_code)), NULL);
    define_function_object("UNWRAP-FUNCTION-CODE", "(DEFUN (UNWRAP-FUNCTION-CODE FUNCTION-CODE) ((WRAPPER FUNCTION-CODE-WRAPPER)) :DOCUMENTATION \"Unwrap `wrapper' and return the result.\nReturn NULL if `wrapper' is NULL.\" :PUBLIC? TRUE)", ((cpp_function_code)(&unwrap_function_code)), NULL);
    define_function_object("WRAP-METHOD-CODE", "(DEFUN (WRAP-METHOD-CODE METHOD-CODE-WRAPPER) ((VALUE METHOD-CODE)) :DOCUMENTATION \"Return a literal object whose value is the METHOD-CODE 'value'.\" :PUBLIC? TRUE)", ((cpp_function_code)(&wrap_method_code)), NULL);
    define_function_object("UNWRAP-METHOD-CODE", "(DEFUN (UNWRAP-METHOD-CODE METHOD-CODE) ((WRAPPER METHOD-CODE-WRAPPER)) :DOCUMENTATION \"Unwrap `wrapper' and return the result.\nReturn NULL if `wrapper' is NULL.\" :PUBLIC? TRUE)", ((cpp_function_code)(&unwrap_method_code)), NULL);
    define_function_object("WRAP-BOOLEAN", "(DEFUN (WRAP-BOOLEAN BOOLEAN-WRAPPER) ((VALUE BOOLEAN)) :DOCUMENTATION \"Return a literal object whose value is the BOOLEAN 'value'.\" :PUBLIC? TRUE :GLOBALLY-INLINE? TRUE (RETURN (CHOOSE VALUE TRUE-WRAPPER FALSE-WRAPPER)))", ((cpp_function_code)(&wrap_boolean)), NULL);
    define_function_object("UNWRAP-BOOLEAN", "(DEFUN (UNWRAP-BOOLEAN BOOLEAN) ((WRAPPER BOOLEAN-WRAPPER)) :DOCUMENTATION \"Unwrap `wrapper' and return its values as a regular BOOLEAN.\nMap NULL onto FALSE.\" :PUBLIC? TRUE :GLOBALLY-INLINE? TRUE (RETURN (EQ? WRAPPER TRUE-WRAPPER)))", ((cpp_function_code)(&unwrap_boolean)), NULL);
    define_function_object("COERCE-WRAPPED-BOOLEAN-TO-BOOLEAN", "(DEFUN (COERCE-WRAPPED-BOOLEAN-TO-BOOLEAN BOOLEAN) ((WRAPPER BOOLEAN-WRAPPER)) :PUBLIC? TRUE)", ((cpp_function_code)(&coerce_wrapped_boolean_to_boolean)), NULL);
    define_function_object("INLINE-WRAP-BOOLEAN-EXPANDER", "(DEFUN (INLINE-WRAP-BOOLEAN-EXPANDER OBJECT) ((EXPRESSION OBJECT)))", ((cpp_function_code)(&inline_wrap_boolean_expander)), NULL);
    define_function_object("INLINE-UNWRAP-BOOLEAN-EXPANDER", "(DEFUN (INLINE-UNWRAP-BOOLEAN-EXPANDER OBJECT) ((EXPRESSION OBJECT)))", ((cpp_function_code)(&inline_unwrap_boolean_expander)), NULL);
    define_method_object("(DEFMETHOD (WRAP-LITERAL INTEGER-WRAPPER) ((VALUE INTEGER)) :GLOBALLY-INLINE? TRUE :PUBLIC? TRUE (RETURN (WRAP-INTEGER VALUE)))", ((cpp_method_code)(NULL)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (WRAP-LITERAL FLOAT-WRAPPER) ((VALUE FLOAT)) :GLOBALLY-INLINE? TRUE :PUBLIC? TRUE (RETURN (WRAP-FLOAT VALUE)))", ((cpp_method_code)(NULL)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (WRAP-LITERAL MUTABLE-STRING-WRAPPER) ((VALUE MUTABLE-STRING)) :GLOBALLY-INLINE? TRUE :PUBLIC? TRUE (RETURN (WRAP-MUTABLE-STRING VALUE)))", ((cpp_method_code)(NULL)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (WRAP-LITERAL STRING-WRAPPER) ((VALUE STRING)) :GLOBALLY-INLINE? TRUE :PUBLIC? TRUE (RETURN (WRAP-STRING VALUE)))", ((cpp_method_code)(NULL)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (WRAP-LITERAL CHARACTER-WRAPPER) ((VALUE CHARACTER)) :GLOBALLY-INLINE? TRUE :PUBLIC? TRUE (RETURN (WRAP-CHARACTER VALUE)))", ((cpp_method_code)(NULL)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (WRAP-LITERAL FUNCTION-CODE-WRAPPER) ((VALUE FUNCTION-CODE)) :GLOBALLY-INLINE? TRUE :PUBLIC? TRUE (RETURN (WRAP-FUNCTION-CODE VALUE)))", ((cpp_method_code)(NULL)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (WRAP-LITERAL METHOD-CODE-WRAPPER) ((VALUE METHOD-CODE)) :GLOBALLY-INLINE? TRUE :PUBLIC? TRUE (RETURN (WRAP-METHOD-CODE VALUE)))", ((cpp_method_code)(NULL)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (COPY-WRAPPED-LITERAL OBJECT) ((SELF STANDARD-OBJECT)))", ((cpp_method_code)(&Standard_Object::copy_wrapped_literal)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (COPY-WRAPPED-LITERAL OBJECT) ((SELF LITERAL-WRAPPER)))", ((cpp_method_code)(&Literal_Wrapper::copy_wrapped_literal)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (COPY-WRAPPED-LITERAL OBJECT) ((SELF INTEGER-WRAPPER)))", ((cpp_method_code)(&Integer_Wrapper::copy_wrapped_literal)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (COPY-WRAPPED-LITERAL OBJECT) ((SELF FLOAT-WRAPPER)))", ((cpp_method_code)(&Float_Wrapper::copy_wrapped_literal)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (COPY-WRAPPED-LITERAL OBJECT) ((SELF STRING-WRAPPER)))", ((cpp_method_code)(&String_Wrapper::copy_wrapped_literal)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (COPY-WRAPPED-LITERAL OBJECT) ((SELF MUTABLE-STRING-WRAPPER)))", ((cpp_method_code)(&Mutable_String_Wrapper::copy_wrapped_literal)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (COPY-WRAPPED-LITERAL OBJECT) ((SELF CHARACTER-WRAPPER)))", ((cpp_method_code)(&Character_Wrapper::copy_wrapped_literal)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (COPY-WRAPPED-LITERAL OBJECT) ((SELF FUNCTION-CODE-WRAPPER)))", ((cpp_method_code)(&Function_Code_Wrapper::copy_wrapped_literal)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (COPY-WRAPPED-LITERAL OBJECT) ((SELF METHOD-CODE-WRAPPER)))", ((cpp_method_code)(&Method_Code_Wrapper::copy_wrapped_literal)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (OBJECT-EQL? BOOLEAN) ((X OBJECT) (Y OBJECT)))", ((cpp_method_code)(&Object::object_eqlP)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (OBJECT-EQL? BOOLEAN) ((X STANDARD-OBJECT) (Y OBJECT)))", ((cpp_method_code)(&Standard_Object::object_eqlP)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (OBJECT-EQL? BOOLEAN) ((X INTEGER-WRAPPER) (Y OBJECT)))", ((cpp_method_code)(&Integer_Wrapper::object_eqlP)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (OBJECT-EQL? BOOLEAN) ((X FLOAT-WRAPPER) (Y OBJECT)))", ((cpp_method_code)(&Float_Wrapper::object_eqlP)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (OBJECT-EQL? BOOLEAN) ((X BOOLEAN-WRAPPER) (Y OBJECT)))", ((cpp_method_code)(&Boolean_Wrapper::object_eqlP)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (OBJECT-EQL? BOOLEAN) ((X STRING-WRAPPER) (Y OBJECT)))", ((cpp_method_code)(&String_Wrapper::object_eqlP)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (OBJECT-EQL? BOOLEAN) ((X MUTABLE-STRING-WRAPPER) (Y OBJECT)))", ((cpp_method_code)(&Mutable_String_Wrapper::object_eqlP)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (OBJECT-EQL? BOOLEAN) ((X CHARACTER-WRAPPER) (Y OBJECT)))", ((cpp_method_code)(&Character_Wrapper::object_eqlP)), ((cpp_method_code)(NULL)));
    define_function_object("EQL-TO-BOOLEAN?", "(DEFUN (EQL-TO-BOOLEAN? BOOLEAN) ((Y OBJECT) (X BOOLEAN)))", ((cpp_function_code)(&eql_to_booleanP)), NULL);
    define_function_object("EQL-TO-INTEGER?", "(DEFUN (EQL-TO-INTEGER? BOOLEAN) ((Y OBJECT) (X INTEGER)))", ((cpp_function_code)(&eql_to_integerP)), NULL);
    define_function_object("EQL-TO-FLOAT?", "(DEFUN (EQL-TO-FLOAT? BOOLEAN) ((Y OBJECT) (X FLOAT)))", ((cpp_function_code)(&eql_to_floatP)), NULL);
    define_function_object("EQL-TO-STRING?", "(DEFUN (EQL-TO-STRING? BOOLEAN) ((Y OBJECT) (X STRING)))", ((cpp_function_code)(&eql_to_stringP)), NULL);
    define_function_object("EQL-TO-CHARACTER?", "(DEFUN (EQL-TO-CHARACTER? BOOLEAN) ((Y OBJECT) (X CHARACTER)))", ((cpp_function_code)(&eql_to_characterP)), NULL);
    define_function_object("HELP-BQUOTIFY", "(DEFUN (HELP-BQUOTIFY OBJECT) ((TREE OBJECT)))", ((cpp_function_code)(&help_bquotify)), NULL);
    define_function_object("BQUOTIFY", "(DEFUN (BQUOTIFY OBJECT) ((TREE OBJECT)))", ((cpp_function_code)(&bquotify)), NULL);
    define_function_object("EXPAND-BQUOTE-TREE", "(DEFUN (EXPAND-BQUOTE-TREE OBJECT) ((TREE OBJECT)))", ((cpp_function_code)(&expand_bquote_tree)), NULL);
    define_function_object("SIMPLIFY-BQUOTE-TREE", "(DEFUN (SIMPLIFY-BQUOTE-TREE OBJECT) ((TREE OBJECT)))", ((cpp_function_code)(&simplify_bquote_tree)), NULL);
    define_method_object("(DEFMETHOD (PERMANENTIFY OBJECT) ((SELF OBJECT)) :PUBLIC? TRUE)", ((cpp_method_code)(&Object::permanentify)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (PERMANENTIFY SYMBOL) ((SELF SYMBOL)) :PUBLIC? TRUE)", ((cpp_method_code)(&Symbol::permanentify)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (PERMANENTIFY SYMBOL) ((SELF TRANSIENT-SYMBOL)) :PUBLIC? TRUE)", ((cpp_method_code)(&Transient_Symbol::permanentify)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (PERMANENTIFY LITERAL-WRAPPER) ((SELF LITERAL-WRAPPER)) :PUBLIC? TRUE)", ((cpp_method_code)(&Literal_Wrapper::permanentify)), ((cpp_method_code)(NULL)));
  }
}

void startup_literals() {
  if (current_startup_time_phaseP(0)) {
    ZERO_WRAPPER = new_integer_wrapper(0);
    ONE_WRAPPER = new_integer_wrapper(1);
    FALSE_WRAPPER = new_boolean_wrapper(FALSE);
    TRUE_WRAPPER = new_boolean_wrapper(TRUE);
    NULL_INTEGER_WRAPPER = new_integer_wrapper(NULL_INTEGER);
    NULL_FLOAT_WRAPPER = new_float_wrapper(NULL_FLOAT);
    NULL_STRING_WRAPPER = new_string_wrapper(NULL);
    NULL_MUTABLE_STRING_WRAPPER = new_mutable_string_wrapper(NULL);
    NULL_CHARACTER_WRAPPER = new_character_wrapper(NULL_CHARACTER);
    NULL_FUNCTION_CODE_WRAPPER = new_function_code_wrapper(NULL);
    NULL_METHOD_CODE_WRAPPER = new_method_code_wrapper(((cpp_method_code)(NULL)));
  }
  { 
    BIND_STELLA_SPECIAL(oMODULEo, Module*, oSTELLA_MODULEo);
    BIND_STELLA_SPECIAL(oCONTEXTo, Context*, oMODULEo.get());
    if (current_startup_time_phaseP(2)) {
      help_startup_literals1();
      KWD_LITERALS_WHITE_SPACE = ((Keyword*)(intern_rigid_symbol_wrt_module("WHITE-SPACE", NULL, 2)));
      SYM_LITERALS_STELLA_STARTUP_LITERALS = ((Symbol*)(intern_rigid_symbol_wrt_module("STARTUP-LITERALS", NULL, 0)));
      SYM_LITERALS_STELLA_METHOD_STARTUP_CLASSNAME = ((Symbol*)(intern_rigid_symbol_wrt_module("METHOD-STARTUP-CLASSNAME", NULL, 0)));
    }
    if (current_startup_time_phaseP(4)) {
      help_startup_literals2();
    }
    if (current_startup_time_phaseP(6)) {
      finalize_classes();
    }
    if (current_startup_time_phaseP(7)) {
      help_startup_literals3();
      define_function_object("PERMANENT-COPY", "(DEFUN (PERMANENT-COPY OBJECT) ((TREE OBJECT)) :PUBLIC? TRUE)", ((cpp_function_code)(&permanent_copy)), NULL);
      define_method_object("(DEFMETHOD (SOFT-PERMANENTIFY SYMBOL) ((SYMBOL SYMBOL)))", ((cpp_method_code)(&Symbol::soft_permanentify)), ((cpp_method_code)(NULL)));
      define_method_object("(DEFMETHOD (SOFT-PERMANENTIFY SYMBOL) ((SYMBOL TRANSIENT-SYMBOL)))", ((cpp_method_code)(&Transient_Symbol::soft_permanentify)), ((cpp_method_code)(NULL)));
      define_function_object("PRINT-CHARACTER", "(DEFUN PRINT-CHARACTER ((CHAR CHARACTER) (STREAM NATIVE-OUTPUT-STREAM)) :PUBLIC? TRUE)", ((cpp_function_code)(&print_character)), NULL);
      define_function_object("CREATE-CHARACTER-TYPE-TABLE", "(DEFUN (CREATE-CHARACTER-TYPE-TABLE (ARRAY (256) OF KEYWORD)) ())", ((cpp_function_code)(&create_character_type_table)), NULL);
      define_function_object("DIGIT-CHARACTER?", "(DEFUN (DIGIT-CHARACTER? BOOLEAN) ((CH CHARACTER)) :DOCUMENTATION \"Return TRUE if `ch' represents a digit.\" :PUBLIC? TRUE :GLOBALLY-INLINE? TRUE (RETURN (EQL? (AREF *CHARACTER-TYPE-TABLE* (CHARACTER-CODE CH)) :DIGIT)))", ((cpp_function_code)(&digit_characterP)), NULL);
      define_function_object("LETTER-CHARACTER?", "(DEFUN (LETTER-CHARACTER? BOOLEAN) ((CH CHARACTER)) :DOCUMENTATION \"Return TRUE if `ch' represents a letter.\" :PUBLIC? TRUE :GLOBALLY-INLINE? TRUE (RETURN (EQL? (AREF *CHARACTER-TYPE-TABLE* (CHARACTER-CODE CH)) :LETTER)))", ((cpp_function_code)(&letter_characterP)), NULL);
      define_function_object("UPPER-CASE-CHARACTER?", "(DEFUN (UPPER-CASE-CHARACTER? BOOLEAN) ((CH CHARACTER)) :PUBLIC? TRUE :DOCUMENTATION \"Return TRUE if `ch' represents an upper-case character.\")", ((cpp_function_code)(&upper_case_characterP)), NULL);
      define_function_object("LOWER-CASE-CHARACTER?", "(DEFUN (LOWER-CASE-CHARACTER? BOOLEAN) ((CH CHARACTER)) :PUBLIC? TRUE :DOCUMENTATION \"Return TRUE if `ch' represents a lower-case character.\")", ((cpp_function_code)(&lower_case_characterP)), NULL);
      define_function_object("WHITE-SPACE-CHARACTER?", "(DEFUN (WHITE-SPACE-CHARACTER? BOOLEAN) ((CH CHARACTER)) :DOCUMENTATION \"Return TRUE if `ch' is a white space character.\" :PUBLIC? TRUE :GLOBALLY-INLINE? TRUE (RETURN (EQL? (AREF *CHARACTER-TYPE-TABLE* (CHARACTER-CODE CH)) :WHITE-SPACE)))", ((cpp_function_code)(&white_space_characterP)), NULL);
      define_function_object("ALL-UPPER-CASE-STRING?", "(DEFUN (ALL-UPPER-CASE-STRING? BOOLEAN) ((S STRING)) :DOCUMENTATION \"Return TRUE if all letters in `s' are upper case.\" :PUBLIC? TRUE)", ((cpp_function_code)(&all_upper_case_stringP)), NULL);
      define_function_object("ALL-LOWER-CASE-STRING?", "(DEFUN (ALL-LOWER-CASE-STRING? BOOLEAN) ((S STRING)) :DOCUMENTATION \"Return TRUE if all letters in `s' are lower case.\" :PUBLIC? TRUE)", ((cpp_function_code)(&all_lower_case_stringP)), NULL);
      define_function_object("INITIALIZE-CHARACTER-UPCASE-TABLE", "(DEFUN (INITIALIZE-CHARACTER-UPCASE-TABLE STRING) ())", ((cpp_function_code)(&initialize_character_upcase_table)), NULL);
      define_function_object("INITIALIZE-CHARACTER-DOWNCASE-TABLE", "(DEFUN (INITIALIZE-CHARACTER-DOWNCASE-TABLE STRING) ())", ((cpp_function_code)(&initialize_character_downcase_table)), NULL);
      define_function_object("UPCASE-CHARACTER", "(DEFUN (UPCASE-CHARACTER CHARACTER) ((CHAR CHARACTER)) :PUBLIC? TRUE :GLOBALLY-INLINE? TRUE :DOCUMENTATION \"If `char' is lowercase, return its uppercase version,\notherwise, return 'char' unmodified.\" (RETURN (NTH *CHARACTER-UPCASE-TABLE* (CHARACTER-CODE CHAR))))", ((cpp_function_code)(&upcase_character)), NULL);
      define_function_object("DOWNCASE-CHARACTER", "(DEFUN (DOWNCASE-CHARACTER CHARACTER) ((CHAR CHARACTER)) :PUBLIC? TRUE :GLOBALLY-INLINE? TRUE :DOCUMENTATION \"If `char' is uppercase, return its lowercase version,\notherwise, return 'char' unmodified.\" (RETURN (NTH *CHARACTER-DOWNCASE-TABLE* (CHARACTER-CODE CHAR))))", ((cpp_function_code)(&downcase_character)), NULL);
      define_function_object("PRINT-STRING-READABLY", "(DEFUN PRINT-STRING-READABLY ((STRING STRING) (STREAM NATIVE-OUTPUT-STREAM)))", ((cpp_function_code)(&print_string_readably)), NULL);
      define_function_object("STRING-TO-MUTABLE-STRING", "(DEFUN (STRING-TO-MUTABLE-STRING MUTABLE-STRING) ((S STRING)) :PUBLIC? TRUE :GLOBALLY-INLINE? TRUE (RETURN (VERBATIM :COMMON-LISP (CL:COPY-SEQ S) :CPP \"strcpy(new (GC) char[strlen(s)+1], s)\" :JAVA \"new StringBuffer(s)\")))", ((cpp_function_code)(&string_to_mutable_string)), NULL);
      define_function_object("MUTABLE-STRING-TO-STRING", "(DEFUN (MUTABLE-STRING-TO-STRING STRING) ((S MUTABLE-STRING)) :PUBLIC? TRUE :GLOBALLY-INLINE? TRUE (RETURN (VERBATIM :COMMON-LISP S :CPP \"s\" :JAVA \"s.toString()\")))", ((cpp_function_code)(&mutable_string_to_string)), NULL);
      define_method_object("(DEFMETHOD (NUMBER-WRAPPER-TO-FLOAT FLOAT) ((SELF NUMBER-WRAPPER)) :PUBLIC? TRUE)", ((cpp_method_code)(&Number_Wrapper::number_wrapper_to_float)), ((cpp_method_code)(NULL)));
      define_method_object("(DEFMETHOD (NUMBER-WRAPPER-TO-FLOAT FLOAT) ((SELF INTEGER-WRAPPER)) :PUBLIC? TRUE)", ((cpp_method_code)(&Integer_Wrapper::number_wrapper_to_float)), ((cpp_method_code)(NULL)));
      define_method_object("(DEFMETHOD (NUMBER-WRAPPER-TO-FLOAT FLOAT) ((SELF FLOAT-WRAPPER)) :PUBLIC? TRUE)", ((cpp_method_code)(&Float_Wrapper::number_wrapper_to_float)), ((cpp_method_code)(NULL)));
      define_function_object("STARTUP-LITERALS", "(DEFUN STARTUP-LITERALS () :PUBLIC? TRUE)", ((cpp_function_code)(&startup_literals)), NULL);
      { Method_Slot* function = lookup_function(SYM_LITERALS_STELLA_STARTUP_LITERALS);

        set_dynamic_slot_value(function->dynamic_slots, SYM_LITERALS_STELLA_METHOD_STARTUP_CLASSNAME, wrap_string("Startup-Literals"), NULL_STRING_WRAPPER);
      }
    }
    if (current_startup_time_phaseP(8)) {
      finalize_slots();
      cleanup_unfinalized_classes();
    }
    if (current_startup_time_phaseP(9)) {
      define_stella_global_variable_from_stringified_source("(DEFGLOBAL ZERO-WRAPPER INTEGER-WRAPPER NULL :PUBLIC? TRUE)");
      define_stella_global_variable_from_stringified_source("(DEFGLOBAL ONE-WRAPPER INTEGER-WRAPPER NULL :PUBLIC? TRUE)");
      define_stella_global_variable_from_stringified_source("(DEFGLOBAL TRUE-WRAPPER BOOLEAN-WRAPPER NULL :PUBLIC? TRUE)");
      define_stella_global_variable_from_stringified_source("(DEFGLOBAL FALSE-WRAPPER BOOLEAN-WRAPPER NULL :PUBLIC? TRUE)");
      define_stella_global_variable_from_stringified_source("(DEFGLOBAL NULL-INTEGER-WRAPPER INTEGER-WRAPPER NULL :PUBLIC? TRUE)");
      define_stella_global_variable_from_stringified_source("(DEFGLOBAL NULL-FLOAT-WRAPPER FLOAT-WRAPPER NULL :PUBLIC? TRUE)");
      define_stella_global_variable_from_stringified_source("(DEFGLOBAL NULL-STRING-WRAPPER STRING-WRAPPER NULL :PUBLIC? TRUE)");
      define_stella_global_variable_from_stringified_source("(DEFGLOBAL NULL-MUTABLE-STRING-WRAPPER MUTABLE-STRING-WRAPPER NULL :PUBLIC? TRUE)");
      define_stella_global_variable_from_stringified_source("(DEFGLOBAL NULL-CHARACTER-WRAPPER CHARACTER-WRAPPER NULL :PUBLIC? TRUE)");
      define_stella_global_variable_from_stringified_source("(DEFGLOBAL NULL-FUNCTION-CODE-WRAPPER FUNCTION-CODE-WRAPPER NULL :PUBLIC? TRUE)");
      define_stella_global_variable_from_stringified_source("(DEFGLOBAL NULL-METHOD-CODE-WRAPPER METHOD-CODE-WRAPPER NULL :PUBLIC? TRUE)");
      define_stella_global_variable_from_stringified_source("(DEFGLOBAL *LITERAL-TYPE-INFO-TABLE* (HASH-TABLE OF TYPE (KEY-VALUE-LIST OF KEYWORD OBJECT)) (NEW HASH-TABLE) :DOCUMENTATION \"Table that holds a variety of information about literal\ntypes, e.g., the name of their null-wrapper, wrap-function, etc.\")");
      define_stella_global_variable_from_stringified_source("(DEFGLOBAL *LITERAL-TYPES* (LIST OF TYPE) (NEW LIST) :DOCUMENTATION \"List of literal types stored in '*literal-type-info-table*'.\nMaintained for iteration purposes.\")");
      store_macro(SYM_LITERALS_STELLA_INLINE_WRAP_BOOLEAN, SYM_LITERALS_STELLA_INLINE_WRAP_BOOLEAN_EXPANDER, ((cpp_function_code)(&inline_wrap_boolean_expander)));
      store_macro(SYM_LITERALS_STELLA_INLINE_UNWRAP_BOOLEAN, SYM_LITERALS_STELLA_INLINE_UNWRAP_BOOLEAN_EXPANDER, ((cpp_function_code)(&inline_unwrap_boolean_expander)));
      define_stella_global_variable_from_stringified_source("(DEFGLOBAL *CHARACTER-TYPE-TABLE* (ARRAY (256) OF KEYWORD) (CREATE-CHARACTER-TYPE-TABLE) :DOCUMENTATION \"Table of character types.  Entry 'i' represents the type\nof the character whose 'char-code' equals 'i'.  Each character is classified \nby one of the following keywords: :DIGIT, :LETTER, :SYMBOL-CONSTITUENT, \n:SYMBOL-QUALIFIER, :ESCAPE, :DELIMITER, :WHITE-SPACE, or :OTHER.\" :PUBLIC? TRUE)");
      define_stella_global_variable_from_stringified_source("(DEFGLOBAL *CHARACTER-UPCASE-TABLE* STRING (INITIALIZE-CHARACTER-UPCASE-TABLE))");
      define_stella_global_variable_from_stringified_source("(DEFGLOBAL *CHARACTER-DOWNCASE-TABLE* STRING (INITIALIZE-CHARACTER-DOWNCASE-TABLE))");
    }
  }
}

Surrogate* SGT_LITERALS_STELLA_BOOLEAN = NULL;

Keyword* KWD_LITERALS_NULL_WRAPPER = NULL;

Symbol* SYM_LITERALS_STELLA_FALSE_WRAPPER = NULL;

Surrogate* SGT_LITERALS_STELLA_INTEGER = NULL;

Symbol* SYM_LITERALS_STELLA_NULL_INTEGER_WRAPPER = NULL;

Surrogate* SGT_LITERALS_STELLA_FLOAT = NULL;

Symbol* SYM_LITERALS_STELLA_NULL_FLOAT_WRAPPER = NULL;

Surrogate* SGT_LITERALS_STELLA_STRING = NULL;

Symbol* SYM_LITERALS_STELLA_NULL_STRING_WRAPPER = NULL;

Surrogate* SGT_LITERALS_STELLA_MUTABLE_STRING = NULL;

Symbol* SYM_LITERALS_STELLA_NULL_MUTABLE_STRING_WRAPPER = NULL;

Surrogate* SGT_LITERALS_STELLA_CHARACTER = NULL;

Symbol* SYM_LITERALS_STELLA_NULL_CHARACTER_WRAPPER = NULL;

Surrogate* SGT_LITERALS_STELLA_FUNCTION_CODE = NULL;

Symbol* SYM_LITERALS_STELLA_NULL_FUNCTION_CODE_WRAPPER = NULL;

Surrogate* SGT_LITERALS_STELLA_METHOD_CODE = NULL;

Symbol* SYM_LITERALS_STELLA_NULL_METHOD_CODE_WRAPPER = NULL;

Symbol* SYM_LITERALS_STELLA_TRUE = NULL;

Symbol* SYM_LITERALS_STELLA_TRUE_WRAPPER = NULL;

Symbol* SYM_LITERALS_STELLA_FALSE = NULL;

Symbol* SYM_LITERALS_STELLA_WRAP_BOOLEAN = NULL;

Symbol* SYM_LITERALS_STELLA_INLINE_WRAP_BOOLEAN = NULL;

Symbol* SYM_LITERALS_STELLA_INLINE_WRAP_BOOLEAN_EXPANDER = NULL;

Symbol* SYM_LITERALS_STELLA_COERCE_WRAPPED_BOOLEAN_TO_BOOLEAN = NULL;

Symbol* SYM_LITERALS_STELLA_INLINE_UNWRAP_BOOLEAN = NULL;

Symbol* SYM_LITERALS_STELLA_INLINE_UNWRAP_BOOLEAN_EXPANDER = NULL;

Keyword* KWD_LITERALS_WRAP_FUNCTION = NULL;

Symbol* SYM_LITERALS_STELLA_WRAP_INTEGER = NULL;

Symbol* SYM_LITERALS_STELLA_WRAP_FLOAT = NULL;

Symbol* SYM_LITERALS_STELLA_WRAP_STRING = NULL;

Symbol* SYM_LITERALS_STELLA_WRAP_CHARACTER = NULL;

Symbol* SYM_LITERALS_STELLA_WRAP_FUNCTION_CODE = NULL;

Symbol* SYM_LITERALS_STELLA_WRAP_METHOD_CODE = NULL;

Surrogate* SGT_LITERALS_STELLA_INTEGER_WRAPPER = NULL;

Surrogate* SGT_LITERALS_STELLA_FLOAT_WRAPPER = NULL;

Surrogate* SGT_LITERALS_STELLA_STRING_WRAPPER = NULL;

Surrogate* SGT_LITERALS_STELLA_VERBATIM_STRING_WRAPPER = NULL;

Surrogate* SGT_LITERALS_STELLA_CHARACTER_WRAPPER = NULL;

Symbol* SYM_LITERALS_STELLA_NULL = NULL;

Surrogate* SGT_LITERALS_STELLA_CONS = NULL;

Symbol* SYM_LITERALS_STELLA_NIL = NULL;

Symbol* SYM_LITERALS_STELLA_CONS = NULL;

Symbol* SYM_LITERALS_STELLA_WRAP_LITERAL = NULL;

Symbol* SYM_LITERALS_STELLA_GET_KWD = NULL;

Symbol* SYM_LITERALS_STELLA_TYPED_SYS = NULL;

Surrogate* SGT_LITERALS_STELLA_SYMBOL = NULL;

Symbol* SYM_LITERALS_STELLA_GET_SGT = NULL;

Symbol* SYM_LITERALS_STELLA_INTERN_COMMON_LISP_SYMBOL = NULL;

Symbol* SYM_LITERALS_STELLA_GET_SYM = NULL;

Symbol* SYM_LITERALS_STELLA_LISTo = NULL;

Symbol* SYM_LITERALS_STELLA_a = NULL;

Symbol* SYM_LITERALS_STELLA_aa = NULL;

Symbol* SYM_LITERALS_STELLA_CONCATENATE = NULL;

Keyword* KWD_LITERALS_OTHER = NULL;

Keyword* KWD_LITERALS_DIGIT = NULL;

Keyword* KWD_LITERALS_LETTER = NULL;

Keyword* KWD_LITERALS_SYMBOL_CONSTITUENT = NULL;

Keyword* KWD_LITERALS_SYMBOL_QUALIFIER = NULL;

Keyword* KWD_LITERALS_ESCAPE = NULL;

Keyword* KWD_LITERALS_DELIMITER = NULL;

Keyword* KWD_LITERALS_WHITE_SPACE = NULL;

Symbol* SYM_LITERALS_STELLA_STARTUP_LITERALS = NULL;

Symbol* SYM_LITERALS_STELLA_METHOD_STARTUP_CLASSNAME = NULL;

} // end of namespace stella
