// modules.cc

/*--------------------------- BEGIN LICENSE BLOCK ---------------------------+
|                                                                            |
| Version: MPL 1.1/GPL 2.0/LGPL 2.1                                          |
|                                                                            |
| The contents of this file are subject to the Mozilla Public License        |
| Version 1.1 (the "License"); you may not use this file except in           |
| compliance with the License. You may obtain a copy of the License at       |
| http://www.mozilla.org/MPL/                                                |
|                                                                            |
| Software distributed under the License is distributed on an "AS IS" basis, |
| WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License   |
| for the specific language governing rights and limitations under the       |
| License.                                                                   |
|                                                                            |
| The Original Code is the STELLA Programming Language.                      |
|                                                                            |
| The Initial Developer of the Original Code is                              |
| UNIVERSITY OF SOUTHERN CALIFORNIA, INFORMATION SCIENCES INSTITUTE          |
| 4676 Admiralty Way, Marina Del Rey, California 90292, U.S.A.               |
|                                                                            |
| Portions created by the Initial Developer are Copyright (C) 1996-2001      |
| the Initial Developer. All Rights Reserved.                                |
|                                                                            |
| Contributor(s):                                                            |
|                                                                            |
| Alternatively, the contents of this file may be used under the terms of    |
| either the GNU General Public License Version 2 or later (the "GPL"), or   |
| the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),   |
| in which case the provisions of the GPL or the LGPL are applicable instead |
| of those above. If you wish to allow use of your version of this file only |
| under the terms of either the GPL or the LGPL, and not to allow others to  |
| use your version of this file under the terms of the MPL, indicate your    |
| decision by deleting the provisions above and replace them with the notice |
| and other provisions required by the GPL or the LGPL. If you do not delete |
| the provisions above, a recipient may use your version of this file under  |
| the terms of any one of the MPL, the GPL or the LGPL.                      |
|                                                                            |
+---------------------------- END LICENSE BLOCK ----------------------------*/

#include "stella/stella-system.hh"

namespace stella {

// Controls actions reflexive transitive closure of
//    subcontexts when a context is revised.
//    Values are ':destroy' -- destroy subcontexts;
//    ':clear' -- clear contents of subcontexts;
//    ':preserve' -- don't disturb subcontexts.
Keyword* oSUBCONTEXT_REVISION_POLICYo = NULL;

Iterator* Context::parent_contexts() {
  { Context* self = this;

    return (NULL);
  }
}

Iterator* Module::parent_contexts() {
  { Module* self = this;

    return (((List_Iterator*)(self->parent_modules->allocate_iterator())));
  }
}

Iterator* World::parent_contexts() {
  { World* self = this;

    return (((Cons*)(cons(self->parent_context, NIL)))->allocate_destructive_list_iterator());
  }
}

Context* compute_context_or_module_from_pathname(char* pathname, boolean moduleP, boolean symbolnameP, int& _return1) {
  { Module* context = ((pathname[0] == MODULE_SEPARATOR_CHARACTER) ? oROOT_MODULEo : ((Module*)(NULL)));
    int componentend = (((boolean)(context)) ? 0 : -1);
    int componentstart = NULL_INTEGER;
    int lastcharpos = strlen(pathname) - 1;

    while (componentend < lastcharpos) {
      componentstart = componentend + 1;
      componentend = componentstart;
      { int i = NULL_INTEGER;
        int iter_000 = componentstart;
        int upper_bound_000 = lastcharpos;
        boolean unboundedP_000 = upper_bound_000 == NULL_INTEGER;

        while (unboundedP_000 ||
            (iter_000 <= upper_bound_000)) {
          i = iter_000;
          iter_000 = iter_000 + 1;
          if (!(pathname[i] == MODULE_SEPARATOR_CHARACTER)) {
            componentend = componentend + 1;
          }
          else {
            break;
          }
        }
      }
      if (symbolnameP &&
          (componentend > lastcharpos)) {
        _return1 = componentstart;
        return ((((boolean)(context)) ? context : ((moduleP ? oMODULEo.get() : oCONTEXTo.get()))));
      }
      if (!((boolean)(context))) {
        { char* startstring = string_subsequence(pathname, componentstart, componentend);

          context = ((Module*)(string_get_stella_context_slowly(startstring)));
        }
      }
      else {
        { Context* value_000 = NULL;

          { Context* child = NULL;
            Cons* iter_001 = context->child_contexts->the_cons_list;

            while (!(iter_001 == NIL)) {
              child = ((Context*)(iter_001->value));
              iter_001 = iter_001->rest;
              { boolean test_value_000 = FALSE;

                test_value_000 = (!moduleP) ||
                    isaP(child, SGT_MODULES_STELLA_MODULE);
                if (test_value_000) {
                  if (component_matchP((moduleP ? ((Module*)(child))->module_name : ((Context*)(child))->context_name()), pathname, componentstart, componentend)) {
                    test_value_000 = TRUE;
                  }
                  else {
                    {
                      test_value_000 = moduleP;
                      if (test_value_000) {
                        test_value_000 = ((boolean)(((Module*)(child))->nicknames));
                        if (test_value_000) {
                          { boolean foundP_000 = FALSE;

                            { String_Wrapper* nickname = NULL;
                              Cons* iter_002 = ((Module*)(child))->nicknames->the_cons_list;

                              while (!(iter_002 == NIL)) {
                                nickname = ((String_Wrapper*)(iter_002->value));
                                iter_002 = iter_002->rest;
                                if (component_matchP(nickname->wrapper_value, pathname, componentstart, componentend)) {
                                  foundP_000 = TRUE;
                                  break;
                                }
                              }
                            }
                            test_value_000 = foundP_000;
                          }
                        }
                      }
                    }
                  }
                }
                if (test_value_000) {
                  value_000 = child;
                  break;
                }
              }
            }
          }
          context = ((Module*)(value_000));
        }
      }
      if (!((boolean)(context))) {
        break;
      }
    }
    _return1 = NULL_INTEGER;
    return (context);
  }
}

boolean component_matchP(char* component, char* string, int start, int end) {
  { boolean test_value_000 = FALSE;

    test_value_000 = strlen(component) == (end - start);
    if (test_value_000) {
      { boolean alwaysP_000 = TRUE;

        { char renamed_Char = NULL_CHARACTER;
          int i = NULL_INTEGER;
          char* vector_000 = component;
          int index_000 = 0;
          int length_000 = strlen(vector_000);
          int iter_000 = start;

          while (index_000 < length_000) {
            renamed_Char = vector_000[index_000];
            index_000 = index_000 + 1;
            i = iter_000;
            iter_000 = iter_000 + 1;
            if (!(renamed_Char == string[i])) {
              alwaysP_000 = FALSE;
              break;
            }
          }
        }
        test_value_000 = alwaysP_000;
      }
    }
    { boolean value_000 = test_value_000;

      return (value_000);
    }
  }
}

Module* compute_module_and_bare_name(char* name, char*& _return1) {
  { Context* module = NULL;
    int barenamestart = NULL_INTEGER;

    module = compute_context_or_module_from_pathname(name, TRUE, TRUE, barenamestart);
    { Module* _return_temp = ((Module*)(module));

      _return1 = ((barenamestart != NULL_INTEGER) ? string_subsequence(name, barenamestart, NULL_INTEGER) : ((char*)(NULL)));
      return (_return_temp);
    }
  }
}

char* qualified_name_module_name(char* qualifiedname) {
  { int end = strlen(qualifiedname);

    { int i = NULL_INTEGER;
      int iter_000 = 1;
      int upper_bound_000 = end;
      boolean unboundedP_000 = upper_bound_000 == NULL_INTEGER;

      while (unboundedP_000 ||
          (iter_000 <= upper_bound_000)) {
        i = iter_000;
        iter_000 = iter_000 + 1;
        if (qualifiedname[(end - i)] == MODULE_SEPARATOR_CHARACTER) {
          return (string_subsequence(qualifiedname, 0, end - i));
        }
      }
    }
    return (NULL);
  }
}

char* qualified_name_bare_name(char* qualifiedname) {
  { int end = strlen(qualifiedname);

    { int i = NULL_INTEGER;
      int iter_000 = 1;
      int upper_bound_000 = end;
      boolean unboundedP_000 = upper_bound_000 == NULL_INTEGER;

      while (unboundedP_000 ||
          (iter_000 <= upper_bound_000)) {
        i = iter_000;
        iter_000 = iter_000 + 1;
        if (qualifiedname[(end - i)] == MODULE_SEPARATOR_CHARACTER) {
          return (string_subsequence(qualifiedname, (end - i) + 1, end));
        }
      }
    }
    return (qualifiedname);
  }
}

Module* dwim_module_and_bare_name(char* name, boolean warnP, char*& _return1) {
  { Context* module = NULL;
    int barenamestart = NULL_INTEGER;

    module = compute_context_or_module_from_pathname(name, TRUE, TRUE, barenamestart);
    if ((!((boolean)(module))) &&
        (barenamestart != NULL_INTEGER)) {
      { char* modulename = qualified_name_bare_name(qualified_name_module_name(name));

        module = get_stella_module(modulename, FALSE);
        if (!((boolean)(module))) {
          module = get_stella_module(string_upcase(modulename), FALSE);
        }
      }
      if (warnP) {
        if (!((boolean)(module))) {
          *(STANDARD_WARNING->native_stream) << "Warning: " << "Module " << "`" << string_subsequence(name, 0, barenamestart - 1) << "'" << " of symbol " << "`" << name << "'" << " does not exist." << endl;
        }
        else {
          *(STANDARD_WARNING->native_stream) << "Warning: " << "Module " << "`" << string_subsequence(name, 0, barenamestart - 1) << "'" << " of symbol " << "`" << name << "'" << " does not exist." << endl << "    Assuming you mean " << "`" << ((Module*)(module))->module_full_name << "'" << "." << endl;
        }
      }
    }
    { Module* _return_temp = ((Module*)(module));

      _return1 = ((barenamestart != NULL_INTEGER) ? string_subsequence(name, barenamestart, NULL_INTEGER) : ((char*)(NULL)));
      return (_return_temp);
    }
  }
}

Generalized_Symbol* intern_qualified_name(char* name, boolean enable_case_conversionP) {
  { Module* module = NULL;
    char* barename = NULL;

    module = dwim_module_and_bare_name(name, TRUE, barename);
    if (!((boolean)(module))) {
      if ((barename == NULL) ||
          surrogate_nameP(barename)) {
        return (SGT_MODULES_STELLA_UNKNOWN_SURROGATE);
      }
      else {
        return (SYM_MODULES_STELLA_UNKNOWN_SYMBOL);
      }
    }
    if (string_eqlP(barename, "")) {
      if ((module == oMODULEo.get()) ||
          (module == oROOT_MODULEo)) {
        return (intern_rigid_symbol_wrt_module("/", oSTELLA_MODULEo, SYMBOL_SYM));
      }
      else {
        {
          *(STANDARD_WARNING->native_stream) << "Warning: " << "Empty symbol name encountered" << endl;
          return (SYM_MODULES_STELLA_UNKNOWN_SYMBOL);
        }
      }
    }
    if (enable_case_conversionP &&
        (!module->case_sensitiveP)) {
      barename = string_upcase(barename);
    }
    if (surrogate_nameP(barename)) {
      return (intern_rigid_symbol_wrt_module(string_subsequence(barename, 1, NULL_INTEGER), module, SURROGATE_SYM));
    }
    else {
      return (intern_rigid_symbol_wrt_module(barename, module, SYMBOL_SYM));
    }
  }
}

Context* get_stella_context(char* pathname, boolean warnP) {
  // Return the context located at 'pathName', or NULL
  // if no such context exists.  If 'warn?' is TRUE, print a warning
  // message if no context is found.
  { int dummy1;

    { Context* context = NULL;

      if (qualified_nameP(pathname)) {
        context = compute_context_or_module_from_pathname(pathname, FALSE, FALSE, dummy1);
      }
      else {
        { Context* context = string_get_stella_context_slowly(pathname);

          if (((boolean)(context))) {
            return (context);
          }
        }
      }
      if ((!((boolean)(context))) &&
          warnP) {
        *(STANDARD_WARNING->native_stream) << "Warning: " << "Can't find a context located at " << "`" << pathname << "'" << "." << endl;
      }
      return (context);
    }
  }
}

Module* get_stella_module(char* pathname, boolean warnP) {
  // Return the module located at 'pathName', or NULL
  // if no such module exists.  The search looks at ancestors and top-most
  //  (cardinal) modules.  If 'warn?' is TRUE, print a warning
  // message if no module is found.
  { int dummy1;

    if (string_eqlP(pathname, oMODULEo.get()->module_full_name)) {
      return (oMODULEo.get());
    }
    { Module* module = NULL;

      if (qualified_nameP(pathname)) {
        module = ((Module*)(compute_context_or_module_from_pathname(pathname, TRUE, FALSE, dummy1)));
      }
      else {
        { Context* context = string_get_stella_context_slowly(pathname);

          if (((boolean)(context)) &&
              isaP(context, SGT_MODULES_STELLA_MODULE)) {
            return (((Module*)(context)));
          }
        }
      }
      if ((!((boolean)(module))) &&
          warnP) {
        *(STANDARD_WARNING->native_stream) << "Warning: " << "Can't find a module located at " << "`" << pathname << "'" << "." << endl;
      }
      return (module);
    }
  }
}

char* coerce_to_module_name(Object* namespec, boolean warnP) {
  { Surrogate* test_value_000 = safe_primary_type(namespec);

    if (subtype_of_stringP(test_value_000)) {
      { Object* namespec_000 = namespec;
        String_Wrapper* namespec = ((String_Wrapper*)(namespec_000));

        return (namespec->wrapper_value);
      }
    }
    else if (subtype_ofP(test_value_000, SGT_MODULES_STELLA_MUTABLE_STRING_WRAPPER)) {
      { Object* namespec_001 = namespec;
        Mutable_String_Wrapper* namespec = ((Mutable_String_Wrapper*)(namespec_001));

        return ((namespec->wrapper_value));
      }
    }
    else if (subtype_of_keywordP(test_value_000)) {
      { Object* namespec_002 = namespec;
        Keyword* namespec = ((Keyword*)(namespec_002));

        return (namespec->symbol_name);
      }
    }
    else if (subtype_of_symbolP(test_value_000)) {
      { Object* namespec_003 = namespec;
        Symbol* namespec = ((Symbol*)(namespec_003));

        { Module* module = ((Module*)(namespec->home_context));

          if (!((boolean)(module))) {
            return (namespec->symbol_name);
          }
          else {
            return (namespec->visible_name());
          }
        }
      }
    }
    else if (subtype_of_surrogateP(test_value_000)) {
      { Object* namespec_004 = namespec;
        Surrogate* namespec = ((Surrogate*)(namespec_004));

        { Module* module = ((Module*)(namespec->home_context));

          if (!((boolean)(module))) {
            return (namespec->symbol_name);
          }
          else {
            return (namespec->visible_name());
          }
        }
      }
    }
    else {
      if (warnP) {
        *(STANDARD_WARNING->native_stream) << "Warning: " << "Illegal module name: " << "`" << namespec << "'" << endl;
      }
      return (NULL);
    }
  }
}

Module* coerce_to_module(Object* object, boolean warnP) {
  if (!((boolean)(object))) {
    return (oMODULEo.get());
  }
  if (subtype_ofP(safe_primary_type(object), SGT_MODULES_STELLA_MODULE)) {
    { Object* object_000 = object;
      Module* object = ((Module*)(object_000));

      return (object);
    }
  }
  else {
    { char* modulename = coerce_to_module_name(object, warnP);
      Module* module = NULL;

      if (modulename != NULL) {
        if (qualified_nameP(modulename)) {
          return (get_stella_module(modulename, warnP));
        }
        return (get_stella_module(modulename, warnP));
      }
      return (NULL);
    }
  }
}

Module* find_or_create_module(char* pathname) {
  // Return a module located at `pathname' if one exists,
  //   otherwise create one
  { Module* module = get_stella_module(pathname, FALSE);

    if (!(((boolean)(module)))) {
      module = define_module(pathname, NIL);
    }
    return (module);
  }
}

Module* change_current_module(Module* module) {
  oMODULEo.set(module);
  oCONTEXTo.set(module);
  run_hooks(oCHANGE_MODULE_HOOKSo, module);
  return (module);
}

Context* change_current_context(Context* context) {
  if (!(context->base_module == oMODULEo.get())) {
    change_current_module(context->base_module);
  }
  oCONTEXTo.set(context);
  return (context);
}

Context* Context::change_context() {
  // Change the current context to be the context
  // 'context'.
  { Context* context = this;

    if (context == oCONTEXTo.get()) {
      return (context);
    }
    else {
      return (change_current_context(context));
    }
  }
}

Context* string_change_context(char* contextname) {
  // Change the current context to be the context named
  // 'contextName'.
  { Context* context = get_stella_context(contextname, TRUE);

    if (!((boolean)(context))) {
      return (oCONTEXTo.get());
    }
    else {
      return (change_current_context(context));
    }
  }
}

Module* Module::change_module() {
  // Change the current module to be the module
  // 'module'.
  { Module* module = this;

    return (change_current_module(module));
  }
}

Module* string_change_module(char* modulename) {
  // Change the current module to be the module named
  // 'moduleName'.
  { Module* module = get_stella_module(modulename, TRUE);

    if (!((boolean)(module))) {
      return (oMODULEo.get());
    }
    else {
      return (change_current_module(module));
    }
  }
}

boolean Module::multiple_parentsP() {
  // Return TRUE if 'module' has more than one parent.
  { Module* module = this;

    return (!(module->parent_modules->rest() == NIL));
  }
}

void normalize_parent_modules(Module* self) {
  if (!self->multiple_parentsP()) {
    return;
  }
  { List* parents = self->parent_modules;

    { Module* supermodule = NULL;
      Cons* iter_000 = parents->the_cons_list;

      while (!(iter_000 == NIL)) {
        supermodule = ((Module*)(iter_000->value));
        iter_000 = iter_000->rest;
        { Module* othersupermodule = NULL;
          Cons* iter_001 = parents->the_cons_list;

          while (!(iter_001 == NIL)) {
            othersupermodule = ((Module*)(iter_001->value));
            iter_001 = iter_001->rest;
            if (!(othersupermodule == supermodule)) {
              if (supermodule->all_super_contexts->memberP(othersupermodule)) {
                parents->remove(othersupermodule);
              }
            }
          }
        }
      }
    }
  }
}

void inherit_supercontexts(Module* module) {
  if (!module->multiple_parentsP()) {
    { Module* onlyparent = ((Module*)(module->parent_modules->first()));

      if (((boolean)(onlyparent))) {
        module->all_super_contexts = cons(onlyparent, onlyparent->all_super_contexts);
        return;
      }
    }
  }
  { Cons* allsupercontexts = NIL;
    Cons* sublist = NULL;

    normalize_parent_modules(module);
    module->parent_modules->reverse();
    { Module* parent = NULL;
      Cons* iter_000 = module->parent_modules->the_cons_list;

      while (!(iter_000 == NIL)) {
        parent = ((Module*)(iter_000->value));
        iter_000 = iter_000->rest;
        sublist = NIL;
        { Context* ancestor = NULL;
          Cons* iter_001 = parent->all_super_contexts;
          Cons* collect_000 = NULL;

          while (!(iter_001 == NIL)) {
            ancestor = ((Context*)(iter_001->value));
            iter_001 = iter_001->rest;
            if (!allsupercontexts->memberP(ancestor)) {
              if (!((boolean)(collect_000))) {
                {
                  collect_000 = cons(ancestor, NIL);
                  if (sublist == NIL) {
                    sublist = collect_000;
                  }
                  else {
                    add_cons_to_end_of_cons_list(sublist, collect_000);
                  }
                }
              }
              else {
                {
                  collect_000->rest = cons(ancestor, NIL);
                  collect_000 = collect_000->rest;
                }
              }
            }
          }
        }
        sublist = cons(parent, sublist);
        allsupercontexts = sublist->concatenate(allsupercontexts, 0);
      }
    }
    module->all_super_contexts = allsupercontexts;
    module->parent_modules->reverse();
  }
}

void uninherit_supercontexts(Module* module) {
  module->all_super_contexts = NIL;
}

void inherit_used_modules(Module* module) {
  { List* usees = module->uses;

    if (usees->emptyP()) {
      if (module->parent_modules->emptyP()) {
        usees->push(oSTELLA_MODULEo);
        oSTELLA_MODULEo->used_by->insert_new(module);
      }
      else {
        { Module* parent = NULL;
          Cons* iter_000 = module->parent_modules->the_cons_list;

          while (!(iter_000 == NIL)) {
            parent = ((Module*)(iter_000->value));
            iter_000 = iter_000->rest;
            { Module* parentusees = NULL;
              Cons* iter_001 = parent->uses->the_cons_list;

              while (!(iter_001 == NIL)) {
                parentusees = ((Module*)(iter_001->value));
                iter_001 = iter_001->rest;
                usees->insert_new(parentusees);
                parentusees->used_by->insert_new(module);
              }
            }
          }
        }
        usees->reverse();
      }
    }
    else if (usees->memberP(module)) {
      usees->remove(module);
    }
    else {
    }
  }
}

void uninherit_used_modules(Module* module) {
  { List* usees = module->uses;

    if (usees->emptyP()) {
      usees->push(module);
    }
    else if (string_search(module->stringified_options, ":USES ", 0) == NULL_INTEGER) {
      { Module* usee = NULL;
        Cons* iter_000 = usees->the_cons_list;

        while (!(iter_000 == NIL)) {
          usee = ((Module*)(iter_000->value));
          iter_000 = iter_000->rest;
          usee->used_by->remove(module);
        }
      }
      usees->clear();
    }
  }
}

// Holds list of symbols representing surrogates
// to be shadowed during module finalization.
DEFINE_STELLA_SPECIAL(oSHADOWEDSURROGATESo, Cons* , NULL);

void finalize_module(Module* self) {
  inherit_supercontexts(self);
  inherit_used_modules(self);
  oCONTEXT_NUMBER_COUNTERo = oCONTEXT_NUMBER_COUNTERo + 2;
  self->context_number = oCONTEXT_NUMBER_COUNTERo;
  self->cardinal_module = ((!((boolean)(self->parent_module()))) ? self : self->parent_module()->cardinal_module);
  self->symbol_offset_table = new_string_to_integer_hash_table();
  self->surrogate_offset_table = new_string_to_integer_hash_table();
  if (!(oSHADOWEDSURROGATESo.get() == NIL)) {
    set_dynamic_slot_value(self->dynamic_slots, SYM_MODULES_STELLA_SHADOWED_SURROGATES, new_list(), NULL);
    { 
      BIND_STELLA_SPECIAL(oMODULEo, Module*, self);
      { Symbol* sym = NULL;
        Cons* iter_000 = oSHADOWEDSURROGATESo.get();

        while (!(iter_000 == NIL)) {
          sym = ((Symbol*)(iter_000->value));
          iter_000 = iter_000->rest;
          self->shadowed_surrogates_reader()->insert_last(shadow_surrogate(sym->symbol_name));
        }
      }
    }
  }
}

void finalize_world(World* self) {
  { Context* parentcontext = self->parent_context;

    self->all_super_contexts = cons(parentcontext, parentcontext->all_super_contexts);
    oCONTEXT_NUMBER_COUNTERo = oCONTEXT_NUMBER_COUNTERo + 2;
    self->context_number = oCONTEXT_NUMBER_COUNTERo;
  }
}

void Module::unfinalize_module() {
  { Module* self = this;

    uninherit_supercontexts(self);
    uninherit_used_modules(self);
    if (cardinal_moduleP(self)) {
      oROOT_MODULEo->child_contexts->remove(self);
    }
    else {
      { Module* parent = NULL;
        Cons* iter_000 = self->parent_modules->the_cons_list;

        while (!(iter_000 == NIL)) {
          parent = ((Module*)(iter_000->value));
          iter_000 = iter_000->rest;
          parent->child_contexts->remove(self);
        }
      }
    }
    self->parent_modules->clear();
    self->context_number = self->context_number - 1;
    self->module_full_name = NULL;
  }
}

void World::unfinalize_world() {
  { World* self = this;

    self->parent_context->child_contexts->remove(self);
    self->context_number = self->context_number - 1;
    self->all_super_contexts = NULL;
    self->parent_context = NULL;
  }
}

void destroy_module(Module* self) {
  // Destroy the module 'self', and recursively destroy
  // all contexts that inherit 'self'.
  if ((self == oROOT_MODULEo) ||
      (self == oSTELLA_MODULEo)) {
    *(STANDARD_WARNING->native_stream) << "Warning: " << "Can't destroy the root module or the STELLA module." << endl;
    return;
  }
  clear_context(self);
  help_destroy_module(self);
}

void help_destroy_module(Module* self) {
  { Context* child = NULL;
    Cons* iter_000 = self->child_contexts->copy()->the_cons_list;

    while (!(iter_000 == NIL)) {
      child = ((Context*)(iter_000->value));
      iter_000 = iter_000->rest;
      if (subtype_ofP(safe_primary_type(child), SGT_MODULES_STELLA_MODULE)) {
        { Context* child_000 = child;
          Module* child = ((Module*)(child_000));

          help_destroy_module(child);
        }
      }
      else {
        child->destroy_context();
      }
    }
  }
  { Module* parentmodule = self->parent_module();

    run_hooks(oDESTROY_CONTEXT_HOOKSo, self);
    if (!((boolean)(parentmodule))) {
      parentmodule = self->cardinal_module;
    }
    if (!((boolean)(parentmodule))) {
      parentmodule = oSTELLA_MODULEo;
    }
    self->unfinalize_module();
    if (self == oMODULEo.get()) {
      parentmodule->change_module();
    }
  }
}

void destroy_world(World* self) {
  if (self->child_contexts->non_emptyP()) {
    { World* child = NULL;
      Cons* iter_000 = self->child_contexts->copy()->the_cons_list;

      while (!(iter_000 == NIL)) {
        child = ((World*)(iter_000->value));
        iter_000 = iter_000->rest;
        if (!child->deletedP()) {
          destroy_world(child);
        }
      }
    }
  }
  { Context* parentcontext = self->parent_context;

    run_hooks(oDESTROY_CONTEXT_HOOKSo, self);
    self->unfinalize_world();
    if (self == oCONTEXTo.get()) {
      oCONTEXTo.set(parentcontext);
    }
  }
}

void Module::destroy_context() {
  // Destroy the context 'self', and recursively destroy
  // all contexts that inherit 'self'.
  { Module* self = this;

    destroy_module(self);
  }
}

void World::destroy_context() {
  // Destroy the context 'self', and recursively destroy
  // all contexts that inherit 'self'.
  { World* self = this;

    destroy_world(self);
  }
}

void string_destroy_context(char* self) {
  // Destroy the context 'self', and recursively destroy
  // all contexts that inherit 'self'.
  { Context* context = get_stella_context(self, TRUE);

    if (!(!((boolean)(context)))) {
      context->destroy_context();
    }
  }
}

void Context::destroy_context() {
  // Make the translator happy.
  { Context* self = this;

  }
}

void link_to_parent_module(Module* self, Module* parent, boolean insert_firstP) {
  if ((!(parent == oROOT_MODULEo)) &&
      (!self->parent_modules->memberP(parent))) {
    if (insert_firstP) {
      self->parent_modules->insert(parent);
    }
    else {
      self->parent_modules->insert_last(parent);
    }
  }
  parent->child_contexts->insert_new(self);
}

void incorporate_module_name(Module* module, char* name) {
  { Module* parentmodule = NULL;
    char* barename = NULL;

    if (qualified_nameP(name)) {
      {
        parentmodule = compute_module_and_bare_name(name, barename);
        if (!((boolean)(parentmodule))) {
          *(STANDARD_WARNING->native_stream) << "Warning: " << "Bad path name " << "`" << name << "'" << " when defining the module " << "`" << barename << "'" << endl;
          return;
        }
        { boolean test_value_000 = FALSE;

          if ((parentmodule == oROOT_MODULEo) &&
              module->parent_modules->non_emptyP()) {
            test_value_000 = TRUE;
          }
          else {
            { boolean foundP_000 = FALSE;

              { Module* includedmodule = NULL;
                Cons* iter_000 = module->parent_modules->the_cons_list;

                while (!(iter_000 == NIL)) {
                  includedmodule = ((Module*)(iter_000->value));
                  iter_000 = iter_000->rest;
                  if ((!(includedmodule == parentmodule)) &&
                      ((includedmodule == parentmodule) ||
                       includedmodule->all_super_contexts->membP(parentmodule))) {
                    foundP_000 = TRUE;
                    break;
                  }
                }
              }
              test_value_000 = foundP_000;
            }
          }
          if (test_value_000) {
            *(STANDARD_WARNING->native_stream) << "Warning: " << "Bad path name " << "`" << name << "'" << " when defining the module " << "`" << barename << "'" << "," << endl << "   since it points to a parent of an included module." << endl;
          }
          else {
            link_to_parent_module(module, parentmodule, TRUE);
          }
        }
      }
    }
    else {
      {
        if (module->parent_modules->emptyP()) {
          link_to_parent_module(module, oROOT_MODULEo, TRUE);
        }
        barename = name;
      }
    }
    module->module_name = barename;
    module->module_full_name = compute_full_name("", module);
  }
}

void incorporate_includes_modules(Module* module, Object* includees) {
  { Surrogate* test_value_000 = safe_primary_type(includees);

    if (test_value_000 == SGT_MODULES_STELLA_CONS) {
      { Object* includees_000 = includees;
        Cons* includees = ((Cons*)(includees_000));

        { Object* name = NULL;
          Cons* iter_000 = includees;

          while (!(iter_000 == NIL)) {
            name = iter_000->value;
            iter_000 = iter_000->rest;
            incorporate_includes_modules(module, name);
          }
        }
      }
    }
    else if (subtype_of_stringP(test_value_000)) {
      { Object* includees_001 = includees;
        String_Wrapper* includees = ((String_Wrapper*)(includees_001));

        { Module* includeemodule = coerce_to_module(includees, TRUE);

          if (((boolean)(includeemodule))) {
            if (!module->parent_modules->memberP(includeemodule)) {
              module->parent_modules->insert_last(includeemodule);
            }
            includeemodule->child_contexts->insert_new(module);
          }
        }
      }
    }
    else if (subtype_of_symbolP(test_value_000)) {
      { Object* includees_002 = includees;
        Symbol* includees = ((Symbol*)(includees_002));

        { Module* includeemodule = coerce_to_module(includees, TRUE);

          if (((boolean)(includeemodule))) {
            if (!module->parent_modules->memberP(includeemodule)) {
              module->parent_modules->insert_last(includeemodule);
            }
            includeemodule->child_contexts->insert_new(module);
          }
        }
      }
    }
    else {
      { 
        BIND_STELLA_SPECIAL(oPRINTREADABLYpo, boolean, TRUE);
        signal_translation_error();
        if (!(suppress_warningsP())) {
          print_error_context(">> ERROR: ", STANDARD_ERROR);
          *(STANDARD_ERROR->native_stream) << endl << " " << "Illegal argument to ':includes' option" << "`" << de_uglify_parse_tree(includees) << "'" << "." << endl;
        }
      }
    }
  }
}

void incorporate_uses_modules(Module* module, Object* usees) {
  { Surrogate* test_value_000 = safe_primary_type(usees);

    if (test_value_000 == SGT_MODULES_STELLA_CONS) {
      { Object* usees_000 = usees;
        Cons* usees = ((Cons*)(usees_000));

        { Object* name = NULL;
          Cons* iter_000 = usees;

          while (!(iter_000 == NIL)) {
            name = iter_000->value;
            iter_000 = iter_000->rest;
            incorporate_uses_modules(module, name);
          }
        }
        if (module->uses->emptyP()) {
          module->uses->insert(module);
        }
      }
    }
    else if (subtype_of_stringP(test_value_000)) {
      { Object* usees_001 = usees;
        String_Wrapper* usees = ((String_Wrapper*)(usees_001));

        { Module* useemodule = coerce_to_module(usees, TRUE);

          if (((boolean)(useemodule))) {
            if (!module->uses->memberP(useemodule)) {
              module->uses->insert_last(useemodule);
            }
            useemodule->used_by->insert_new(module);
          }
        }
      }
    }
    else if (subtype_of_symbolP(test_value_000)) {
      { Object* usees_002 = usees;
        Symbol* usees = ((Symbol*)(usees_002));

        { Module* useemodule = coerce_to_module(usees, TRUE);

          if (((boolean)(useemodule))) {
            if (!module->uses->memberP(useemodule)) {
              module->uses->insert_last(useemodule);
            }
            useemodule->used_by->insert_new(module);
          }
        }
      }
    }
    else {
      { 
        BIND_STELLA_SPECIAL(oPRINTREADABLYpo, boolean, TRUE);
        signal_translation_error();
        if (!(suppress_warningsP())) {
          print_error_context(">> ERROR: ", STANDARD_ERROR);
          *(STANDARD_ERROR->native_stream) << endl << " " << "Illegal argument to ':uses' option " << "." << endl;
        }
      }
    }
  }
}

void incorporate_module_options(Module* self, Cons* options) {
  { Property_List* self_000 = new_property_list();

    self_000->the_plist = options;
    { Property_List* plist = vet_options(self_000, get_quoted_tree("((:NICKNAME :INCLUDES :USES :DOCUMENTATION :SHADOW :CASE-SENSITIVE? :LISP-PACKAGE :CPP-PACKAGE :JAVA-PACKAGE :JAVA-CATCHALL-CLASS :API? :CLEARABLE? :PROTECT-SURROGATES? :CODE-ONLY?) \"/STELLA\")", "/STELLA"));

      { Object* key = NULL;
        Object* value = NULL;
        Cons* iter_000 = plist->the_plist;

        while (!(iter_000 == NIL)) {
          key = iter_000->value;
          value = iter_000->rest->value;
          iter_000 = iter_000->rest->rest;
          { Keyword* test_value_000 = ((Keyword*)(key));

            if (test_value_000 == KWD_MODULES_INCLUDES) {
              incorporate_includes_modules(self, value);
            }
            else if (test_value_000 == KWD_MODULES_USES) {
              incorporate_uses_modules(self, value);
            }
            else if (test_value_000 == KWD_MODULES_SHADOW) {
              oSHADOWEDSURROGATESo.set(((Cons*)(value)));
            }
            else {
              value = permanent_copy(value);
              { Keyword* test_value_001 = ((Keyword*)(key));

                if (test_value_001 == KWD_MODULES_DOCUMENTATION) {
                  self->documentation = ((String_Wrapper*)(value))->wrapper_value;
                }
                else if (test_value_001 == KWD_MODULES_CASE_SENSITIVEp) {
                  self->case_sensitiveP = value == SYM_MODULES_STELLA_TRUE;
                }
                else if (test_value_001 == KWD_MODULES_APIp) {
                  set_dynamic_slot_value(self->dynamic_slots, SYM_MODULES_STELLA_APIp, ((value == SYM_MODULES_STELLA_TRUE) ? TRUE_WRAPPER : FALSE_WRAPPER), FALSE_WRAPPER);
                }
                else if (test_value_001 == KWD_MODULES_LISP_PACKAGE) {
                  set_dynamic_slot_value(self->dynamic_slots, SYM_MODULES_STELLA_MODULE_LISP_PACKAGE, wrap_string(((String_Wrapper*)(value))->wrapper_value), NULL_STRING_WRAPPER);
                }
                else if (test_value_001 == KWD_MODULES_CPP_PACKAGE) {
                  set_dynamic_slot_value(self->dynamic_slots, SYM_MODULES_STELLA_MODULE_CPP_PACKAGE, wrap_string(((String_Wrapper*)(value))->wrapper_value), NULL_STRING_WRAPPER);
                }
                else if (test_value_001 == KWD_MODULES_JAVA_PACKAGE) {
                  set_dynamic_slot_value(self->dynamic_slots, SYM_MODULES_STELLA_JAVA_PACKAGE, wrap_string(((String_Wrapper*)(value))->wrapper_value), NULL_STRING_WRAPPER);
                }
                else if (test_value_001 == KWD_MODULES_JAVA_CATCHALL_CLASS) {
                  set_dynamic_slot_value(self->dynamic_slots, SYM_MODULES_STELLA_JAVA_FLOTSAM_CLASS, wrap_string(((String_Wrapper*)(value))->wrapper_value), NULL_STRING_WRAPPER);
                }
                else if (test_value_001 == KWD_MODULES_NICKNAME) {
                  if (!((boolean)(self->nicknames))) {
                    self->nicknames = new_list();
                  }
                  self->nicknames->insert(((String_Wrapper*)(value)));
                }
                else if (test_value_001 == KWD_MODULES_CLEARABLEp) {
                  set_dynamic_slot_value(self->dynamic_slots, SYM_MODULES_STELLA_CLEARABLEp, ((value == SYM_MODULES_STELLA_TRUE) ? TRUE_WRAPPER : FALSE_WRAPPER), FALSE_WRAPPER);
                }
                else if (test_value_001 == KWD_MODULES_PROTECT_SURROGATESp) {
                  set_dynamic_slot_value(self->dynamic_slots, SYM_MODULES_STELLA_PROTECT_SURROGATESp, ((value == SYM_MODULES_STELLA_TRUE) ? TRUE_WRAPPER : FALSE_WRAPPER), FALSE_WRAPPER);
                }
                else if (test_value_001 == KWD_MODULES_CODE_ONLYp) {
                  set_dynamic_slot_value(self->dynamic_slots, SYM_MODULES_STELLA_CODE_ONLYp, ((value == SYM_MODULES_STELLA_TRUE) ? TRUE_WRAPPER : FALSE_WRAPPER), FALSE_WRAPPER);
                }
                else if (test_value_001 == KWD_MODULES_REQUIRES) {
                  self->requires = ((List*)(value));
                }
                else {
                  { Output_String_Stream* stream_000 = new_output_string_stream();

                    *(stream_000->native_stream) << "`" << test_value_001 << "'" << " is not a valid case option";
                    throw *new_stella_exception(stream_000->the_string_reader());
                  }
                }
              }
            }
          }
        }
      }
      plist->free();
    }
  }
}

void undefine_module(Module* oldmodule, Module* newmodule) {
  cout << "Redefining the module " << "`" << oldmodule->context_name() << "'" << endl;
  if (oSUBCONTEXT_REVISION_POLICYo == KWD_MODULES_DESTROY) {
    oldmodule->destroy_context();
    return;
  }
  else if (oSUBCONTEXT_REVISION_POLICYo == KWD_MODULES_PRESERVE) {
  }
  else if (oSUBCONTEXT_REVISION_POLICYo == KWD_MODULES_CLEAR) {
    { Context* c = NULL;
      All_Purpose_Iterator* iter_000 = all_subcontexts(oldmodule, KWD_MODULES_PREORDER);

      while (iter_000->nextP()) {
        c = ((Context*)(iter_000->value));
        clear_context(c);
      }
    }
  }
  else {
    { Output_String_Stream* stream_000 = new_output_string_stream();

      *(stream_000->native_stream) << "`" << oSUBCONTEXT_REVISION_POLICYo << "'" << " is not a valid case option";
      throw *new_stella_exception(stream_000->the_string_reader());
    }
  }
  if (cardinal_moduleP(oldmodule)) {
    oROOT_MODULEo->child_contexts->remove(oldmodule);
  }
  else {
    { Module* p = NULL;
      Cons* iter_001 = oldmodule->parent_modules->the_cons_list;

      while (!(iter_001 == NIL)) {
        p = ((Module*)(iter_001->value));
        iter_001 = iter_001->rest;
        p->child_contexts->remove(oldmodule);
      }
    }
  }
  { Context* c = NULL;
    Cons* iter_002 = oldmodule->child_contexts->the_cons_list;

    while (!(iter_002 == NIL)) {
      c = ((Context*)(iter_002->value));
      iter_002 = iter_002->rest;
      { Surrogate* test_value_000 = safe_primary_type(c);

        if (subtype_ofP(test_value_000, SGT_MODULES_STELLA_MODULE)) {
          { Context* c_000 = c;
            Module* c = ((Module*)(c_000));

            c->parent_modules->the_cons_list->substitute(newmodule, oldmodule);
          }
        }
        else if (subtype_ofP(test_value_000, SGT_MODULES_STELLA_WORLD)) {
          { Context* c_001 = c;
            World* c = ((World*)(c_001));

            c->parent_context = newmodule;
          }
        }
        else {
          { Output_String_Stream* stream_001 = new_output_string_stream();

            *(stream_001->native_stream) << "`" << test_value_000 << "'" << " is not a valid case option";
            throw *new_stella_exception(stream_001->the_string_reader());
          }
        }
      }
    }
  }
  newmodule->child_contexts->the_cons_list = oldmodule->child_contexts->the_cons_list;
  oldmodule->child_contexts->the_cons_list = NIL;
  oldmodule->surrogate_value_inverse = NULL;
  oldmodule->free();
}

// HOOK-LIST called by 'define-module', applied to a
// 'module' argument.
Hook_List* oDEFINE_MODULE_HOOKSo = NULL;

Module* define_module(char* name, Cons* options) {
  // Define or redefine a module named 'name' having the
  // options 'options'.  Return the new module.
  { 
    BIND_STELLA_SPECIAL(oSHADOWEDSURROGATESo, Cons*, NIL);
    { Module* module = NULL;
      char* stringifiedoptions = stringify(options);
      Module* oldmodule = NULL;

      oldmodule = get_stella_module(name, FALSE);
      if (((boolean)(oldmodule)) &&
          identical_module_signaturesP(oldmodule, NULL, stringifiedoptions)) {
        return (oldmodule);
      }
      module = new_module();
      module->stringified_options = stringifiedoptions;
      incorporate_module_options(module, options);
      incorporate_module_name(module, name);
      module->base_module = module;
      if (((boolean)(oldmodule))) {
        if (identical_module_signaturesP(oldmodule, module, NULL)) {
          module->unfinalize_module();
          return (oldmodule);
        }
        if (oldmodule == oMODULEo.get()) {
          *(STANDARD_WARNING->native_stream) << "Warning: " << "Can't define a module inside of itself." << endl;
          cout << "Skipping redefinition of module " << name << "." << endl;
          module->unfinalize_module();
          return (oldmodule);
        }
        undefine_module(oldmodule, module);
      }
      finalize_module(module);
      run_hooks(oDEFINE_MODULE_HOOKSo, module);
      return (module);
    }
  }
}

boolean identical_module_signaturesP(Module* oldmodule, Module* newmodule, char* newoptions) {
  if (((boolean)(newmodule)) &&
      (!string_eqlP(oldmodule->module_full_name, newmodule->module_full_name))) {
    return (FALSE);
  }
  { char* oldoptions = oldmodule->stringified_options;

    newoptions = ((!((boolean)(newmodule))) ? newoptions : newmodule->stringified_options);
    if (newoptions == NULL) {
      return (FALSE);
    }
    if (string_eqlP(oldoptions, newoptions)) {
      return (TRUE);
    }
    if ((string_search(oldoptions, ":INCLUDES", 0) == NULL_INTEGER) &&
        (string_search(newoptions, ":INCLUDES", 0) == NULL_INTEGER)) {
      return (FALSE);
    }
    if (((boolean)(newmodule)) &&
        equal_cons_treesP(oldmodule->parent_modules->the_cons_list, newmodule->parent_modules->the_cons_list)) {
      { Property_List* self_000 = new_property_list();

        self_000->the_plist = ((Cons*)(read_s_expression_from_string(oldoptions)));
        { Property_List* oldplist = self_000;

          { Property_List* self_001 = new_property_list();

            self_001->the_plist = ((Cons*)(read_s_expression_from_string(newoptions)));
            { Property_List* newplist = self_001;

              oldplist->remove_at(KWD_MODULES_INCLUDES);
              newplist->remove_at(KWD_MODULES_INCLUDES);
              if (!(oldplist->length() == newplist->length())) {
                return (FALSE);
              }
              { Object* key = NULL;
                Object* value = NULL;
                Cons* iter_000 = oldplist->the_plist;

                while (!(iter_000 == NIL)) {
                  key = iter_000->value;
                  value = iter_000->rest->value;
                  iter_000 = iter_000->rest->rest;
                  if (!equal_cons_treesP(value, newplist->lookup(key))) {
                    return (FALSE);
                  }
                }
              }
              return (TRUE);
            }
          }
        }
      }
    }
    return (FALSE);
  }
}

Module* define_module_from_stringified_source(char* name, char* stringifiedoptions) {
  return (define_module(name, ((Cons*)(read_s_expression_from_string(stringifiedoptions)))));
}

Cons* yield_define_module(Module* module) {
  return (listO(3, SYM_MODULES_STELLA_DEFINE_MODULE_FROM_STRINGIFIED_SOURCE, wrap_string(module->module_full_name), cons(yield_string_constant_tree(module->stringified_options), NIL)));
}

void defmodule(Object* name, Cons* options) {
  // Define (or redefine) a module named `name'.
  // The accepted syntax is:
  // 	 
  //   (defmodule <module-name>
  //      [:documentation <docstring>]
  //      [:includes {<module-name> | (<module-name>*)}]
  //      [:uses {<module-name> | (<module-name>*)}]
  //      [:lisp-package <package-name-string>]
  //      [:java-package <package-specification-string>]
  //      [:cpp-namespace <namespace-name-string>]
  //      [:java-catchall-class
  //      [:api? {TRUE | FALSE}]
  //      [:case-sensitive? {TRUE | FALSE}]
  //      [:shadow (<symbol>*)]
  //      [:java-catchall-class <class-name-string>]
  //      [<other-options>*])
  // 	
  // `name' can be a string or a symbol.
  // 
  // Modules include objects from other modules via two separate mechanisms:
  // (1) they inherit from their parents specified via the `:includes' option
  // and/or a fully qualified module name, and (2) they inherit from used
  // modules specified via the `:uses' option.  The main difference between
  // the two mechanisms is that inheritance from parents is transitive, while
  // uses-links are only followed one level deep.  I.e., a module A that uses
  // B will see all objects of B (and any of B's parents) but not see anything
  // from modules used by B.  Another difference is that only objects declared
  // as public can be inherited via uses-links (this is not yet enforced).
  // Note that - contrary to Lisp - there are separate name spaces for classes,
  // functions, and variables.  For example, a module could inherit the class
  // `CONS' from the `STELLA' module, but shadow the function of the same name.
  // 
  // The above discussion of `:includes' and `:uses' semantics keyed on the
  // inheritance/visibility of symbols. The PowerLoom system makes another
  // very important distinction: If a module 'A' is inherited directly or
  // indirectly via `:includes' specification(s) by a submodule 'B', then all
  // definitions and facts asserted in 'A' are visible in 'B'. This is not the
  // cases for `:uses'; the `:uses' options does not impact inheritance of
  // propositions at all.
  // 
  // The list of modules specified in the
  // `:includes' option plus (if supplied) the parent in the path used for
  // `name' become the new module's parents. If no `:uses' option was
  // supplied, the new module will use the `STELLA' module by default,
  // otherwise, it will use the set of specified modules.
  // If `:case-sensitive?' is supplied as TRUE, symbols in the module will be
  // interned case-sensitively, otherwise (the default), they will be
  // converted to uppercase before they get interned. Modules can shadow
  // definitions of functions and classes inherited from parents or used
  // modules. Shadowing is done automatically, but generates a warning unless
  // the shadowed type or function name is listed in the `:shadow' option of
  // the module definition .
  // 
  // Examples:
  // 	 
  //   (defmodule "PL-KERNEL/PL-USER"
  //     :uses ("LOGIC" "STELLA")
  //     :package "PL-USER")
  // 
  //   (defmodule PL-USER/GENEALOGY)
  // 	
  // The remaining options are relevant only for modules that contain STELLA
  // code.  Modules used only to contain knowledge base definitions and assertions
  // have no use for them:
  // 
  // The keywords `:lisp-package', `:java-package', and `:cpp-package' specify
  // the name of a native package or name space in which symbols of the module
  // should be allocated when they get translated into one of Lisp, Java, or
  // C++. By default, Lisp symbols are allocated in the `STELLA' package, and
  // C++ names are translated without any prefixes. The rules that the STELLA
  // translator uses to attach translated Java objects to classes and packages
  // are somewhat complex. Use :java-package option to specify a list of
  // package names (separated by periods) that prefix the Java object in this
  // module.  Use :java-catchall-class to specify the name of the Java class to
  // contain all global & special variables, parameter-less functions and functions 
  // defined on arguments that are not classes in the current module.
  // The default value will be the name of the module.
  // 
  // When set to TRUE, the :api? option tells the PowerLoom User Manual
  // generator that all functions defined in this module should be included in
  // the API section. Additionally, the Java translator makes all API
  // functions `synchronized'.
  // 
  { char* modulename = coerce_to_module_name(name, TRUE);

    if (modulename != NULL) {
      define_module(modulename, options);
    }
  }
}

void defmodule_evaluator_wrapper(Cons* arguments) {
  defmodule(arguments->value, arguments->rest);
}

Module* in_module(Object* name) {
  // Change the current module to the module named `name'.
  { char* modulename = coerce_to_module_name(name, TRUE);
    Module* module = ((modulename != NULL) ? get_stella_module(modulename, FALSE) : ((Module*)(NULL)));

    if (((boolean)(module))) {
      oMODULEo.set(module);
      oCONTEXTo.set(module);
    }
    else {
      { Output_String_Stream* stream_000 = new_output_string_stream();

        *(stream_000->native_stream) << "Illegal or undefined module: " << "`" << name << "'";
        throw *new_stella_exception(stream_000->the_string_reader());
      }
    }
    return (oMODULEo.get());
  }
}

World* create_world(Context* parentcontext, char* name) {
  // Create a new world below the world or module 'parentContext'.
  // Optionally, specify a name.
  { World* world = new_world();

    if (name != NULL) {
      set_dynamic_slot_value(world->dynamic_slots, SYM_MODULES_STELLA_WORLD_NAME, wrap_string(name), NULL_STRING_WRAPPER);
      { boolean foundP_000 = FALSE;

        { Context* sibling = NULL;
          Cons* iter_000 = parentcontext->child_contexts->the_cons_list;

          while (!(iter_000 == NIL)) {
            sibling = ((Context*)(iter_000->value));
            iter_000 = iter_000->rest;
            if (string_eqlP(sibling->context_name(), name)) {
              foundP_000 = TRUE;
              break;
            }
          }
        }
        if (foundP_000) {
          *(STANDARD_WARNING->native_stream) << "Warning: " << "Can't create a world named " << "`" << name << "'" << " because a sibling already has that name." << endl;
          return (NULL);
        }
      }
    }
    world->parent_context = parentcontext;
    parentcontext->child_contexts->insert(world);
    world->base_module = parentcontext->base_module;
    finalize_world(world);
    return (world);
  }
}

World* push_world() {
  // Spawn a new world that is a child of the current context,
  // and change the current context to the new world.
  return (((World*)(oCONTEXTo.set(create_world(oCONTEXTo.get(), NULL)))));
}

Context* pop_world() {
  // Destroy the current world and change the current
  // context to be its parent.  Return the current context. Nothing happens
  // if there is no current world.
  if (!(oCONTEXTo.get() == oMODULEo.get())) {
    { World* world = ((World*)(oCONTEXTo.get()));

      oCONTEXTo.set(world->parent_context);
      destroy_world(world);
    }
  }
  return (oCONTEXTo.get());
}

All_Purpose_Iterator* all_subcontexts(Context* context, Keyword* traversal) {
  // Return an iterator that generates all subcontexts of
  // 'self' (not including 'self') in the order specified by 'traversal' (one
  // of :preorder, :inorder, or :postorder).
  { All_Purpose_Iterator* contextsiterator = new_all_purpose_iterator();
    Cons* subcontexts = NIL;

    { Context* child = NULL;
      Cons* iter_000 = context->child_contexts->the_cons_list;
      Cons* collect_000 = NULL;

      while (!(iter_000 == NIL)) {
        child = ((Context*)(iter_000->value));
        iter_000 = iter_000->rest;
        if (!((boolean)(collect_000))) {
          {
            collect_000 = cons(child, NIL);
            if (subcontexts == NIL) {
              subcontexts = collect_000;
            }
            else {
              add_cons_to_end_of_cons_list(subcontexts, collect_000);
            }
          }
        }
        else {
          {
            collect_000->rest = cons(child, NIL);
            collect_000 = collect_000->rest;
          }
        }
      }
    }
    if (traversal == KWD_MODULES_POSTORDER) {
      subcontexts = subcontexts->reverse();
    }
    contextsiterator->iterator_cons_list = subcontexts;
    contextsiterator->iterator_object = traversal;
    contextsiterator->iterator_next_code = ((cpp_function_code)(&all_subcontexts_nextP));
    return (contextsiterator);
  }
}

boolean all_subcontexts_nextP(All_Purpose_Iterator* self) {
  { Keyword* traversal = ((Keyword*)(self->iterator_object));
    Cons* remainingcontexts = self->iterator_cons_list;
    Context* thiscontext = NULL;
    Cons* subcontexts = NIL;

    if (remainingcontexts == NIL) {
      self->value = NULL;
      return (FALSE);
    }
    thiscontext = ((Context*)(remainingcontexts->value));
    remainingcontexts = remainingcontexts->rest;
    { Context* child = NULL;
      Cons* iter_000 = thiscontext->child_contexts->the_cons_list;
      Cons* collect_000 = NULL;

      while (!(iter_000 == NIL)) {
        child = ((Context*)(iter_000->value));
        iter_000 = iter_000->rest;
        if ((((Context*)(child->parent_contexts()->pop())) == thiscontext) ||
            child->parent_contexts()->emptyP()) {
          if (!((boolean)(collect_000))) {
            {
              collect_000 = cons(child, NIL);
              if (subcontexts == NIL) {
                subcontexts = collect_000;
              }
              else {
                add_cons_to_end_of_cons_list(subcontexts, collect_000);
              }
            }
          }
          else {
            {
              collect_000->rest = cons(child, NIL);
              collect_000 = collect_000->rest;
            }
          }
        }
      }
    }
    if (subcontexts == NIL) {
      self->iterator_cons_list = remainingcontexts;
    }
    else {
      if (traversal == KWD_MODULES_PREORDER) {
        last_cons(subcontexts)->rest = remainingcontexts;
        self->iterator_cons_list = subcontexts;
      }
      else if (traversal == KWD_MODULES_POSTORDER) {
        subcontexts = subcontexts->reverse();
        last_cons(subcontexts)->rest = remainingcontexts;
        self->iterator_cons_list = subcontexts;
      }
      else if (traversal == KWD_MODULES_INORDER) {
        if (remainingcontexts == NIL) {
          self->iterator_cons_list = subcontexts;
        }
        else {
          last_cons(remainingcontexts)->rest = subcontexts;
          self->iterator_cons_list = remainingcontexts;
        }
      }
      else {
        { Output_String_Stream* stream_000 = new_output_string_stream();

          *(stream_000->native_stream) << "`" << traversal << "'" << " is not a valid case option";
          throw *new_stella_exception(stream_000->the_string_reader());
        }
      }
    }
    self->value = thiscontext;
    return (TRUE);
  }
}

Iterator* all_contexts() {
  // Return an iterator that generates all contexts.
  { All_Purpose_Iterator* iterator = all_subcontexts(oROOT_MODULEo, KWD_MODULES_PREORDER);

    iterator->iterator_cons_list = cons(oROOT_MODULEo, NIL);
    return (iterator);
  }
}

boolean filter_moduleP(Object* self, All_Purpose_Iterator* iterator) {
  return (self->primary_type() == SGT_MODULES_STELLA_MODULE);
}

Iterator* all_modules() {
  // Return an iterator that generates all modules.
  { Iterator* contextsiterator = all_contexts();
    All_Purpose_Iterator* iterator = new_all_purpose_iterator();

    iterator->first_iterationP = TRUE;
    iterator->iterator_nested_iterator = contextsiterator;
    iterator->iterator_next_code = ((cpp_function_code)(&filtered_nested_iterator_nextP));
    iterator->iterator_filter_code = ((cpp_function_code)(&filter_moduleP));
    return (iterator);
  }
}

Iterator* all_included_modules(Module* self) {
  // Generate a sequence of all modules included
  // by 'self', inclusive, starting from the highest ancestor and working
  // down to 'self' (which is last).
  { Cons* copy = copy_cons_list(((Cons*)(self->all_super_contexts)));

    copy = cons(self, copy);
    return (copy->reverse()->allocate_destructive_list_iterator());
  }
}

Iterator* visible_modules(Module* from) {
  // Return an iterator that generates all modules visible from module `from'.
  // The generated modules are generated from most- to least-specific and 
  // will start with the module `from'.
  { Cons* visiblemodules = NIL;

    if (!((boolean)(from))) {
      from = oMODULEo.get();
    }
    visiblemodules = cons(from, visiblemodules);
    { Context* super = NULL;
      Cons* iter_000 = from->all_super_contexts;

      while (!(iter_000 == NIL)) {
        super = ((Context*)(iter_000->value));
        iter_000 = iter_000->rest;
        visiblemodules = cons(super, visiblemodules);
      }
    }
    { Module* usee = NULL;
      Cons* iter_001 = from->uses->the_cons_list;

      while (!(iter_001 == NIL)) {
        usee = ((Module*)(iter_001->value));
        iter_001 = iter_001->rest;
        if (!visiblemodules->membP(usee)) {
          visiblemodules = cons(usee, visiblemodules);
        }
        { Context* useesuper = NULL;
          Cons* iter_002 = usee->all_super_contexts;

          while (!(iter_002 == NIL)) {
            useesuper = ((Context*)(iter_002->value));
            iter_002 = iter_002->rest;
            if (!visiblemodules->membP(useesuper)) {
              visiblemodules = cons(useesuper, visiblemodules);
            }
          }
        }
      }
    }
    return (visiblemodules->reverse()->allocate_destructive_list_iterator());
  }
}

boolean cardinal_moduleP(Module* self) {
  return (self->cardinal_module == self);
}

boolean visible_fromP(Context* viewedcontext, Context* fromcontext) {
  { boolean test_value_000 = FALSE;

    if (viewedcontext == fromcontext) {
      test_value_000 = TRUE;
    }
    else {
      if (fromcontext->all_super_contexts->membP(viewedcontext)) {
        test_value_000 = TRUE;
      }
      else {
        { boolean foundP_000 = FALSE;

          { Module* usesmodule = NULL;
            Cons* iter_000 = ((isaP(fromcontext, SGT_MODULES_STELLA_MODULE) ? ((Module*)(fromcontext))->uses : fromcontext->base_module->uses))->the_cons_list;

            while (!(iter_000 == NIL)) {
              usesmodule = ((Module*)(iter_000->value));
              iter_000 = iter_000->rest;
              if ((viewedcontext == usesmodule) ||
                  usesmodule->all_super_contexts->membP(viewedcontext)) {
                foundP_000 = TRUE;
                break;
              }
            }
          }
          test_value_000 = foundP_000;
        }
      }
    }
    { boolean value_000 = test_value_000;

      return (value_000);
    }
  }
}

void clear_one_context(Context* self) {
  if (subtype_ofP(safe_primary_type(self), SGT_MODULES_STELLA_MODULE)) {
    { Context* self_000 = self;
      Module* self = ((Module*)(self_000));

      if (!self->clearableP_reader()) {
        *(STANDARD_WARNING->native_stream) << "Warning: " << "Module " << "`" << self->module_full_name << "'" << " cannot be cleared!" << endl;
        return;
      }
      run_hooks(oCLEAR_MODULE_HOOKSo, self);
      { Method_Slot* function = NULL;
        Iterator* iter_000 = all_functions(self, TRUE);

        while (iter_000->nextP()) {
          function = ((Method_Slot*)(iter_000->value));
          destroy_function(function);
        }
      }
      { Global_Variable* variable = NULL;
        Iterator* iter_001 = all_variables(self, TRUE);

        while (iter_001->nextP()) {
          variable = ((Global_Variable*)(iter_001->value));
          destroy_variable(variable);
        }
      }
      { Class* renamed_Class = NULL;
        Iterator* iter_002 = all_classes(self, TRUE);

        while (iter_002->nextP()) {
          renamed_Class = ((Class*)(iter_002->value));
          renamed_Class->destroy_class();
        }
      }
      { Surrogate* surrogate = NULL;
        Iterator* iter_003 = all_surrogates(self, TRUE);

        while (iter_003->nextP()) {
          surrogate = ((Surrogate*)(iter_003->value));
          if (((Boolean_Wrapper*)(dynamic_slot_value(self->dynamic_slots, SYM_MODULES_STELLA_PROTECT_SURROGATESp, FALSE_WRAPPER)))->wrapper_value) {
            surrogate->surrogate_value = NULL;
          }
          else {
            unintern_surrogate(surrogate);
          }
        }
      }
      if (!(((Boolean_Wrapper*)(dynamic_slot_value(self->dynamic_slots, SYM_MODULES_STELLA_PROTECT_SURROGATESp, FALSE_WRAPPER)))->wrapper_value)) {
        { Symbol* symbol = NULL;
          Iterator* iter_004 = all_symbols(self, TRUE);

          while (iter_004->nextP()) {
            symbol = ((Symbol*)(iter_004->value));
            unintern_symbol(symbol);
          }
        }
      }
      { 
        BIND_STELLA_SPECIAL(oMODULEo, Module*, self);
        { List_Iterator* it = ((List_Iterator*)(self->shadowed_surrogates_reader()->allocate_iterator()));

          while (it->nextP()) {
            it->value_setter(shadow_surrogate(((Surrogate*)(it->value))->symbol_name));
          }
        }
      }
    }
  }
  else {
  }
}

void help_clear_context(Context* self) {
  { Context* child = NULL;
    Cons* iter_000 = self->child_contexts->the_cons_list;

    while (!(iter_000 == NIL)) {
      child = ((Context*)(iter_000->value));
      iter_000 = iter_000->rest;
      help_clear_context(child);
    }
  }
  clear_one_context(self);
}

void clear_context(Context* self) {
  // Destroy all objects belonging to 'self' or any of its subcontexts.
  help_clear_context(self);
  finalize_classes_and_slots();
}

void call_clear_module(Cons* name) {
  // Destroy all objects belonging to module `name' or any of its children.
  // If no `name' is supplied, the current module will be cleared after
  // confirming with the user.  Important modules such as STELLA are protected
  // against accidental clearing.
  { Cons* arglist = name;
    Module* module = coerce_to_module(arglist->value, TRUE);

    if (((boolean)(module))) {
      if ((!(arglist == NIL)) ||
          y_or_nP(string_concatenate("Really clear module ", string_concatenate(module->module_full_name, "? ", 0), 0))) {
        clear_context(module);
      }
    }
  }
}

void call_clear_module_evaluator_wrapper(Cons* arguments) {
  call_clear_module(arguments);
}

void clear_module(Cons* name) {
  // Destroy all objects belonging to module `name' or any of its children.
  // If no `name' is supplied, the current module will be cleared after
  // confirming with the user.  Important modules such as STELLA are protected
  // against accidental clearing.
  if (name->length() == 0) {
    call_clear_module(NIL);
  }
  else {
    call_clear_module(cons_list(1, name->value));
  }
}

void clear_module_evaluator_wrapper(Cons* arguments) {
  clear_module(arguments);
}

void help_startup_modules1() {
  {
    KWD_MODULES_CLEAR = ((Keyword*)(intern_rigid_symbol_wrt_module("CLEAR", NULL, 2)));
    SGT_MODULES_STELLA_MODULE = ((Surrogate*)(intern_rigid_symbol_wrt_module("MODULE", NULL, 1)));
    SGT_MODULES_STELLA_UNKNOWN_SURROGATE = ((Surrogate*)(intern_rigid_symbol_wrt_module("UNKNOWN-SURROGATE", NULL, 1)));
    SYM_MODULES_STELLA_UNKNOWN_SYMBOL = ((Symbol*)(intern_rigid_symbol_wrt_module("UNKNOWN-SYMBOL", NULL, 0)));
    SYM_MODULES_STELLA_NAME = ((Symbol*)(intern_rigid_symbol_wrt_module("NAME", NULL, 0)));
    SGT_MODULES_STELLA_MUTABLE_STRING_WRAPPER = ((Surrogate*)(intern_rigid_symbol_wrt_module("MUTABLE-STRING-WRAPPER", NULL, 1)));
    SYM_MODULES_STELLA_SHADOWED_SURROGATES = ((Symbol*)(intern_rigid_symbol_wrt_module("SHADOWED-SURROGATES", NULL, 0)));
    SGT_MODULES_STELLA_CONS = ((Surrogate*)(intern_rigid_symbol_wrt_module("CONS", NULL, 1)));
    KWD_MODULES_INCLUDES = ((Keyword*)(intern_rigid_symbol_wrt_module("INCLUDES", NULL, 2)));
    KWD_MODULES_USES = ((Keyword*)(intern_rigid_symbol_wrt_module("USES", NULL, 2)));
    KWD_MODULES_SHADOW = ((Keyword*)(intern_rigid_symbol_wrt_module("SHADOW", NULL, 2)));
    KWD_MODULES_DOCUMENTATION = ((Keyword*)(intern_rigid_symbol_wrt_module("DOCUMENTATION", NULL, 2)));
    KWD_MODULES_CASE_SENSITIVEp = ((Keyword*)(intern_rigid_symbol_wrt_module("CASE-SENSITIVE?", NULL, 2)));
    SYM_MODULES_STELLA_TRUE = ((Symbol*)(intern_rigid_symbol_wrt_module("TRUE", NULL, 0)));
    KWD_MODULES_APIp = ((Keyword*)(intern_rigid_symbol_wrt_module("API?", NULL, 2)));
    SYM_MODULES_STELLA_APIp = ((Symbol*)(intern_rigid_symbol_wrt_module("API?", NULL, 0)));
    KWD_MODULES_LISP_PACKAGE = ((Keyword*)(intern_rigid_symbol_wrt_module("LISP-PACKAGE", NULL, 2)));
    SYM_MODULES_STELLA_MODULE_LISP_PACKAGE = ((Symbol*)(intern_rigid_symbol_wrt_module("MODULE-LISP-PACKAGE", NULL, 0)));
    KWD_MODULES_CPP_PACKAGE = ((Keyword*)(intern_rigid_symbol_wrt_module("CPP-PACKAGE", NULL, 2)));
    SYM_MODULES_STELLA_MODULE_CPP_PACKAGE = ((Symbol*)(intern_rigid_symbol_wrt_module("MODULE-CPP-PACKAGE", NULL, 0)));
    KWD_MODULES_JAVA_PACKAGE = ((Keyword*)(intern_rigid_symbol_wrt_module("JAVA-PACKAGE", NULL, 2)));
    SYM_MODULES_STELLA_JAVA_PACKAGE = ((Symbol*)(intern_rigid_symbol_wrt_module("JAVA-PACKAGE", NULL, 0)));
    KWD_MODULES_JAVA_CATCHALL_CLASS = ((Keyword*)(intern_rigid_symbol_wrt_module("JAVA-CATCHALL-CLASS", NULL, 2)));
    SYM_MODULES_STELLA_JAVA_FLOTSAM_CLASS = ((Symbol*)(intern_rigid_symbol_wrt_module("JAVA-FLOTSAM-CLASS", NULL, 0)));
    KWD_MODULES_NICKNAME = ((Keyword*)(intern_rigid_symbol_wrt_module("NICKNAME", NULL, 2)));
    KWD_MODULES_CLEARABLEp = ((Keyword*)(intern_rigid_symbol_wrt_module("CLEARABLE?", NULL, 2)));
    SYM_MODULES_STELLA_CLEARABLEp = ((Symbol*)(intern_rigid_symbol_wrt_module("CLEARABLE?", NULL, 0)));
    KWD_MODULES_PROTECT_SURROGATESp = ((Keyword*)(intern_rigid_symbol_wrt_module("PROTECT-SURROGATES?", NULL, 2)));
    SYM_MODULES_STELLA_PROTECT_SURROGATESp = ((Symbol*)(intern_rigid_symbol_wrt_module("PROTECT-SURROGATES?", NULL, 0)));
    KWD_MODULES_CODE_ONLYp = ((Keyword*)(intern_rigid_symbol_wrt_module("CODE-ONLY?", NULL, 2)));
    SYM_MODULES_STELLA_CODE_ONLYp = ((Symbol*)(intern_rigid_symbol_wrt_module("CODE-ONLY?", NULL, 0)));
    KWD_MODULES_REQUIRES = ((Keyword*)(intern_rigid_symbol_wrt_module("REQUIRES", NULL, 2)));
    KWD_MODULES_DESTROY = ((Keyword*)(intern_rigid_symbol_wrt_module("DESTROY", NULL, 2)));
    KWD_MODULES_PRESERVE = ((Keyword*)(intern_rigid_symbol_wrt_module("PRESERVE", NULL, 2)));
    KWD_MODULES_PREORDER = ((Keyword*)(intern_rigid_symbol_wrt_module("PREORDER", NULL, 2)));
    SGT_MODULES_STELLA_WORLD = ((Surrogate*)(intern_rigid_symbol_wrt_module("WORLD", NULL, 1)));
    SYM_MODULES_STELLA_DEFINE_MODULE_FROM_STRINGIFIED_SOURCE = ((Symbol*)(intern_rigid_symbol_wrt_module("DEFINE-MODULE-FROM-STRINGIFIED-SOURCE", NULL, 0)));
    SYM_MODULES_STELLA_WORLD_NAME = ((Symbol*)(intern_rigid_symbol_wrt_module("WORLD-NAME", NULL, 0)));
    KWD_MODULES_POSTORDER = ((Keyword*)(intern_rigid_symbol_wrt_module("POSTORDER", NULL, 2)));
    KWD_MODULES_INORDER = ((Keyword*)(intern_rigid_symbol_wrt_module("INORDER", NULL, 2)));
    SYM_MODULES_STELLA_STARTUP_MODULES = ((Symbol*)(intern_rigid_symbol_wrt_module("STARTUP-MODULES", NULL, 0)));
    SYM_MODULES_STELLA_METHOD_STARTUP_CLASSNAME = ((Symbol*)(intern_rigid_symbol_wrt_module("METHOD-STARTUP-CLASSNAME", NULL, 0)));
  }
}

void help_startup_modules2() {
  {
    define_method_object("(DEFMETHOD (PARENT-CONTEXTS (ITERATOR OF CONTEXT)) ((SELF CONTEXT)))", ((cpp_method_code)(&Context::parent_contexts)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (PARENT-CONTEXTS (ITERATOR OF CONTEXT)) ((SELF MODULE)) :PUBLIC? TRUE)", ((cpp_method_code)(&Module::parent_contexts)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (PARENT-CONTEXTS (ITERATOR OF CONTEXT)) ((SELF WORLD)) :PUBLIC? TRUE)", ((cpp_method_code)(&World::parent_contexts)), ((cpp_method_code)(NULL)));
    define_function_object("COMPUTE-CONTEXT-OR-MODULE-FROM-PATHNAME", "(DEFUN (COMPUTE-CONTEXT-OR-MODULE-FROM-PATHNAME CONTEXT INTEGER) ((PATHNAME STRING) (MODULE? BOOLEAN) (SYMBOLNAME? BOOLEAN)))", ((cpp_function_code)(&compute_context_or_module_from_pathname)), NULL);
    define_function_object("COMPONENT-MATCH?", "(DEFUN (COMPONENT-MATCH? BOOLEAN) ((COMPONENT STRING) (STRING STRING) (START INTEGER) (END INTEGER)))", ((cpp_function_code)(&component_matchP)), NULL);
    define_function_object("COMPUTE-MODULE-AND-BARE-NAME", "(DEFUN (COMPUTE-MODULE-AND-BARE-NAME MODULE STRING) ((NAME STRING)) :PUBLIC? TRUE)", ((cpp_function_code)(&compute_module_and_bare_name)), NULL);
    define_function_object("QUALIFIED-NAME-MODULE-NAME", "(DEFUN (QUALIFIED-NAME-MODULE-NAME STRING) ((QUALIFIEDNAME STRING)))", ((cpp_function_code)(&qualified_name_module_name)), NULL);
    define_function_object("QUALIFIED-NAME-BARE-NAME", "(DEFUN (QUALIFIED-NAME-BARE-NAME STRING) ((QUALIFIEDNAME STRING)))", ((cpp_function_code)(&qualified_name_bare_name)), NULL);
    define_function_object("DWIM-MODULE-AND-BARE-NAME", "(DEFUN (DWIM-MODULE-AND-BARE-NAME MODULE STRING) ((NAME STRING) (WARN? BOOLEAN)))", ((cpp_function_code)(&dwim_module_and_bare_name)), NULL);
    define_function_object("INTERN-QUALIFIED-NAME", "(DEFUN (INTERN-QUALIFIED-NAME GENERALIZED-SYMBOL) ((NAME STRING) (ENABLE-CASE-CONVERSION? BOOLEAN)))", ((cpp_function_code)(&intern_qualified_name)), NULL);
    define_function_object("GET-STELLA-CONTEXT", "(DEFUN (GET-STELLA-CONTEXT CONTEXT) ((PATHNAME STRING) (WARN? BOOLEAN)) :DOCUMENTATION \"Return the context located at 'pathName', or NULL\nif no such context exists.  If 'warn?' is TRUE, print a warning\nmessage if no context is found.\" :PUBLIC? TRUE)", ((cpp_function_code)(&get_stella_context)), NULL);
    define_function_object("GET-STELLA-MODULE", "(DEFUN (GET-STELLA-MODULE MODULE) ((PATHNAME STRING) (WARN? BOOLEAN)) :DOCUMENTATION \"Return the module located at 'pathName', or NULL\nif no such module exists.  The search looks at ancestors and top-most\n (cardinal) modules.  If 'warn?' is TRUE, print a warning\nmessage if no module is found.\" :PUBLIC? TRUE)", ((cpp_function_code)(&get_stella_module)), NULL);
    define_function_object("COERCE-TO-MODULE-NAME", "(DEFUN (COERCE-TO-MODULE-NAME STRING) ((NAMESPEC NAME) (WARN? BOOLEAN)))", ((cpp_function_code)(&coerce_to_module_name)), NULL);
    define_function_object("COERCE-TO-MODULE", "(DEFUN (COERCE-TO-MODULE MODULE) ((OBJECT OBJECT) (WARN? BOOLEAN)) :PUBLIC? TRUE)", ((cpp_function_code)(&coerce_to_module)), NULL);
    define_function_object("FIND-OR-CREATE-MODULE", "(DEFUN (FIND-OR-CREATE-MODULE MODULE) ((PATHNAME STRING)) :DOCUMENTATION \"Return a module located at `pathname' if one exists,\n  otherwise create one\")", ((cpp_function_code)(&find_or_create_module)), NULL);
    define_function_object("CHANGE-CURRENT-MODULE", "(DEFUN (CHANGE-CURRENT-MODULE MODULE) ((MODULE MODULE)))", ((cpp_function_code)(&change_current_module)), NULL);
    define_function_object("CHANGE-CURRENT-CONTEXT", "(DEFUN (CHANGE-CURRENT-CONTEXT CONTEXT) ((CONTEXT CONTEXT)))", ((cpp_function_code)(&change_current_context)), NULL);
    define_method_object("(DEFMETHOD (CHANGE-CONTEXT CONTEXT) ((CONTEXT CONTEXT)) :DOCUMENTATION \"Change the current context to be the context\n'context'.\" :PUBLIC? TRUE)", ((cpp_method_code)(&Context::change_context)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (CHANGE-CONTEXT CONTEXT) ((CONTEXTNAME STRING)) :DOCUMENTATION \"Change the current context to be the context named\n'contextName'.\" :PUBLIC? TRUE)", ((cpp_method_code)(NULL)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (CHANGE-MODULE MODULE) ((MODULE MODULE)) :DOCUMENTATION \"Change the current module to be the module\n'module'.\" :PUBLIC? TRUE)", ((cpp_method_code)(&Module::change_module)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (CHANGE-MODULE MODULE) ((MODULENAME STRING)) :DOCUMENTATION \"Change the current module to be the module named\n'moduleName'.\" :PUBLIC? TRUE)", ((cpp_method_code)(NULL)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (MULTIPLE-PARENTS? BOOLEAN) ((MODULE MODULE)) :DOCUMENTATION \"Return TRUE if 'module' has more than one parent.\")", ((cpp_method_code)(&Module::multiple_parentsP)), ((cpp_method_code)(NULL)));
    define_function_object("NORMALIZE-PARENT-MODULES", "(DEFUN NORMALIZE-PARENT-MODULES ((SELF MODULE)))", ((cpp_function_code)(&normalize_parent_modules)), NULL);
    define_function_object("INHERIT-SUPERCONTEXTS", "(DEFUN INHERIT-SUPERCONTEXTS ((MODULE MODULE)))", ((cpp_function_code)(&inherit_supercontexts)), NULL);
    define_function_object("UNINHERIT-SUPERCONTEXTS", "(DEFUN UNINHERIT-SUPERCONTEXTS ((MODULE MODULE)))", ((cpp_function_code)(&uninherit_supercontexts)), NULL);
    define_function_object("INHERIT-USED-MODULES", "(DEFUN INHERIT-USED-MODULES ((MODULE MODULE)))", ((cpp_function_code)(&inherit_used_modules)), NULL);
    define_function_object("UNINHERIT-USED-MODULES", "(DEFUN UNINHERIT-USED-MODULES ((MODULE MODULE)))", ((cpp_function_code)(&uninherit_used_modules)), NULL);
    define_function_object("FINALIZE-MODULE", "(DEFUN FINALIZE-MODULE ((SELF MODULE)))", ((cpp_function_code)(&finalize_module)), NULL);
    define_function_object("FINALIZE-WORLD", "(DEFUN FINALIZE-WORLD ((SELF WORLD)))", ((cpp_function_code)(&finalize_world)), NULL);
    define_method_object("(DEFMETHOD UNFINALIZE-MODULE ((SELF MODULE)))", ((cpp_method_code)(&Module::unfinalize_module)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD UNFINALIZE-WORLD ((SELF WORLD)))", ((cpp_method_code)(&World::unfinalize_world)), ((cpp_method_code)(NULL)));
    define_function_object("DESTROY-MODULE", "(DEFUN DESTROY-MODULE ((SELF MODULE)) :DOCUMENTATION \"Destroy the module 'self', and recursively destroy\nall contexts that inherit 'self'.\" :PUBLIC? TRUE)", ((cpp_function_code)(&destroy_module)), NULL);
    define_function_object("HELP-DESTROY-MODULE", "(DEFUN HELP-DESTROY-MODULE ((SELF MODULE)))", ((cpp_function_code)(&help_destroy_module)), NULL);
    define_function_object("DESTROY-WORLD", "(DEFUN DESTROY-WORLD ((SELF WORLD)) :PUBLIC? TRUE)", ((cpp_function_code)(&destroy_world)), NULL);
    define_method_object("(DEFMETHOD DESTROY-CONTEXT ((SELF MODULE)) :DOCUMENTATION \"Destroy the context 'self', and recursively destroy\nall contexts that inherit 'self'.\" :PUBLIC? TRUE)", ((cpp_method_code)(&Module::destroy_context)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD DESTROY-CONTEXT ((SELF WORLD)) :DOCUMENTATION \"Destroy the context 'self', and recursively destroy\nall contexts that inherit 'self'.\" :PUBLIC? TRUE)", ((cpp_method_code)(&World::destroy_context)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD DESTROY-CONTEXT ((SELF STRING)) :DOCUMENTATION \"Destroy the context 'self', and recursively destroy\nall contexts that inherit 'self'.\" :PUBLIC? TRUE)", ((cpp_method_code)(NULL)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD DESTROY-CONTEXT ((SELF CONTEXT)) :PUBLIC? TRUE :DOCUMENTATION \"Make the translator happy.\")", ((cpp_method_code)(&Context::destroy_context)), ((cpp_method_code)(NULL)));
    define_function_object("LINK-TO-PARENT-MODULE", "(DEFUN LINK-TO-PARENT-MODULE ((SELF MODULE) (PARENT MODULE) (INSERT-FIRST? BOOLEAN)))", ((cpp_function_code)(&link_to_parent_module)), NULL);
    define_function_object("INCORPORATE-MODULE-NAME", "(DEFUN INCORPORATE-MODULE-NAME ((MODULE MODULE) (NAME STRING)))", ((cpp_function_code)(&incorporate_module_name)), NULL);
    define_function_object("INCORPORATE-INCLUDES-MODULES", "(DEFUN INCORPORATE-INCLUDES-MODULES ((MODULE MODULE) (INCLUDEES OBJECT)))", ((cpp_function_code)(&incorporate_includes_modules)), NULL);
    define_function_object("INCORPORATE-USES-MODULES", "(DEFUN INCORPORATE-USES-MODULES ((MODULE MODULE) (USEES OBJECT)))", ((cpp_function_code)(&incorporate_uses_modules)), NULL);
    define_function_object("INCORPORATE-MODULE-OPTIONS", "(DEFUN INCORPORATE-MODULE-OPTIONS ((SELF MODULE) (OPTIONS CONS)))", ((cpp_function_code)(&incorporate_module_options)), NULL);
    define_function_object("UNDEFINE-MODULE", "(DEFUN UNDEFINE-MODULE ((OLDMODULE MODULE) (NEWMODULE MODULE)))", ((cpp_function_code)(&undefine_module)), NULL);
    define_function_object("DEFINE-MODULE", "(DEFUN (DEFINE-MODULE MODULE) ((NAME STRING) (OPTIONS CONS)) :DOCUMENTATION \"Define or redefine a module named 'name' having the\noptions 'options'.  Return the new module.\" :PUBLIC? TRUE)", ((cpp_function_code)(&define_module)), NULL);
    define_function_object("IDENTICAL-MODULE-SIGNATURES?", "(DEFUN (IDENTICAL-MODULE-SIGNATURES? BOOLEAN) ((OLDMODULE MODULE) (NEWMODULE MODULE) (NEWOPTIONS STRING)))", ((cpp_function_code)(&identical_module_signaturesP)), NULL);
    define_function_object("DEFINE-MODULE-FROM-STRINGIFIED-SOURCE", "(DEFUN (DEFINE-MODULE-FROM-STRINGIFIED-SOURCE MODULE) ((NAME STRING) (STRINGIFIEDOPTIONS STRING)) :PUBLIC? TRUE)", ((cpp_function_code)(&define_module_from_stringified_source)), NULL);
    define_function_object("YIELD-DEFINE-MODULE", "(DEFUN (YIELD-DEFINE-MODULE CONS) ((MODULE MODULE)))", ((cpp_function_code)(&yield_define_module)), NULL);
    define_function_object("DEFMODULE", string_concatenate("(DEFUN DEFMODULE ((NAME NAME) |&REST| (OPTIONS OBJECT)) :PUBLIC? TRUE :COMMAND? TRUE :EVALUATE-ARGUMENTS? FALSE :DOCUMENTATION \"Define (or redefine) a module named `name'.\nThe accepted syntax is:\n	 \n  (defmodule <module-name>\n     [:documentation <docstring>]\n     [:includes {<module-name> | (<module-name>*)}]\n     [:uses {<module-name> | (<module-name>*)}]\n     [:lisp-package <package-name-string>]\n     [:java-package <package-specification-string>]\n     [:cpp-namespace <namespace-name-string>]\n     [:java-catchall-class\n     [:api? {TRUE | FALSE}]\n     [:case-sensitive? {TRUE | FALSE}]\n     [:shadow (<symbol>*)]\n     [:java-catchall-class <class-name-string>]\n     [<other-options>*])\n	\n`name' can be a string or a symbol.\n\nModules include objects from other modules via two separate mechanisms:\n(1) they inherit from their parents specified via the `:includes' option\nand/or a fully qualified module name, and (2) they inherit from used\nmodules specified via the `:uses' option.  The main difference between\nthe two mechanisms is that inheritance from parents is transitive, while\nuses-links are only followed one level deep.  I.e., a module A that uses\nB will see all objects of B (and any of B's parents) but not see anything\nfrom modules used by B.  Another difference is that only objects declared\nas public can be inherited via uses-links (this is not yet enforced).\nNote that - contrary to Lisp - there are separate name spaces for classes,\nfunctions, and variables.  For example, a module could inherit the class\n`CONS' from the `STELLA' module, but shadow the function of the same name.\n\nThe above discussion of `:includes' and `:uses' semantics keyed on the\ninheritance/visibility of symbols. The PowerLoom system makes another\nvery important distinction: If a module 'A' is inherited directly or\nindirectly via `:includes' specification(s) by a submodule 'B', then all\ndefinitions and facts asserted in 'A' are visible in 'B'. This is not the\ncases for `:uses'; the `:uses' options does not impact inheritance of\npropositions at all.\n\nThe list of modules specified in the\n`:includes' option plus (if supplied) the parent in the path used for\n`name' become the new module's parents. If no `:uses' option was\nsupplied, the new module will use the `STELLA' module by default,\notherwise, it will use the set of specified modules.\nIf `:case-sensitive?' is supplied as TRUE, symbols in the module will be\ninterned case-sensitively, otherwise (the default), they will be\nconverted to uppercase before they get interned. Modules can shadow\ndefinitions of functions and classes inherited from parents or used\nmodules. Shadowing is done automatically, but generates a warning unless\nthe shadowed type or function name is listed in the `:shadow' option of\nthe module definition .\n\nExamples:\n	 \n  (defmodule \\\"PL-KERNEL/PL-USER\\\"\n    :uses (\\\"LOGIC\\\" \\\"STELLA\\\")\n    :package \\\"PL-USER\\\")\n\n  (defmodule PL-USER/GENEALOGY)\n	\nThe remaining options are relevant only for modules that contain STELLA\ncode.  Modules used only to contain knowledge base definitions and assertions\nhave no use for them:\n\nThe keywords `:lisp-package', `:java-package', and `:cpp-package' specify\nthe name of a native package or name space in which symbols of the module\nshould be allocated when they get translated into one of Lisp, Java, or\nC++. By default, Lisp symbols are allocated in the `STELLA' package, and\nC++ names are translated without any prefixes. The rules that the STELLA\ntranslator uses to attach translated Java objects to classes and packages\nare somewhat complex. Use :java-package option to specify a list of\npackage names (separated by periods) that prefix the Java object in this\nmodule.  Use :java-catchall-class to specify the name of the Java class to\ncontain all global & special variables, parameter-less functions and functions \ndefined on arguments that are not classes in the current module.\nThe default value will be the name of the module.\n\nWhen set to TRUE, the :api? option tells t", "he PowerLoom User Manual\ngenerator that all functions defined in this module should be included in\nthe API section. Additionally, the Java translator makes all API\nfunctions `synchronized'.\n\")", 0), ((cpp_function_code)(&defmodule)), ((cpp_function_code)(&defmodule_evaluator_wrapper)));
    define_function_object("IN-MODULE", "(DEFUN (IN-MODULE MODULE) ((NAME NAME)) :PUBLIC? TRUE :COMMAND? TRUE :EVALUATE-ARGUMENTS? FALSE :LISP-MACRO? TRUE :DOCUMENTATION \"Change the current module to the module named `name'.\")", ((cpp_function_code)(&in_module)), NULL);
    define_function_object("CREATE-WORLD", "(DEFUN (CREATE-WORLD WORLD) ((PARENTCONTEXT CONTEXT) (NAME STRING)) :DOCUMENTATION \"Create a new world below the world or module 'parentContext'.\nOptionally, specify a name.\" :PUBLIC? TRUE :INLINE FINALIZE-WORLD)", ((cpp_function_code)(&create_world)), NULL);
    define_function_object("PUSH-WORLD", "(DEFUN (PUSH-WORLD WORLD) () :DOCUMENTATION \"Spawn a new world that is a child of the current context,\nand change the current context to the new world.\" :PUBLIC? TRUE)", ((cpp_function_code)(&push_world)), NULL);
    define_function_object("POP-WORLD", "(DEFUN (POP-WORLD CONTEXT) () :DOCUMENTATION \"Destroy the current world and change the current\ncontext to be its parent.  Return the current context. Nothing happens\nif there is no current world.\" :PUBLIC? TRUE)", ((cpp_function_code)(&pop_world)), NULL);
    define_function_object("ALL-SUBCONTEXTS", "(DEFUN (ALL-SUBCONTEXTS (ALL-PURPOSE-ITERATOR OF CONTEXT)) ((CONTEXT CONTEXT) (TRAVERSAL KEYWORD)) :DOCUMENTATION \"Return an iterator that generates all subcontexts of\n'self' (not including 'self') in the order specified by 'traversal' (one\nof :preorder, :inorder, or :postorder).\")", ((cpp_function_code)(&all_subcontexts)), NULL);
    define_function_object("ALL-SUBCONTEXTS-NEXT?", "(DEFUN (ALL-SUBCONTEXTS-NEXT? BOOLEAN) ((SELF ALL-PURPOSE-ITERATOR)))", ((cpp_function_code)(&all_subcontexts_nextP)), NULL);
    define_function_object("ALL-CONTEXTS", "(DEFUN (ALL-CONTEXTS (ITERATOR OF CONTEXT)) () :PUBLIC? TRUE :DOCUMENTATION \"Return an iterator that generates all contexts.\")", ((cpp_function_code)(&all_contexts)), NULL);
    define_function_object("FILTER-MODULE?", "(DEFUN (FILTER-MODULE? BOOLEAN) ((SELF OBJECT) (ITERATOR ALL-PURPOSE-ITERATOR)))", ((cpp_function_code)(&filter_moduleP)), NULL);
    define_function_object("ALL-MODULES", "(DEFUN (ALL-MODULES (ITERATOR OF MODULE)) () :PUBLIC? TRUE :DOCUMENTATION \"Return an iterator that generates all modules.\")", ((cpp_function_code)(&all_modules)), NULL);
    define_function_object("ALL-INCLUDED-MODULES", "(DEFUN (ALL-INCLUDED-MODULES (ITERATOR OF MODULE)) ((SELF MODULE)) :PUBLIC? TRUE :DOCUMENTATION \"Generate a sequence of all modules included\nby 'self', inclusive, starting from the highest ancestor and working\ndown to 'self' (which is last).\")", ((cpp_function_code)(&all_included_modules)), NULL);
    define_function_object("VISIBLE-MODULES", "(DEFUN (VISIBLE-MODULES (ITERATOR OF MODULE)) ((FROM MODULE)) :PUBLIC? TRUE :DOCUMENTATION \"Return an iterator that generates all modules visible from module `from'.\nThe generated modules are generated from most- to least-specific and \nwill start with the module `from'.\")", ((cpp_function_code)(&visible_modules)), NULL);
  }
}

void startup_modules() {
  { 
    BIND_STELLA_SPECIAL(oMODULEo, Module*, oSTELLA_MODULEo);
    BIND_STELLA_SPECIAL(oCONTEXTo, Context*, oMODULEo.get());
    if (current_startup_time_phaseP(2)) {
      help_startup_modules1();
    }
    if (current_startup_time_phaseP(4)) {
      oSUBCONTEXT_REVISION_POLICYo = KWD_MODULES_CLEAR;
      oDEFINE_MODULE_HOOKSo = new_hook_list();
    }
    if (current_startup_time_phaseP(5)) {
      define_stella_type_from_stringified_source("(DEFTYPE NAME OBJECT)");
    }
    if (current_startup_time_phaseP(6)) {
      finalize_classes();
    }
    if (current_startup_time_phaseP(7)) {
      help_startup_modules2();
      define_function_object("CARDINAL-MODULE?", "(DEFUN (CARDINAL-MODULE? BOOLEAN) ((SELF MODULE)))", ((cpp_function_code)(&cardinal_moduleP)), NULL);
      define_function_object("VISIBLE-FROM?", "(DEFUN (VISIBLE-FROM? BOOLEAN) ((VIEWEDCONTEXT CONTEXT) (FROMCONTEXT CONTEXT)) :PUBLIC? TRUE)", ((cpp_function_code)(&visible_fromP)), NULL);
      define_function_object("CLEAR-ONE-CONTEXT", "(DEFUN CLEAR-ONE-CONTEXT ((SELF CONTEXT)))", ((cpp_function_code)(&clear_one_context)), NULL);
      define_function_object("HELP-CLEAR-CONTEXT", "(DEFUN HELP-CLEAR-CONTEXT ((SELF CONTEXT)))", ((cpp_function_code)(&help_clear_context)), NULL);
      define_function_object("CLEAR-CONTEXT", "(DEFUN CLEAR-CONTEXT ((SELF CONTEXT)) :PUBLIC? TRUE :DOCUMENTATION \"Destroy all objects belonging to 'self' or any of its subcontexts.\")", ((cpp_function_code)(&clear_context)), NULL);
      define_function_object("CALL-CLEAR-MODULE", "(DEFUN CALL-CLEAR-MODULE (|&REST| (NAME NAME)) :PUBLIC? TRUE :COMMAND? TRUE :EVALUATE-ARGUMENTS? TRUE :LISP-MACRO? FALSE :DOCUMENTATION \"Destroy all objects belonging to module `name' or any of its children.\nIf no `name' is supplied, the current module will be cleared after\nconfirming with the user.  Important modules such as STELLA are protected\nagainst accidental clearing.\")", ((cpp_function_code)(&call_clear_module)), ((cpp_function_code)(&call_clear_module_evaluator_wrapper)));
      define_function_object("CLEAR-MODULE", "(DEFUN CLEAR-MODULE (|&REST| (NAME NAME)) :PUBLIC? TRUE :COMMAND? TRUE :EVALUATE-ARGUMENTS? FALSE :DOCUMENTATION \"Destroy all objects belonging to module `name' or any of its children.\nIf no `name' is supplied, the current module will be cleared after\nconfirming with the user.  Important modules such as STELLA are protected\nagainst accidental clearing.\")", ((cpp_function_code)(&clear_module)), ((cpp_function_code)(&clear_module_evaluator_wrapper)));
      define_function_object("STARTUP-MODULES", "(DEFUN STARTUP-MODULES () :PUBLIC? TRUE)", ((cpp_function_code)(&startup_modules)), NULL);
      { Method_Slot* function = lookup_function(SYM_MODULES_STELLA_STARTUP_MODULES);

        set_dynamic_slot_value(function->dynamic_slots, SYM_MODULES_STELLA_METHOD_STARTUP_CLASSNAME, wrap_string("Startup-Modules"), NULL_STRING_WRAPPER);
      }
    }
    if (current_startup_time_phaseP(8)) {
      finalize_slots();
      cleanup_unfinalized_classes();
    }
    if (current_startup_time_phaseP(9)) {
      define_stella_global_variable_from_stringified_source("(DEFGLOBAL *SUBCONTEXT-REVISION-POLICY* KEYWORD :CLEAR :DOCUMENTATION \"Controls actions reflexive transitive closure of\n   subcontexts when a context is revised.\n   Values are ':destroy' -- destroy subcontexts;\n   ':clear' -- clear contents of subcontexts;\n   ':preserve' -- don't disturb subcontexts.\")");
      define_stella_global_variable_from_stringified_source("(DEFSPECIAL *SHADOWEDSURROGATES* (CONS OF SYMBOL) NULL :DOCUMENTATION \"Holds list of symbols representing surrogates\nto be shadowed during module finalization.\")");
      define_stella_global_variable_from_stringified_source("(DEFGLOBAL *DEFINE-MODULE-HOOKS* HOOK-LIST (NEW HOOK-LIST) :DOCUMENTATION \"HOOK-LIST called by 'define-module', applied to a\n'module' argument.\")");
      set_dynamic_slot_value(oROOT_MODULEo->dynamic_slots, SYM_MODULES_STELLA_CLEARABLEp, (FALSE ? TRUE_WRAPPER : FALSE_WRAPPER), FALSE_WRAPPER);
      set_dynamic_slot_value(oSTELLA_MODULEo->dynamic_slots, SYM_MODULES_STELLA_CLEARABLEp, (FALSE ? TRUE_WRAPPER : FALSE_WRAPPER), FALSE_WRAPPER);
      set_dynamic_slot_value(oCOMMON_LISP_MODULEo->dynamic_slots, SYM_MODULES_STELLA_CLEARABLEp, (FALSE ? TRUE_WRAPPER : FALSE_WRAPPER), FALSE_WRAPPER);
    }
  }
}

Keyword* KWD_MODULES_CLEAR = NULL;

Surrogate* SGT_MODULES_STELLA_MODULE = NULL;

Surrogate* SGT_MODULES_STELLA_UNKNOWN_SURROGATE = NULL;

Symbol* SYM_MODULES_STELLA_UNKNOWN_SYMBOL = NULL;

Symbol* SYM_MODULES_STELLA_NAME = NULL;

Surrogate* SGT_MODULES_STELLA_MUTABLE_STRING_WRAPPER = NULL;

Symbol* SYM_MODULES_STELLA_SHADOWED_SURROGATES = NULL;

Surrogate* SGT_MODULES_STELLA_CONS = NULL;

Keyword* KWD_MODULES_INCLUDES = NULL;

Keyword* KWD_MODULES_USES = NULL;

Keyword* KWD_MODULES_SHADOW = NULL;

Keyword* KWD_MODULES_DOCUMENTATION = NULL;

Keyword* KWD_MODULES_CASE_SENSITIVEp = NULL;

Symbol* SYM_MODULES_STELLA_TRUE = NULL;

Keyword* KWD_MODULES_APIp = NULL;

Symbol* SYM_MODULES_STELLA_APIp = NULL;

Keyword* KWD_MODULES_LISP_PACKAGE = NULL;

Symbol* SYM_MODULES_STELLA_MODULE_LISP_PACKAGE = NULL;

Keyword* KWD_MODULES_CPP_PACKAGE = NULL;

Symbol* SYM_MODULES_STELLA_MODULE_CPP_PACKAGE = NULL;

Keyword* KWD_MODULES_JAVA_PACKAGE = NULL;

Symbol* SYM_MODULES_STELLA_JAVA_PACKAGE = NULL;

Keyword* KWD_MODULES_JAVA_CATCHALL_CLASS = NULL;

Symbol* SYM_MODULES_STELLA_JAVA_FLOTSAM_CLASS = NULL;

Keyword* KWD_MODULES_NICKNAME = NULL;

Keyword* KWD_MODULES_CLEARABLEp = NULL;

Symbol* SYM_MODULES_STELLA_CLEARABLEp = NULL;

Keyword* KWD_MODULES_PROTECT_SURROGATESp = NULL;

Symbol* SYM_MODULES_STELLA_PROTECT_SURROGATESp = NULL;

Keyword* KWD_MODULES_CODE_ONLYp = NULL;

Symbol* SYM_MODULES_STELLA_CODE_ONLYp = NULL;

Keyword* KWD_MODULES_REQUIRES = NULL;

Keyword* KWD_MODULES_DESTROY = NULL;

Keyword* KWD_MODULES_PRESERVE = NULL;

Keyword* KWD_MODULES_PREORDER = NULL;

Surrogate* SGT_MODULES_STELLA_WORLD = NULL;

Symbol* SYM_MODULES_STELLA_DEFINE_MODULE_FROM_STRINGIFIED_SOURCE = NULL;

Symbol* SYM_MODULES_STELLA_WORLD_NAME = NULL;

Keyword* KWD_MODULES_POSTORDER = NULL;

Keyword* KWD_MODULES_INORDER = NULL;

Symbol* SYM_MODULES_STELLA_STARTUP_MODULES = NULL;

Symbol* SYM_MODULES_STELLA_METHOD_STARTUP_CLASSNAME = NULL;

} // end of namespace stella
