// /tmp/build-stella/stella-3.0.0.beta/native/cpp/stella/streams.hh

/*--------------------------- BEGIN LICENSE BLOCK ---------------------------+
|                                                                            |
| Version: MPL 1.1/GPL 2.0/LGPL 2.1                                          |
|                                                                            |
| The contents of this file are subject to the Mozilla Public License        |
| Version 1.1 (the "License"); you may not use this file except in           |
| compliance with the License. You may obtain a copy of the License at       |
| http://www.mozilla.org/MPL/                                                |
|                                                                            |
| Software distributed under the License is distributed on an "AS IS" basis, |
| WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License   |
| for the specific language governing rights and limitations under the       |
| License.                                                                   |
|                                                                            |
| The Original Code is the STELLA Programming Language.                      |
|                                                                            |
| The Initial Developer of the Original Code is                              |
| UNIVERSITY OF SOUTHERN CALIFORNIA, INFORMATION SCIENCES INSTITUTE          |
| 4676 Admiralty Way, Marina Del Rey, California 90292, U.S.A.               |
|                                                                            |
| Portions created by the Initial Developer are Copyright (C) 1996-2001      |
| the Initial Developer. All Rights Reserved.                                |
|                                                                            |
| Contributor(s):                                                            |
|                                                                            |
| Alternatively, the contents of this file may be used under the terms of    |
| either the GNU General Public License Version 2 or later (the "GPL"), or   |
| the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),   |
| in which case the provisions of the GPL or the LGPL are applicable instead |
| of those above. If you wish to allow use of your version of this file only |
| under the terms of either the GPL or the LGPL, and not to allow others to  |
| use your version of this file under the terms of the MPL, indicate your    |
| decision by deleting the provisions above and replace them with the notice |
| and other provisions required by the GPL or the LGPL. If you do not delete |
| the provisions above, a recipient may use your version of this file under  |
| the terms of any one of the MPL, the GPL or the LGPL.                      |
|                                                                            |
+---------------------------- END LICENSE BLOCK ----------------------------*/


namespace stella {

// Class definitions:
class Stream_Iterator : public Abstract_Iterator {
// Iterator that yields tokens from an input stream.
public:
  Input_Stream* the_stream;
};

class S_Expression_Iterator : public Stream_Iterator {
// Iterator that yields s-expressions from an input stream.
public:
  Object* value;
public:
  virtual void free();
  virtual boolean nextP();
  virtual Surrogate* primary_type();
};

class Line_Iterator : public Stream_Iterator {
// Iterator that yields lines from an input stream.
public:
  char* value;
public:
  virtual void free();
  virtual boolean nextP();
  virtual Surrogate* primary_type();
};

class Character_Iterator : public Stream_Iterator {
// Iterator that yields characters from an input stream.
public:
  char value;
public:
  virtual void free();
  virtual boolean nextP();
  virtual Surrogate* primary_type();
};


// Global declarations:
extern Symbol* EOL;
extern char* EOL_STRING;
extern Input_Stream* STANDARD_INPUT;
extern Output_Stream* STANDARD_OUTPUT;
extern Output_Stream* STANDARD_WARNING;
extern Output_Stream* STANDARD_ERROR;
extern String_Wrapper* SINGLE_QUOTE_STRING;
extern String_Wrapper* SINGLE_BQUOTE_STRING;
extern List* oOPEN_FILE_STREAMSo;
extern char* oHTML_QUOTED_CHARACTERSo;
extern Key_Value_List* oHTML_ESCAPE_TABLEo;

// Function signatures:
Cons* format_message_arguments(Cons* messageargumentstree, boolean trailingeolP);
Object* print_expander(Cons* body);
Object* inform_expander(Cons* body);
Object* warn_expander(Cons* body);
Object* signal_expander(Symbol* type, Cons* body);
Object* error_expander(Cons* body);
Object* continuable_error_expander(Cons* body);
Object* signal_read_error_expander(Cons* body);
Cons* format_walk_message_arguments(Cons* messageargumentstree);
Object* walk_error_expander(Cons* body);
Object* walk_inform_expander(Cons* body);
Object* walk_warn_expander(Cons* body);
Object* safety_expander(Integer_Wrapper* level, Object* test, Cons* body);
Object* evaluation_error_expander(Cons* body);
Cons* format_string_message_arguments(Cons* messagearguments, boolean trailingeolP);
Object* message_to_string_expander(Cons* body);
Cons* expand_markup_tag_function(Cons* tag_and_stream, Cons* body, boolean xmlP);
Object* with_html_tag_expander(Cons* tag_and_stream, Cons* body);
Object* with_xml_tag_expander(Cons* tag_and_stream, Cons* body);
boolean terminate_output_streamP(Output_Stream* self);
boolean terminate_input_streamP(Input_Stream* self);
void initialize_file_output_stream(Output_File_Stream* self);
boolean terminate_file_output_streamP(Output_File_Stream* self);
void initialize_file_input_stream(Input_File_Stream* self);
boolean terminate_file_input_streamP(Input_File_Stream* self);
void initialize_string_output_stream(Output_String_Stream* self);
void initialize_string_input_stream(Input_String_Stream* self);
void close_stream(Stream* self);
void flush_output(Output_Stream* self);
void close_all_files();
Input_Stream* open_network_stream(char* host, int port, Output_Stream*& _return1);
boolean html_character_needs_quotingP(char ch);
void write_html_character_quoting_special_characters(ostream* stream, char ch);
void write_html_quoting_special_characters(ostream* stream, char* input);
void write_html_escaping_url_special_characters(ostream* stream, char* input);
char* unescape_html_string(char* input);
Object* access_stream_iterator_slot_value(Stream_Iterator* self, Symbol* slotname, Object* value, boolean setvalueP);
S_Expression_Iterator* new_s_expression_iterator();
Line_Iterator* new_line_iterator();
Character_Iterator* new_character_iterator();
boolean terminate_stream_iteratorP(Stream_Iterator* self);
S_Expression_Iterator* s_expressions(Input_Stream* stream);
Line_Iterator* lines(Input_Stream* stream);
Character_Iterator* characters(Input_Stream* stream);
void help_startup_streams1();
void help_startup_streams2();
void help_startup_streams3();
void startup_streams();

// Auxiliary global declarations:
extern Symbol* SYM_STREAMS_STELLA_EOL;
extern Symbol* SYM_STREAMS_STELLA_PRINT_STREAM;
extern Symbol* SYM_STREAMS_STELLA_STANDARD_OUTPUT;
extern Symbol* SYM_STREAMS_STELLA_PRINT;
extern Symbol* SYM_STREAMS_STELLA_PRINT_EXPANDER;
extern Symbol* SYM_STREAMS_STELLA_INFORM;
extern Symbol* SYM_STREAMS_STELLA_INFORM_EXPANDER;
extern Keyword* KWD_STREAMS_COMMON_LISP;
extern Keyword* KWD_STREAMS_WARN;
extern Symbol* SYM_STREAMS_STELLA_STANDARD_WARNING;
extern Symbol* SYM_STREAMS_STELLA_WARN;
extern Symbol* SYM_STREAMS_STELLA_WARN_EXPANDER;
extern Symbol* SYM_STREAMS_STELLA_NULL;
extern Symbol* SYM_STREAMS_STELLA_STELLA_EXCEPTION;
extern Surrogate* SGT_STREAMS_STELLA_STRING_WRAPPER;
extern Symbol* SYM_STREAMS_STELLA_SIGNAL_EXCEPTION;
extern Symbol* SYM_STREAMS_STELLA_NEW;
extern Keyword* KWD_STREAMS_MESSAGE;
extern Symbol* SYM_STREAMS_STELLA_LET;
extern Symbol* SYM_STREAMS_STELLA_OUTPUT_STRING_STREAM;
extern Symbol* SYM_STREAMS_STELLA_THE_STRING;
extern Symbol* SYM_STREAMS_STELLA_SIGNAL;
extern Symbol* SYM_STREAMS_STELLA_SIGNAL_EXPANDER;
extern Symbol* SYM_STREAMS_STELLA_ERROR;
extern Symbol* SYM_STREAMS_STELLA_ERROR_EXPANDER;
extern Keyword* KWD_STREAMS_JAVA;
extern Symbol* SYM_STREAMS_STELLA_ERROR_MESSAGE_;
extern Symbol* SYM_STREAMS_STELLA_VERBATIM;
extern Keyword* KWD_STREAMS_CONTINUABLE_ERROR;
extern Symbol* SYM_STREAMS_STELLA_CONTINUABLE_ERROR;
extern Symbol* SYM_STREAMS_STELLA_CONTINUABLE_ERROR_EXPANDER;
extern Symbol* SYM_STREAMS_STELLA_READ_EXCEPTION;
extern Symbol* SYM_STREAMS_STELLA_SIGNAL_READ_ERROR;
extern Symbol* SYM_STREAMS_STELLA_SIGNAL_READ_ERROR_EXPANDER;
extern Symbol* SYM_STREAMS_STELLA_DE_UGLIFY_PARSE_TREE;
extern Symbol* SYM_STREAMS_STELLA_SPECIAL;
extern Symbol* SYM_STREAMS_STELLA_oPRINTREADABLYpo;
extern Symbol* SYM_STREAMS_STELLA_TRUE;
extern Symbol* SYM_STREAMS_STELLA_SIGNAL_TRANSLATION_ERROR;
extern Symbol* SYM_STREAMS_STELLA_UNLESS;
extern Symbol* SYM_STREAMS_STELLA_SUPPRESS_WARNINGSp;
extern Symbol* SYM_STREAMS_STELLA_PRINT_ERROR_CONTEXT;
extern Symbol* SYM_STREAMS_STELLA_STANDARD_ERROR;
extern Symbol* SYM_STREAMS_STELLA_WALK_ERROR;
extern Symbol* SYM_STREAMS_STELLA_WALK_ERROR_EXPANDER;
extern Symbol* SYM_STREAMS_STELLA_SIGNAL_TRANSLATION_WARNING;
extern Symbol* SYM_STREAMS_STELLA_WALK_INFORM;
extern Symbol* SYM_STREAMS_STELLA_WALK_INFORM_EXPANDER;
extern Symbol* SYM_STREAMS_STELLA_WALK_WARN;
extern Symbol* SYM_STREAMS_STELLA_WALK_WARN_EXPANDER;
extern Symbol* SYM_STREAMS_STELLA_WHEN;
extern Symbol* SYM_STREAMS_STELLA_NOT;
extern Keyword* KWD_STREAMS_ERROR;
extern Symbol* SYM_STREAMS_STELLA_SAFETY;
extern Symbol* SYM_STREAMS_STELLA_SAFETY_EXPANDER;
extern Symbol* SYM_STREAMS_STELLA_oEVALUATIONTREEo;
extern Symbol* SYM_STREAMS_STELLA_DEFINEDp;
extern Symbol* SYM_STREAMS_STELLA_oEVALUATIONPARENTTREEo;
extern Symbol* SYM_STREAMS_STELLA_EVALUATION_EXCEPTION;
extern Symbol* SYM_STREAMS_STELLA_EVALUATION_ERROR;
extern Symbol* SYM_STREAMS_STELLA_EVALUATION_ERROR_EXPANDER;
extern Surrogate* SGT_STREAMS_STELLA_LITERAL_WRAPPER;
extern Symbol* SYM_STREAMS_STELLA_STRINGIFY;
extern Symbol* SYM_STREAMS_STELLA_VRLET;
extern Symbol* SYM_STREAMS_STELLA_STRING;
extern Symbol* SYM_STREAMS_STELLA_SETQ;
extern Symbol* SYM_STREAMS_STELLA_CONCATENATE;
extern Symbol* SYM_STREAMS_STELLA_MESSAGE_TO_STRING;
extern Symbol* SYM_STREAMS_STELLA_MESSAGE_TO_STRING_EXPANDER;
extern Surrogate* SGT_STREAMS_STELLA_OBJECT;
extern Symbol* SYM_STREAMS_STELLA_PROGN;
extern Symbol* SYM_STREAMS_STELLA_SUBSEQUENCE;
extern Symbol* SYM_STREAMS_STELLA_POSITION;
extern Symbol* SYM_STREAMS_STELLA_WITH_HTML_TAG;
extern Symbol* SYM_STREAMS_STELLA_WITH_HTML_TAG_EXPANDER;
extern Symbol* SYM_STREAMS_STELLA_WITH_XML_TAG;
extern Symbol* SYM_STREAMS_STELLA_WITH_XML_TAG_EXPANDER;
extern Keyword* KWD_STREAMS_CLOSED;
extern Keyword* KWD_STREAMS_OPEN;
extern Surrogate* SGT_STREAMS_STELLA_FILE_INPUT_STREAM;
extern Surrogate* SGT_STREAMS_STELLA_FILE_OUTPUT_STREAM;
extern Surrogate* SGT_STREAMS_STELLA_INPUT_STREAM;
extern Surrogate* SGT_STREAMS_STELLA_OUTPUT_STREAM;
extern Symbol* SYM_STREAMS_STELLA_THE_STREAM;
extern Surrogate* SGT_STREAMS_STELLA_S_EXPRESSION_ITERATOR;
extern Surrogate* SGT_STREAMS_STELLA_LINE_ITERATOR;
extern Surrogate* SGT_STREAMS_STELLA_CHARACTER_ITERATOR;
extern Symbol* SYM_STREAMS_STELLA_STARTUP_STREAMS;
extern Symbol* SYM_STREAMS_STELLA_METHOD_STARTUP_CLASSNAME;


} // end of namespace stella
