// hierarchy.cc

/*--------------------------- BEGIN LICENSE BLOCK ---------------------------+
|                                                                            |
| Version: MPL 1.1/GPL 2.0/LGPL 2.1                                          |
|                                                                            |
| The contents of this file are subject to the Mozilla Public License        |
| Version 1.1 (the "License"); you may not use this file except in           |
| compliance with the License. You may obtain a copy of the License at       |
| http://www.mozilla.org/MPL/                                                |
|                                                                            |
| Software distributed under the License is distributed on an "AS IS" basis, |
| WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License   |
| for the specific language governing rights and limitations under the       |
| License.                                                                   |
|                                                                            |
| The Original Code is the STELLA Programming Language.                      |
|                                                                            |
| The Initial Developer of the Original Code is                              |
| UNIVERSITY OF SOUTHERN CALIFORNIA, INFORMATION SCIENCES INSTITUTE          |
| 4676 Admiralty Way, Marina Del Rey, California 90292, U.S.A.               |
|                                                                            |
| Portions created by the Initial Developer are Copyright (C) 1996-2001      |
| the Initial Developer. All Rights Reserved.                                |
|                                                                            |
| Contributor(s):                                                            |
|                                                                            |
| Alternatively, the contents of this file may be used under the terms of    |
| either the GNU General Public License Version 2 or later (the "GPL"), or   |
| the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),   |
| in which case the provisions of the GPL or the LGPL are applicable instead |
| of those above. If you wish to allow use of your version of this file only |
| under the terms of either the GPL or the LGPL, and not to allow others to  |
| use your version of this file under the terms of the MPL, indicate your    |
| decision by deleting the provisions above and replace them with the notice |
| and other provisions required by the GPL or the LGPL. If you do not delete |
| the provisions above, a recipient may use your version of this file under  |
| the terms of any one of the MPL, the GPL or the LGPL.                      |
|                                                                            |
+---------------------------- END LICENSE BLOCK ----------------------------*/

#include "stella/stella-system.hh"

namespace stella {

Surrogate* Object::primary_type() {
  // Returns the primary type of 'self'.
  // Gets defined automatically for every non-abstract subclass of OBJECT.
  { Object* self = this;

    return (NULL);
  }
}

Module* Object::home_module() {
  // Return the home module of 'self'.
  { Object* self = this;

    return (NULL);
  }
}

boolean Object::deletedP() {
  // Default 'deleted?' method which always returns FALSE.  Objects
  // that inherit DYNAMIC-SLOTS-MIXIN also inherit the dynamically-allocated slot
  // 'deleted-object?' which is read/writable with specializations of this method.
  { Object* self = this;

    return (FALSE);
  }
}

void Object::increment_reference_count() {
  { Object* self = this;

  }
}

void Object::decrement_reference_count() {
  { Object* self = this;

  }
}

void Object::print_object(ostream* stream) {
  { Object* self = this;

    *(stream) << "|i|" << self->primary_type();
  }
}

ostream& operator<< (ostream& stream, Object* self) {
  if (!((boolean)(self))) {
    stream << "!NULL!";
  }
  else {
    self->print_object(&stream);
  }
  return (stream);
}

void Reference_Count_Object::increment_reference_count() {
  { Reference_Count_Object* self = this;

    self->reference_count = self->reference_count + 1;
  }
}

void Reference_Count_Object::decrement_reference_count() {
  { Reference_Count_Object* self = this;

    self->reference_count = self->reference_count - 1;
  }
}

Object* access_reference_count_object_slot_value(Reference_Count_Object* self, Symbol* slotname, Object* value, boolean setvalueP) {
  if (slotname == SYM_HIERARCHY_STELLA_REFERENCE_COUNT) {
    if (setvalueP) {
      self->reference_count = ((Integer_Wrapper*)(value))->wrapper_value;
    }
    else {
      value = wrap_integer(self->reference_count);
    }
  }
  else {
    { Output_String_Stream* stream_000 = new_output_string_stream();

      *(stream_000->native_stream) << "`" << slotname << "'" << " is not a valid case option";
      throw *new_stella_exception(stream_000->the_string_reader());
    }
  }
  return (value);
}

Module* Context_Sensitive_Object::home_module() {
  { Context_Sensitive_Object* self = this;

    return (self->home_context->base_module);
  }
}

Object* access_context_sensitive_object_slot_value(Context_Sensitive_Object* self, Symbol* slotname, Object* value, boolean setvalueP) {
  if (slotname == SYM_HIERARCHY_STELLA_HOME_CONTEXT) {
    if (setvalueP) {
      self->home_context = ((Context*)(value));
    }
    else {
      value = self->home_context;
    }
  }
  else {
    { Output_String_Stream* stream_000 = new_output_string_stream();

      *(stream_000->native_stream) << "`" << slotname << "'" << " is not a valid case option";
      throw *new_stella_exception(stream_000->the_string_reader());
    }
  }
  return (value);
}

Boolean_Wrapper* Active_Object::badP_reader() {
  { Active_Object* self = this;

    { Boolean_Wrapper* answer = ((Boolean_Wrapper*)(dynamic_slot_value(self->dynamic_slots, SYM_HIERARCHY_STELLA_BADp, NULL)));

      if (!((boolean)(answer))) {
        return ((FALSE ? TRUE_WRAPPER : FALSE_WRAPPER));
      }
      else {
        return (answer);
      }
    }
  }
}

Object* access_active_object_slot_value(Active_Object* self, Symbol* slotname, Object* value, boolean setvalueP) {
  if (TRUE) {
    if (setvalueP) {
      set_dynamic_slot_value(self->dynamic_slots, slotname, value, NULL);
    }
    else {
      value = self->dynamic_slots->lookup(slotname);
    }
  }
  return (value);
}

boolean Active_Object::deletedP() {
  { Active_Object* self = this;

    { Boolean_Wrapper* deletedP = ((Boolean_Wrapper*)(dynamic_slot_value(self->dynamic_slots, SYM_HIERARCHY_STELLA_DELETED_OBJECTp, NULL)));

      if (((boolean)(deletedP))) {
        return (coerce_wrapped_boolean_to_boolean(deletedP));
      }
      else {
        return (FALSE);
      }
    }
  }
}

boolean Active_Object::deletedP_setter(boolean value) {
  { Active_Object* self = this;

    set_dynamic_slot_value(self->dynamic_slots, SYM_HIERARCHY_STELLA_DELETED_OBJECTp, (value ? TRUE_WRAPPER : FALSE_WRAPPER), NULL);
    return (value);
  }
}

boolean Abstract_Iterator::nextP() {
  { Abstract_Iterator* self = this;

    return (FALSE);
  }
}

Object* access_abstract_iterator_slot_value(Abstract_Iterator* self, Symbol* slotname, Object* value, boolean setvalueP) {
  if (slotname == SYM_HIERARCHY_STELLA_FIRST_ITERATIONp) {
    if (setvalueP) {
      self->first_iterationP = coerce_wrapped_boolean_to_boolean(((Boolean_Wrapper*)(value)));
    }
    else {
      value = (self->first_iterationP ? TRUE_WRAPPER : FALSE_WRAPPER);
    }
  }
  else {
    { Output_String_Stream* stream_000 = new_output_string_stream();

      *(stream_000->native_stream) << "`" << slotname << "'" << " is not a valid case option";
      throw *new_stella_exception(stream_000->the_string_reader());
    }
  }
  return (value);
}

Cons_Iterator* new_cons_iterator() {
  { Cons_Iterator* self = NULL;

    self = new Cons_Iterator();
    self->first_iterationP = TRUE;
    self->value = NULL;
    self->cons_iterator_cursor = NULL;
    return (self);
  }
}

void Cons_Iterator::free() {
  { Cons_Iterator* self = this;

    if (terminate_cons_iteratorP(self)) {
      unmake(self);
    }
  }
}

Surrogate* Cons_Iterator::primary_type() {
  { Cons_Iterator* self = this;

    return (SGT_HIERARCHY_STELLA_CONS_ITERATOR);
  }
}

Object* access_cons_iterator_slot_value(Cons_Iterator* self, Symbol* slotname, Object* value, boolean setvalueP) {
  if (slotname == SYM_HIERARCHY_STELLA_CONS_ITERATOR_CURSOR) {
    if (setvalueP) {
      self->cons_iterator_cursor = ((Cons*)(value));
    }
    else {
      value = self->cons_iterator_cursor;
    }
  }
  else {
    { Output_String_Stream* stream_000 = new_output_string_stream();

      *(stream_000->native_stream) << "`" << slotname << "'" << " is not a valid case option";
      throw *new_stella_exception(stream_000->the_string_reader());
    }
  }
  return (value);
}

List_Iterator* new_list_iterator() {
  { List_Iterator* self = NULL;

    self = new List_Iterator();
    self->first_iterationP = TRUE;
    self->value = NULL;
    self->list_iterator_collection = NULL;
    self->list_iterator_cursor = NULL;
    return (self);
  }
}

void List_Iterator::free() {
  { List_Iterator* self = this;

    if (terminate_list_iteratorP(self)) {
      unmake(self);
    }
  }
}

Surrogate* List_Iterator::primary_type() {
  { List_Iterator* self = this;

    return (SGT_HIERARCHY_STELLA_LIST_ITERATOR);
  }
}

Object* access_list_iterator_slot_value(List_Iterator* self, Symbol* slotname, Object* value, boolean setvalueP) {
  if (slotname == SYM_HIERARCHY_STELLA_LIST_ITERATOR_CURSOR) {
    if (setvalueP) {
      self->list_iterator_cursor = ((Cons*)(value));
    }
    else {
      value = self->list_iterator_cursor;
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_LIST_ITERATOR_COLLECTION) {
    if (setvalueP) {
      self->list_iterator_collection = ((List*)(value));
    }
    else {
      value = self->list_iterator_collection;
    }
  }
  else {
    { Output_String_Stream* stream_000 = new_output_string_stream();

      *(stream_000->native_stream) << "`" << slotname << "'" << " is not a valid case option";
      throw *new_stella_exception(stream_000->the_string_reader());
    }
  }
  return (value);
}

Destructive_List_Iterator* new_destructive_list_iterator() {
  { Destructive_List_Iterator* self = NULL;

    self = new Destructive_List_Iterator();
    self->first_iterationP = TRUE;
    self->value = NULL;
    self->list_iterator_cursor = NULL;
    self->the_cons_list = NIL;
    return (self);
  }
}

void Destructive_List_Iterator::free() {
  { Destructive_List_Iterator* self = this;

    if (terminate_destructive_list_iteratorP(self)) {
      unmake(self);
    }
  }
}

Surrogate* Destructive_List_Iterator::primary_type() {
  { Destructive_List_Iterator* self = this;

    return (SGT_HIERARCHY_STELLA_DESTRUCTIVE_LIST_ITERATOR);
  }
}

Object* access_destructive_list_iterator_slot_value(Destructive_List_Iterator* self, Symbol* slotname, Object* value, boolean setvalueP) {
  if (slotname == SYM_HIERARCHY_STELLA_THE_CONS_LIST) {
    if (setvalueP) {
      self->the_cons_list = ((Cons*)(value));
    }
    else {
      value = self->the_cons_list;
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_LIST_ITERATOR_CURSOR) {
    if (setvalueP) {
      self->list_iterator_cursor = ((Cons*)(value));
    }
    else {
      value = self->list_iterator_cursor;
    }
  }
  else {
    { Output_String_Stream* stream_000 = new_output_string_stream();

      *(stream_000->native_stream) << "`" << slotname << "'" << " is not a valid case option";
      throw *new_stella_exception(stream_000->the_string_reader());
    }
  }
  return (value);
}

Set_Iterator* new_set_iterator() {
  { Set_Iterator* self = NULL;

    self = new Set_Iterator();
    self->first_iterationP = TRUE;
    self->value = NULL;
    self->set_iterator_collection = NULL;
    self->set_iterator_cursor = NULL;
    return (self);
  }
}

void Set_Iterator::free() {
  { Set_Iterator* self = this;

    if (terminate_set_iteratorP(self)) {
      unmake(self);
    }
  }
}

Surrogate* Set_Iterator::primary_type() {
  { Set_Iterator* self = this;

    return (SGT_HIERARCHY_STELLA_SET_ITERATOR);
  }
}

Object* access_set_iterator_slot_value(Set_Iterator* self, Symbol* slotname, Object* value, boolean setvalueP) {
  if (slotname == SYM_HIERARCHY_STELLA_SET_ITERATOR_CURSOR) {
    if (setvalueP) {
      self->set_iterator_cursor = ((Cons*)(value));
    }
    else {
      value = self->set_iterator_cursor;
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_SET_ITERATOR_COLLECTION) {
    if (setvalueP) {
      self->set_iterator_collection = ((Set*)(value));
    }
    else {
      value = self->set_iterator_collection;
    }
  }
  else {
    { Output_String_Stream* stream_000 = new_output_string_stream();

      *(stream_000->native_stream) << "`" << slotname << "'" << " is not a valid case option";
      throw *new_stella_exception(stream_000->the_string_reader());
    }
  }
  return (value);
}

All_Purpose_Iterator* new_all_purpose_iterator() {
  { All_Purpose_Iterator* self = NULL;

    self = new All_Purpose_Iterator();
    self->first_iterationP = TRUE;
    self->value = NULL;
    self->iterator_second_integer = NULL_INTEGER;
    self->iterator_integer = NULL_INTEGER;
    self->iterator_second_object = NULL;
    self->iterator_object = NULL;
    self->iterator_cons_list = NULL;
    self->iterator_nested_iterator = NULL;
    self->iterator_empty_code = NULL;
    self->iterator_filter_code = NULL;
    self->iterator_next_code = NULL;
    self->key = NULL;
    return (self);
  }
}

void All_Purpose_Iterator::free() {
  { All_Purpose_Iterator* self = this;

    if (self->terminate_all_purpose_iteratorP()) {
      unmake(self);
    }
  }
}

Surrogate* All_Purpose_Iterator::primary_type() {
  { All_Purpose_Iterator* self = this;

    return (SGT_HIERARCHY_STELLA_ALL_PURPOSE_ITERATOR);
  }
}

Object* access_all_purpose_iterator_slot_value(All_Purpose_Iterator* self, Symbol* slotname, Object* value, boolean setvalueP) {
  if (slotname == SYM_HIERARCHY_STELLA_KEY) {
    if (setvalueP) {
      self->key = value;
    }
    else {
      value = self->key;
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_ITERATOR_NEXT_CODE) {
    if (setvalueP) {
      self->iterator_next_code = ((Function_Code_Wrapper*)(value))->wrapper_value;
    }
    else {
      value = wrap_function_code(self->iterator_next_code);
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_ITERATOR_FILTER_CODE) {
    if (setvalueP) {
      self->iterator_filter_code = ((Function_Code_Wrapper*)(value))->wrapper_value;
    }
    else {
      value = wrap_function_code(self->iterator_filter_code);
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_ITERATOR_EMPTY_CODE) {
    if (setvalueP) {
      self->iterator_empty_code = ((Function_Code_Wrapper*)(value))->wrapper_value;
    }
    else {
      value = wrap_function_code(self->iterator_empty_code);
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_ITERATOR_NESTED_ITERATOR) {
    if (setvalueP) {
      self->iterator_nested_iterator = ((Iterator*)(value));
    }
    else {
      value = self->iterator_nested_iterator;
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_ITERATOR_CONS_LIST) {
    if (setvalueP) {
      self->iterator_cons_list = ((Cons*)(value));
    }
    else {
      value = self->iterator_cons_list;
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_ITERATOR_OBJECT) {
    if (setvalueP) {
      self->iterator_object = value;
    }
    else {
      value = self->iterator_object;
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_ITERATOR_SECOND_OBJECT) {
    if (setvalueP) {
      self->iterator_second_object = value;
    }
    else {
      value = self->iterator_second_object;
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_ITERATOR_INTEGER) {
    if (setvalueP) {
      self->iterator_integer = ((Integer_Wrapper*)(value))->wrapper_value;
    }
    else {
      value = wrap_integer(self->iterator_integer);
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_ITERATOR_SECOND_INTEGER) {
    if (setvalueP) {
      self->iterator_second_integer = ((Integer_Wrapper*)(value))->wrapper_value;
    }
    else {
      value = wrap_integer(self->iterator_second_integer);
    }
  }
  else {
    { Output_String_Stream* stream_000 = new_output_string_stream();

      *(stream_000->native_stream) << "`" << slotname << "'" << " is not a valid case option";
      throw *new_stella_exception(stream_000->the_string_reader());
    }
  }
  return (value);
}

Types_To_Classes_Iterator* new_types_to_classes_iterator(Cons* iterator_cursor) {
  { Types_To_Classes_Iterator* self = NULL;

    self = new Types_To_Classes_Iterator();
    self->iterator_cursor = iterator_cursor;
    self->first_iterationP = TRUE;
    self->value = NULL;
    return (self);
  }
}

Surrogate* Types_To_Classes_Iterator::primary_type() {
  { Types_To_Classes_Iterator* self = this;

    return (SGT_HIERARCHY_STELLA_TYPES_TO_CLASSES_ITERATOR);
  }
}

boolean Types_To_Classes_Iterator::nextP() {
  { Types_To_Classes_Iterator* self = this;

    if (self->first_iterationP) {
      self->first_iterationP = FALSE;
    }
    else {
      self->iterator_cursor = self->iterator_cursor->rest;
    }
    if (!(self->iterator_cursor == NIL)) {
      self->value = ((Class*)(((Surrogate*)(self->iterator_cursor->value))->surrogate_value));
      return (TRUE);
    }
    else {
      return (FALSE);
    }
  }
}

Object* access_types_to_classes_iterator_slot_value(Types_To_Classes_Iterator* self, Symbol* slotname, Object* value, boolean setvalueP) {
  if (slotname == SYM_HIERARCHY_STELLA_ITERATOR_CURSOR) {
    if (setvalueP) {
      self->iterator_cursor = ((Cons*)(value));
    }
    else {
      value = self->iterator_cursor;
    }
  }
  else {
    { Output_String_Stream* stream_000 = new_output_string_stream();

      *(stream_000->native_stream) << "`" << slotname << "'" << " is not a valid case option";
      throw *new_stella_exception(stream_000->the_string_reader());
    }
  }
  return (value);
}

Property_List_Iterator* new_property_list_iterator() {
  { Property_List_Iterator* self = NULL;

    self = new Property_List_Iterator();
    self->first_iterationP = TRUE;
    self->value = NULL;
    self->key = NULL;
    self->plist_iterator_collection = NULL;
    self->plist_iterator_cursor = NULL;
    return (self);
  }
}

Surrogate* Property_List_Iterator::primary_type() {
  { Property_List_Iterator* self = this;

    return (SGT_HIERARCHY_STELLA_PROPERTY_LIST_ITERATOR);
  }
}

Object* access_property_list_iterator_slot_value(Property_List_Iterator* self, Symbol* slotname, Object* value, boolean setvalueP) {
  if (slotname == SYM_HIERARCHY_STELLA_PLIST_ITERATOR_CURSOR) {
    if (setvalueP) {
      self->plist_iterator_cursor = ((Cons*)(value));
    }
    else {
      value = self->plist_iterator_cursor;
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_PLIST_ITERATOR_COLLECTION) {
    if (setvalueP) {
      self->plist_iterator_collection = ((Property_List*)(value));
    }
    else {
      value = self->plist_iterator_collection;
    }
  }
  else {
    { Output_String_Stream* stream_000 = new_output_string_stream();

      *(stream_000->native_stream) << "`" << slotname << "'" << " is not a valid case option";
      throw *new_stella_exception(stream_000->the_string_reader());
    }
  }
  return (value);
}

Kv_List_Iterator* new_kv_list_iterator() {
  { Kv_List_Iterator* self = NULL;

    self = new Kv_List_Iterator();
    self->first_iterationP = TRUE;
    self->value = NULL;
    self->key = NULL;
    self->kv_list_iterator_cursor = NULL;
    self->the_kv_list = NULL;
    return (self);
  }
}

Surrogate* Kv_List_Iterator::primary_type() {
  { Kv_List_Iterator* self = this;

    return (SGT_HIERARCHY_STELLA_KV_LIST_ITERATOR);
  }
}

Object* access_kv_list_iterator_slot_value(Kv_List_Iterator* self, Symbol* slotname, Object* value, boolean setvalueP) {
  if (slotname == SYM_HIERARCHY_STELLA_THE_KV_LIST) {
    if (setvalueP) {
      self->the_kv_list = ((Key_Value_List*)(value));
    }
    else {
      value = self->the_kv_list;
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_KV_LIST_ITERATOR_CURSOR) {
    if (setvalueP) {
      self->kv_list_iterator_cursor = ((Kv_Cons*)(value));
    }
    else {
      value = self->kv_list_iterator_cursor;
    }
  }
  else {
    { Output_String_Stream* stream_000 = new_output_string_stream();

      *(stream_000->native_stream) << "`" << slotname << "'" << " is not a valid case option";
      throw *new_stella_exception(stream_000->the_string_reader());
    }
  }
  return (value);
}

Integer_Interval_Iterator* new_integer_interval_iterator(int lower_bound, int upper_bound) {
  { Integer_Interval_Iterator* self = NULL;

    self = new Integer_Interval_Iterator();
    self->lower_bound = lower_bound;
    self->upper_bound = upper_bound;
    self->first_iterationP = TRUE;
    self->value = NULL_INTEGER;
    self->interval_cursor = NULL_INTEGER;
    self->initialize_integer_interval_iterator();
    return (self);
  }
}

Surrogate* Integer_Interval_Iterator::primary_type() {
  { Integer_Interval_Iterator* self = this;

    return (SGT_HIERARCHY_STELLA_INTEGER_INTERVAL_ITERATOR);
  }
}

Object* access_integer_interval_iterator_slot_value(Integer_Interval_Iterator* self, Symbol* slotname, Object* value, boolean setvalueP) {
  if (slotname == SYM_HIERARCHY_STELLA_INTERVAL_CURSOR) {
    if (setvalueP) {
      self->interval_cursor = ((Integer_Wrapper*)(value))->wrapper_value;
    }
    else {
      value = wrap_integer(self->interval_cursor);
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_LOWER_BOUND) {
    if (setvalueP) {
      self->lower_bound = ((Integer_Wrapper*)(value))->wrapper_value;
    }
    else {
      value = wrap_integer(self->lower_bound);
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_UPPER_BOUND) {
    if (setvalueP) {
      self->upper_bound = ((Integer_Wrapper*)(value))->wrapper_value;
    }
    else {
      value = wrap_integer(self->upper_bound);
    }
  }
  else {
    { Output_String_Stream* stream_000 = new_output_string_stream();

      *(stream_000->native_stream) << "`" << slotname << "'" << " is not a valid case option";
      throw *new_stella_exception(stream_000->the_string_reader());
    }
  }
  return (value);
}

Reverse_Integer_Interval_Iterator* new_reverse_integer_interval_iterator(int lower_bound, int upper_bound) {
  { Reverse_Integer_Interval_Iterator* self = NULL;

    self = new Reverse_Integer_Interval_Iterator();
    self->lower_bound = lower_bound;
    self->upper_bound = upper_bound;
    self->first_iterationP = TRUE;
    self->value = NULL_INTEGER;
    self->interval_cursor = NULL_INTEGER;
    initialize_reverse_integer_interval_iterator(self);
    return (self);
  }
}

Surrogate* Reverse_Integer_Interval_Iterator::primary_type() {
  { Reverse_Integer_Interval_Iterator* self = this;

    return (SGT_HIERARCHY_STELLA_REVERSE_INTEGER_INTERVAL_ITERATOR);
  }
}

Object* access_reverse_integer_interval_iterator_slot_value(Reverse_Integer_Interval_Iterator* self, Symbol* slotname, Object* value, boolean setvalueP) {
  if (slotname == SYM_HIERARCHY_STELLA_INTERVAL_CURSOR) {
    if (setvalueP) {
      self->interval_cursor = ((Integer_Wrapper*)(value))->wrapper_value;
    }
    else {
      value = wrap_integer(self->interval_cursor);
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_LOWER_BOUND) {
    if (setvalueP) {
      self->lower_bound = ((Integer_Wrapper*)(value))->wrapper_value;
    }
    else {
      value = wrap_integer(self->lower_bound);
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_UPPER_BOUND) {
    if (setvalueP) {
      self->upper_bound = ((Integer_Wrapper*)(value))->wrapper_value;
    }
    else {
      value = wrap_integer(self->upper_bound);
    }
  }
  else {
    { Output_String_Stream* stream_000 = new_output_string_stream();

      *(stream_000->native_stream) << "`" << slotname << "'" << " is not a valid case option";
      throw *new_stella_exception(stream_000->the_string_reader());
    }
  }
  return (value);
}

String_Iterator* new_string_iterator() {
  { String_Iterator* self = NULL;

    self = new String_Iterator();
    self->first_iterationP = TRUE;
    self->value = NULL_CHARACTER;
    self->end = NULL_INTEGER;
    self->cursor = NULL_INTEGER;
    self->the_string = NULL;
    return (self);
  }
}

Surrogate* String_Iterator::primary_type() {
  { String_Iterator* self = this;

    return (SGT_HIERARCHY_STELLA_STRING_ITERATOR);
  }
}

Object* access_string_iterator_slot_value(String_Iterator* self, Symbol* slotname, Object* value, boolean setvalueP) {
  if (slotname == SYM_HIERARCHY_STELLA_THE_STRING) {
    if (setvalueP) {
      self->the_string = ((String_Wrapper*)(value))->wrapper_value;
    }
    else {
      value = wrap_string(self->the_string);
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_CURSOR) {
    if (setvalueP) {
      self->cursor = ((Integer_Wrapper*)(value))->wrapper_value;
    }
    else {
      value = wrap_integer(self->cursor);
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_END) {
    if (setvalueP) {
      self->end = ((Integer_Wrapper*)(value))->wrapper_value;
    }
    else {
      value = wrap_integer(self->end);
    }
  }
  else {
    { Output_String_Stream* stream_000 = new_output_string_stream();

      *(stream_000->native_stream) << "`" << slotname << "'" << " is not a valid case option";
      throw *new_stella_exception(stream_000->the_string_reader());
    }
  }
  return (value);
}

Abstract_Iterator* Abstract_Collection::allocate_iterator() {
  { Abstract_Collection* self = this;

    return (NULL);
  }
}

int Abstract_Collection::length() {
  { Abstract_Collection* self = this;

    return (NULL_INTEGER);
  }
}

Abstract_Iterator* Collection::allocate_iterator() {
  { Collection* self = this;

    return (NULL);
  }
}

int Collection::length() {
  { Collection* self = this;

    return (NULL_INTEGER);
  }
}

void Collection::insert(Object* value) {
  { Collection* self = this;

    return;
  }
}

Abstract_Collection* Collection::remove(Object* value) {
  { Collection* self = this;

    return (NULL);
  }
}

Abstract_Iterator* Sequence::allocate_iterator() {
  { Sequence* self = this;

    return (NULL);
  }
}

int Sequence::length() {
  { Sequence* sequence = this;

    return (NULL_INTEGER);
  }
}

List* new_list() {
  { List* self = NULL;

    self = new List();
    self->the_cons_list = NIL;
    return (self);
  }
}

Surrogate* List::primary_type() {
  { List* self = this;

    return (SGT_HIERARCHY_STELLA_LIST);
  }
}

Object* access_list_slot_value(List* self, Symbol* slotname, Object* value, boolean setvalueP) {
  if (slotname == SYM_HIERARCHY_STELLA_THE_CONS_LIST) {
    if (setvalueP) {
      self->the_cons_list = ((Cons*)(value));
    }
    else {
      value = self->the_cons_list;
    }
  }
  else {
    { Output_String_Stream* stream_000 = new_output_string_stream();

      *(stream_000->native_stream) << "`" << slotname << "'" << " is not a valid case option";
      throw *new_stella_exception(stream_000->the_string_reader());
    }
  }
  return (value);
}

void List::print_object(ostream* stream) {
  { List* self = this;

    print_cons_list(self->the_cons_list, stream);
  }
}

Set* new_set() {
  { Set* self = NULL;

    self = new Set();
    self->the_cons_list = NIL;
    return (self);
  }
}

Surrogate* Set::primary_type() {
  { Set* self = this;

    return (SGT_HIERARCHY_STELLA_SET);
  }
}

Object* access_set_slot_value(Set* self, Symbol* slotname, Object* value, boolean setvalueP) {
  if (slotname == SYM_HIERARCHY_STELLA_THE_CONS_LIST) {
    if (setvalueP) {
      self->the_cons_list = ((Cons*)(value));
    }
    else {
      value = self->the_cons_list;
    }
  }
  else {
    { Output_String_Stream* stream_000 = new_output_string_stream();

      *(stream_000->native_stream) << "`" << slotname << "'" << " is not a valid case option";
      throw *new_stella_exception(stream_000->the_string_reader());
    }
  }
  return (value);
}

void Set::print_object(ostream* stream) {
  { Set* self = this;

    print_cons_list(self->the_cons_list, stream);
  }
}

Class_Extension* new_class_extension() {
  { Class_Extension* self = NULL;

    self = new Class_Extension();
    self->the_cons_list = NIL;
    return (self);
  }
}

Surrogate* Class_Extension::primary_type() {
  { Class_Extension* self = this;

    return (SGT_HIERARCHY_STELLA_CLASS_EXTENSION);
  }
}

Cons* new_cons() {
  { Cons* self = NULL;

    self = new Cons();
    self->rest = NIL;
    self->value = NULL;
    return (self);
  }
}

Surrogate* Cons::primary_type() {
  { Cons* self = this;

    return (SGT_HIERARCHY_STELLA_CONS);
  }
}

Object* access_cons_slot_value(Cons* self, Symbol* slotname, Object* value, boolean setvalueP) {
  if (slotname == SYM_HIERARCHY_STELLA_VALUE) {
    if (setvalueP) {
      self->value = value;
    }
    else {
      value = self->value;
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_REST) {
    if (setvalueP) {
      self->rest = ((Cons*)(value));
    }
    else {
      value = self->rest;
    }
  }
  else {
    { Output_String_Stream* stream_000 = new_output_string_stream();

      *(stream_000->native_stream) << "`" << slotname << "'" << " is not a valid case option";
      throw *new_stella_exception(stream_000->the_string_reader());
    }
  }
  return (value);
}

void Cons::print_object(ostream* stream) {
  { Cons* self = this;

    print_cons(self, stream, "(", ")");
  }
}

Abstract_Iterator* Abstract_Dictionary::allocate_iterator() {
  { Abstract_Dictionary* self = this;

    return (NULL);
  }
}

Abstract_Iterator* Dictionary::allocate_iterator() {
  { Dictionary* self = this;

    return (NULL);
  }
}

Property_List* new_property_list() {
  { Property_List* self = NULL;

    self = new Property_List();
    self->the_plist = NIL;
    return (self);
  }
}

Surrogate* Property_List::primary_type() {
  { Property_List* self = this;

    return (SGT_HIERARCHY_STELLA_PROPERTY_LIST);
  }
}

Object* access_property_list_slot_value(Property_List* self, Symbol* slotname, Object* value, boolean setvalueP) {
  if (slotname == SYM_HIERARCHY_STELLA_THE_PLIST) {
    if (setvalueP) {
      self->the_plist = ((Cons*)(value));
    }
    else {
      value = self->the_plist;
    }
  }
  else {
    { Output_String_Stream* stream_000 = new_output_string_stream();

      *(stream_000->native_stream) << "`" << slotname << "'" << " is not a valid case option";
      throw *new_stella_exception(stream_000->the_string_reader());
    }
  }
  return (value);
}

void Property_List::print_object(ostream* stream) {
  { Property_List* self = this;

    print_cons_list(self->the_plist, stream);
  }
}

Kv_Cons* new_kv_cons() {
  { Kv_Cons* self = NULL;

    self = new Kv_Cons();
    self->rest = NULL;
    self->value = NULL;
    self->key = NULL;
    return (self);
  }
}

Surrogate* Kv_Cons::primary_type() {
  { Kv_Cons* self = this;

    return (SGT_HIERARCHY_STELLA_KV_CONS);
  }
}

Object* access_kv_cons_slot_value(Kv_Cons* self, Symbol* slotname, Object* value, boolean setvalueP) {
  if (slotname == SYM_HIERARCHY_STELLA_KEY) {
    if (setvalueP) {
      self->key = value;
    }
    else {
      value = self->key;
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_VALUE) {
    if (setvalueP) {
      self->value = value;
    }
    else {
      value = self->value;
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_REST) {
    if (setvalueP) {
      self->rest = ((Kv_Cons*)(value));
    }
    else {
      value = self->rest;
    }
  }
  else {
    { Output_String_Stream* stream_000 = new_output_string_stream();

      *(stream_000->native_stream) << "`" << slotname << "'" << " is not a valid case option";
      throw *new_stella_exception(stream_000->the_string_reader());
    }
  }
  return (value);
}

void Kv_Cons::print_object(ostream* stream) {
  { Kv_Cons* self = this;

    *(stream) << "<" << self->key << "," << self->value << ">";
  }
}

Key_Value_List* new_key_value_list() {
  { Key_Value_List* self = NULL;

    self = new Key_Value_List();
    self->the_kv_list = NULL;
    return (self);
  }
}

Surrogate* Key_Value_List::primary_type() {
  { Key_Value_List* self = this;

    return (SGT_HIERARCHY_STELLA_KEY_VALUE_LIST);
  }
}

Object* access_key_value_list_slot_value(Key_Value_List* self, Symbol* slotname, Object* value, boolean setvalueP) {
  if (slotname == SYM_HIERARCHY_STELLA_THE_KV_LIST) {
    if (setvalueP) {
      self->the_kv_list = ((Kv_Cons*)(value));
    }
    else {
      value = self->the_kv_list;
    }
  }
  else {
    { Output_String_Stream* stream_000 = new_output_string_stream();

      *(stream_000->native_stream) << "`" << slotname << "'" << " is not a valid case option";
      throw *new_stella_exception(stream_000->the_string_reader());
    }
  }
  return (value);
}

void Key_Value_List::print_object(ostream* stream) {
  { Key_Value_List* self = this;

    print_key_value_list(self, stream);
  }
}

Recycle_List* new_recycle_list() {
  { Recycle_List* self = NULL;

    self = new Recycle_List();
    self->item_size = NULL_INTEGER;
    self->current_length = 0;
    self->unused_items = NULL;
    self->all_items = NULL;
    self->list_of_recycled_items = make_non_recycled_list();
    self->recycled_items = NULL;
    self->recycle_list_of = NULL;
    return (self);
  }
}

Surrogate* Recycle_List::primary_type() {
  { Recycle_List* self = this;

    return (SGT_HIERARCHY_STELLA_RECYCLE_LIST);
  }
}

Object* access_recycle_list_slot_value(Recycle_List* self, Symbol* slotname, Object* value, boolean setvalueP) {
  if (slotname == SYM_HIERARCHY_STELLA_RECYCLE_LIST_OF) {
    if (setvalueP) {
      self->recycle_list_of = ((Surrogate*)(value));
    }
    else {
      value = self->recycle_list_of;
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_RECYCLED_ITEMS) {
    if (setvalueP) {
      self->recycled_items = value;
    }
    else {
      value = self->recycled_items;
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_LIST_OF_RECYCLED_ITEMS) {
    if (setvalueP) {
      self->list_of_recycled_items = ((List*)(value));
    }
    else {
      value = self->list_of_recycled_items;
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_ALL_ITEMS) {
    if (setvalueP) {
      self->all_items = value;
    }
    else {
      value = self->all_items;
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_UNUSED_ITEMS) {
    if (setvalueP) {
      self->unused_items = value;
    }
    else {
      value = self->unused_items;
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_CURRENT_LENGTH) {
    if (setvalueP) {
      self->current_length = ((Integer_Wrapper*)(value))->wrapper_value;
    }
    else {
      value = wrap_integer(self->current_length);
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_ITEM_SIZE) {
    if (setvalueP) {
      self->item_size = ((Integer_Wrapper*)(value))->wrapper_value;
    }
    else {
      value = wrap_integer(self->item_size);
    }
  }
  else {
    { Output_String_Stream* stream_000 = new_output_string_stream();

      *(stream_000->native_stream) << "`" << slotname << "'" << " is not a valid case option";
      throw *new_stella_exception(stream_000->the_string_reader());
    }
  }
  return (value);
}

void Recycle_List::print_object(ostream* stream) {
  { Recycle_List* self = this;

    if (((boolean)(self->recycle_list_of))) {
      *(stream) << "|i|[RECYCLE-LIST OF " << self->current_length << " " << self->recycle_list_of << "'s]";
    }
    else {
      *(stream) << "|i|@RECYCLE-LIST";
    }
  }
}

Object* access_abstract_hash_table_slot_value(Abstract_Hash_Table* self, Symbol* slotname, Object* value, boolean setvalueP) {
  if (slotname == SYM_HIERARCHY_STELLA_THE_STELLA_HASH_TABLE) {
    if (setvalueP) {
      self->the_stella_hash_table = ((Stella_Hash_Table*)(value));
    }
    else {
      value = self->the_stella_hash_table;
    }
  }
  else {
    { Output_String_Stream* stream_000 = new_output_string_stream();

      *(stream_000->native_stream) << "`" << slotname << "'" << " is not a valid case option";
      throw *new_stella_exception(stream_000->the_string_reader());
    }
  }
  return (value);
}

Hash_Table* new_hash_table() {
  { Hash_Table* self = NULL;

    self = new Hash_Table();
    self->the_stella_hash_table = NULL;
    self->the_hash_table = NULL;
    self->initialize_hash_table();
    self->initialize_hash_table();
    return (self);
  }
}

Surrogate* Hash_Table::primary_type() {
  { Hash_Table* self = this;

    return (SGT_HIERARCHY_STELLA_HASH_TABLE);
  }
}

Integer_Hash_Table* new_integer_hash_table() {
  { Integer_Hash_Table* self = NULL;

    self = new Integer_Hash_Table();
    self->the_stella_hash_table = NULL;
    self->the_hash_table = NULL;
    self->initialize_hash_table();
    self->initialize_hash_table();
    return (self);
  }
}

Surrogate* Integer_Hash_Table::primary_type() {
  { Integer_Hash_Table* self = this;

    return (SGT_HIERARCHY_STELLA_INTEGER_HASH_TABLE);
  }
}

Float_Hash_Table* new_float_hash_table() {
  { Float_Hash_Table* self = NULL;

    self = new Float_Hash_Table();
    self->the_stella_hash_table = NULL;
    self->the_hash_table = NULL;
    self->initialize_hash_table();
    self->initialize_hash_table();
    return (self);
  }
}

Surrogate* Float_Hash_Table::primary_type() {
  { Float_Hash_Table* self = this;

    return (SGT_HIERARCHY_STELLA_FLOAT_HASH_TABLE);
  }
}

String_Hash_Table* new_string_hash_table() {
  { String_Hash_Table* self = NULL;

    self = new String_Hash_Table();
    self->the_stella_hash_table = NULL;
    self->the_hash_table = NULL;
    self->initialize_hash_table();
    self->initialize_hash_table();
    return (self);
  }
}

Surrogate* String_Hash_Table::primary_type() {
  { String_Hash_Table* self = this;

    return (SGT_HIERARCHY_STELLA_STRING_HASH_TABLE);
  }
}

String_To_Integer_Hash_Table* new_string_to_integer_hash_table() {
  { String_To_Integer_Hash_Table* self = NULL;

    self = new String_To_Integer_Hash_Table();
    self->the_stella_hash_table = NULL;
    self->the_hash_table = NULL;
    self->initialize_hash_table();
    self->initialize_hash_table();
    return (self);
  }
}

Surrogate* String_To_Integer_Hash_Table::primary_type() {
  { String_To_Integer_Hash_Table* self = this;

    return (SGT_HIERARCHY_STELLA_STRING_TO_INTEGER_HASH_TABLE);
  }
}

Vector* new_vector(int array_size) {
  { Vector* self = NULL;

    self = new Vector();
    self->array_size = array_size;
    self->the_array = NULL;
    self->initialize_vector();
    return (self);
  }
}

Surrogate* Vector::primary_type() {
  { Vector* self = this;

    return (SGT_HIERARCHY_STELLA_VECTOR);
  }
}

Object* access_vector_slot_value(Vector* self, Symbol* slotname, Object* value, boolean setvalueP) {
  if (slotname == SYM_HIERARCHY_STELLA_ARRAY_SIZE) {
    if (setvalueP) {
      self->array_size = ((Integer_Wrapper*)(value))->wrapper_value;
    }
    else {
      value = wrap_integer(self->array_size);
    }
  }
  else {
    { Output_String_Stream* stream_000 = new_output_string_stream();

      *(stream_000->native_stream) << "`" << slotname << "'" << " is not a valid case option";
      throw *new_stella_exception(stream_000->the_string_reader());
    }
  }
  return (value);
}

void Vector::print_object(ostream* stream) {
  { Vector* self = this;

    self->print_vector(stream);
  }
}

Extensible_Vector* new_extensible_vector(int array_size) {
  { Extensible_Vector* self = NULL;

    self = new Extensible_Vector();
    self->array_size = array_size;
    self->the_array = NULL;
    self->initialize_vector();
    self->initialize_vector();
    return (self);
  }
}

Surrogate* Extensible_Vector::primary_type() {
  { Extensible_Vector* self = this;

    return (SGT_HIERARCHY_STELLA_EXTENSIBLE_VECTOR);
  }
}

Extensible_Symbol_Array* new_extensible_symbol_array(int array_size) {
  { Extensible_Symbol_Array* self = NULL;

    self = new Extensible_Symbol_Array();
    self->array_size = array_size;
    self->the_array = NULL;
    self->potential_free_symbol_offset = 0;
    self->top_symbol_offset = -1;
    self->initialize_vector();
    self->initialize_vector();
    self->initialize_vector();
    return (self);
  }
}

Surrogate* Extensible_Symbol_Array::primary_type() {
  { Extensible_Symbol_Array* self = this;

    return (SGT_HIERARCHY_STELLA_EXTENSIBLE_SYMBOL_ARRAY);
  }
}

Object* access_extensible_symbol_array_slot_value(Extensible_Symbol_Array* self, Symbol* slotname, Object* value, boolean setvalueP) {
  if (slotname == SYM_HIERARCHY_STELLA_TOP_SYMBOL_OFFSET) {
    if (setvalueP) {
      self->top_symbol_offset = ((Integer_Wrapper*)(value))->wrapper_value;
    }
    else {
      value = wrap_integer(self->top_symbol_offset);
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_POTENTIAL_FREE_SYMBOL_OFFSET) {
    if (setvalueP) {
      self->potential_free_symbol_offset = ((Integer_Wrapper*)(value))->wrapper_value;
    }
    else {
      value = wrap_integer(self->potential_free_symbol_offset);
    }
  }
  else {
    { Output_String_Stream* stream_000 = new_output_string_stream();

      *(stream_000->native_stream) << "`" << slotname << "'" << " is not a valid case option";
      throw *new_stella_exception(stream_000->the_string_reader());
    }
  }
  return (value);
}

int Vector_Sequence::resize_increment_reader() {
  { Vector_Sequence* self = this;

    return (oHARDWIRED_RESIZE_INCREMENT_ON_VECTOR_SEQUENCEo);
  }
}

Object* access_vector_sequence_slot_value(Vector_Sequence* self, Symbol* slotname, Object* value, boolean setvalueP) {
  if (slotname == SYM_HIERARCHY_STELLA_RESIZE_INCREMENT) {
    value = wrap_integer(self->resize_increment_reader());
  }
  else if (slotname == SYM_HIERARCHY_STELLA_SEQUENCE_LENGTH) {
    if (setvalueP) {
      self->sequence_length = ((Integer_Wrapper*)(value))->wrapper_value;
    }
    else {
      value = wrap_integer(self->sequence_length);
    }
  }
  else {
    { Output_String_Stream* stream_000 = new_output_string_stream();

      *(stream_000->native_stream) << "`" << slotname << "'" << " is not a valid case option";
      throw *new_stella_exception(stream_000->the_string_reader());
    }
  }
  return (value);
}

Short_Vector_Sequence* new_short_vector_sequence(int array_size) {
  { Short_Vector_Sequence* self = NULL;

    self = new Short_Vector_Sequence();
    self->array_size = array_size;
    self->the_array = NULL;
    self->sequence_length = 0;
    self->initialize_vector();
    self->initialize_vector();
    self->initialize_vector();
    return (self);
  }
}

Surrogate* Short_Vector_Sequence::primary_type() {
  { Short_Vector_Sequence* self = this;

    return (SGT_HIERARCHY_STELLA_SHORT_VECTOR_SEQUENCE);
  }
}

int Short_Vector_Sequence::resize_increment_reader() {
  { Short_Vector_Sequence* self = this;

    return (oHARDWIRED_RESIZE_INCREMENT_ON_SHORT_VECTOR_SEQUENCEo);
  }
}

Object* access_short_vector_sequence_slot_value(Short_Vector_Sequence* self, Symbol* slotname, Object* value, boolean setvalueP) {
  if (slotname == SYM_HIERARCHY_STELLA_RESIZE_INCREMENT) {
    value = wrap_integer(self->resize_increment_reader());
  }
  else if (slotname == SYM_HIERARCHY_STELLA_SEQUENCE_LENGTH) {
    if (setvalueP) {
      self->sequence_length = ((Integer_Wrapper*)(value))->wrapper_value;
    }
    else {
      value = wrap_integer(self->sequence_length);
    }
  }
  else {
    { Output_String_Stream* stream_000 = new_output_string_stream();

      *(stream_000->native_stream) << "`" << slotname << "'" << " is not a valid case option";
      throw *new_stella_exception(stream_000->the_string_reader());
    }
  }
  return (value);
}

Boolean_Vector* new_boolean_vector(int array_size) {
  { Boolean_Vector* self = NULL;

    self = new Boolean_Vector();
    self->array_size = array_size;
    self->the_array = NULL;
    self->initialize_vector();
    return (self);
  }
}

Surrogate* Boolean_Vector::primary_type() {
  { Boolean_Vector* self = this;

    return (SGT_HIERARCHY_STELLA_BOOLEAN_VECTOR);
  }
}

Integer_Vector* new_integer_vector(int array_size) {
  { Integer_Vector* self = NULL;

    self = new Integer_Vector();
    self->array_size = array_size;
    self->the_array = NULL;
    self->initialize_vector();
    return (self);
  }
}

Surrogate* Integer_Vector::primary_type() {
  { Integer_Vector* self = this;

    return (SGT_HIERARCHY_STELLA_INTEGER_VECTOR);
  }
}

Active_List* new_active_list() {
  { Active_List* self = NULL;

    self = new Active_List();
    self->owner_instance = NULL;
    self->active_slot = NULL;
    self->the_cons_list = NIL;
    return (self);
  }
}

Surrogate* Active_List::primary_type() {
  { Active_List* self = this;

    return (SGT_HIERARCHY_STELLA_ACTIVE_LIST);
  }
}

Active_Set* new_active_set() {
  { Active_Set* self = NULL;

    self = new Active_Set();
    self->owner_instance = NULL;
    self->active_slot = NULL;
    self->the_cons_list = NIL;
    return (self);
  }
}

Surrogate* Active_Set::primary_type() {
  { Active_Set* self = this;

    return (SGT_HIERARCHY_STELLA_ACTIVE_SET);
  }
}

Demon* new_demon() {
  { Demon* self = NULL;

    self = new Demon();
    self->demon_inheritP = FALSE;
    self->demon_allP = FALSE;
    self->demon_guardP = FALSE;
    self->demon_documentation = NULL;
    self->demon_method = NULL;
    self->demon_code = NULL;
    self->demon_slot_refs = new_list();
    self->demon_class_refs = new_list();
    self->demon_action = NULL;
    self->demon_name = NULL;
    return (self);
  }
}

Surrogate* Demon::primary_type() {
  { Demon* self = this;

    return (SGT_HIERARCHY_STELLA_DEMON);
  }
}

Object* access_demon_slot_value(Demon* self, Symbol* slotname, Object* value, boolean setvalueP) {
  if (slotname == SYM_HIERARCHY_STELLA_DEMON_NAME) {
    if (setvalueP) {
      self->demon_name = ((String_Wrapper*)(value))->wrapper_value;
    }
    else {
      value = wrap_string(self->demon_name);
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_DEMON_ACTION) {
    if (setvalueP) {
      self->demon_action = ((Keyword*)(value));
    }
    else {
      value = self->demon_action;
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_DEMON_CLASS_REFS) {
    if (setvalueP) {
      self->demon_class_refs = ((List*)(value));
    }
    else {
      value = self->demon_class_refs;
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_DEMON_SLOT_REFS) {
    if (setvalueP) {
      self->demon_slot_refs = ((List*)(value));
    }
    else {
      value = self->demon_slot_refs;
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_DEMON_CODE) {
    if (setvalueP) {
      self->demon_code = ((Function_Code_Wrapper*)(value))->wrapper_value;
    }
    else {
      value = wrap_function_code(self->demon_code);
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_DEMON_METHOD) {
    if (setvalueP) {
      self->demon_method = ((Method_Slot*)(value));
    }
    else {
      value = self->demon_method;
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_DEMON_DOCUMENTATION) {
    if (setvalueP) {
      self->demon_documentation = ((String_Wrapper*)(value))->wrapper_value;
    }
    else {
      value = wrap_string(self->demon_documentation);
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_DEMON_GUARDp) {
    if (setvalueP) {
      self->demon_guardP = coerce_wrapped_boolean_to_boolean(((Boolean_Wrapper*)(value)));
    }
    else {
      value = (self->demon_guardP ? TRUE_WRAPPER : FALSE_WRAPPER);
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_DEMON_ALLp) {
    if (setvalueP) {
      self->demon_allP = coerce_wrapped_boolean_to_boolean(((Boolean_Wrapper*)(value)));
    }
    else {
      value = (self->demon_allP ? TRUE_WRAPPER : FALSE_WRAPPER);
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_DEMON_INHERITp) {
    if (setvalueP) {
      self->demon_inheritP = coerce_wrapped_boolean_to_boolean(((Boolean_Wrapper*)(value)));
    }
    else {
      value = (self->demon_inheritP ? TRUE_WRAPPER : FALSE_WRAPPER);
    }
  }
  else {
    { Output_String_Stream* stream_000 = new_output_string_stream();

      *(stream_000->native_stream) << "`" << slotname << "'" << " is not a valid case option";
      throw *new_stella_exception(stream_000->the_string_reader());
    }
  }
  return (value);
}

Keyword_Key_Value_List* new_keyword_key_value_list() {
  { Keyword_Key_Value_List* self = NULL;

    self = new Keyword_Key_Value_List();
    self->the_kv_list = NULL;
    return (self);
  }
}

Surrogate* Keyword_Key_Value_List::primary_type() {
  { Keyword_Key_Value_List* self = this;

    return (SGT_HIERARCHY_STELLA_KEYWORD_KEY_VALUE_LIST);
  }
}

Object* access_generalized_symbol_slot_value(Generalized_Symbol* self, Symbol* slotname, Object* value, boolean setvalueP) {
  if (slotname == SYM_HIERARCHY_STELLA_SYMBOL_NAME) {
    if (setvalueP) {
      self->symbol_name = ((String_Wrapper*)(value))->wrapper_value;
    }
    else {
      value = wrap_string(self->symbol_name);
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_SYMBOL_ID) {
    if (setvalueP) {
      self->symbol_id = ((Integer_Wrapper*)(value))->wrapper_value;
    }
    else {
      value = wrap_integer(self->symbol_id);
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_INTERNED_IN) {
    if (setvalueP) {
      self->home_context = ((Module*)(value));
    }
    else {
      value = ((Module*)(self->home_context));
    }
  }
  else {
    { Output_String_Stream* stream_000 = new_output_string_stream();

      *(stream_000->native_stream) << "`" << slotname << "'" << " is not a valid case option";
      throw *new_stella_exception(stream_000->the_string_reader());
    }
  }
  return (value);
}

Symbol* new_symbol(char* symbol_name) {
  { Symbol* self = NULL;

    self = new Symbol();
    self->symbol_name = symbol_name;
    self->home_context = oCONTEXTo.get();
    self->symbol_id = NULL_INTEGER;
    self->symbol_value_and_plist = NIL;
    self->symbol_slot_offset = NULL_INTEGER;
    return (self);
  }
}

Surrogate* Symbol::primary_type() {
  { Symbol* self = this;

    return (SGT_HIERARCHY_STELLA_SYMBOL);
  }
}

Object* access_symbol_slot_value(Symbol* self, Symbol* slotname, Object* value, boolean setvalueP) {
  if (slotname == SYM_HIERARCHY_STELLA_SYMBOL_SLOT_OFFSET) {
    if (setvalueP) {
      self->symbol_slot_offset = ((Integer_Wrapper*)(value))->wrapper_value;
    }
    else {
      value = wrap_integer(self->symbol_slot_offset);
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_SYMBOL_VALUE_AND_PLIST) {
    if (setvalueP) {
      self->symbol_value_and_plist = ((Cons*)(value));
    }
    else {
      value = self->symbol_value_and_plist;
    }
  }
  else {
    { Output_String_Stream* stream_000 = new_output_string_stream();

      *(stream_000->native_stream) << "`" << slotname << "'" << " is not a valid case option";
      throw *new_stella_exception(stream_000->the_string_reader());
    }
  }
  return (value);
}

void Symbol::print_object(ostream* stream) {
  { Symbol* self = this;

    print_symbol(self, stream);
  }
}

Surrogate* new_surrogate(char* symbol_name) {
  { Surrogate* self = NULL;

    self = new Surrogate();
    self->symbol_name = symbol_name;
    self->home_context = oCONTEXTo.get();
    self->symbol_id = NULL_INTEGER;
    self->surrogate_value = NULL;
    return (self);
  }
}

Surrogate* Surrogate::primary_type() {
  { Surrogate* self = this;

    return (SGT_HIERARCHY_STELLA_SURROGATE);
  }
}

Object* access_surrogate_slot_value(Surrogate* self, Symbol* slotname, Object* value, boolean setvalueP) {
  if (slotname == SYM_HIERARCHY_STELLA_SURROGATE_VALUE) {
    if (setvalueP) {
      self->surrogate_value = value;
    }
    else {
      value = self->surrogate_value;
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_SURROGATE_NAME) {
    if (setvalueP) {
      self->symbol_name = ((String_Wrapper*)(value))->wrapper_value;
    }
    else {
      value = wrap_string(self->symbol_name);
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_TYPE_CLASS) {
    if (setvalueP) {
      self->surrogate_value = ((Class*)(value));
    }
    else {
      value = ((Class*)(self->surrogate_value));
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_TYPE_NAME) {
    if (setvalueP) {
      self->symbol_name = ((String_Wrapper*)(value))->wrapper_value;
    }
    else {
      value = wrap_string(self->symbol_name);
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_SLOTREF_SLOT) {
    if (setvalueP) {
      self->surrogate_value = ((Slot*)(value));
    }
    else {
      value = ((Slot*)(self->surrogate_value));
    }
  }
  else {
    { Output_String_Stream* stream_000 = new_output_string_stream();

      *(stream_000->native_stream) << "`" << slotname << "'" << " is not a valid case option";
      throw *new_stella_exception(stream_000->the_string_reader());
    }
  }
  return (value);
}

void Surrogate::print_object(ostream* stream) {
  { Surrogate* self = this;

    print_surrogate(self, stream);
  }
}

Keyword* new_keyword(char* symbol_name) {
  { Keyword* self = NULL;

    self = new Keyword();
    self->symbol_name = symbol_name;
    self->home_context = oCONTEXTo.get();
    self->symbol_id = NULL_INTEGER;
    return (self);
  }
}

Surrogate* Keyword::primary_type() {
  { Keyword* self = this;

    return (SGT_HIERARCHY_STELLA_KEYWORD);
  }
}

Object* access_keyword_slot_value(Keyword* self, Symbol* slotname, Object* value, boolean setvalueP) {
  if (slotname == SYM_HIERARCHY_STELLA_KEYWORD_NAME) {
    if (setvalueP) {
      self->symbol_name = ((String_Wrapper*)(value))->wrapper_value;
    }
    else {
      value = wrap_string(self->symbol_name);
    }
  }
  else {
    { Output_String_Stream* stream_000 = new_output_string_stream();

      *(stream_000->native_stream) << "`" << slotname << "'" << " is not a valid case option";
      throw *new_stella_exception(stream_000->the_string_reader());
    }
  }
  return (value);
}

void Keyword::print_object(ostream* stream) {
  { Keyword* self = this;

    print_keyword(self, stream);
  }
}

Transient_Symbol* new_transient_symbol(char* symbol_name) {
  { Transient_Symbol* self = NULL;

    self = new Transient_Symbol();
    self->symbol_name = symbol_name;
    self->home_context = oCONTEXTo.get();
    self->symbol_id = NULL_INTEGER;
    self->symbol_value_and_plist = NIL;
    self->symbol_slot_offset = NULL_INTEGER;
    return (self);
  }
}

Surrogate* Transient_Symbol::primary_type() {
  { Transient_Symbol* self = this;

    return (SGT_HIERARCHY_STELLA_TRANSIENT_SYMBOL);
  }
}

boolean Transient_Symbol::transientP_reader() {
  { Transient_Symbol* self = this;

    return (oHARDWIRED_TRANSIENTp_ON_TRANSIENT_MIXINo);
  }
}

List* Mappable_Object::projects_onto_reader() {
  { Mappable_Object* self = this;

    { List* answer = ((List*)(dynamic_slot_value(self->dynamic_slots, SYM_HIERARCHY_STELLA_PROJECTS_ONTO, NULL)));

      if (!((boolean)(answer))) {
        return (NIL_LIST);
      }
      else {
        return (answer);
      }
    }
  }
}

List* Mappable_Object::projected_from_reader() {
  { Mappable_Object* self = this;

    { List* answer = ((List*)(dynamic_slot_value(self->dynamic_slots, SYM_HIERARCHY_STELLA_PROJECTED_FROM, NULL)));

      if (!((boolean)(answer))) {
        return (NIL_LIST);
      }
      else {
        return (answer);
      }
    }
  }
}

Boolean_Wrapper* Mappable_Object::badP_reader() {
  { Mappable_Object* self = this;

    { Boolean_Wrapper* answer = ((Boolean_Wrapper*)(dynamic_slot_value(self->dynamic_slots, SYM_HIERARCHY_STELLA_BADp, NULL)));

      if (!((boolean)(answer))) {
        return ((FALSE ? TRUE_WRAPPER : FALSE_WRAPPER));
      }
      else {
        return (answer);
      }
    }
  }
}

Object* access_mappable_object_slot_value(Mappable_Object* self, Symbol* slotname, Object* value, boolean setvalueP) {
  if (slotname == SYM_HIERARCHY_STELLA_PROJECTS_ONTO) {
    if (setvalueP) {
      set_dynamic_slot_value(self->dynamic_slots, SYM_HIERARCHY_STELLA_PROJECTS_ONTO, ((List*)(value)), NULL);
    }
    else {
      value = self->projects_onto_reader();
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_PROJECTED_FROM) {
    if (setvalueP) {
      set_dynamic_slot_value(self->dynamic_slots, SYM_HIERARCHY_STELLA_PROJECTED_FROM, ((List*)(value)), NULL);
    }
    else {
      value = self->projected_from_reader();
    }
  }
  else {
    if (setvalueP) {
      set_dynamic_slot_value(self->dynamic_slots, slotname, value, NULL);
    }
    else {
      value = self->dynamic_slots->lookup(slotname);
    }
  }
  return (value);
}

boolean Mappable_Object::deletedP() {
  { Mappable_Object* self = this;

    { Boolean_Wrapper* deletedP = ((Boolean_Wrapper*)(dynamic_slot_value(self->dynamic_slots, SYM_HIERARCHY_STELLA_DELETED_OBJECTp, NULL)));

      if (((boolean)(deletedP))) {
        return (coerce_wrapped_boolean_to_boolean(deletedP));
      }
      else {
        return (FALSE);
      }
    }
  }
}

boolean Mappable_Object::deletedP_setter(boolean value) {
  { Mappable_Object* self = this;

    set_dynamic_slot_value(self->dynamic_slots, SYM_HIERARCHY_STELLA_DELETED_OBJECTp, (value ? TRUE_WRAPPER : FALSE_WRAPPER), NULL);
    return (value);
  }
}

char* Relation::name() {
  { Relation* self = this;

    return (NULL);
  }
}

Module* Relation::home_module() {
  { Relation* self = this;

    return (NULL);
  }
}

int Relation::arity() {
  { Relation* self = this;

    return (NULL_INTEGER);
  }
}

boolean Relation::publicP() {
  { Relation* self = this;

    return (FALSE);
  }
}

List* Relation::direct_supers() {
  { Relation* self = this;

    return (NULL);
  }
}

List* Relation::all_supers() {
  { Relation* self = this;

    return (NULL);
  }
}

Iterator* Relation::slots() {
  { Relation* self = this;

    return (NULL);
  }
}

List* Relation::properties_reader() {
  { Relation* self = this;

    { List* answer = ((List*)(dynamic_slot_value(self->dynamic_slots, SYM_HIERARCHY_STELLA_PROPERTIES, NULL)));

      if (!((boolean)(answer))) {
        return (NIL_LIST);
      }
      else {
        return (answer);
      }
    }
  }
}

Object* access_relation_slot_value(Relation* self, Symbol* slotname, Object* value, boolean setvalueP) {
  if (slotname == SYM_HIERARCHY_STELLA_ABSTRACTp) {
    if (setvalueP) {
      self->abstractP = coerce_wrapped_boolean_to_boolean(((Boolean_Wrapper*)(value)));
    }
    else {
      value = (self->abstractP ? TRUE_WRAPPER : FALSE_WRAPPER);
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_PROPERTIES) {
    if (setvalueP) {
      set_dynamic_slot_value(self->dynamic_slots, SYM_HIERARCHY_STELLA_PROPERTIES, ((List*)(value)), NULL);
    }
    else {
      value = self->properties_reader();
    }
  }
  else {
    if (setvalueP) {
      set_dynamic_slot_value(self->dynamic_slots, slotname, value, NULL);
    }
    else {
      value = self->dynamic_slots->lookup(slotname);
    }
  }
  return (value);
}

Class* new_class() {
  { Class* self = NULL;

    self = new Class();
    self->dynamic_slots = new_key_value_list();
    self->class_taxonomy_node = NULL;
    self->class_markedP = FALSE;
    self->class_slot_accessor_code = NULL;
    self->class_constructor_code = NULL;
    self->class_stringified_source = NULL;
    self->class_slots_finalizedP = FALSE;
    self->class_finalizedP = FALSE;
    self->class_publicP = TRUE;
    self->cl_structP = FALSE;
    self->class_collectionP = FALSE;
    self->mixinP = FALSE;
    self->abstractP = FALSE;
    self->class_slot_and_method_cache = NULL;
    self->class_local_slots = NULL;
    self->class_all_slots = NULL;
    self->class_all_super_classes = NIL;
    self->class_direct_subs = new_list();
    self->class_direct_supers = new_list();
    self->class_type = NULL;
    return (self);
  }
}

Surrogate* Class::primary_type() {
  { Class* self = this;

    return (SGT_HIERARCHY_STELLA_CLASS);
  }
}

char* Class::name() {
  { Class* self = this;

    return (self->class_type->symbol_name);
  }
}

Module* Class::home_module() {
  { Class* self = this;

    return (((Module*)(self->class_type->home_context)));
  }
}

int Class::arity() {
  { Class* self = this;

    return (1);
  }
}

boolean Class::publicP() {
  { Class* self = this;

    return (self->class_publicP);
  }
}

boolean Class::activeP() {
  { Class* self = this;

    return ((((boolean)(((Boolean_Wrapper*)(dynamic_slot_value(self->dynamic_slots, SYM_HIERARCHY_STELLA_STORED_ACTIVEp, NULL))))) &&
        coerce_wrapped_boolean_to_boolean(((Boolean_Wrapper*)(dynamic_slot_value(self->dynamic_slots, SYM_HIERARCHY_STELLA_STORED_ACTIVEp, NULL))))) ||
        subtype_ofP(self->class_type, SGT_HIERARCHY_STELLA_ACTIVE_OBJECT));
  }
}

Symbol* Class::creator() {
  { Class* self = this;

    return (((Symbol*)(dynamic_slot_value(self->dynamic_slots, SYM_HIERARCHY_STELLA_CLASS_CREATOR, NULL))));
  }
}

Symbol* Class::initializer() {
  { Class* self = this;

    { Class* cursor = self;
      Symbol* value = ((Symbol*)(dynamic_slot_value(cursor->dynamic_slots, SYM_HIERARCHY_STELLA_CLASS_INITIALIZER, NULL)));

      for (;;) {
        if (((boolean)(value))) {
          return (value);
        }
        cursor = ((Class*)(cursor->class_all_super_classes->value));
        if (((boolean)(cursor))) {
          value = ((Symbol*)(dynamic_slot_value(cursor->dynamic_slots, SYM_HIERARCHY_STELLA_CLASS_INITIALIZER, NULL)));
        }
        else {
          return (NULL);
        }
      }
    }
  }
}

Symbol* Class::terminator() {
  { Class* self = this;

    { Class* cursor = self;
      Symbol* value = ((Symbol*)(dynamic_slot_value(cursor->dynamic_slots, SYM_HIERARCHY_STELLA_CLASS_TERMINATOR, NULL)));

      for (;;) {
        if (((boolean)(value))) {
          return (value);
        }
        cursor = ((Class*)(cursor->class_all_super_classes->value));
        if (((boolean)(cursor))) {
          value = ((Symbol*)(dynamic_slot_value(cursor->dynamic_slots, SYM_HIERARCHY_STELLA_CLASS_TERMINATOR, NULL)));
        }
        else {
          return (NULL);
        }
      }
    }
  }
}

Symbol* Class::destructor() {
  { Class* self = this;

    return (((Symbol*)(dynamic_slot_value(self->dynamic_slots, SYM_HIERARCHY_STELLA_CLASS_DESTRUCTOR, NULL))));
  }
}

List* Class::required_slots() {
  // Returns a list of names of required slots for 'self'.
  { Class* self = this;

    return (self->class_required_slot_names_reader());
  }
}

List* Class::parameters() {
  // Returns the list of parameters names of 'self'.
  { Class* self = this;

    { Class* cursor = self;
      List* value = ((List*)(dynamic_slot_value(cursor->dynamic_slots, SYM_HIERARCHY_STELLA_CLASS_PARAMETERS, NULL)));

      for (;;) {
        if (((boolean)(value))) {
          return (value);
        }
        cursor = ((Class*)(cursor->class_all_super_classes->value));
        if (((boolean)(cursor))) {
          value = ((List*)(dynamic_slot_value(cursor->dynamic_slots, SYM_HIERARCHY_STELLA_CLASS_PARAMETERS, NULL)));
        }
        else {
          return (NIL_LIST);
        }
      }
    }
  }
}

Object* Class::initial_value() {
  // Return an initial value for the class 'self'.
  { Class* self = this;

    { Class* cursor = self;
      Object* value = dynamic_slot_value(cursor->dynamic_slots, SYM_HIERARCHY_STELLA_CLASS_INITIAL_VALUE, NULL);

      for (;;) {
        if (((boolean)(value))) {
          return (value);
        }
        cursor = ((Class*)(cursor->class_all_super_classes->value));
        if (((boolean)(cursor))) {
          value = dynamic_slot_value(cursor->dynamic_slots, SYM_HIERARCHY_STELLA_CLASS_INITIAL_VALUE, NULL);
        }
        else {
          return (NULL);
        }
      }
    }
  }
}

Class_Extension* Class::extension() {
  // Return the nearest class extension that records instances
  // of the class 'self'.
  { Class* self = this;

    { Class* cursor = self;
      Class_Extension* value = ((Class_Extension*)(dynamic_slot_value(cursor->dynamic_slots, SYM_HIERARCHY_STELLA_CLASS_EXTENSION, NULL)));

      for (;;) {
        if (((boolean)(value))) {
          return (value);
        }
        cursor = ((Class*)(cursor->class_all_super_classes->value));
        if (((boolean)(cursor))) {
          value = ((Class_Extension*)(dynamic_slot_value(cursor->dynamic_slots, SYM_HIERARCHY_STELLA_CLASS_EXTENSION, NULL)));
        }
        else {
          return (NULL);
        }
      }
    }
  }
}

char* Class::cl_native_type() {
  { Class* self = this;

    return (((String_Wrapper*)(dynamic_slot_value(self->dynamic_slots, SYM_HIERARCHY_STELLA_CLASS_CL_NATIVE_TYPE, NULL_STRING_WRAPPER)))->wrapper_value);
  }
}

char* Class::cpp_native_type() {
  { Class* self = this;

    return (((String_Wrapper*)(dynamic_slot_value(self->dynamic_slots, SYM_HIERARCHY_STELLA_CLASS_CPP_NATIVE_TYPE, NULL_STRING_WRAPPER)))->wrapper_value);
  }
}

char* Class::idl_native_type() {
  { Class* self = this;

    return (((String_Wrapper*)(dynamic_slot_value(self->dynamic_slots, SYM_HIERARCHY_STELLA_CLASS_CPP_NATIVE_TYPE, NULL_STRING_WRAPPER)))->wrapper_value);
  }
}

char* Class::java_native_type() {
  { Class* self = this;

    { char* nativetype = ((String_Wrapper*)(dynamic_slot_value(self->dynamic_slots, SYM_HIERARCHY_STELLA_CLASS_JAVA_NATIVE_TYPE, NULL_STRING_WRAPPER)))->wrapper_value;

      if (nativetype != NULL) {
        return (substitute_template_variables_in_string(nativetype, oJAVA_STELLA_PACKAGE_MAPPINGo));
      }
      else {
        return (NULL);
      }
    }
  }
}

int Class::class_arity_reader() {
  { Class* self = this;

    return (oHARDWIRED_CLASS_ARITY_ON_CLASSo);
  }
}

List* Class::class_cl_struct_slots_reader() {
  { Class* self = this;

    { List* answer = ((List*)(dynamic_slot_value(self->dynamic_slots, SYM_HIERARCHY_STELLA_CLASS_CL_STRUCT_SLOTS, NULL)));

      if (!((boolean)(answer))) {
        return (NIL_LIST);
      }
      else {
        return (answer);
      }
    }
  }
}

Keyword* Class::class_recycle_method_reader() {
  { Class* self = this;

    { Keyword* answer = ((Keyword*)(dynamic_slot_value(self->dynamic_slots, SYM_HIERARCHY_STELLA_CLASS_RECYCLE_METHOD, NULL)));

      if (!((boolean)(answer))) {
        return (KWD_HIERARCHY_NONE);
      }
      else {
        return (answer);
      }
    }
  }
}

List* Class::class_parameters_reader() {
  { Class* self = this;

    { List* answer = ((List*)(dynamic_slot_value(self->dynamic_slots, SYM_HIERARCHY_STELLA_CLASS_PARAMETERS, NULL)));

      if (!((boolean)(answer))) {
        return (NIL_LIST);
      }
      else {
        return (answer);
      }
    }
  }
}

List* Class::class_required_slot_names_reader() {
  { Class* self = this;

    { List* answer = ((List*)(dynamic_slot_value(self->dynamic_slots, SYM_HIERARCHY_STELLA_CLASS_REQUIRED_SLOT_NAMES, NULL)));

      if (!((boolean)(answer))) {
        return (NIL_LIST);
      }
      else {
        return (answer);
      }
    }
  }
}

List* Class::class_guard_constructor_demons_reader() {
  { Class* self = this;

    { List* answer = ((List*)(dynamic_slot_value(self->dynamic_slots, SYM_HIERARCHY_STELLA_CLASS_GUARD_CONSTRUCTOR_DEMONS, NULL)));

      if (!((boolean)(answer))) {
        return (NIL_LIST);
      }
      else {
        return (answer);
      }
    }
  }
}

List* Class::class_constructor_demons_reader() {
  { Class* self = this;

    { List* answer = ((List*)(dynamic_slot_value(self->dynamic_slots, SYM_HIERARCHY_STELLA_CLASS_CONSTRUCTOR_DEMONS, NULL)));

      if (!((boolean)(answer))) {
        return (NIL_LIST);
      }
      else {
        return (answer);
      }
    }
  }
}

List* Class::class_guard_destructor_demons_reader() {
  { Class* self = this;

    { List* answer = ((List*)(dynamic_slot_value(self->dynamic_slots, SYM_HIERARCHY_STELLA_CLASS_GUARD_DESTRUCTOR_DEMONS, NULL)));

      if (!((boolean)(answer))) {
        return (NIL_LIST);
      }
      else {
        return (answer);
      }
    }
  }
}

List* Class::class_destructor_demons_reader() {
  { Class* self = this;

    { List* answer = ((List*)(dynamic_slot_value(self->dynamic_slots, SYM_HIERARCHY_STELLA_CLASS_DESTRUCTOR_DEMONS, NULL)));

      if (!((boolean)(answer))) {
        return (NIL_LIST);
      }
      else {
        return (answer);
      }
    }
  }
}

List* Class::class_key_reader() {
  { Class* self = this;

    { List* answer = ((List*)(dynamic_slot_value(self->dynamic_slots, SYM_HIERARCHY_STELLA_CLASS_KEY, NULL)));

      if (!((boolean)(answer))) {
        return (NIL_LIST);
      }
      else {
        return (answer);
      }
    }
  }
}

List* Class::class_synonyms_reader() {
  { Class* self = this;

    { List* answer = ((List*)(dynamic_slot_value(self->dynamic_slots, SYM_HIERARCHY_STELLA_CLASS_SYNONYMS, NULL)));

      if (!((boolean)(answer))) {
        return (NIL_LIST);
      }
      else {
        return (answer);
      }
    }
  }
}

Object* access_class_slot_value(Class* self, Symbol* slotname, Object* value, boolean setvalueP) {
  if (slotname == SYM_HIERARCHY_STELLA_CL_STRUCTp) {
    if (setvalueP) {
      self->cl_structP = coerce_wrapped_boolean_to_boolean(((Boolean_Wrapper*)(value)));
    }
    else {
      value = (self->cl_structP ? TRUE_WRAPPER : FALSE_WRAPPER);
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_MIXINp) {
    if (setvalueP) {
      self->mixinP = coerce_wrapped_boolean_to_boolean(((Boolean_Wrapper*)(value)));
    }
    else {
      value = (self->mixinP ? TRUE_WRAPPER : FALSE_WRAPPER);
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_CLASS_TYPE) {
    if (setvalueP) {
      self->class_type = ((Surrogate*)(value));
    }
    else {
      value = self->class_type;
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_CLASS_ARITY) {
    value = wrap_integer(self->class_arity_reader());
  }
  else if (slotname == SYM_HIERARCHY_STELLA_CLASS_DIRECT_SUPERS) {
    if (setvalueP) {
      self->class_direct_supers = ((List*)(value));
    }
    else {
      value = self->class_direct_supers;
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_CLASS_DIRECT_SUBS) {
    if (setvalueP) {
      self->class_direct_subs = ((List*)(value));
    }
    else {
      value = self->class_direct_subs;
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_CLASS_ALL_SUPER_CLASSES) {
    if (setvalueP) {
      self->class_all_super_classes = ((Cons*)(value));
    }
    else {
      value = self->class_all_super_classes;
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_CLASS_ALL_SLOTS) {
    if (setvalueP) {
      self->class_all_slots = ((Cons*)(value));
    }
    else {
      value = self->class_all_slots;
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_CLASS_LOCAL_SLOTS) {
    if (setvalueP) {
      self->class_local_slots = ((List*)(value));
    }
    else {
      value = self->class_local_slots;
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_CLASS_SLOT_AND_METHOD_CACHE) {
    if (setvalueP) {
      self->class_slot_and_method_cache = ((Vector*)(value));
    }
    else {
      value = self->class_slot_and_method_cache;
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_CLASS_ABSTRACTp) {
    if (setvalueP) {
      self->abstractP = coerce_wrapped_boolean_to_boolean(((Boolean_Wrapper*)(value)));
    }
    else {
      value = (self->abstractP ? TRUE_WRAPPER : FALSE_WRAPPER);
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_CLASS_MIXINp) {
    if (setvalueP) {
      self->mixinP = coerce_wrapped_boolean_to_boolean(((Boolean_Wrapper*)(value)));
    }
    else {
      value = (self->mixinP ? TRUE_WRAPPER : FALSE_WRAPPER);
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_CLASS_COLLECTIONp) {
    if (setvalueP) {
      self->class_collectionP = coerce_wrapped_boolean_to_boolean(((Boolean_Wrapper*)(value)));
    }
    else {
      value = (self->class_collectionP ? TRUE_WRAPPER : FALSE_WRAPPER);
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_CLASS_CL_STRUCTp) {
    if (setvalueP) {
      self->cl_structP = coerce_wrapped_boolean_to_boolean(((Boolean_Wrapper*)(value)));
    }
    else {
      value = (self->cl_structP ? TRUE_WRAPPER : FALSE_WRAPPER);
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_CLASS_CL_STRUCT_SLOTS) {
    if (setvalueP) {
      set_dynamic_slot_value(self->dynamic_slots, SYM_HIERARCHY_STELLA_CLASS_CL_STRUCT_SLOTS, ((List*)(value)), NULL);
    }
    else {
      value = self->class_cl_struct_slots_reader();
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_CLASS_PUBLICp) {
    if (setvalueP) {
      self->class_publicP = coerce_wrapped_boolean_to_boolean(((Boolean_Wrapper*)(value)));
    }
    else {
      value = (self->class_publicP ? TRUE_WRAPPER : FALSE_WRAPPER);
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_CLASS_RECYCLE_METHOD) {
    if (setvalueP) {
      set_dynamic_slot_value(self->dynamic_slots, SYM_HIERARCHY_STELLA_CLASS_RECYCLE_METHOD, ((Keyword*)(value)), NULL);
    }
    else {
      value = self->class_recycle_method_reader();
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_CLASS_FINALIZEDp) {
    if (setvalueP) {
      self->class_finalizedP = coerce_wrapped_boolean_to_boolean(((Boolean_Wrapper*)(value)));
    }
    else {
      value = (self->class_finalizedP ? TRUE_WRAPPER : FALSE_WRAPPER);
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_CLASS_SLOTS_FINALIZEDp) {
    if (setvalueP) {
      self->class_slots_finalizedP = coerce_wrapped_boolean_to_boolean(((Boolean_Wrapper*)(value)));
    }
    else {
      value = (self->class_slots_finalizedP ? TRUE_WRAPPER : FALSE_WRAPPER);
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_CLASS_STRINGIFIED_SOURCE) {
    if (setvalueP) {
      self->class_stringified_source = ((String_Wrapper*)(value))->wrapper_value;
    }
    else {
      value = wrap_string(self->class_stringified_source);
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_CLASS_PARAMETERS) {
    if (setvalueP) {
      set_dynamic_slot_value(self->dynamic_slots, SYM_HIERARCHY_STELLA_CLASS_PARAMETERS, ((List*)(value)), NULL);
    }
    else {
      value = self->class_parameters_reader();
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_CLASS_CONSTRUCTOR_CODE) {
    if (setvalueP) {
      self->class_constructor_code = ((Function_Code_Wrapper*)(value))->wrapper_value;
    }
    else {
      value = wrap_function_code(self->class_constructor_code);
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_CLASS_SLOT_ACCESSOR_CODE) {
    if (setvalueP) {
      self->class_slot_accessor_code = ((Function_Code_Wrapper*)(value))->wrapper_value;
    }
    else {
      value = wrap_function_code(self->class_slot_accessor_code);
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_CLASS_REQUIRED_SLOT_NAMES) {
    if (setvalueP) {
      set_dynamic_slot_value(self->dynamic_slots, SYM_HIERARCHY_STELLA_CLASS_REQUIRED_SLOT_NAMES, ((List*)(value)), NULL);
    }
    else {
      value = self->class_required_slot_names_reader();
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_CLASS_GUARD_CONSTRUCTOR_DEMONS) {
    if (setvalueP) {
      set_dynamic_slot_value(self->dynamic_slots, SYM_HIERARCHY_STELLA_CLASS_GUARD_CONSTRUCTOR_DEMONS, ((List*)(value)), NULL);
    }
    else {
      value = self->class_guard_constructor_demons_reader();
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_CLASS_CONSTRUCTOR_DEMONS) {
    if (setvalueP) {
      set_dynamic_slot_value(self->dynamic_slots, SYM_HIERARCHY_STELLA_CLASS_CONSTRUCTOR_DEMONS, ((List*)(value)), NULL);
    }
    else {
      value = self->class_constructor_demons_reader();
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_CLASS_GUARD_DESTRUCTOR_DEMONS) {
    if (setvalueP) {
      set_dynamic_slot_value(self->dynamic_slots, SYM_HIERARCHY_STELLA_CLASS_GUARD_DESTRUCTOR_DEMONS, ((List*)(value)), NULL);
    }
    else {
      value = self->class_guard_destructor_demons_reader();
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_CLASS_DESTRUCTOR_DEMONS) {
    if (setvalueP) {
      set_dynamic_slot_value(self->dynamic_slots, SYM_HIERARCHY_STELLA_CLASS_DESTRUCTOR_DEMONS, ((List*)(value)), NULL);
    }
    else {
      value = self->class_destructor_demons_reader();
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_CLASS_KEY) {
    if (setvalueP) {
      set_dynamic_slot_value(self->dynamic_slots, SYM_HIERARCHY_STELLA_CLASS_KEY, ((List*)(value)), NULL);
    }
    else {
      value = self->class_key_reader();
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_CLASS_SYNONYMS) {
    if (setvalueP) {
      set_dynamic_slot_value(self->dynamic_slots, SYM_HIERARCHY_STELLA_CLASS_SYNONYMS, ((List*)(value)), NULL);
    }
    else {
      value = self->class_synonyms_reader();
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_CLASS_MARKEDp) {
    if (setvalueP) {
      self->class_markedP = coerce_wrapped_boolean_to_boolean(((Boolean_Wrapper*)(value)));
    }
    else {
      value = (self->class_markedP ? TRUE_WRAPPER : FALSE_WRAPPER);
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_CLASS_TAXONOMY_NODE) {
    if (setvalueP) {
      self->class_taxonomy_node = ((Taxonomy_Node*)(value));
    }
    else {
      value = self->class_taxonomy_node;
    }
  }
  else {
    if (slotname == SYM_HIERARCHY_STELLA_CLASS_DOCUMENTATION) {
      slotname = SYM_HIERARCHY_STELLA_DOCUMENTATION;
    }
    else if (slotname == SYM_HIERARCHY_STELLA_CLASS_PRINT_FORM) {
      slotname = SYM_HIERARCHY_STELLA_PRINT_FORM;
    }
    else {
    }
    if (setvalueP) {
      set_dynamic_slot_value(self->dynamic_slots, slotname, value, NULL);
    }
    else {
      value = self->dynamic_slots->lookup(slotname);
    }
  }
  return (value);
}

void Class::print_object(ostream* stream) {
  { Class* self = this;

    *(stream) << "|C|" << ((((boolean)(self->class_type)) ? class_symbol(self)->relative_name() : (char*)"??"));
  }
}

Surrogate* Polymorphic_Relation::owner() {
  { Polymorphic_Relation* self = this;

    return (NULL);
  }
}

Symbol* Polymorphic_Relation::renames() {
  { Polymorphic_Relation* self = this;

    return (NULL);
  }
}

boolean Polymorphic_Relation::activeP() {
  // True if 'self' or a superslot of 'self' is marked active.
  { Polymorphic_Relation* self = this;

    { Polymorphic_Relation* cursor = self;
      Boolean_Wrapper* value = ((Boolean_Wrapper*)(dynamic_slot_value(cursor->dynamic_slots, SYM_HIERARCHY_STELLA_STORED_ACTIVEp, NULL)));

      for (;;) {
        if (((boolean)(value))) {
          return (coerce_wrapped_boolean_to_boolean(value));
        }
        cursor = cursor->slot_direct_equivalent;
        if (((boolean)(cursor))) {
          value = ((Boolean_Wrapper*)(dynamic_slot_value(cursor->dynamic_slots, SYM_HIERARCHY_STELLA_STORED_ACTIVEp, NULL)));
        }
        else {
          return (FALSE);
        }
      }
    }
  }
}

Object* access_polymorphic_relation_slot_value(Polymorphic_Relation* self, Symbol* slotname, Object* value, boolean setvalueP) {
  if (slotname == SYM_HIERARCHY_STELLA_SLOT_DIRECT_EQUIVALENT) {
    if (setvalueP) {
      self->slot_direct_equivalent = ((Slot*)(value));
    }
    else {
      value = self->slot_direct_equivalent;
    }
  }
  else {
    if (setvalueP) {
      set_dynamic_slot_value(self->dynamic_slots, slotname, value, NULL);
    }
    else {
      value = self->dynamic_slots->lookup(slotname);
    }
  }
  return (value);
}

char* Slot::name() {
  { Slot* self = this;

    return (self->slot_name->symbol_name);
  }
}

Surrogate* Slot::type() {
  // The type of a storage slot is its base type.
  { Slot* self = this;

    { Slot* cursor = self;
      Surrogate* value = cursor->slot_base_type;

      for (;;) {
        if (((boolean)(value))) {
          return (value);
        }
        cursor = cursor->slot_direct_equivalent;
        if (((boolean)(cursor))) {
          value = cursor->slot_base_type;
        }
        else {
          return (NULL);
        }
      }
    }
  }
}

Standard_Object* Slot::type_specifier() {
  // If 'self' has a complex type return its
  // type specifier, otherwise, return 'type' of 'self'.
  { Slot* self = this;

    { Compound_Type_Specifier* tspec = ((Compound_Type_Specifier*)(dynamic_slot_value(self->dynamic_slots, SYM_HIERARCHY_STELLA_SLOT_TYPE_SPECIFIER, NULL)));

      return ((((boolean)(tspec)) ? ((Standard_Object*)(tspec)) : ((Standard_Object*)(self->type()))));
    }
  }
}

Surrogate* Slot::owner() {
  { Slot* self = this;

    return (self->slot_owner);
  }
}

Module* Slot::home_module() {
  { Slot* self = this;

    { Module* module = ((Module*)(dynamic_slot_value(self->dynamic_slots, SYM_HIERARCHY_STELLA_SLOT_HOME_MODULE, NULL)));

      if (((boolean)(module))) {
        return (module);
      }
      else {
        return (((Module*)(self->slot_name->home_context)));
      }
    }
  }
}

Symbol* Slot::renames() {
  { Slot* self = this;

    return (((Symbol*)(dynamic_slot_value(self->dynamic_slots, SYM_HIERARCHY_STELLA_SLOT_RENAMES, NULL))));
  }
}

boolean Slot::collection_valuedP() {
  // True if slot values are collections.
  { Slot* self = this;

    return (subtype_ofP(self->slot_base_type, SGT_HIERARCHY_STELLA_COLLECTION));
  }
}

Keyword* Slot::closure_assumption() {
  { Slot* self = this;

    { Slot* cursor = self;
      Keyword* value = ((Keyword*)(dynamic_slot_value(cursor->dynamic_slots, SYM_HIERARCHY_STELLA_SLOT_CLOSURE_ASSUMPTION, NULL)));

      for (;;) {
        if (((boolean)(value))) {
          return (value);
        }
        cursor = cursor->slot_direct_equivalent;
        if (((boolean)(cursor))) {
          value = ((Keyword*)(dynamic_slot_value(cursor->dynamic_slots, SYM_HIERARCHY_STELLA_SLOT_CLOSURE_ASSUMPTION, NULL)));
        }
        else {
          return (NULL);
        }
      }
    }
  }
}

List* Slot::slot_direct_supers_reader() {
  { Slot* self = this;

    { List* answer = ((List*)(dynamic_slot_value(self->dynamic_slots, SYM_HIERARCHY_STELLA_SLOT_DIRECT_SUPERS, NULL)));

      if (!((boolean)(answer))) {
        return (NIL_LIST);
      }
      else {
        return (answer);
      }
    }
  }
}

List* Slot::slot_direct_subs_reader() {
  { Slot* self = this;

    { List* answer = ((List*)(dynamic_slot_value(self->dynamic_slots, SYM_HIERARCHY_STELLA_SLOT_DIRECT_SUBS, NULL)));

      if (!((boolean)(answer))) {
        return (NIL_LIST);
      }
      else {
        return (answer);
      }
    }
  }
}

Object* access_slot_slot_value(Slot* self, Symbol* slotname, Object* value, boolean setvalueP) {
  if (slotname == SYM_HIERARCHY_STELLA_SLOT_NAME) {
    if (setvalueP) {
      self->slot_name = ((Symbol*)(value));
    }
    else {
      value = self->slot_name;
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_SLOT_OWNER) {
    if (setvalueP) {
      self->slot_owner = ((Surrogate*)(value));
    }
    else {
      value = self->slot_owner;
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_SLOT_BASE_TYPE) {
    if (setvalueP) {
      self->slot_base_type = ((Surrogate*)(value));
    }
    else {
      value = self->slot_base_type;
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_SLOT_SLOTREF) {
    if (setvalueP) {
      self->slot_slotref = ((Surrogate*)(value));
    }
    else {
      value = self->slot_slotref;
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_SLOT_DIRECT_SUPERS) {
    if (setvalueP) {
      set_dynamic_slot_value(self->dynamic_slots, SYM_HIERARCHY_STELLA_SLOT_DIRECT_SUPERS, ((List*)(value)), NULL);
    }
    else {
      value = self->slot_direct_supers_reader();
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_SLOT_DIRECT_SUBS) {
    if (setvalueP) {
      set_dynamic_slot_value(self->dynamic_slots, SYM_HIERARCHY_STELLA_SLOT_DIRECT_SUBS, ((List*)(value)), NULL);
    }
    else {
      value = self->slot_direct_subs_reader();
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_SLOT_PUBLICp) {
    if (setvalueP) {
      self->slot_publicP = coerce_wrapped_boolean_to_boolean(((Boolean_Wrapper*)(value)));
    }
    else {
      value = (self->slot_publicP ? TRUE_WRAPPER : FALSE_WRAPPER);
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_SLOT_RENAMEDp) {
    if (setvalueP) {
      self->slot_renamedP = coerce_wrapped_boolean_to_boolean(((Boolean_Wrapper*)(value)));
    }
    else {
      value = (self->slot_renamedP ? TRUE_WRAPPER : FALSE_WRAPPER);
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_SLOT_EXTERNALp) {
    if (setvalueP) {
      self->slot_externalP = coerce_wrapped_boolean_to_boolean(((Boolean_Wrapper*)(value)));
    }
    else {
      value = (self->slot_externalP ? TRUE_WRAPPER : FALSE_WRAPPER);
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_SLOT_MARKEDp) {
    if (setvalueP) {
      self->slot_markedP = coerce_wrapped_boolean_to_boolean(((Boolean_Wrapper*)(value)));
    }
    else {
      value = (self->slot_markedP ? TRUE_WRAPPER : FALSE_WRAPPER);
    }
  }
  else {
    if (slotname == SYM_HIERARCHY_STELLA_SLOT_DOCUMENTATION) {
      slotname = SYM_HIERARCHY_STELLA_DOCUMENTATION;
    }
    else {
    }
    if (setvalueP) {
      set_dynamic_slot_value(self->dynamic_slots, slotname, value, NULL);
    }
    else {
      value = self->dynamic_slots->lookup(slotname);
    }
  }
  return (value);
}

void Slot::print_object(ostream* stream) {
  { Slot* self = this;

    *(stream) << "|S|" << self->slot_owner->symbol_name << "." << self->slot_name->symbol_name;
  }
}

Storage_Slot* new_storage_slot() {
  { Storage_Slot* self = NULL;

    self = new Storage_Slot();
    self->dynamic_slots = new_key_value_list();
    self->abstractP = FALSE;
    self->slot_direct_equivalent = NULL;
    self->slot_markedP = FALSE;
    self->slot_externalP = FALSE;
    self->slot_renamedP = FALSE;
    self->slot_publicP = TRUE;
    self->slot_slotref = NULL;
    self->slot_base_type = NULL;
    self->slot_owner = NULL;
    self->slot_name = NULL;
    self->slot_context_sensitiveP = FALSE;
    self->slot_hardwiredP = FALSE;
    self->slot_read_onlyP = FALSE;
    self->slot_componentP = FALSE;
    self->slot_requiredP = FALSE;
    return (self);
  }
}

Surrogate* Storage_Slot::primary_type() {
  { Storage_Slot* self = this;

    return (SGT_HIERARCHY_STELLA_STORAGE_SLOT);
  }
}

Keyword* Storage_Slot::allocation() {
  // Return the most specific :allocation facet, or
  // :instance if all inherited values are NULL.
  { Storage_Slot* self = this;

    { Storage_Slot* cursor = self;
      Keyword* value = ((Keyword*)(dynamic_slot_value(cursor->dynamic_slots, SYM_HIERARCHY_STELLA_SLOT_ALLOCATION, NULL)));

      for (;;) {
        if (((boolean)(value))) {
          return (value);
        }
        cursor = ((Storage_Slot*)(cursor->slot_direct_equivalent));
        if (((boolean)(cursor))) {
          value = ((Keyword*)(dynamic_slot_value(cursor->dynamic_slots, SYM_HIERARCHY_STELLA_SLOT_ALLOCATION, NULL)));
        }
        else {
          return (KWD_HIERARCHY_INSTANCE);
        }
      }
    }
  }
}

Object* Storage_Slot::default_form() {
  // Returns the current value of default expression when the
  // slot has not been assigned a value.
  { Storage_Slot* self = this;

    { Storage_Slot* cursor = self;
      Object* value = dynamic_slot_value(cursor->dynamic_slots, SYM_HIERARCHY_STELLA_SLOT_DEFAULT_EXPRESSION, NULL);

      for (;;) {
        if (((boolean)(value))) {
          return (value);
        }
        cursor = ((Storage_Slot*)(cursor->slot_direct_equivalent));
        if (((boolean)(cursor))) {
          value = dynamic_slot_value(cursor->dynamic_slots, SYM_HIERARCHY_STELLA_SLOT_DEFAULT_EXPRESSION, NULL);
        }
        else {
          return (NULL);
        }
      }
    }
  }
}

boolean Storage_Slot::requiredP() {
  // True if a value must be assigned to this slot at
  // creation time.
  { Storage_Slot* self = this;

    { Storage_Slot* cursor = self;
      boolean value = cursor->slot_requiredP;

      for (;;) {
        if (value) {
          return (value);
        }
        cursor = ((Storage_Slot*)(cursor->slot_direct_equivalent));
        if (((boolean)(cursor))) {
          value = cursor->slot_requiredP;
        }
        else {
          return (FALSE);
        }
      }
    }
  }
}

boolean Storage_Slot::componentP() {
  // True if fillers of this slot are components of the
  // owner slot, and therefore should be deleted if the owner is deleted.
  { Storage_Slot* self = this;

    { Storage_Slot* cursor = self;
      boolean value = cursor->slot_componentP;

      for (;;) {
        if (value) {
          return (value);
        }
        cursor = ((Storage_Slot*)(cursor->slot_direct_equivalent));
        if (((boolean)(cursor))) {
          value = cursor->slot_componentP;
        }
        else {
          return (FALSE);
        }
      }
    }
  }
}

Symbol* Storage_Slot::reader() {
  // Name of a method called to read the value of the slot
  // 'self'.
  { Storage_Slot* self = this;

    { Storage_Slot* cursor = self;
      Symbol* value = ((Symbol*)(dynamic_slot_value(cursor->dynamic_slots, SYM_HIERARCHY_STELLA_SLOT_READER, NULL)));

      for (;;) {
        if (((boolean)(value))) {
          return (value);
        }
        cursor = ((Storage_Slot*)(cursor->slot_direct_equivalent));
        if (((boolean)(cursor))) {
          value = ((Symbol*)(dynamic_slot_value(cursor->dynamic_slots, SYM_HIERARCHY_STELLA_SLOT_READER, NULL)));
        }
        else {
          return (NULL);
        }
      }
    }
  }
}

Symbol* Storage_Slot::writer() {
  // Name of a method called to write the value of the slot
  // 'self'.
  { Storage_Slot* self = this;

    { Storage_Slot* cursor = self;
      Symbol* value = ((Symbol*)(dynamic_slot_value(cursor->dynamic_slots, SYM_HIERARCHY_STELLA_SLOT_WRITER, NULL)));

      for (;;) {
        if (((boolean)(value))) {
          return (value);
        }
        cursor = ((Storage_Slot*)(cursor->slot_direct_equivalent));
        if (((boolean)(cursor))) {
          value = ((Symbol*)(dynamic_slot_value(cursor->dynamic_slots, SYM_HIERARCHY_STELLA_SLOT_WRITER, NULL)));
        }
        else {
          return (NULL);
        }
      }
    }
  }
}

Keyword* Storage_Slot::slot_allocation_reader() {
  { Storage_Slot* self = this;

    { Keyword* answer = ((Keyword*)(dynamic_slot_value(self->dynamic_slots, SYM_HIERARCHY_STELLA_SLOT_ALLOCATION, NULL)));

      if (!((boolean)(answer))) {
        return (KWD_HIERARCHY_INSTANCE);
      }
      else {
        return (answer);
      }
    }
  }
}

List* Storage_Slot::slot_guard_demons_reader() {
  { Storage_Slot* self = this;

    { List* answer = ((List*)(dynamic_slot_value(self->dynamic_slots, SYM_HIERARCHY_STELLA_SLOT_GUARD_DEMONS, NULL)));

      if (!((boolean)(answer))) {
        return (NIL_LIST);
      }
      else {
        return (answer);
      }
    }
  }
}

List* Storage_Slot::slot_demons_reader() {
  { Storage_Slot* self = this;

    { List* answer = ((List*)(dynamic_slot_value(self->dynamic_slots, SYM_HIERARCHY_STELLA_SLOT_DEMONS, NULL)));

      if (!((boolean)(answer))) {
        return (NIL_LIST);
      }
      else {
        return (answer);
      }
    }
  }
}

Object* access_storage_slot_slot_value(Storage_Slot* self, Symbol* slotname, Object* value, boolean setvalueP) {
  if (slotname == SYM_HIERARCHY_STELLA_SLOT_ALLOCATION) {
    if (setvalueP) {
      set_dynamic_slot_value(self->dynamic_slots, SYM_HIERARCHY_STELLA_SLOT_ALLOCATION, ((Keyword*)(value)), NULL);
    }
    else {
      value = self->slot_allocation_reader();
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_SLOT_GUARD_DEMONS) {
    if (setvalueP) {
      set_dynamic_slot_value(self->dynamic_slots, SYM_HIERARCHY_STELLA_SLOT_GUARD_DEMONS, ((List*)(value)), NULL);
    }
    else {
      value = self->slot_guard_demons_reader();
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_SLOT_DEMONS) {
    if (setvalueP) {
      set_dynamic_slot_value(self->dynamic_slots, SYM_HIERARCHY_STELLA_SLOT_DEMONS, ((List*)(value)), NULL);
    }
    else {
      value = self->slot_demons_reader();
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_SLOT_REQUIREDp) {
    if (setvalueP) {
      self->slot_requiredP = coerce_wrapped_boolean_to_boolean(((Boolean_Wrapper*)(value)));
    }
    else {
      value = (self->slot_requiredP ? TRUE_WRAPPER : FALSE_WRAPPER);
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_SLOT_COMPONENTp) {
    if (setvalueP) {
      self->slot_componentP = coerce_wrapped_boolean_to_boolean(((Boolean_Wrapper*)(value)));
    }
    else {
      value = (self->slot_componentP ? TRUE_WRAPPER : FALSE_WRAPPER);
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_SLOT_READ_ONLYp) {
    if (setvalueP) {
      self->slot_read_onlyP = coerce_wrapped_boolean_to_boolean(((Boolean_Wrapper*)(value)));
    }
    else {
      value = (self->slot_read_onlyP ? TRUE_WRAPPER : FALSE_WRAPPER);
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_SLOT_HARDWIREDp) {
    if (setvalueP) {
      self->slot_hardwiredP = coerce_wrapped_boolean_to_boolean(((Boolean_Wrapper*)(value)));
    }
    else {
      value = (self->slot_hardwiredP ? TRUE_WRAPPER : FALSE_WRAPPER);
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_SLOT_CONTEXT_SENSITIVEp) {
    if (setvalueP) {
      self->slot_context_sensitiveP = coerce_wrapped_boolean_to_boolean(((Boolean_Wrapper*)(value)));
    }
    else {
      value = (self->slot_context_sensitiveP ? TRUE_WRAPPER : FALSE_WRAPPER);
    }
  }
  else {
    if (setvalueP) {
      set_dynamic_slot_value(self->dynamic_slots, slotname, value, NULL);
    }
    else {
      value = self->dynamic_slots->lookup(slotname);
    }
  }
  return (value);
}

Method_Slot* new_method_slot() {
  { Method_Slot* self = NULL;

    self = new Method_Slot();
    self->dynamic_slots = new_key_value_list();
    self->abstractP = FALSE;
    self->slot_direct_equivalent = NULL;
    self->slot_markedP = FALSE;
    self->slot_externalP = FALSE;
    self->slot_renamedP = FALSE;
    self->slot_publicP = TRUE;
    self->slot_slotref = NULL;
    self->slot_base_type = NULL;
    self->slot_owner = NULL;
    self->slot_name = NULL;
    self->method_functionP = FALSE;
    self->function_code = NULL;
    self->method_code = ((cpp_method_code)(NULL));
    self->method_stringified_source = NULL;
    self->method_return_type_specifiers = NULL;
    self->method_parameter_type_specifiers = NULL;
    self->method_parameter_names = NULL;
    self->method_setterP = FALSE;
    return (self);
  }
}

Surrogate* Method_Slot::primary_type() {
  { Method_Slot* self = this;

    return (SGT_HIERARCHY_STELLA_METHOD_SLOT);
  }
}

boolean Method_Slot::setterP() {
  { Method_Slot* self = this;

    return (self->method_setterP);
  }
}

List* Method_Slot::method_parameter_names_reader() {
  { Method_Slot* self = this;

    { List* answer = self->method_parameter_names;

      if (!((boolean)(answer))) {
        return (NIL_LIST);
      }
      else {
        return (answer);
      }
    }
  }
}

List* Method_Slot::method_parameter_type_specifiers_reader() {
  { Method_Slot* self = this;

    { List* answer = self->method_parameter_type_specifiers;

      if (!((boolean)(answer))) {
        return (NIL_LIST);
      }
      else {
        return (answer);
      }
    }
  }
}

List* Method_Slot::method_parameter_directions_reader() {
  { Method_Slot* self = this;

    { List* answer = ((List*)(dynamic_slot_value(self->dynamic_slots, SYM_HIERARCHY_STELLA_METHOD_PARAMETER_DIRECTIONS, NULL)));

      if (!((boolean)(answer))) {
        return (NIL_LIST);
      }
      else {
        return (answer);
      }
    }
  }
}

List* Method_Slot::method_return_type_specifiers_reader() {
  { Method_Slot* self = this;

    { List* answer = self->method_return_type_specifiers;

      if (!((boolean)(answer))) {
        return (NIL_LIST);
      }
      else {
        return (answer);
      }
    }
  }
}

List* Method_Slot::method_inlined_functions_reader() {
  { Method_Slot* self = this;

    { List* answer = ((List*)(dynamic_slot_value(self->dynamic_slots, SYM_HIERARCHY_STELLA_METHOD_INLINED_FUNCTIONS, NULL)));

      if (!((boolean)(answer))) {
        return (NIL_LIST);
      }
      else {
        return (answer);
      }
    }
  }
}

boolean Method_Slot::method_evaluate_argumentsP_reader() {
  { Method_Slot* self = this;

    return (((Boolean_Wrapper*)(dynamic_slot_value(self->dynamic_slots, SYM_HIERARCHY_STELLA_METHOD_EVALUATE_ARGUMENTSp, TRUE_WRAPPER)))->wrapper_value);
  }
}

Object* access_method_slot_slot_value(Method_Slot* self, Symbol* slotname, Object* value, boolean setvalueP) {
  if (slotname == SYM_HIERARCHY_STELLA_METHOD_SETTERp) {
    if (setvalueP) {
      self->method_setterP = coerce_wrapped_boolean_to_boolean(((Boolean_Wrapper*)(value)));
    }
    else {
      value = (self->method_setterP ? TRUE_WRAPPER : FALSE_WRAPPER);
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_METHOD_PARAMETER_NAMES) {
    if (setvalueP) {
      self->method_parameter_names = ((List*)(value));
    }
    else {
      value = self->method_parameter_names_reader();
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_METHOD_PARAMETER_TYPE_SPECIFIERS) {
    if (setvalueP) {
      self->method_parameter_type_specifiers = ((List*)(value));
    }
    else {
      value = self->method_parameter_type_specifiers_reader();
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_METHOD_PARAMETER_DIRECTIONS) {
    if (setvalueP) {
      set_dynamic_slot_value(self->dynamic_slots, SYM_HIERARCHY_STELLA_METHOD_PARAMETER_DIRECTIONS, ((List*)(value)), NULL);
    }
    else {
      value = self->method_parameter_directions_reader();
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_METHOD_RETURN_TYPE_SPECIFIERS) {
    if (setvalueP) {
      self->method_return_type_specifiers = ((List*)(value));
    }
    else {
      value = self->method_return_type_specifiers_reader();
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_METHOD_STRINGIFIED_SOURCE) {
    if (setvalueP) {
      self->method_stringified_source = ((String_Wrapper*)(value))->wrapper_value;
    }
    else {
      value = wrap_string(self->method_stringified_source);
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_METHOD_CODE) {
    if (setvalueP) {
      self->method_code = ((Method_Code_Wrapper*)(value))->wrapper_value;
    }
    else {
      value = wrap_method_code(self->method_code);
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_FUNCTION_CODE) {
    if (setvalueP) {
      self->function_code = ((Function_Code_Wrapper*)(value))->wrapper_value;
    }
    else {
      value = wrap_function_code(self->function_code);
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_METHOD_FUNCTIONp) {
    if (setvalueP) {
      self->method_functionP = coerce_wrapped_boolean_to_boolean(((Boolean_Wrapper*)(value)));
    }
    else {
      value = (self->method_functionP ? TRUE_WRAPPER : FALSE_WRAPPER);
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_METHOD_INLINED_FUNCTIONS) {
    if (setvalueP) {
      set_dynamic_slot_value(self->dynamic_slots, SYM_HIERARCHY_STELLA_METHOD_INLINED_FUNCTIONS, ((List*)(value)), NULL);
    }
    else {
      value = self->method_inlined_functions_reader();
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_METHOD_EVALUATE_ARGUMENTSp) {
    if (setvalueP) {
      set_dynamic_slot_value(self->dynamic_slots, SYM_HIERARCHY_STELLA_METHOD_EVALUATE_ARGUMENTSp, (coerce_wrapped_boolean_to_boolean(((Boolean_Wrapper*)(value))) ? TRUE_WRAPPER : FALSE_WRAPPER), FALSE_WRAPPER);
    }
    else {
      value = (self->method_evaluate_argumentsP_reader() ? TRUE_WRAPPER : FALSE_WRAPPER);
    }
  }
  else {
    if (slotname == SYM_HIERARCHY_STELLA_METHOD_DOCUMENTATION) {
      slotname = SYM_HIERARCHY_STELLA_DOCUMENTATION;
    }
    else if (slotname == SYM_HIERARCHY_STELLA_METHOD_AUXILIARYp) {
      slotname = SYM_HIERARCHY_STELLA_SLOT_AUXILIARYp;
    }
    else if (slotname == SYM_HIERARCHY_STELLA_METHOD_STORAGE_SLOT) {
      slotname = SYM_HIERARCHY_STELLA_STORAGE_SLOT;
    }
    else {
    }
    if (setvalueP) {
      set_dynamic_slot_value(self->dynamic_slots, slotname, value, NULL);
    }
    else {
      value = self->dynamic_slots->lookup(slotname);
    }
  }
  return (value);
}

void Method_Slot::print_object(ostream* stream) {
  { Method_Slot* self = this;

    if (self->method_functionP) {
      *(stream) << "|F|" << self->slot_name->symbol_name;
    }
    else {
      *(stream) << "|M|" << self->slot_owner->symbol_name << "." << self->slot_name->symbol_name;
    }
  }
}

void Compound_Type_Specifier::print_object(ostream* stream) {
  { Compound_Type_Specifier* self = this;

    *(stream) << "|TS|" << yield_type_spec_tree(self);
  }
}

Parametric_Type_Specifier* new_parametric_type_specifier() {
  { Parametric_Type_Specifier* self = NULL;

    self = new Parametric_Type_Specifier();
    self->specifier_dimensions = NULL;
    self->specifier_parameter_types = new_list();
    self->specifier_base_type = NULL;
    return (self);
  }
}

Surrogate* Parametric_Type_Specifier::primary_type() {
  { Parametric_Type_Specifier* self = this;

    return (SGT_HIERARCHY_STELLA_PARAMETRIC_TYPE_SPECIFIER);
  }
}

Object* access_parametric_type_specifier_slot_value(Parametric_Type_Specifier* self, Symbol* slotname, Object* value, boolean setvalueP) {
  if (slotname == SYM_HIERARCHY_STELLA_SPECIFIER_BASE_TYPE) {
    if (setvalueP) {
      self->specifier_base_type = ((Surrogate*)(value));
    }
    else {
      value = self->specifier_base_type;
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_SPECIFIER_PARAMETER_TYPES) {
    if (setvalueP) {
      self->specifier_parameter_types = ((List*)(value));
    }
    else {
      value = self->specifier_parameter_types;
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_SPECIFIER_DIMENSIONS) {
    if (setvalueP) {
      self->specifier_dimensions = ((List*)(value));
    }
    else {
      value = self->specifier_dimensions;
    }
  }
  else {
    { Output_String_Stream* stream_000 = new_output_string_stream();

      *(stream_000->native_stream) << "`" << slotname << "'" << " is not a valid case option";
      throw *new_stella_exception(stream_000->the_string_reader());
    }
  }
  return (value);
}

Anchored_Type_Specifier* new_anchored_type_specifier() {
  { Anchored_Type_Specifier* self = NULL;

    self = new Anchored_Type_Specifier();
    self->specifier_parameter_name = NULL;
    return (self);
  }
}

Surrogate* Anchored_Type_Specifier::primary_type() {
  { Anchored_Type_Specifier* self = this;

    return (SGT_HIERARCHY_STELLA_ANCHORED_TYPE_SPECIFIER);
  }
}

Object* access_anchored_type_specifier_slot_value(Anchored_Type_Specifier* self, Symbol* slotname, Object* value, boolean setvalueP) {
  if (slotname == SYM_HIERARCHY_STELLA_SPECIFIER_PARAMETER_NAME) {
    if (setvalueP) {
      self->specifier_parameter_name = ((Symbol*)(value));
    }
    else {
      value = self->specifier_parameter_name;
    }
  }
  else {
    { Output_String_Stream* stream_000 = new_output_string_stream();

      *(stream_000->native_stream) << "`" << slotname << "'" << " is not a valid case option";
      throw *new_stella_exception(stream_000->the_string_reader());
    }
  }
  return (value);
}

Table* new_table() {
  { Table* self = NULL;

    self = new Table();
    self->dynamic_slots = new_key_value_list();
    self->abstractP = FALSE;
    self->slot_direct_equivalent = NULL;
    self->slot_markedP = FALSE;
    self->slot_externalP = FALSE;
    self->slot_renamedP = FALSE;
    self->slot_publicP = TRUE;
    self->slot_slotref = NULL;
    self->slot_base_type = NULL;
    self->slot_owner = NULL;
    self->slot_name = NULL;
    self->tuple_domains = NULL;
    return (self);
  }
}

Surrogate* Table::primary_type() {
  { Table* self = this;

    return (SGT_HIERARCHY_STELLA_TABLE);
  }
}

Object* access_table_slot_value(Table* self, Symbol* slotname, Object* value, boolean setvalueP) {
  if (slotname == SYM_HIERARCHY_STELLA_TUPLE_DOMAINS) {
    if (setvalueP) {
      self->tuple_domains = ((List*)(value));
    }
    else {
      value = self->tuple_domains;
    }
  }
  else {
    if (setvalueP) {
      set_dynamic_slot_value(self->dynamic_slots, slotname, value, NULL);
    }
    else {
      value = self->dynamic_slots->lookup(slotname);
    }
  }
  return (value);
}

void Table::print_object(ostream* stream) {
  { Table* self = this;

    *(stream) << "|T|" << self->slot_name->symbol_name;
  }
}

Global_Variable* new_global_variable() {
  { Global_Variable* self = NULL;

    self = new Global_Variable();
    self->dynamic_slots = new_key_value_list();
    self->variable_stringified_source = NULL;
    self->documentation = NULL;
    self->variable_value_stack = NULL;
    self->variable_set_value_code = NULL;
    self->variable_get_value_code = NULL;
    self->variable_auxiliaryP = FALSE;
    self->variable_publicP = TRUE;
    self->variable_constantP = FALSE;
    self->variable_specialP = FALSE;
    self->variable_type = NULL;
    self->variable_name = NULL;
    return (self);
  }
}

Surrogate* Global_Variable::primary_type() {
  { Global_Variable* self = this;

    return (SGT_HIERARCHY_STELLA_GLOBAL_VARIABLE);
  }
}

char* Global_Variable::name() {
  { Global_Variable* self = this;

    return (self->variable_name->symbol_name);
  }
}

Surrogate* Global_Variable::type() {
  { Global_Variable* self = this;

    return (self->variable_type);
  }
}

Module* Global_Variable::home_module() {
  { Global_Variable* self = this;

    { Module* module = ((Module*)(dynamic_slot_value(self->dynamic_slots, SYM_HIERARCHY_STELLA_VARIABLE_HOME_MODULE, NULL)));

      if (((boolean)(module))) {
        return (module);
      }
      else {
        return (((Module*)(self->variable_name->home_context)));
      }
    }
  }
}

Object* access_global_variable_slot_value(Global_Variable* self, Symbol* slotname, Object* value, boolean setvalueP) {
  if (slotname == SYM_HIERARCHY_STELLA_DOCUMENTATION) {
    if (setvalueP) {
      self->documentation = ((String_Wrapper*)(value))->wrapper_value;
    }
    else {
      value = wrap_string(self->documentation);
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_VARIABLE_NAME) {
    if (setvalueP) {
      self->variable_name = ((Symbol*)(value));
    }
    else {
      value = self->variable_name;
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_VARIABLE_TYPE) {
    if (setvalueP) {
      self->variable_type = ((Surrogate*)(value));
    }
    else {
      value = self->variable_type;
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_VARIABLE_SPECIALp) {
    if (setvalueP) {
      self->variable_specialP = coerce_wrapped_boolean_to_boolean(((Boolean_Wrapper*)(value)));
    }
    else {
      value = (self->variable_specialP ? TRUE_WRAPPER : FALSE_WRAPPER);
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_VARIABLE_CONSTANTp) {
    if (setvalueP) {
      self->variable_constantP = coerce_wrapped_boolean_to_boolean(((Boolean_Wrapper*)(value)));
    }
    else {
      value = (self->variable_constantP ? TRUE_WRAPPER : FALSE_WRAPPER);
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_VARIABLE_PUBLICp) {
    if (setvalueP) {
      self->variable_publicP = coerce_wrapped_boolean_to_boolean(((Boolean_Wrapper*)(value)));
    }
    else {
      value = (self->variable_publicP ? TRUE_WRAPPER : FALSE_WRAPPER);
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_VARIABLE_AUXILIARYp) {
    if (setvalueP) {
      self->variable_auxiliaryP = coerce_wrapped_boolean_to_boolean(((Boolean_Wrapper*)(value)));
    }
    else {
      value = (self->variable_auxiliaryP ? TRUE_WRAPPER : FALSE_WRAPPER);
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_VARIABLE_GET_VALUE_CODE) {
    if (setvalueP) {
      self->variable_get_value_code = ((Function_Code_Wrapper*)(value))->wrapper_value;
    }
    else {
      value = wrap_function_code(self->variable_get_value_code);
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_VARIABLE_SET_VALUE_CODE) {
    if (setvalueP) {
      self->variable_set_value_code = ((Function_Code_Wrapper*)(value))->wrapper_value;
    }
    else {
      value = wrap_function_code(self->variable_set_value_code);
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_VARIABLE_VALUE_STACK) {
    if (setvalueP) {
      self->variable_value_stack = ((List*)(value));
    }
    else {
      value = self->variable_value_stack;
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_VARIABLE_DOCUMENTATION) {
    if (setvalueP) {
      self->documentation = ((String_Wrapper*)(value))->wrapper_value;
    }
    else {
      value = wrap_string(self->documentation);
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_VARIABLE_STRINGIFIED_SOURCE) {
    if (setvalueP) {
      self->variable_stringified_source = ((String_Wrapper*)(value))->wrapper_value;
    }
    else {
      value = wrap_string(self->variable_stringified_source);
    }
  }
  else {
    if (setvalueP) {
      set_dynamic_slot_value(self->dynamic_slots, slotname, value, NULL);
    }
    else {
      value = self->dynamic_slots->lookup(slotname);
    }
  }
  return (value);
}

Quoted_Expression* new_quoted_expression() {
  { Quoted_Expression* self = NULL;

    self = new Quoted_Expression();
    self->quotation_table_offset = NULL_INTEGER;
    return (self);
  }
}

Surrogate* Quoted_Expression::primary_type() {
  { Quoted_Expression* self = this;

    return (SGT_HIERARCHY_STELLA_QUOTED_EXPRESSION);
  }
}

Object* access_quoted_expression_slot_value(Quoted_Expression* self, Symbol* slotname, Object* value, boolean setvalueP) {
  if (slotname == SYM_HIERARCHY_STELLA_QUOTATION_TABLE_OFFSET) {
    if (setvalueP) {
      self->quotation_table_offset = ((Integer_Wrapper*)(value))->wrapper_value;
    }
    else {
      value = wrap_integer(self->quotation_table_offset);
    }
  }
  else {
    { Output_String_Stream* stream_000 = new_output_string_stream();

      *(stream_000->native_stream) << "`" << slotname << "'" << " is not a valid case option";
      throw *new_stella_exception(stream_000->the_string_reader());
    }
  }
  return (value);
}

Boolean_Wrapper* Thing::badP_reader() {
  { Thing* self = this;

    { Boolean_Wrapper* answer = ((Boolean_Wrapper*)(dynamic_slot_value(self->dynamic_slots, SYM_HIERARCHY_STELLA_BADp, NULL)));

      if (!((boolean)(answer))) {
        return ((FALSE ? TRUE_WRAPPER : FALSE_WRAPPER));
      }
      else {
        return (answer);
      }
    }
  }
}

Object* access_thing_slot_value(Thing* self, Symbol* slotname, Object* value, boolean setvalueP) {
  if (slotname == SYM_HIERARCHY_STELLA_SURROGATE_VALUE_INVERSE) {
    if (setvalueP) {
      self->surrogate_value_inverse = ((Surrogate*)(value));
    }
    else {
      value = self->surrogate_value_inverse;
    }
  }
  else {
    if (setvalueP) {
      set_dynamic_slot_value(self->dynamic_slots, slotname, value, NULL);
    }
    else {
      value = self->dynamic_slots->lookup(slotname);
    }
  }
  return (value);
}

boolean Thing::deletedP() {
  { Thing* self = this;

    { Boolean_Wrapper* deletedP = ((Boolean_Wrapper*)(dynamic_slot_value(self->dynamic_slots, SYM_HIERARCHY_STELLA_DELETED_OBJECTp, NULL)));

      if (((boolean)(deletedP))) {
        return (coerce_wrapped_boolean_to_boolean(deletedP));
      }
      else {
        return (FALSE);
      }
    }
  }
}

boolean Thing::deletedP_setter(boolean value) {
  { Thing* self = this;

    set_dynamic_slot_value(self->dynamic_slots, SYM_HIERARCHY_STELLA_DELETED_OBJECTp, (value ? TRUE_WRAPPER : FALSE_WRAPPER), NULL);
    return (value);
  }
}

void Thing::print_object(ostream* stream) {
  { Thing* self = this;

    if (((boolean)(self->surrogate_value_inverse))) {
      *(stream) << "|i|" << self->surrogate_value_inverse->symbol_name;
    }
    else {
      *(stream) << "|i|" << self->primary_type();
    }
  }
}

char* Context::context_name() {
  { Context* self = this;

    { Context* aux = self;

      { Surrogate* test_value_000 = safe_primary_type(aux);

        if (subtype_ofP(test_value_000, SGT_HIERARCHY_STELLA_MODULE)) {
          { Context* aux_000 = aux;
            Module* aux = ((Module*)(aux_000));

            return (aux->module_name);
          }
        }
        else if (subtype_ofP(test_value_000, SGT_HIERARCHY_STELLA_WORLD)) {
          { Context* aux_001 = aux;
            World* aux = ((World*)(aux_001));

            return (((String_Wrapper*)(dynamic_slot_value(aux->dynamic_slots, SYM_HIERARCHY_STELLA_WORLD_NAME, NULL_STRING_WRAPPER)))->wrapper_value);
          }
        }
        else {
          { Output_String_Stream* stream_000 = new_output_string_stream();

            *(stream_000->native_stream) << "`" << test_value_000 << "'" << " is not a valid case option";
            throw *new_stella_exception(stream_000->the_string_reader());
          }
        }
      }
    }
  }
}

Module* Context::home_module() {
  { Context* self = this;

    return (self->base_module);
  }
}

Object* access_context_slot_value(Context* self, Symbol* slotname, Object* value, boolean setvalueP) {
  if (slotname == SYM_HIERARCHY_STELLA_CHILD_CONTEXTS) {
    if (setvalueP) {
      self->child_contexts = ((List*)(value));
    }
    else {
      value = self->child_contexts;
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_BASE_MODULE) {
    if (setvalueP) {
      self->base_module = ((Module*)(value));
    }
    else {
      value = self->base_module;
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_ALL_SUPER_CONTEXTS) {
    if (setvalueP) {
      self->all_super_contexts = ((Cons*)(value));
    }
    else {
      value = self->all_super_contexts;
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_CONTEXT_NUMBER) {
    if (setvalueP) {
      self->context_number = ((Integer_Wrapper*)(value))->wrapper_value;
    }
    else {
      value = wrap_integer(self->context_number);
    }
  }
  else {
    if (setvalueP) {
      set_dynamic_slot_value(self->dynamic_slots, slotname, value, NULL);
    }
    else {
      value = self->dynamic_slots->lookup(slotname);
    }
  }
  return (value);
}

void Context::print_object(ostream* stream) {
  { Context* self = this;

    print_context(self, stream);
  }
}

Module* new_module() {
  { Module* self = NULL;

    self = new Module();
    self->dynamic_slots = new_key_value_list();
    self->surrogate_value_inverse = NULL;
    self->context_number = NULL_INTEGER;
    self->all_super_contexts = NULL;
    self->base_module = NULL;
    self->child_contexts = new_list();
    self->prototype_inference_cache = NULL;
    self->default_inference_cache = NULL;
    self->strict_inference_cache = NULL;
    self->surrogate_offset_table = NULL;
    self->symbol_offset_table = NULL;
    self->cardinal_module = NULL;
    self->stringified_options = NULL;
    self->module_stringified_source = NULL;
    self->module_name = NULL;
    self->module_full_name = NULL;
    self->case_sensitiveP = FALSE;
    self->requires = NULL;
    self->used_by = new_list();
    self->uses = new_list();
    self->nicknames = NULL;
    self->documentation = NULL;
    self->parent_modules = new_list();
    return (self);
  }
}

Surrogate* Module::primary_type() {
  { Module* self = this;

    return (SGT_HIERARCHY_STELLA_MODULE);
  }
}

char* Module::name() {
  { Module* self = this;

    return (self->module_name);
  }
}

Module* Module::parent_module() {
  { Module* self = this;

    return (((Module*)(((Context*)(self->parent_contexts()->pop())))));
  }
}

char* Module::lisp_package() {
  { Module* self = this;

    { Module* cursor = self;
      char* value = ((String_Wrapper*)(dynamic_slot_value(cursor->dynamic_slots, SYM_HIERARCHY_STELLA_MODULE_LISP_PACKAGE, NULL_STRING_WRAPPER)))->wrapper_value;

      for (;;) {
        if (value != NULL) {
          return (value);
        }
        cursor = ((Module*)(cursor->parent_modules->first()));
        if (((boolean)(cursor))) {
          value = ((String_Wrapper*)(dynamic_slot_value(cursor->dynamic_slots, SYM_HIERARCHY_STELLA_MODULE_LISP_PACKAGE, NULL_STRING_WRAPPER)))->wrapper_value;
        }
        else {
          return ("STELLA");
        }
      }
    }
  }
}

char* Module::cpp_package() {
  { Module* self = this;

    { Module* cursor = self;
      char* value = ((String_Wrapper*)(dynamic_slot_value(cursor->dynamic_slots, SYM_HIERARCHY_STELLA_MODULE_CPP_PACKAGE, NULL_STRING_WRAPPER)))->wrapper_value;

      for (;;) {
        if (value != NULL) {
          return (value);
        }
        cursor = ((Module*)(cursor->parent_modules->first()));
        if (((boolean)(cursor))) {
          value = ((String_Wrapper*)(dynamic_slot_value(cursor->dynamic_slots, SYM_HIERARCHY_STELLA_MODULE_CPP_PACKAGE, NULL_STRING_WRAPPER)))->wrapper_value;
        }
        else {
          return ("stella");
        }
      }
    }
  }
}

List* Module::shadowed_surrogates_reader() {
  { Module* self = this;

    { List* answer = ((List*)(dynamic_slot_value(self->dynamic_slots, SYM_HIERARCHY_STELLA_SHADOWED_SURROGATES, NULL)));

      if (!((boolean)(answer))) {
        return (NIL_LIST);
      }
      else {
        return (answer);
      }
    }
  }
}

char* Module::module_lisp_package_reader() {
  { Module* self = this;

    { char* answer = ((String_Wrapper*)(dynamic_slot_value(self->dynamic_slots, SYM_HIERARCHY_STELLA_MODULE_LISP_PACKAGE, NULL_STRING_WRAPPER)))->wrapper_value;

      if (answer == NULL) {
        return ("STELLA");
      }
      else {
        return (answer);
      }
    }
  }
}

char* Module::module_cpp_package_reader() {
  { Module* self = this;

    { char* answer = ((String_Wrapper*)(dynamic_slot_value(self->dynamic_slots, SYM_HIERARCHY_STELLA_MODULE_CPP_PACKAGE, NULL_STRING_WRAPPER)))->wrapper_value;

      if (answer == NULL) {
        return ("stella");
      }
      else {
        return (answer);
      }
    }
  }
}

char* Module::java_package_reader() {
  { Module* self = this;

    { char* answer = ((String_Wrapper*)(dynamic_slot_value(self->dynamic_slots, SYM_HIERARCHY_STELLA_JAVA_PACKAGE, NULL_STRING_WRAPPER)))->wrapper_value;

      if (answer == NULL) {
        return (NULL);
      }
      else {
        return (answer);
      }
    }
  }
}

boolean Module::clearableP_reader() {
  { Module* self = this;

    return (((Boolean_Wrapper*)(dynamic_slot_value(self->dynamic_slots, SYM_HIERARCHY_STELLA_CLEARABLEp, TRUE_WRAPPER)))->wrapper_value);
  }
}

Object* access_module_slot_value(Module* self, Symbol* slotname, Object* value, boolean setvalueP) {
  if (slotname == SYM_HIERARCHY_STELLA_PARENT_MODULES) {
    if (setvalueP) {
      self->parent_modules = ((List*)(value));
    }
    else {
      value = self->parent_modules;
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_DOCUMENTATION) {
    if (setvalueP) {
      self->documentation = ((String_Wrapper*)(value))->wrapper_value;
    }
    else {
      value = wrap_string(self->documentation);
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_NICKNAMES) {
    if (setvalueP) {
      self->nicknames = ((List*)(value));
    }
    else {
      value = self->nicknames;
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_USES) {
    if (setvalueP) {
      self->uses = ((List*)(value));
    }
    else {
      value = self->uses;
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_USED_BY) {
    if (setvalueP) {
      self->used_by = ((List*)(value));
    }
    else {
      value = self->used_by;
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_SHADOWED_SURROGATES) {
    if (setvalueP) {
      set_dynamic_slot_value(self->dynamic_slots, SYM_HIERARCHY_STELLA_SHADOWED_SURROGATES, ((List*)(value)), NULL);
    }
    else {
      value = self->shadowed_surrogates_reader();
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_REQUIRES) {
    if (setvalueP) {
      self->requires = ((List*)(value));
    }
    else {
      value = self->requires;
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_MODULE_LISP_PACKAGE) {
    if (setvalueP) {
      set_dynamic_slot_value(self->dynamic_slots, SYM_HIERARCHY_STELLA_MODULE_LISP_PACKAGE, wrap_string(((String_Wrapper*)(value))->wrapper_value), NULL_STRING_WRAPPER);
    }
    else {
      value = wrap_string(self->module_lisp_package_reader());
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_MODULE_CPP_PACKAGE) {
    if (setvalueP) {
      set_dynamic_slot_value(self->dynamic_slots, SYM_HIERARCHY_STELLA_MODULE_CPP_PACKAGE, wrap_string(((String_Wrapper*)(value))->wrapper_value), NULL_STRING_WRAPPER);
    }
    else {
      value = wrap_string(self->module_cpp_package_reader());
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_JAVA_PACKAGE) {
    if (setvalueP) {
      set_dynamic_slot_value(self->dynamic_slots, SYM_HIERARCHY_STELLA_JAVA_PACKAGE, wrap_string(((String_Wrapper*)(value))->wrapper_value), NULL_STRING_WRAPPER);
    }
    else {
      value = wrap_string(self->java_package_reader());
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_CASE_SENSITIVEp) {
    if (setvalueP) {
      self->case_sensitiveP = coerce_wrapped_boolean_to_boolean(((Boolean_Wrapper*)(value)));
    }
    else {
      value = (self->case_sensitiveP ? TRUE_WRAPPER : FALSE_WRAPPER);
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_CLEARABLEp) {
    if (setvalueP) {
      set_dynamic_slot_value(self->dynamic_slots, SYM_HIERARCHY_STELLA_CLEARABLEp, (coerce_wrapped_boolean_to_boolean(((Boolean_Wrapper*)(value))) ? TRUE_WRAPPER : FALSE_WRAPPER), FALSE_WRAPPER);
    }
    else {
      value = (self->clearableP_reader() ? TRUE_WRAPPER : FALSE_WRAPPER);
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_MODULE_FULL_NAME) {
    if (setvalueP) {
      self->module_full_name = ((String_Wrapper*)(value))->wrapper_value;
    }
    else {
      value = wrap_string(self->module_full_name);
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_MODULE_NAME) {
    if (setvalueP) {
      self->module_name = ((String_Wrapper*)(value))->wrapper_value;
    }
    else {
      value = wrap_string(self->module_name);
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_MODULE_STRINGIFIED_SOURCE) {
    if (setvalueP) {
      self->module_stringified_source = ((String_Wrapper*)(value))->wrapper_value;
    }
    else {
      value = wrap_string(self->module_stringified_source);
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_STRINGIFIED_OPTIONS) {
    if (setvalueP) {
      self->stringified_options = ((String_Wrapper*)(value))->wrapper_value;
    }
    else {
      value = wrap_string(self->stringified_options);
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_CARDINAL_MODULE) {
    if (setvalueP) {
      self->cardinal_module = ((Module*)(value));
    }
    else {
      value = self->cardinal_module;
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_SYMBOL_OFFSET_TABLE) {
    if (setvalueP) {
      self->symbol_offset_table = ((String_To_Integer_Hash_Table*)(value));
    }
    else {
      value = self->symbol_offset_table;
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_SURROGATE_OFFSET_TABLE) {
    if (setvalueP) {
      self->surrogate_offset_table = ((String_To_Integer_Hash_Table*)(value));
    }
    else {
      value = self->surrogate_offset_table;
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_STRICT_INFERENCE_CACHE) {
    if (setvalueP) {
      self->strict_inference_cache = ((World*)(value));
    }
    else {
      value = self->strict_inference_cache;
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_DEFAULT_INFERENCE_CACHE) {
    if (setvalueP) {
      self->default_inference_cache = ((World*)(value));
    }
    else {
      value = self->default_inference_cache;
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_PROTOTYPE_INFERENCE_CACHE) {
    if (setvalueP) {
      self->prototype_inference_cache = ((World*)(value));
    }
    else {
      value = self->prototype_inference_cache;
    }
  }
  else {
    if (setvalueP) {
      set_dynamic_slot_value(self->dynamic_slots, slotname, value, NULL);
    }
    else {
      value = self->dynamic_slots->lookup(slotname);
    }
  }
  return (value);
}

World* new_world() {
  { World* self = NULL;

    self = new World();
    self->dynamic_slots = new_key_value_list();
    self->surrogate_value_inverse = NULL;
    self->context_number = NULL_INTEGER;
    self->all_super_contexts = NULL;
    self->base_module = NULL;
    self->child_contexts = new_list();
    self->parent_context = NULL;
    return (self);
  }
}

Surrogate* World::primary_type() {
  { World* self = this;

    return (SGT_HIERARCHY_STELLA_WORLD);
  }
}

Object* access_world_slot_value(World* self, Symbol* slotname, Object* value, boolean setvalueP) {
  if (slotname == SYM_HIERARCHY_STELLA_PARENT_CONTEXT) {
    if (setvalueP) {
      self->parent_context = ((Context*)(value));
    }
    else {
      value = self->parent_context;
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_CHILD_CONTEXTS) {
    if (setvalueP) {
      self->child_contexts = ((List*)(value));
    }
    else {
      value = self->child_contexts;
    }
  }
  else {
    if (setvalueP) {
      set_dynamic_slot_value(self->dynamic_slots, slotname, value, NULL);
    }
    else {
      value = self->dynamic_slots->lookup(slotname);
    }
  }
  return (value);
}

Cs_Value* new_cs_value() {
  { Cs_Value* self = NULL;

    self = new Cs_Value();
    self->the_kv_list = NULL;
    return (self);
  }
}

Surrogate* Cs_Value::primary_type() {
  { Cs_Value* self = this;

    return (SGT_HIERARCHY_STELLA_CS_VALUE);
  }
}

Interval* new_interval() {
  { Interval* self = NULL;

    self = new Interval();
    self->upper_bound = NULL_INTEGER;
    self->lower_bound = NULL_INTEGER;
    return (self);
  }
}

Surrogate* Interval::primary_type() {
  { Interval* self = this;

    return (SGT_HIERARCHY_STELLA_INTERVAL);
  }
}

Object* access_interval_slot_value(Interval* self, Symbol* slotname, Object* value, boolean setvalueP) {
  if (slotname == SYM_HIERARCHY_STELLA_LOWER_BOUND) {
    if (setvalueP) {
      self->lower_bound = ((Integer_Wrapper*)(value))->wrapper_value;
    }
    else {
      value = wrap_integer(self->lower_bound);
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_UPPER_BOUND) {
    if (setvalueP) {
      self->upper_bound = ((Integer_Wrapper*)(value))->wrapper_value;
    }
    else {
      value = wrap_integer(self->upper_bound);
    }
  }
  else {
    { Output_String_Stream* stream_000 = new_output_string_stream();

      *(stream_000->native_stream) << "`" << slotname << "'" << " is not a valid case option";
      throw *new_stella_exception(stream_000->the_string_reader());
    }
  }
  return (value);
}

void Interval::print_object(ostream* stream) {
  { Interval* self = this;

    *(stream) << "[" << self->lower_bound << "," << self->upper_bound << "]";
  }
}

Taxonomy_Node* new_taxonomy_node() {
  { Taxonomy_Node* self = NULL;

    self = new Taxonomy_Node();
    self->marker = NULL;
    self->tree_children = NIL;
    self->total_ancestors = 1;
    self->children = NIL;
    self->parents = NIL;
    self->first_interval_upper_bound = -1;
    self->first_interval_lower_bound = -1;
    self->initial_interval = NULL;
    self->intervals = NIL;
    self->label = NULL_INTEGER;
    self->native_object = NULL;
    return (self);
  }
}

Surrogate* Taxonomy_Node::primary_type() {
  { Taxonomy_Node* self = this;

    return (SGT_HIERARCHY_STELLA_TAXONOMY_NODE);
  }
}

Object* access_taxonomy_node_slot_value(Taxonomy_Node* self, Symbol* slotname, Object* value, boolean setvalueP) {
  if (slotname == SYM_HIERARCHY_STELLA_NATIVE_OBJECT) {
    if (setvalueP) {
      self->native_object = value;
    }
    else {
      value = self->native_object;
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_LABEL) {
    if (setvalueP) {
      self->label = ((Integer_Wrapper*)(value))->wrapper_value;
    }
    else {
      value = wrap_integer(self->label);
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_INTERVALS) {
    if (setvalueP) {
      self->intervals = ((Cons*)(value));
    }
    else {
      value = self->intervals;
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_INITIAL_INTERVAL) {
    if (setvalueP) {
      self->initial_interval = ((Interval*)(value));
    }
    else {
      value = self->initial_interval;
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_FIRST_INTERVAL_LOWER_BOUND) {
    if (setvalueP) {
      self->first_interval_lower_bound = ((Integer_Wrapper*)(value))->wrapper_value;
    }
    else {
      value = wrap_integer(self->first_interval_lower_bound);
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_FIRST_INTERVAL_UPPER_BOUND) {
    if (setvalueP) {
      self->first_interval_upper_bound = ((Integer_Wrapper*)(value))->wrapper_value;
    }
    else {
      value = wrap_integer(self->first_interval_upper_bound);
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_PARENTS) {
    if (setvalueP) {
      self->parents = ((Cons*)(value));
    }
    else {
      value = self->parents;
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_CHILDREN) {
    if (setvalueP) {
      self->children = ((Cons*)(value));
    }
    else {
      value = self->children;
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_TOTAL_ANCESTORS) {
    if (setvalueP) {
      self->total_ancestors = ((Integer_Wrapper*)(value))->wrapper_value;
    }
    else {
      value = wrap_integer(self->total_ancestors);
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_TREE_CHILDREN) {
    if (setvalueP) {
      self->tree_children = ((Cons*)(value));
    }
    else {
      value = self->tree_children;
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_MARKER) {
    if (setvalueP) {
      self->marker = value;
    }
    else {
      value = self->marker;
    }
  }
  else {
    { Output_String_Stream* stream_000 = new_output_string_stream();

      *(stream_000->native_stream) << "`" << slotname << "'" << " is not a valid case option";
      throw *new_stella_exception(stream_000->the_string_reader());
    }
  }
  return (value);
}

void Taxonomy_Node::print_object(ostream* stream) {
  { Taxonomy_Node* self = this;

    *(stream) << "|TN|" << self->label << " " << self->intervals << " " << self->native_object;
  }
}

Taxonomy_Graph* new_taxonomy_graph() {
  { Taxonomy_Graph* self = NULL;

    self = new Taxonomy_Graph();
    self->removed_links = NULL;
    self->added_links = NULL;
    self->broken_links = NULL;
    self->roots = NULL;
    self->number_of_foreign_interval_nodes = NULL_INTEGER;
    self->number_of_nodes = NULL_INTEGER;
    self->largest_postorder_number = NULL_INTEGER;
    self->incremental_modeP = FALSE;
    self->renumber_ratio = 0.1;
    self->renumber_if_out_of_numbersP = FALSE;
    initialize_taxonomy_graph(self);
    return (self);
  }
}

Surrogate* Taxonomy_Graph::primary_type() {
  { Taxonomy_Graph* self = this;

    return (SGT_HIERARCHY_STELLA_TAXONOMY_GRAPH);
  }
}

Object* access_taxonomy_graph_slot_value(Taxonomy_Graph* self, Symbol* slotname, Object* value, boolean setvalueP) {
  if (slotname == SYM_HIERARCHY_STELLA_RENUMBER_IF_OUT_OF_NUMBERSp) {
    if (setvalueP) {
      self->renumber_if_out_of_numbersP = coerce_wrapped_boolean_to_boolean(((Boolean_Wrapper*)(value)));
    }
    else {
      value = (self->renumber_if_out_of_numbersP ? TRUE_WRAPPER : FALSE_WRAPPER);
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_RENUMBER_RATIO) {
    if (setvalueP) {
      self->renumber_ratio = ((Float_Wrapper*)(value))->wrapper_value;
    }
    else {
      value = wrap_float(self->renumber_ratio);
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_INCREMENTAL_MODEp) {
    if (setvalueP) {
      self->incremental_modeP = coerce_wrapped_boolean_to_boolean(((Boolean_Wrapper*)(value)));
    }
    else {
      value = (self->incremental_modeP ? TRUE_WRAPPER : FALSE_WRAPPER);
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_LARGEST_POSTORDER_NUMBER) {
    if (setvalueP) {
      self->largest_postorder_number = ((Integer_Wrapper*)(value))->wrapper_value;
    }
    else {
      value = wrap_integer(self->largest_postorder_number);
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_NUMBER_OF_NODES) {
    if (setvalueP) {
      self->number_of_nodes = ((Integer_Wrapper*)(value))->wrapper_value;
    }
    else {
      value = wrap_integer(self->number_of_nodes);
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_NUMBER_OF_FOREIGN_INTERVAL_NODES) {
    if (setvalueP) {
      self->number_of_foreign_interval_nodes = ((Integer_Wrapper*)(value))->wrapper_value;
    }
    else {
      value = wrap_integer(self->number_of_foreign_interval_nodes);
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_ROOTS) {
    if (setvalueP) {
      self->roots = ((List*)(value));
    }
    else {
      value = self->roots;
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_BROKEN_LINKS) {
    if (setvalueP) {
      self->broken_links = ((List*)(value));
    }
    else {
      value = self->broken_links;
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_ADDED_LINKS) {
    if (setvalueP) {
      self->added_links = ((List*)(value));
    }
    else {
      value = self->added_links;
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_REMOVED_LINKS) {
    if (setvalueP) {
      self->removed_links = ((List*)(value));
    }
    else {
      value = self->removed_links;
    }
  }
  else {
    { Output_String_Stream* stream_000 = new_output_string_stream();

      *(stream_000->native_stream) << "`" << slotname << "'" << " is not a valid case option";
      throw *new_stella_exception(stream_000->the_string_reader());
    }
  }
  return (value);
}

void Taxonomy_Graph::print_object(ostream* stream) {
  { Taxonomy_Graph* self = this;

    *(stream) << "|TG|" << self->roots;
  }
}

Stella_Exception* new_stella_exception(char* message) {
  { Stella_Exception* self = NULL;

    self = new Stella_Exception(message);
    return (self);
  }
}

Read_Exception* new_read_exception(char* message) {
  { Read_Exception* self = NULL;

    self = new Read_Exception(message);
    return (self);
  }
}

Unhandled_Exception* new_unhandled_exception(char* message) {
  { Unhandled_Exception* self = NULL;

    self = new Unhandled_Exception(message);
    return (self);
  }
}

Evaluation_Exception* new_evaluation_exception(char* message) {
  { Evaluation_Exception* self = NULL;

    self = new Evaluation_Exception(message);
    return (self);
  }
}

No_Such_Object_Exception* new_no_such_object_exception(char* message) {
  { No_Such_Object_Exception* self = NULL;

    self = new No_Such_Object_Exception(message);
    return (self);
  }
}

No_Such_Context_Exception* new_no_such_context_exception(char* message) {
  { No_Such_Context_Exception* self = NULL;

    self = new No_Such_Context_Exception(message);
    return (self);
  }
}

Integer_Wrapper* new_integer_wrapper(int wrapper_value) {
  { Integer_Wrapper* self = NULL;

    self = new Integer_Wrapper();
    self->wrapper_value = wrapper_value;
    return (self);
  }
}

Surrogate* Integer_Wrapper::primary_type() {
  { Integer_Wrapper* self = this;

    return (SGT_HIERARCHY_STELLA_INTEGER_WRAPPER);
  }
}

Object* access_integer_wrapper_slot_value(Integer_Wrapper* self, Symbol* slotname, Object* value, boolean setvalueP) {
  if (slotname == SYM_HIERARCHY_STELLA_WRAPPER_VALUE) {
    if (setvalueP) {
      self->wrapper_value = ((Integer_Wrapper*)(value))->wrapper_value;
    }
    else {
      value = wrap_integer(self->wrapper_value);
    }
  }
  else {
    { Output_String_Stream* stream_000 = new_output_string_stream();

      *(stream_000->native_stream) << "`" << slotname << "'" << " is not a valid case option";
      throw *new_stella_exception(stream_000->the_string_reader());
    }
  }
  return (value);
}

void Integer_Wrapper::print_object(ostream* stream) {
  { Integer_Wrapper* self = this;

    if (oPRINTREADABLYpo.get()) {
      *(stream) << self->wrapper_value;
    }
    else {
      *(stream) << "|L|" << self->wrapper_value;
    }
  }
}

Float_Wrapper* new_float_wrapper(double wrapper_value) {
  { Float_Wrapper* self = NULL;

    self = new Float_Wrapper();
    self->wrapper_value = wrapper_value;
    return (self);
  }
}

Surrogate* Float_Wrapper::primary_type() {
  { Float_Wrapper* self = this;

    return (SGT_HIERARCHY_STELLA_FLOAT_WRAPPER);
  }
}

Object* access_float_wrapper_slot_value(Float_Wrapper* self, Symbol* slotname, Object* value, boolean setvalueP) {
  if (slotname == SYM_HIERARCHY_STELLA_WRAPPER_VALUE) {
    if (setvalueP) {
      self->wrapper_value = ((Float_Wrapper*)(value))->wrapper_value;
    }
    else {
      value = wrap_float(self->wrapper_value);
    }
  }
  else {
    { Output_String_Stream* stream_000 = new_output_string_stream();

      *(stream_000->native_stream) << "`" << slotname << "'" << " is not a valid case option";
      throw *new_stella_exception(stream_000->the_string_reader());
    }
  }
  return (value);
}

void Float_Wrapper::print_object(ostream* stream) {
  { Float_Wrapper* self = this;

    if (oPRINTREADABLYpo.get()) {
      *(stream) << self->wrapper_value;
    }
    else {
      *(stream) << "|L|" << self->wrapper_value;
    }
  }
}

String_Wrapper* new_string_wrapper(char* wrapper_value) {
  { String_Wrapper* self = NULL;

    self = new String_Wrapper();
    self->wrapper_value = wrapper_value;
    return (self);
  }
}

Surrogate* String_Wrapper::primary_type() {
  { String_Wrapper* self = this;

    return (SGT_HIERARCHY_STELLA_STRING_WRAPPER);
  }
}

Object* access_string_wrapper_slot_value(String_Wrapper* self, Symbol* slotname, Object* value, boolean setvalueP) {
  if (slotname == SYM_HIERARCHY_STELLA_WRAPPER_VALUE) {
    if (setvalueP) {
      self->wrapper_value = ((String_Wrapper*)(value))->wrapper_value;
    }
    else {
      value = wrap_string(self->wrapper_value);
    }
  }
  else {
    { Output_String_Stream* stream_000 = new_output_string_stream();

      *(stream_000->native_stream) << "`" << slotname << "'" << " is not a valid case option";
      throw *new_stella_exception(stream_000->the_string_reader());
    }
  }
  return (value);
}

void String_Wrapper::print_object(ostream* stream) {
  { String_Wrapper* self = this;

    { char* value = self->wrapper_value;

      if (oPRINTREADABLYpo.get()) {
        print_string_readably(value, stream);
      }
      else {
        if (value != NULL) {
          *(stream) << "|L|" << "\"" << value << "\"";
        }
        else {
          *(stream) << "|L|" << "NULL-STRING";
        }
      }
    }
  }
}

Mutable_String_Wrapper* new_mutable_string_wrapper(char* wrapper_value) {
  { Mutable_String_Wrapper* self = NULL;

    self = new Mutable_String_Wrapper();
    self->wrapper_value = wrapper_value;
    return (self);
  }
}

Surrogate* Mutable_String_Wrapper::primary_type() {
  { Mutable_String_Wrapper* self = this;

    return (SGT_HIERARCHY_STELLA_MUTABLE_STRING_WRAPPER);
  }
}

void Mutable_String_Wrapper::print_object(ostream* stream) {
  { Mutable_String_Wrapper* self = this;

    { char* value = self->wrapper_value;

      if (oPRINTREADABLYpo.get()) {
        print_string_readably(value, stream);
      }
      else {
        if (value != NULL) {
          *(stream) << "|L|" << "\"" << value << "\"";
        }
        else {
          *(stream) << "|L|" << "NULL-MUTABLE-STRING";
        }
      }
    }
  }
}

Character_Wrapper* new_character_wrapper(char wrapper_value) {
  { Character_Wrapper* self = NULL;

    self = new Character_Wrapper();
    self->wrapper_value = wrapper_value;
    return (self);
  }
}

Surrogate* Character_Wrapper::primary_type() {
  { Character_Wrapper* self = this;

    return (SGT_HIERARCHY_STELLA_CHARACTER_WRAPPER);
  }
}

Object* access_character_wrapper_slot_value(Character_Wrapper* self, Symbol* slotname, Object* value, boolean setvalueP) {
  if (slotname == SYM_HIERARCHY_STELLA_WRAPPER_VALUE) {
    if (setvalueP) {
      self->wrapper_value = ((Character_Wrapper*)(value))->wrapper_value;
    }
    else {
      value = wrap_character(self->wrapper_value);
    }
  }
  else {
    { Output_String_Stream* stream_000 = new_output_string_stream();

      *(stream_000->native_stream) << "`" << slotname << "'" << " is not a valid case option";
      throw *new_stella_exception(stream_000->the_string_reader());
    }
  }
  return (value);
}

void Character_Wrapper::print_object(ostream* stream) {
  { Character_Wrapper* self = this;

    if (oPRINTREADABLYpo.get()) {
      print_character(self->wrapper_value, stream);
    }
    else {
      {
        *(stream) << "|L|";
        print_character(self->wrapper_value, stream);
      }
    }
  }
}

Boolean_Wrapper* new_boolean_wrapper(boolean wrapper_value) {
  { Boolean_Wrapper* self = NULL;

    self = new Boolean_Wrapper();
    self->wrapper_value = wrapper_value;
    return (self);
  }
}

Surrogate* Boolean_Wrapper::primary_type() {
  { Boolean_Wrapper* self = this;

    return (SGT_HIERARCHY_STELLA_BOOLEAN_WRAPPER);
  }
}

Object* access_boolean_wrapper_slot_value(Boolean_Wrapper* self, Symbol* slotname, Object* value, boolean setvalueP) {
  if (slotname == SYM_HIERARCHY_STELLA_WRAPPER_VALUE) {
    if (setvalueP) {
      self->wrapper_value = coerce_wrapped_boolean_to_boolean(((Boolean_Wrapper*)(value)));
    }
    else {
      value = (self->wrapper_value ? TRUE_WRAPPER : FALSE_WRAPPER);
    }
  }
  else {
    { Output_String_Stream* stream_000 = new_output_string_stream();

      *(stream_000->native_stream) << "`" << slotname << "'" << " is not a valid case option";
      throw *new_stella_exception(stream_000->the_string_reader());
    }
  }
  return (value);
}

void Boolean_Wrapper::print_object(ostream* stream) {
  { Boolean_Wrapper* self = this;

    *(stream) << ((oPRINTREADABLYpo.get() ? (char*)"" : (char*)"|L|")) << ((self->wrapper_value ? SYM_HIERARCHY_STELLA_TRUE : SYM_HIERARCHY_STELLA_FALSE));
  }
}

Function_Code_Wrapper* new_function_code_wrapper(cpp_function_code wrapper_value) {
  { Function_Code_Wrapper* self = NULL;

    self = new Function_Code_Wrapper();
    self->wrapper_value = wrapper_value;
    return (self);
  }
}

Surrogate* Function_Code_Wrapper::primary_type() {
  { Function_Code_Wrapper* self = this;

    return (SGT_HIERARCHY_STELLA_FUNCTION_CODE_WRAPPER);
  }
}

Object* access_function_code_wrapper_slot_value(Function_Code_Wrapper* self, Symbol* slotname, Object* value, boolean setvalueP) {
  if (slotname == SYM_HIERARCHY_STELLA_WRAPPER_VALUE) {
    if (setvalueP) {
      self->wrapper_value = ((Function_Code_Wrapper*)(value))->wrapper_value;
    }
    else {
      value = wrap_function_code(self->wrapper_value);
    }
  }
  else {
    { Output_String_Stream* stream_000 = new_output_string_stream();

      *(stream_000->native_stream) << "`" << slotname << "'" << " is not a valid case option";
      throw *new_stella_exception(stream_000->the_string_reader());
    }
  }
  return (value);
}

Method_Code_Wrapper* new_method_code_wrapper(cpp_method_code wrapper_value) {
  { Method_Code_Wrapper* self = NULL;

    self = new Method_Code_Wrapper();
    self->wrapper_value = wrapper_value;
    return (self);
  }
}

Surrogate* Method_Code_Wrapper::primary_type() {
  { Method_Code_Wrapper* self = this;

    return (SGT_HIERARCHY_STELLA_METHOD_CODE_WRAPPER);
  }
}

Object* access_method_code_wrapper_slot_value(Method_Code_Wrapper* self, Symbol* slotname, Object* value, boolean setvalueP) {
  if (slotname == SYM_HIERARCHY_STELLA_WRAPPER_VALUE) {
    if (setvalueP) {
      self->wrapper_value = ((Method_Code_Wrapper*)(value))->wrapper_value;
    }
    else {
      value = wrap_method_code(self->wrapper_value);
    }
  }
  else {
    { Output_String_Stream* stream_000 = new_output_string_stream();

      *(stream_000->native_stream) << "`" << slotname << "'" << " is not a valid case option";
      throw *new_stella_exception(stream_000->the_string_reader());
    }
  }
  return (value);
}

Verbatim_String_Wrapper* new_verbatim_string_wrapper(char* wrapper_value) {
  { Verbatim_String_Wrapper* self = NULL;

    self = new Verbatim_String_Wrapper();
    self->wrapper_value = wrapper_value;
    return (self);
  }
}

Surrogate* Verbatim_String_Wrapper::primary_type() {
  { Verbatim_String_Wrapper* self = this;

    return (SGT_HIERARCHY_STELLA_VERBATIM_STRING_WRAPPER);
  }
}

void Verbatim_String_Wrapper::print_object(ostream* stream) {
  { Verbatim_String_Wrapper* self = this;

    *(stream) << ((oPRINTREADABLYpo.get() ? (char*)"" : (char*)"|V|")) << self->wrapper_value;
  }
}

System_Definition* new_system_definition() {
  { System_Definition* self = NULL;

    self = new System_Definition();
    self->finalization_function = NULL;
    self->development_settings = NULL;
    self->production_settings = NULL;
    self->banner = NULL;
    self->binary_root_directory = NULL;
    self->native_root_directory = NULL;
    self->source_root_directory = NULL;
    self->cardinal_module = NULL;
    self->up_to_dateP = FALSE;
    self->loadedP = FALSE;
    self->required_systems = NULL;
    self->preprocessed_files = NULL;
    self->java_only_files = NULL;
    self->cpp_only_files = NULL;
    self->lisp_only_files = NULL;
    self->files = NULL;
    self->directory = NULL;
    self->name = NULL;
    return (self);
  }
}

Surrogate* System_Definition::primary_type() {
  { System_Definition* self = this;

    return (SGT_HIERARCHY_STELLA_SYSTEM_DEFINITION);
  }
}

Object* access_system_definition_slot_value(System_Definition* self, Symbol* slotname, Object* value, boolean setvalueP) {
  if (slotname == SYM_HIERARCHY_STELLA_NAME) {
    if (setvalueP) {
      self->name = ((String_Wrapper*)(value))->wrapper_value;
    }
    else {
      value = wrap_string(self->name);
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_DIRECTORY) {
    if (setvalueP) {
      self->directory = ((String_Wrapper*)(value))->wrapper_value;
    }
    else {
      value = wrap_string(self->directory);
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_FILES) {
    if (setvalueP) {
      self->files = ((Cons*)(value));
    }
    else {
      value = self->files;
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_LISP_ONLY_FILES) {
    if (setvalueP) {
      self->lisp_only_files = ((Cons*)(value));
    }
    else {
      value = self->lisp_only_files;
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_CPP_ONLY_FILES) {
    if (setvalueP) {
      self->cpp_only_files = ((Cons*)(value));
    }
    else {
      value = self->cpp_only_files;
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_JAVA_ONLY_FILES) {
    if (setvalueP) {
      self->java_only_files = ((Cons*)(value));
    }
    else {
      value = self->java_only_files;
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_PREPROCESSED_FILES) {
    if (setvalueP) {
      self->preprocessed_files = ((Cons*)(value));
    }
    else {
      value = self->preprocessed_files;
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_REQUIRED_SYSTEMS) {
    if (setvalueP) {
      self->required_systems = ((Cons*)(value));
    }
    else {
      value = self->required_systems;
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_LOADEDp) {
    if (setvalueP) {
      self->loadedP = coerce_wrapped_boolean_to_boolean(((Boolean_Wrapper*)(value)));
    }
    else {
      value = (self->loadedP ? TRUE_WRAPPER : FALSE_WRAPPER);
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_UP_TO_DATEp) {
    if (setvalueP) {
      self->up_to_dateP = coerce_wrapped_boolean_to_boolean(((Boolean_Wrapper*)(value)));
    }
    else {
      value = (self->up_to_dateP ? TRUE_WRAPPER : FALSE_WRAPPER);
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_CARDINAL_MODULE) {
    if (setvalueP) {
      self->cardinal_module = ((String_Wrapper*)(value))->wrapper_value;
    }
    else {
      value = wrap_string(self->cardinal_module);
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_SOURCE_ROOT_DIRECTORY) {
    if (setvalueP) {
      self->source_root_directory = ((String_Wrapper*)(value))->wrapper_value;
    }
    else {
      value = wrap_string(self->source_root_directory);
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_NATIVE_ROOT_DIRECTORY) {
    if (setvalueP) {
      self->native_root_directory = ((String_Wrapper*)(value))->wrapper_value;
    }
    else {
      value = wrap_string(self->native_root_directory);
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_BINARY_ROOT_DIRECTORY) {
    if (setvalueP) {
      self->binary_root_directory = ((String_Wrapper*)(value))->wrapper_value;
    }
    else {
      value = wrap_string(self->binary_root_directory);
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_BANNER) {
    if (setvalueP) {
      self->banner = ((String_Wrapper*)(value))->wrapper_value;
    }
    else {
      value = wrap_string(self->banner);
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_COPYRIGHT_HEADER) {
    if (setvalueP) {
      self->banner = ((String_Wrapper*)(value))->wrapper_value;
    }
    else {
      value = wrap_string(self->banner);
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_PRODUCTION_SETTINGS) {
    if (setvalueP) {
      self->production_settings = ((Cons*)(value));
    }
    else {
      value = self->production_settings;
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_DEVELOPMENT_SETTINGS) {
    if (setvalueP) {
      self->development_settings = ((Cons*)(value));
    }
    else {
      value = self->development_settings;
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_FINALIZATION_FUNCTION) {
    if (setvalueP) {
      self->finalization_function = ((Symbol*)(value));
    }
    else {
      value = self->finalization_function;
    }
  }
  else {
    { Output_String_Stream* stream_000 = new_output_string_stream();

      *(stream_000->native_stream) << "`" << slotname << "'" << " is not a valid case option";
      throw *new_stella_exception(stream_000->the_string_reader());
    }
  }
  return (value);
}

void System_Definition::print_object(ostream* stream) {
  { System_Definition* self = this;

    *(stream) << "|SYSTEM|" << self->name;
  }
}

Object* access_stream_slot_value(Stream* self, Symbol* slotname, Object* value, boolean setvalueP) {
  if (slotname == SYM_HIERARCHY_STELLA_STATE) {
    if (setvalueP) {
      self->state = ((Keyword*)(value));
    }
    else {
      value = self->state;
    }
  }
  else {
    { Output_String_Stream* stream_000 = new_output_string_stream();

      *(stream_000->native_stream) << "`" << slotname << "'" << " is not a valid case option";
      throw *new_stella_exception(stream_000->the_string_reader());
    }
  }
  return (value);
}

Output_Stream* new_output_stream() {
  { Output_Stream* self = NULL;

    self = new Output_Stream();
    self->state = NULL;
    self->native_stream = NULL;
    return (self);
  }
}

void Output_Stream::free() {
  { Output_Stream* self = this;

    if (terminate_output_streamP(self)) {
      unmake(self);
    }
  }
}

Surrogate* Output_Stream::primary_type() {
  { Output_Stream* self = this;

    return (SGT_HIERARCHY_STELLA_OUTPUT_STREAM);
  }
}

Input_Stream* new_input_stream() {
  { Input_Stream* self = NULL;

    self = new Input_Stream();
    self->state = NULL;
    self->buffering_scheme = KWD_HIERARCHY_LINE;
    self->tokenizer_state = NULL;
    self->echo_stream = NULL;
    self->native_stream = NULL;
    return (self);
  }
}

void Input_Stream::free() {
  { Input_Stream* self = this;

    if (terminate_input_streamP(self)) {
      unmake(self);
    }
  }
}

Surrogate* Input_Stream::primary_type() {
  { Input_Stream* self = this;

    return (SGT_HIERARCHY_STELLA_INPUT_STREAM);
  }
}

Object* access_input_stream_slot_value(Input_Stream* self, Symbol* slotname, Object* value, boolean setvalueP) {
  if (slotname == SYM_HIERARCHY_STELLA_ECHO_STREAM) {
    if (setvalueP) {
      self->echo_stream = ((Output_Stream*)(value));
    }
    else {
      value = self->echo_stream;
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_TOKENIZER_STATE) {
    if (setvalueP) {
      self->tokenizer_state = ((Tokenizer_Stream_State*)(value));
    }
    else {
      value = self->tokenizer_state;
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_BUFFERING_SCHEME) {
    if (setvalueP) {
      self->buffering_scheme = ((Keyword*)(value));
    }
    else {
      value = self->buffering_scheme;
    }
  }
  else {
    { Output_String_Stream* stream_000 = new_output_string_stream();

      *(stream_000->native_stream) << "`" << slotname << "'" << " is not a valid case option";
      throw *new_stella_exception(stream_000->the_string_reader());
    }
  }
  return (value);
}

Output_File_Stream* new_output_file_stream(char* filename) {
  { Output_File_Stream* self = NULL;

    self = new Output_File_Stream();
    self->filename = filename;
    self->state = NULL;
    self->native_stream = NULL;
    self->if_exists_action = KWD_HIERARCHY_SUPERSEDE;
    initialize_file_output_stream(self);
    return (self);
  }
}

void Output_File_Stream::free() {
  { Output_File_Stream* self = this;

    if (terminate_file_output_streamP(self)) {
      unmake(self);
    }
  }
}

Surrogate* Output_File_Stream::primary_type() {
  { Output_File_Stream* self = this;

    return (SGT_HIERARCHY_STELLA_OUTPUT_FILE_STREAM);
  }
}

Object* access_output_file_stream_slot_value(Output_File_Stream* self, Symbol* slotname, Object* value, boolean setvalueP) {
  if (slotname == SYM_HIERARCHY_STELLA_FILENAME) {
    if (setvalueP) {
      self->filename = ((String_Wrapper*)(value))->wrapper_value;
    }
    else {
      value = wrap_string(self->filename);
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_IF_EXISTS_ACTION) {
    if (setvalueP) {
      self->if_exists_action = ((Keyword*)(value));
    }
    else {
      value = self->if_exists_action;
    }
  }
  else {
    { Output_String_Stream* stream_000 = new_output_string_stream();

      *(stream_000->native_stream) << "`" << slotname << "'" << " is not a valid case option";
      throw *new_stella_exception(stream_000->the_string_reader());
    }
  }
  return (value);
}

void Output_File_Stream::print_object(ostream* stream) {
  { Output_File_Stream* self = this;

    *(stream) << "|FOS|'" << self->filename << "'";
  }
}

Input_File_Stream* new_input_file_stream(char* filename) {
  { Input_File_Stream* self = NULL;

    self = new Input_File_Stream();
    self->filename = filename;
    self->state = NULL;
    self->tokenizer_state = NULL;
    self->echo_stream = NULL;
    self->native_stream = NULL;
    self->buffering_scheme = KWD_HIERARCHY_BLOCK;
    self->if_not_exists_action = KWD_HIERARCHY_ERROR;
    initialize_file_input_stream(self);
    return (self);
  }
}

void Input_File_Stream::free() {
  { Input_File_Stream* self = this;

    if (terminate_file_input_streamP(self)) {
      unmake(self);
    }
  }
}

Surrogate* Input_File_Stream::primary_type() {
  { Input_File_Stream* self = this;

    return (SGT_HIERARCHY_STELLA_INPUT_FILE_STREAM);
  }
}

Object* access_input_file_stream_slot_value(Input_File_Stream* self, Symbol* slotname, Object* value, boolean setvalueP) {
  if (slotname == SYM_HIERARCHY_STELLA_FILENAME) {
    if (setvalueP) {
      self->filename = ((String_Wrapper*)(value))->wrapper_value;
    }
    else {
      value = wrap_string(self->filename);
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_IF_NOT_EXISTS_ACTION) {
    if (setvalueP) {
      self->if_not_exists_action = ((Keyword*)(value));
    }
    else {
      value = self->if_not_exists_action;
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_BUFFERING_SCHEME) {
    if (setvalueP) {
      self->buffering_scheme = ((Keyword*)(value));
    }
    else {
      value = self->buffering_scheme;
    }
  }
  else {
    { Output_String_Stream* stream_000 = new_output_string_stream();

      *(stream_000->native_stream) << "`" << slotname << "'" << " is not a valid case option";
      throw *new_stella_exception(stream_000->the_string_reader());
    }
  }
  return (value);
}

void Input_File_Stream::print_object(ostream* stream) {
  { Input_File_Stream* self = this;

    *(stream) << "|FIS|'" << self->filename << "'";
  }
}

Output_String_Stream* new_output_string_stream() {
  { Output_String_Stream* self = NULL;

    self = new Output_String_Stream();
    self->state = NULL;
    self->native_stream = NULL;
    self->the_string = NULL;
    initialize_string_output_stream(self);
    return (self);
  }
}

void Output_String_Stream::free() {
  { Output_String_Stream* self = this;

    if (terminate_output_streamP(self)) {
      unmake(self);
    }
  }
}

Surrogate* Output_String_Stream::primary_type() {
  { Output_String_Stream* self = this;

    return (SGT_HIERARCHY_STELLA_OUTPUT_STRING_STREAM);
  }
}

Object* access_output_string_stream_slot_value(Output_String_Stream* self, Symbol* slotname, Object* value, boolean setvalueP) {
  if (slotname == SYM_HIERARCHY_STELLA_THE_STRING) {
    if (setvalueP) {
      self->the_string = ((String_Wrapper*)(value))->wrapper_value;
    }
    else {
      value = wrap_string(self->the_string_reader());
    }
  }
  else {
    { Output_String_Stream* stream_000 = new_output_string_stream();

      *(stream_000->native_stream) << "`" << slotname << "'" << " is not a valid case option";
      throw *new_stella_exception(stream_000->the_string_reader());
    }
  }
  return (value);
}

Input_String_Stream* new_input_string_stream(char* the_string) {
  { Input_String_Stream* self = NULL;

    self = new Input_String_Stream();
    self->the_string = the_string;
    self->state = NULL;
    self->tokenizer_state = NULL;
    self->echo_stream = NULL;
    self->native_stream = NULL;
    self->buffering_scheme = KWD_HIERARCHY_BLOCK;
    initialize_string_input_stream(self);
    return (self);
  }
}

void Input_String_Stream::free() {
  { Input_String_Stream* self = this;

    if (terminate_input_streamP(self)) {
      unmake(self);
    }
  }
}

Surrogate* Input_String_Stream::primary_type() {
  { Input_String_Stream* self = this;

    return (SGT_HIERARCHY_STELLA_INPUT_STRING_STREAM);
  }
}

Object* access_input_string_stream_slot_value(Input_String_Stream* self, Symbol* slotname, Object* value, boolean setvalueP) {
  if (slotname == SYM_HIERARCHY_STELLA_THE_STRING) {
    if (setvalueP) {
      self->the_string = ((String_Wrapper*)(value))->wrapper_value;
    }
    else {
      value = wrap_string(self->the_string);
    }
  }
  else if (slotname == SYM_HIERARCHY_STELLA_BUFFERING_SCHEME) {
    if (setvalueP) {
      self->buffering_scheme = ((Keyword*)(value));
    }
    else {
      value = self->buffering_scheme;
    }
  }
  else {
    { Output_String_Stream* stream_000 = new_output_string_stream();

      *(stream_000->native_stream) << "`" << slotname << "'" << " is not a valid case option";
      throw *new_stella_exception(stream_000->the_string_reader());
    }
  }
  return (value);
}

void help_startup_hierarchy1() {
  {
    SYM_HIERARCHY_STELLA_TYPE_SPEC = ((Symbol*)(intern_rigid_symbol_wrt_module("TYPE-SPEC", NULL, 0)));
    SYM_HIERARCHY_STELLA_REFERENCE_COUNT = ((Symbol*)(intern_rigid_symbol_wrt_module("REFERENCE-COUNT", NULL, 0)));
    SYM_HIERARCHY_STELLA_HOME_CONTEXT = ((Symbol*)(intern_rigid_symbol_wrt_module("HOME-CONTEXT", NULL, 0)));
    SYM_HIERARCHY_STELLA_BADp = ((Symbol*)(intern_rigid_symbol_wrt_module("BAD?", NULL, 0)));
    SYM_HIERARCHY_STELLA_DELETED_OBJECTp = ((Symbol*)(intern_rigid_symbol_wrt_module("DELETED-OBJECT?", NULL, 0)));
    SYM_HIERARCHY_STELLA_FIRST_ITERATIONp = ((Symbol*)(intern_rigid_symbol_wrt_module("FIRST-ITERATION?", NULL, 0)));
    SYM_HIERARCHY_STELLA_OBJECT_ITERATOR = ((Symbol*)(intern_rigid_symbol_wrt_module("OBJECT-ITERATOR", NULL, 0)));
    SGT_HIERARCHY_STELLA_CONS_ITERATOR = ((Surrogate*)(intern_rigid_symbol_wrt_module("CONS-ITERATOR", NULL, 1)));
    SYM_HIERARCHY_STELLA_CONS_ITERATOR_CURSOR = ((Symbol*)(intern_rigid_symbol_wrt_module("CONS-ITERATOR-CURSOR", NULL, 0)));
    SGT_HIERARCHY_STELLA_LIST_ITERATOR = ((Surrogate*)(intern_rigid_symbol_wrt_module("LIST-ITERATOR", NULL, 1)));
    SYM_HIERARCHY_STELLA_LIST_ITERATOR_CURSOR = ((Symbol*)(intern_rigid_symbol_wrt_module("LIST-ITERATOR-CURSOR", NULL, 0)));
    SYM_HIERARCHY_STELLA_LIST_ITERATOR_COLLECTION = ((Symbol*)(intern_rigid_symbol_wrt_module("LIST-ITERATOR-COLLECTION", NULL, 0)));
    SGT_HIERARCHY_STELLA_DESTRUCTIVE_LIST_ITERATOR = ((Surrogate*)(intern_rigid_symbol_wrt_module("DESTRUCTIVE-LIST-ITERATOR", NULL, 1)));
    SYM_HIERARCHY_STELLA_THE_CONS_LIST = ((Symbol*)(intern_rigid_symbol_wrt_module("THE-CONS-LIST", NULL, 0)));
    SGT_HIERARCHY_STELLA_SET_ITERATOR = ((Surrogate*)(intern_rigid_symbol_wrt_module("SET-ITERATOR", NULL, 1)));
    SYM_HIERARCHY_STELLA_SET_ITERATOR_CURSOR = ((Symbol*)(intern_rigid_symbol_wrt_module("SET-ITERATOR-CURSOR", NULL, 0)));
    SYM_HIERARCHY_STELLA_SET_ITERATOR_COLLECTION = ((Symbol*)(intern_rigid_symbol_wrt_module("SET-ITERATOR-COLLECTION", NULL, 0)));
    SGT_HIERARCHY_STELLA_ALL_PURPOSE_ITERATOR = ((Surrogate*)(intern_rigid_symbol_wrt_module("ALL-PURPOSE-ITERATOR", NULL, 1)));
    SYM_HIERARCHY_STELLA_KEY = ((Symbol*)(intern_rigid_symbol_wrt_module("KEY", NULL, 0)));
    SYM_HIERARCHY_STELLA_ITERATOR_NEXT_CODE = ((Symbol*)(intern_rigid_symbol_wrt_module("ITERATOR-NEXT-CODE", NULL, 0)));
    SYM_HIERARCHY_STELLA_ITERATOR_FILTER_CODE = ((Symbol*)(intern_rigid_symbol_wrt_module("ITERATOR-FILTER-CODE", NULL, 0)));
    SYM_HIERARCHY_STELLA_ITERATOR_EMPTY_CODE = ((Symbol*)(intern_rigid_symbol_wrt_module("ITERATOR-EMPTY-CODE", NULL, 0)));
    SYM_HIERARCHY_STELLA_ITERATOR_NESTED_ITERATOR = ((Symbol*)(intern_rigid_symbol_wrt_module("ITERATOR-NESTED-ITERATOR", NULL, 0)));
    SYM_HIERARCHY_STELLA_ITERATOR_CONS_LIST = ((Symbol*)(intern_rigid_symbol_wrt_module("ITERATOR-CONS-LIST", NULL, 0)));
    SYM_HIERARCHY_STELLA_ITERATOR_OBJECT = ((Symbol*)(intern_rigid_symbol_wrt_module("ITERATOR-OBJECT", NULL, 0)));
    SYM_HIERARCHY_STELLA_ITERATOR_SECOND_OBJECT = ((Symbol*)(intern_rigid_symbol_wrt_module("ITERATOR-SECOND-OBJECT", NULL, 0)));
    SYM_HIERARCHY_STELLA_ITERATOR_INTEGER = ((Symbol*)(intern_rigid_symbol_wrt_module("ITERATOR-INTEGER", NULL, 0)));
    SYM_HIERARCHY_STELLA_ITERATOR_SECOND_INTEGER = ((Symbol*)(intern_rigid_symbol_wrt_module("ITERATOR-SECOND-INTEGER", NULL, 0)));
    SGT_HIERARCHY_STELLA_TYPES_TO_CLASSES_ITERATOR = ((Surrogate*)(intern_rigid_symbol_wrt_module("TYPES-TO-CLASSES-ITERATOR", NULL, 1)));
    SYM_HIERARCHY_STELLA_ITERATOR_CURSOR = ((Symbol*)(intern_rigid_symbol_wrt_module("ITERATOR-CURSOR", NULL, 0)));
    SYM_HIERARCHY_STELLA_OBJECT_DICTIONARY_ITERATOR = ((Symbol*)(intern_rigid_symbol_wrt_module("OBJECT-DICTIONARY-ITERATOR", NULL, 0)));
    SGT_HIERARCHY_STELLA_PROPERTY_LIST_ITERATOR = ((Surrogate*)(intern_rigid_symbol_wrt_module("PROPERTY-LIST-ITERATOR", NULL, 1)));
    SYM_HIERARCHY_STELLA_PLIST_ITERATOR_CURSOR = ((Symbol*)(intern_rigid_symbol_wrt_module("PLIST-ITERATOR-CURSOR", NULL, 0)));
    SYM_HIERARCHY_STELLA_PLIST_ITERATOR_COLLECTION = ((Symbol*)(intern_rigid_symbol_wrt_module("PLIST-ITERATOR-COLLECTION", NULL, 0)));
    SGT_HIERARCHY_STELLA_KV_LIST_ITERATOR = ((Surrogate*)(intern_rigid_symbol_wrt_module("KV-LIST-ITERATOR", NULL, 1)));
    SYM_HIERARCHY_STELLA_THE_KV_LIST = ((Symbol*)(intern_rigid_symbol_wrt_module("THE-KV-LIST", NULL, 0)));
    SYM_HIERARCHY_STELLA_KV_LIST_ITERATOR_CURSOR = ((Symbol*)(intern_rigid_symbol_wrt_module("KV-LIST-ITERATOR-CURSOR", NULL, 0)));
    SGT_HIERARCHY_STELLA_INTEGER_INTERVAL_ITERATOR = ((Surrogate*)(intern_rigid_symbol_wrt_module("INTEGER-INTERVAL-ITERATOR", NULL, 1)));
    SYM_HIERARCHY_STELLA_INTERVAL_CURSOR = ((Symbol*)(intern_rigid_symbol_wrt_module("INTERVAL-CURSOR", NULL, 0)));
    SYM_HIERARCHY_STELLA_LOWER_BOUND = ((Symbol*)(intern_rigid_symbol_wrt_module("LOWER-BOUND", NULL, 0)));
    SYM_HIERARCHY_STELLA_UPPER_BOUND = ((Symbol*)(intern_rigid_symbol_wrt_module("UPPER-BOUND", NULL, 0)));
    SGT_HIERARCHY_STELLA_REVERSE_INTEGER_INTERVAL_ITERATOR = ((Surrogate*)(intern_rigid_symbol_wrt_module("REVERSE-INTEGER-INTERVAL-ITERATOR", NULL, 1)));
    SGT_HIERARCHY_STELLA_STRING_ITERATOR = ((Surrogate*)(intern_rigid_symbol_wrt_module("STRING-ITERATOR", NULL, 1)));
    SYM_HIERARCHY_STELLA_THE_STRING = ((Symbol*)(intern_rigid_symbol_wrt_module("THE-STRING", NULL, 0)));
    SYM_HIERARCHY_STELLA_CURSOR = ((Symbol*)(intern_rigid_symbol_wrt_module("CURSOR", NULL, 0)));
    SYM_HIERARCHY_STELLA_END = ((Symbol*)(intern_rigid_symbol_wrt_module("END", NULL, 0)));
    SYM_HIERARCHY_STELLA_OBJECT_COLLECTION = ((Symbol*)(intern_rigid_symbol_wrt_module("OBJECT-COLLECTION", NULL, 0)));
    SYM_HIERARCHY_STELLA_OBJECT_SEQUENCE = ((Symbol*)(intern_rigid_symbol_wrt_module("OBJECT-SEQUENCE", NULL, 0)));
    SGT_HIERARCHY_STELLA_LIST = ((Surrogate*)(intern_rigid_symbol_wrt_module("LIST", NULL, 1)));
    SGT_HIERARCHY_STELLA_SET = ((Surrogate*)(intern_rigid_symbol_wrt_module("SET", NULL, 1)));
    SGT_HIERARCHY_STELLA_CLASS_EXTENSION = ((Surrogate*)(intern_rigid_symbol_wrt_module("CLASS-EXTENSION", NULL, 1)));
    SGT_HIERARCHY_STELLA_CONS = ((Surrogate*)(intern_rigid_symbol_wrt_module("CONS", NULL, 1)));
    SYM_HIERARCHY_STELLA_VALUE = ((Symbol*)(intern_rigid_symbol_wrt_module("VALUE", NULL, 0)));
    SYM_HIERARCHY_STELLA_REST = ((Symbol*)(intern_rigid_symbol_wrt_module("REST", NULL, 0)));
    SYM_HIERARCHY_STELLA_OBJECT_TO_OBJECT_DICTIONARY = ((Symbol*)(intern_rigid_symbol_wrt_module("OBJECT-TO-OBJECT-DICTIONARY", NULL, 0)));
    SGT_HIERARCHY_STELLA_PROPERTY_LIST = ((Surrogate*)(intern_rigid_symbol_wrt_module("PROPERTY-LIST", NULL, 1)));
    SYM_HIERARCHY_STELLA_THE_PLIST = ((Symbol*)(intern_rigid_symbol_wrt_module("THE-PLIST", NULL, 0)));
    SGT_HIERARCHY_STELLA_KV_CONS = ((Surrogate*)(intern_rigid_symbol_wrt_module("KV-CONS", NULL, 1)));
    SGT_HIERARCHY_STELLA_KEY_VALUE_LIST = ((Surrogate*)(intern_rigid_symbol_wrt_module("KEY-VALUE-LIST", NULL, 1)));
    SGT_HIERARCHY_STELLA_RECYCLE_LIST = ((Surrogate*)(intern_rigid_symbol_wrt_module("RECYCLE-LIST", NULL, 1)));
  }
}

void help_startup_hierarchy2() {
  {
    SYM_HIERARCHY_STELLA_RECYCLE_LIST_OF = ((Symbol*)(intern_rigid_symbol_wrt_module("RECYCLE-LIST-OF", NULL, 0)));
    SYM_HIERARCHY_STELLA_RECYCLED_ITEMS = ((Symbol*)(intern_rigid_symbol_wrt_module("RECYCLED-ITEMS", NULL, 0)));
    SYM_HIERARCHY_STELLA_LIST_OF_RECYCLED_ITEMS = ((Symbol*)(intern_rigid_symbol_wrt_module("LIST-OF-RECYCLED-ITEMS", NULL, 0)));
    SYM_HIERARCHY_STELLA_ALL_ITEMS = ((Symbol*)(intern_rigid_symbol_wrt_module("ALL-ITEMS", NULL, 0)));
    SYM_HIERARCHY_STELLA_UNUSED_ITEMS = ((Symbol*)(intern_rigid_symbol_wrt_module("UNUSED-ITEMS", NULL, 0)));
    SYM_HIERARCHY_STELLA_CURRENT_LENGTH = ((Symbol*)(intern_rigid_symbol_wrt_module("CURRENT-LENGTH", NULL, 0)));
    SYM_HIERARCHY_STELLA_ITEM_SIZE = ((Symbol*)(intern_rigid_symbol_wrt_module("ITEM-SIZE", NULL, 0)));
    SYM_HIERARCHY_STELLA_THE_STELLA_HASH_TABLE = ((Symbol*)(intern_rigid_symbol_wrt_module("THE-STELLA-HASH-TABLE", NULL, 0)));
    SGT_HIERARCHY_STELLA_HASH_TABLE = ((Surrogate*)(intern_rigid_symbol_wrt_module("HASH-TABLE", NULL, 1)));
    SYM_HIERARCHY_STELLA_OBJECT_TO_OBJECT_HASH_TABLE = ((Symbol*)(intern_rigid_symbol_wrt_module("OBJECT-TO-OBJECT-HASH-TABLE", NULL, 0)));
    SGT_HIERARCHY_STELLA_INTEGER_HASH_TABLE = ((Surrogate*)(intern_rigid_symbol_wrt_module("INTEGER-HASH-TABLE", NULL, 1)));
    SGT_HIERARCHY_STELLA_FLOAT_HASH_TABLE = ((Surrogate*)(intern_rigid_symbol_wrt_module("FLOAT-HASH-TABLE", NULL, 1)));
    SGT_HIERARCHY_STELLA_STRING_HASH_TABLE = ((Surrogate*)(intern_rigid_symbol_wrt_module("STRING-HASH-TABLE", NULL, 1)));
    SGT_HIERARCHY_STELLA_STRING_TO_INTEGER_HASH_TABLE = ((Surrogate*)(intern_rigid_symbol_wrt_module("STRING-TO-INTEGER-HASH-TABLE", NULL, 1)));
    SGT_HIERARCHY_STELLA_VECTOR = ((Surrogate*)(intern_rigid_symbol_wrt_module("VECTOR", NULL, 1)));
    SYM_HIERARCHY_STELLA_ARRAY_SIZE = ((Symbol*)(intern_rigid_symbol_wrt_module("ARRAY-SIZE", NULL, 0)));
    SGT_HIERARCHY_STELLA_EXTENSIBLE_VECTOR = ((Surrogate*)(intern_rigid_symbol_wrt_module("EXTENSIBLE-VECTOR", NULL, 1)));
    SGT_HIERARCHY_STELLA_EXTENSIBLE_SYMBOL_ARRAY = ((Surrogate*)(intern_rigid_symbol_wrt_module("EXTENSIBLE-SYMBOL-ARRAY", NULL, 1)));
    SYM_HIERARCHY_STELLA_TOP_SYMBOL_OFFSET = ((Symbol*)(intern_rigid_symbol_wrt_module("TOP-SYMBOL-OFFSET", NULL, 0)));
    SYM_HIERARCHY_STELLA_POTENTIAL_FREE_SYMBOL_OFFSET = ((Symbol*)(intern_rigid_symbol_wrt_module("POTENTIAL-FREE-SYMBOL-OFFSET", NULL, 0)));
    SYM_HIERARCHY_STELLA_RESIZE_INCREMENT = ((Symbol*)(intern_rigid_symbol_wrt_module("RESIZE-INCREMENT", NULL, 0)));
    SYM_HIERARCHY_STELLA_SEQUENCE_LENGTH = ((Symbol*)(intern_rigid_symbol_wrt_module("SEQUENCE-LENGTH", NULL, 0)));
    SYM_HIERARCHY_STELLA_VSEQ = ((Symbol*)(intern_rigid_symbol_wrt_module("VSEQ", NULL, 0)));
    SGT_HIERARCHY_STELLA_SHORT_VECTOR_SEQUENCE = ((Surrogate*)(intern_rigid_symbol_wrt_module("SHORT-VECTOR-SEQUENCE", NULL, 1)));
    SYM_HIERARCHY_STELLA_SVSEQ = ((Symbol*)(intern_rigid_symbol_wrt_module("SVSEQ", NULL, 0)));
    SGT_HIERARCHY_STELLA_BOOLEAN_VECTOR = ((Surrogate*)(intern_rigid_symbol_wrt_module("BOOLEAN-VECTOR", NULL, 1)));
    SGT_HIERARCHY_STELLA_INTEGER_VECTOR = ((Surrogate*)(intern_rigid_symbol_wrt_module("INTEGER-VECTOR", NULL, 1)));
    SGT_HIERARCHY_STELLA_ACTIVE_LIST = ((Surrogate*)(intern_rigid_symbol_wrt_module("ACTIVE-LIST", NULL, 1)));
    SGT_HIERARCHY_STELLA_ACTIVE_SET = ((Surrogate*)(intern_rigid_symbol_wrt_module("ACTIVE-SET", NULL, 1)));
    SGT_HIERARCHY_STELLA_DEMON = ((Surrogate*)(intern_rigid_symbol_wrt_module("DEMON", NULL, 1)));
    SYM_HIERARCHY_STELLA_DEMON_NAME = ((Symbol*)(intern_rigid_symbol_wrt_module("DEMON-NAME", NULL, 0)));
    SYM_HIERARCHY_STELLA_DEMON_ACTION = ((Symbol*)(intern_rigid_symbol_wrt_module("DEMON-ACTION", NULL, 0)));
    SYM_HIERARCHY_STELLA_DEMON_CLASS_REFS = ((Symbol*)(intern_rigid_symbol_wrt_module("DEMON-CLASS-REFS", NULL, 0)));
    SYM_HIERARCHY_STELLA_DEMON_SLOT_REFS = ((Symbol*)(intern_rigid_symbol_wrt_module("DEMON-SLOT-REFS", NULL, 0)));
    SYM_HIERARCHY_STELLA_DEMON_CODE = ((Symbol*)(intern_rigid_symbol_wrt_module("DEMON-CODE", NULL, 0)));
    SYM_HIERARCHY_STELLA_DEMON_METHOD = ((Symbol*)(intern_rigid_symbol_wrt_module("DEMON-METHOD", NULL, 0)));
    SYM_HIERARCHY_STELLA_DEMON_DOCUMENTATION = ((Symbol*)(intern_rigid_symbol_wrt_module("DEMON-DOCUMENTATION", NULL, 0)));
    SYM_HIERARCHY_STELLA_DEMON_GUARDp = ((Symbol*)(intern_rigid_symbol_wrt_module("DEMON-GUARD?", NULL, 0)));
    SYM_HIERARCHY_STELLA_DEMON_ALLp = ((Symbol*)(intern_rigid_symbol_wrt_module("DEMON-ALL?", NULL, 0)));
    SYM_HIERARCHY_STELLA_DEMON_INHERITp = ((Symbol*)(intern_rigid_symbol_wrt_module("DEMON-INHERIT?", NULL, 0)));
    SGT_HIERARCHY_STELLA_KEYWORD_KEY_VALUE_LIST = ((Surrogate*)(intern_rigid_symbol_wrt_module("KEYWORD-KEY-VALUE-LIST", NULL, 1)));
    SYM_HIERARCHY_STELLA_SYMBOL_NAME = ((Symbol*)(intern_rigid_symbol_wrt_module("SYMBOL-NAME", NULL, 0)));
    SYM_HIERARCHY_STELLA_SYMBOL_ID = ((Symbol*)(intern_rigid_symbol_wrt_module("SYMBOL-ID", NULL, 0)));
    SYM_HIERARCHY_STELLA_INTERNED_IN = ((Symbol*)(intern_rigid_symbol_wrt_module("INTERNED-IN", NULL, 0)));
    SGT_HIERARCHY_STELLA_SYMBOL = ((Surrogate*)(intern_rigid_symbol_wrt_module("SYMBOL", NULL, 1)));
    SYM_HIERARCHY_STELLA_SYMBOL_SLOT_OFFSET = ((Symbol*)(intern_rigid_symbol_wrt_module("SYMBOL-SLOT-OFFSET", NULL, 0)));
    SYM_HIERARCHY_STELLA_SYMBOL_VALUE_AND_PLIST = ((Symbol*)(intern_rigid_symbol_wrt_module("SYMBOL-VALUE-AND-PLIST", NULL, 0)));
    SGT_HIERARCHY_STELLA_SURROGATE = ((Surrogate*)(intern_rigid_symbol_wrt_module("SURROGATE", NULL, 1)));
    SYM_HIERARCHY_STELLA_SURROGATE_VALUE = ((Symbol*)(intern_rigid_symbol_wrt_module("SURROGATE-VALUE", NULL, 0)));
    SYM_HIERARCHY_STELLA_SURROGATE_NAME = ((Symbol*)(intern_rigid_symbol_wrt_module("SURROGATE-NAME", NULL, 0)));
    SYM_HIERARCHY_STELLA_TYPE_CLASS = ((Symbol*)(intern_rigid_symbol_wrt_module("TYPE-CLASS", NULL, 0)));
    SYM_HIERARCHY_STELLA_TYPE_NAME = ((Symbol*)(intern_rigid_symbol_wrt_module("TYPE-NAME", NULL, 0)));
    SYM_HIERARCHY_STELLA_SLOTREF_SLOT = ((Symbol*)(intern_rigid_symbol_wrt_module("SLOTREF-SLOT", NULL, 0)));
    SYM_HIERARCHY_STELLA_TYPE = ((Symbol*)(intern_rigid_symbol_wrt_module("TYPE", NULL, 0)));
    SYM_HIERARCHY_STELLA_SLOTREF = ((Symbol*)(intern_rigid_symbol_wrt_module("SLOTREF", NULL, 0)));
    SGT_HIERARCHY_STELLA_KEYWORD = ((Surrogate*)(intern_rigid_symbol_wrt_module("KEYWORD", NULL, 1)));
    SYM_HIERARCHY_STELLA_KEYWORD_NAME = ((Symbol*)(intern_rigid_symbol_wrt_module("KEYWORD-NAME", NULL, 0)));
    SGT_HIERARCHY_STELLA_TRANSIENT_SYMBOL = ((Surrogate*)(intern_rigid_symbol_wrt_module("TRANSIENT-SYMBOL", NULL, 1)));
    SYM_HIERARCHY_STELLA_PROJECTS_ONTO = ((Symbol*)(intern_rigid_symbol_wrt_module("PROJECTS-ONTO", NULL, 0)));
    SYM_HIERARCHY_STELLA_PROJECTED_FROM = ((Symbol*)(intern_rigid_symbol_wrt_module("PROJECTED-FROM", NULL, 0)));
  }
}

void help_startup_hierarchy3() {
  {
    SYM_HIERARCHY_STELLA_PROPERTIES = ((Symbol*)(intern_rigid_symbol_wrt_module("PROPERTIES", NULL, 0)));
    SYM_HIERARCHY_STELLA_ABSTRACTp = ((Symbol*)(intern_rigid_symbol_wrt_module("ABSTRACT?", NULL, 0)));
    SGT_HIERARCHY_STELLA_CLASS = ((Surrogate*)(intern_rigid_symbol_wrt_module("CLASS", NULL, 1)));
    SYM_HIERARCHY_STELLA_STORED_ACTIVEp = ((Symbol*)(intern_rigid_symbol_wrt_module("STORED-ACTIVE?", NULL, 0)));
    SGT_HIERARCHY_STELLA_ACTIVE_OBJECT = ((Surrogate*)(intern_rigid_symbol_wrt_module("ACTIVE-OBJECT", NULL, 1)));
    SYM_HIERARCHY_STELLA_CLASS_CREATOR = ((Symbol*)(intern_rigid_symbol_wrt_module("CLASS-CREATOR", NULL, 0)));
    SYM_HIERARCHY_STELLA_CLASS_INITIALIZER = ((Symbol*)(intern_rigid_symbol_wrt_module("CLASS-INITIALIZER", NULL, 0)));
    SYM_HIERARCHY_STELLA_CLASS_TERMINATOR = ((Symbol*)(intern_rigid_symbol_wrt_module("CLASS-TERMINATOR", NULL, 0)));
    SYM_HIERARCHY_STELLA_CLASS_DESTRUCTOR = ((Symbol*)(intern_rigid_symbol_wrt_module("CLASS-DESTRUCTOR", NULL, 0)));
    SYM_HIERARCHY_STELLA_CLASS_PARAMETERS = ((Symbol*)(intern_rigid_symbol_wrt_module("CLASS-PARAMETERS", NULL, 0)));
    SYM_HIERARCHY_STELLA_CLASS_INITIAL_VALUE = ((Symbol*)(intern_rigid_symbol_wrt_module("CLASS-INITIAL-VALUE", NULL, 0)));
    SYM_HIERARCHY_STELLA_CLASS_EXTENSION = ((Symbol*)(intern_rigid_symbol_wrt_module("CLASS-EXTENSION", NULL, 0)));
    SYM_HIERARCHY_STELLA_CLASS_CL_NATIVE_TYPE = ((Symbol*)(intern_rigid_symbol_wrt_module("CLASS-CL-NATIVE-TYPE", NULL, 0)));
    SYM_HIERARCHY_STELLA_CLASS_CPP_NATIVE_TYPE = ((Symbol*)(intern_rigid_symbol_wrt_module("CLASS-CPP-NATIVE-TYPE", NULL, 0)));
    SYM_HIERARCHY_STELLA_CLASS_JAVA_NATIVE_TYPE = ((Symbol*)(intern_rigid_symbol_wrt_module("CLASS-JAVA-NATIVE-TYPE", NULL, 0)));
    SYM_HIERARCHY_STELLA_CLASS_CL_STRUCT_SLOTS = ((Symbol*)(intern_rigid_symbol_wrt_module("CLASS-CL-STRUCT-SLOTS", NULL, 0)));
    SYM_HIERARCHY_STELLA_CLASS_RECYCLE_METHOD = ((Symbol*)(intern_rigid_symbol_wrt_module("CLASS-RECYCLE-METHOD", NULL, 0)));
    KWD_HIERARCHY_NONE = ((Keyword*)(intern_rigid_symbol_wrt_module("NONE", NULL, 2)));
    SYM_HIERARCHY_STELLA_CLASS_REQUIRED_SLOT_NAMES = ((Symbol*)(intern_rigid_symbol_wrt_module("CLASS-REQUIRED-SLOT-NAMES", NULL, 0)));
    SYM_HIERARCHY_STELLA_CLASS_GUARD_CONSTRUCTOR_DEMONS = ((Symbol*)(intern_rigid_symbol_wrt_module("CLASS-GUARD-CONSTRUCTOR-DEMONS", NULL, 0)));
    SYM_HIERARCHY_STELLA_CLASS_CONSTRUCTOR_DEMONS = ((Symbol*)(intern_rigid_symbol_wrt_module("CLASS-CONSTRUCTOR-DEMONS", NULL, 0)));
    SYM_HIERARCHY_STELLA_CLASS_GUARD_DESTRUCTOR_DEMONS = ((Symbol*)(intern_rigid_symbol_wrt_module("CLASS-GUARD-DESTRUCTOR-DEMONS", NULL, 0)));
    SYM_HIERARCHY_STELLA_CLASS_DESTRUCTOR_DEMONS = ((Symbol*)(intern_rigid_symbol_wrt_module("CLASS-DESTRUCTOR-DEMONS", NULL, 0)));
    SYM_HIERARCHY_STELLA_CLASS_KEY = ((Symbol*)(intern_rigid_symbol_wrt_module("CLASS-KEY", NULL, 0)));
    SYM_HIERARCHY_STELLA_CLASS_SYNONYMS = ((Symbol*)(intern_rigid_symbol_wrt_module("CLASS-SYNONYMS", NULL, 0)));
    SYM_HIERARCHY_STELLA_CL_STRUCTp = ((Symbol*)(intern_rigid_symbol_wrt_module("CL-STRUCT?", NULL, 0)));
    SYM_HIERARCHY_STELLA_MIXINp = ((Symbol*)(intern_rigid_symbol_wrt_module("MIXIN?", NULL, 0)));
    SYM_HIERARCHY_STELLA_CLASS_TYPE = ((Symbol*)(intern_rigid_symbol_wrt_module("CLASS-TYPE", NULL, 0)));
    SYM_HIERARCHY_STELLA_CLASS_ARITY = ((Symbol*)(intern_rigid_symbol_wrt_module("CLASS-ARITY", NULL, 0)));
    SYM_HIERARCHY_STELLA_CLASS_DIRECT_SUPERS = ((Symbol*)(intern_rigid_symbol_wrt_module("CLASS-DIRECT-SUPERS", NULL, 0)));
    SYM_HIERARCHY_STELLA_CLASS_DIRECT_SUBS = ((Symbol*)(intern_rigid_symbol_wrt_module("CLASS-DIRECT-SUBS", NULL, 0)));
    SYM_HIERARCHY_STELLA_CLASS_ALL_SUPER_CLASSES = ((Symbol*)(intern_rigid_symbol_wrt_module("CLASS-ALL-SUPER-CLASSES", NULL, 0)));
    SYM_HIERARCHY_STELLA_CLASS_ALL_SLOTS = ((Symbol*)(intern_rigid_symbol_wrt_module("CLASS-ALL-SLOTS", NULL, 0)));
    SYM_HIERARCHY_STELLA_CLASS_LOCAL_SLOTS = ((Symbol*)(intern_rigid_symbol_wrt_module("CLASS-LOCAL-SLOTS", NULL, 0)));
    SYM_HIERARCHY_STELLA_CLASS_SLOT_AND_METHOD_CACHE = ((Symbol*)(intern_rigid_symbol_wrt_module("CLASS-SLOT-AND-METHOD-CACHE", NULL, 0)));
    SYM_HIERARCHY_STELLA_CLASS_ABSTRACTp = ((Symbol*)(intern_rigid_symbol_wrt_module("CLASS-ABSTRACT?", NULL, 0)));
    SYM_HIERARCHY_STELLA_CLASS_MIXINp = ((Symbol*)(intern_rigid_symbol_wrt_module("CLASS-MIXIN?", NULL, 0)));
    SYM_HIERARCHY_STELLA_CLASS_COLLECTIONp = ((Symbol*)(intern_rigid_symbol_wrt_module("CLASS-COLLECTION?", NULL, 0)));
    SYM_HIERARCHY_STELLA_CLASS_CL_STRUCTp = ((Symbol*)(intern_rigid_symbol_wrt_module("CLASS-CL-STRUCT?", NULL, 0)));
    SYM_HIERARCHY_STELLA_CLASS_PUBLICp = ((Symbol*)(intern_rigid_symbol_wrt_module("CLASS-PUBLIC?", NULL, 0)));
    SYM_HIERARCHY_STELLA_CLASS_FINALIZEDp = ((Symbol*)(intern_rigid_symbol_wrt_module("CLASS-FINALIZED?", NULL, 0)));
    SYM_HIERARCHY_STELLA_CLASS_SLOTS_FINALIZEDp = ((Symbol*)(intern_rigid_symbol_wrt_module("CLASS-SLOTS-FINALIZED?", NULL, 0)));
    SYM_HIERARCHY_STELLA_CLASS_STRINGIFIED_SOURCE = ((Symbol*)(intern_rigid_symbol_wrt_module("CLASS-STRINGIFIED-SOURCE", NULL, 0)));
    SYM_HIERARCHY_STELLA_CLASS_CONSTRUCTOR_CODE = ((Symbol*)(intern_rigid_symbol_wrt_module("CLASS-CONSTRUCTOR-CODE", NULL, 0)));
    SYM_HIERARCHY_STELLA_CLASS_SLOT_ACCESSOR_CODE = ((Symbol*)(intern_rigid_symbol_wrt_module("CLASS-SLOT-ACCESSOR-CODE", NULL, 0)));
    SYM_HIERARCHY_STELLA_CLASS_MARKEDp = ((Symbol*)(intern_rigid_symbol_wrt_module("CLASS-MARKED?", NULL, 0)));
    SYM_HIERARCHY_STELLA_CLASS_TAXONOMY_NODE = ((Symbol*)(intern_rigid_symbol_wrt_module("CLASS-TAXONOMY-NODE", NULL, 0)));
    SYM_HIERARCHY_STELLA_CLASS_DOCUMENTATION = ((Symbol*)(intern_rigid_symbol_wrt_module("CLASS-DOCUMENTATION", NULL, 0)));
    SYM_HIERARCHY_STELLA_CLASS_PRINT_FORM = ((Symbol*)(intern_rigid_symbol_wrt_module("CLASS-PRINT-FORM", NULL, 0)));
    SYM_HIERARCHY_STELLA_DOCUMENTATION = ((Symbol*)(intern_rigid_symbol_wrt_module("DOCUMENTATION", NULL, 0)));
    SYM_HIERARCHY_STELLA_PRINT_FORM = ((Symbol*)(intern_rigid_symbol_wrt_module("PRINT-FORM", NULL, 0)));
    SYM_HIERARCHY_STELLA_SLOT_DIRECT_EQUIVALENT = ((Symbol*)(intern_rigid_symbol_wrt_module("SLOT-DIRECT-EQUIVALENT", NULL, 0)));
    SYM_HIERARCHY_STELLA_SLOT_TYPE_SPECIFIER = ((Symbol*)(intern_rigid_symbol_wrt_module("SLOT-TYPE-SPECIFIER", NULL, 0)));
    SYM_HIERARCHY_STELLA_SLOT_HOME_MODULE = ((Symbol*)(intern_rigid_symbol_wrt_module("SLOT-HOME-MODULE", NULL, 0)));
    SYM_HIERARCHY_STELLA_SLOT_RENAMES = ((Symbol*)(intern_rigid_symbol_wrt_module("SLOT-RENAMES", NULL, 0)));
    SGT_HIERARCHY_STELLA_COLLECTION = ((Surrogate*)(intern_rigid_symbol_wrt_module("COLLECTION", NULL, 1)));
    SYM_HIERARCHY_STELLA_SLOT_CLOSURE_ASSUMPTION = ((Symbol*)(intern_rigid_symbol_wrt_module("SLOT-CLOSURE-ASSUMPTION", NULL, 0)));
    SYM_HIERARCHY_STELLA_SLOT_DIRECT_SUPERS = ((Symbol*)(intern_rigid_symbol_wrt_module("SLOT-DIRECT-SUPERS", NULL, 0)));
    SYM_HIERARCHY_STELLA_SLOT_DIRECT_SUBS = ((Symbol*)(intern_rigid_symbol_wrt_module("SLOT-DIRECT-SUBS", NULL, 0)));
    SYM_HIERARCHY_STELLA_SLOT_NAME = ((Symbol*)(intern_rigid_symbol_wrt_module("SLOT-NAME", NULL, 0)));
  }
}

void help_startup_hierarchy4() {
  {
    SYM_HIERARCHY_STELLA_SLOT_OWNER = ((Symbol*)(intern_rigid_symbol_wrt_module("SLOT-OWNER", NULL, 0)));
    SYM_HIERARCHY_STELLA_SLOT_BASE_TYPE = ((Symbol*)(intern_rigid_symbol_wrt_module("SLOT-BASE-TYPE", NULL, 0)));
    SYM_HIERARCHY_STELLA_SLOT_SLOTREF = ((Symbol*)(intern_rigid_symbol_wrt_module("SLOT-SLOTREF", NULL, 0)));
    SYM_HIERARCHY_STELLA_SLOT_PUBLICp = ((Symbol*)(intern_rigid_symbol_wrt_module("SLOT-PUBLIC?", NULL, 0)));
    SYM_HIERARCHY_STELLA_SLOT_RENAMEDp = ((Symbol*)(intern_rigid_symbol_wrt_module("SLOT-RENAMED?", NULL, 0)));
    SYM_HIERARCHY_STELLA_SLOT_EXTERNALp = ((Symbol*)(intern_rigid_symbol_wrt_module("SLOT-EXTERNAL?", NULL, 0)));
    SYM_HIERARCHY_STELLA_SLOT_MARKEDp = ((Symbol*)(intern_rigid_symbol_wrt_module("SLOT-MARKED?", NULL, 0)));
    SYM_HIERARCHY_STELLA_SLOT_DOCUMENTATION = ((Symbol*)(intern_rigid_symbol_wrt_module("SLOT-DOCUMENTATION", NULL, 0)));
    SGT_HIERARCHY_STELLA_STORAGE_SLOT = ((Surrogate*)(intern_rigid_symbol_wrt_module("STORAGE-SLOT", NULL, 1)));
    SYM_HIERARCHY_STELLA_SLOT_ALLOCATION = ((Symbol*)(intern_rigid_symbol_wrt_module("SLOT-ALLOCATION", NULL, 0)));
    KWD_HIERARCHY_INSTANCE = ((Keyword*)(intern_rigid_symbol_wrt_module("INSTANCE", NULL, 2)));
    SYM_HIERARCHY_STELLA_SLOT_DEFAULT_EXPRESSION = ((Symbol*)(intern_rigid_symbol_wrt_module("SLOT-DEFAULT-EXPRESSION", NULL, 0)));
    SYM_HIERARCHY_STELLA_SLOT_READER = ((Symbol*)(intern_rigid_symbol_wrt_module("SLOT-READER", NULL, 0)));
    SYM_HIERARCHY_STELLA_SLOT_WRITER = ((Symbol*)(intern_rigid_symbol_wrt_module("SLOT-WRITER", NULL, 0)));
    SYM_HIERARCHY_STELLA_SLOT_GUARD_DEMONS = ((Symbol*)(intern_rigid_symbol_wrt_module("SLOT-GUARD-DEMONS", NULL, 0)));
    SYM_HIERARCHY_STELLA_SLOT_DEMONS = ((Symbol*)(intern_rigid_symbol_wrt_module("SLOT-DEMONS", NULL, 0)));
    SYM_HIERARCHY_STELLA_SLOT_REQUIREDp = ((Symbol*)(intern_rigid_symbol_wrt_module("SLOT-REQUIRED?", NULL, 0)));
    SYM_HIERARCHY_STELLA_SLOT_COMPONENTp = ((Symbol*)(intern_rigid_symbol_wrt_module("SLOT-COMPONENT?", NULL, 0)));
    SYM_HIERARCHY_STELLA_SLOT_READ_ONLYp = ((Symbol*)(intern_rigid_symbol_wrt_module("SLOT-READ-ONLY?", NULL, 0)));
    SYM_HIERARCHY_STELLA_SLOT_HARDWIREDp = ((Symbol*)(intern_rigid_symbol_wrt_module("SLOT-HARDWIRED?", NULL, 0)));
    SYM_HIERARCHY_STELLA_SLOT_CONTEXT_SENSITIVEp = ((Symbol*)(intern_rigid_symbol_wrt_module("SLOT-CONTEXT-SENSITIVE?", NULL, 0)));
    SGT_HIERARCHY_STELLA_METHOD_SLOT = ((Surrogate*)(intern_rigid_symbol_wrt_module("METHOD-SLOT", NULL, 1)));
    SYM_HIERARCHY_STELLA_METHOD_PARAMETER_DIRECTIONS = ((Symbol*)(intern_rigid_symbol_wrt_module("METHOD-PARAMETER-DIRECTIONS", NULL, 0)));
    SYM_HIERARCHY_STELLA_METHOD_INLINED_FUNCTIONS = ((Symbol*)(intern_rigid_symbol_wrt_module("METHOD-INLINED-FUNCTIONS", NULL, 0)));
    SYM_HIERARCHY_STELLA_METHOD_EVALUATE_ARGUMENTSp = ((Symbol*)(intern_rigid_symbol_wrt_module("METHOD-EVALUATE-ARGUMENTS?", NULL, 0)));
    SYM_HIERARCHY_STELLA_METHOD_SETTERp = ((Symbol*)(intern_rigid_symbol_wrt_module("METHOD-SETTER?", NULL, 0)));
    SYM_HIERARCHY_STELLA_METHOD_PARAMETER_NAMES = ((Symbol*)(intern_rigid_symbol_wrt_module("METHOD-PARAMETER-NAMES", NULL, 0)));
    SYM_HIERARCHY_STELLA_METHOD_PARAMETER_TYPE_SPECIFIERS = ((Symbol*)(intern_rigid_symbol_wrt_module("METHOD-PARAMETER-TYPE-SPECIFIERS", NULL, 0)));
    SYM_HIERARCHY_STELLA_METHOD_RETURN_TYPE_SPECIFIERS = ((Symbol*)(intern_rigid_symbol_wrt_module("METHOD-RETURN-TYPE-SPECIFIERS", NULL, 0)));
    SYM_HIERARCHY_STELLA_METHOD_STRINGIFIED_SOURCE = ((Symbol*)(intern_rigid_symbol_wrt_module("METHOD-STRINGIFIED-SOURCE", NULL, 0)));
    SYM_HIERARCHY_STELLA_METHOD_CODE = ((Symbol*)(intern_rigid_symbol_wrt_module("METHOD-CODE", NULL, 0)));
    SYM_HIERARCHY_STELLA_FUNCTION_CODE = ((Symbol*)(intern_rigid_symbol_wrt_module("FUNCTION-CODE", NULL, 0)));
    SYM_HIERARCHY_STELLA_METHOD_FUNCTIONp = ((Symbol*)(intern_rigid_symbol_wrt_module("METHOD-FUNCTION?", NULL, 0)));
    SYM_HIERARCHY_STELLA_METHOD_DOCUMENTATION = ((Symbol*)(intern_rigid_symbol_wrt_module("METHOD-DOCUMENTATION", NULL, 0)));
    SYM_HIERARCHY_STELLA_METHOD_AUXILIARYp = ((Symbol*)(intern_rigid_symbol_wrt_module("METHOD-AUXILIARY?", NULL, 0)));
    SYM_HIERARCHY_STELLA_METHOD_STORAGE_SLOT = ((Symbol*)(intern_rigid_symbol_wrt_module("METHOD-STORAGE-SLOT", NULL, 0)));
    SYM_HIERARCHY_STELLA_SLOT_AUXILIARYp = ((Symbol*)(intern_rigid_symbol_wrt_module("SLOT-AUXILIARY?", NULL, 0)));
    SYM_HIERARCHY_STELLA_STORAGE_SLOT = ((Symbol*)(intern_rigid_symbol_wrt_module("STORAGE-SLOT", NULL, 0)));
    SYM_HIERARCHY_STELLA_FUNCTION = ((Symbol*)(intern_rigid_symbol_wrt_module("FUNCTION", NULL, 0)));
    SGT_HIERARCHY_STELLA_PARAMETRIC_TYPE_SPECIFIER = ((Surrogate*)(intern_rigid_symbol_wrt_module("PARAMETRIC-TYPE-SPECIFIER", NULL, 1)));
    SYM_HIERARCHY_STELLA_SPECIFIER_BASE_TYPE = ((Symbol*)(intern_rigid_symbol_wrt_module("SPECIFIER-BASE-TYPE", NULL, 0)));
    SYM_HIERARCHY_STELLA_SPECIFIER_PARAMETER_TYPES = ((Symbol*)(intern_rigid_symbol_wrt_module("SPECIFIER-PARAMETER-TYPES", NULL, 0)));
    SYM_HIERARCHY_STELLA_SPECIFIER_DIMENSIONS = ((Symbol*)(intern_rigid_symbol_wrt_module("SPECIFIER-DIMENSIONS", NULL, 0)));
    SGT_HIERARCHY_STELLA_ANCHORED_TYPE_SPECIFIER = ((Surrogate*)(intern_rigid_symbol_wrt_module("ANCHORED-TYPE-SPECIFIER", NULL, 1)));
    SYM_HIERARCHY_STELLA_SPECIFIER_PARAMETER_NAME = ((Symbol*)(intern_rigid_symbol_wrt_module("SPECIFIER-PARAMETER-NAME", NULL, 0)));
    SGT_HIERARCHY_STELLA_TABLE = ((Surrogate*)(intern_rigid_symbol_wrt_module("TABLE", NULL, 1)));
    SYM_HIERARCHY_STELLA_TUPLE_DOMAINS = ((Symbol*)(intern_rigid_symbol_wrt_module("TUPLE-DOMAINS", NULL, 0)));
    SGT_HIERARCHY_STELLA_GLOBAL_VARIABLE = ((Surrogate*)(intern_rigid_symbol_wrt_module("GLOBAL-VARIABLE", NULL, 1)));
    SYM_HIERARCHY_STELLA_VARIABLE_HOME_MODULE = ((Symbol*)(intern_rigid_symbol_wrt_module("VARIABLE-HOME-MODULE", NULL, 0)));
    SYM_HIERARCHY_STELLA_VARIABLE_NAME = ((Symbol*)(intern_rigid_symbol_wrt_module("VARIABLE-NAME", NULL, 0)));
    SYM_HIERARCHY_STELLA_VARIABLE_TYPE = ((Symbol*)(intern_rigid_symbol_wrt_module("VARIABLE-TYPE", NULL, 0)));
    SYM_HIERARCHY_STELLA_VARIABLE_SPECIALp = ((Symbol*)(intern_rigid_symbol_wrt_module("VARIABLE-SPECIAL?", NULL, 0)));
    SYM_HIERARCHY_STELLA_VARIABLE_CONSTANTp = ((Symbol*)(intern_rigid_symbol_wrt_module("VARIABLE-CONSTANT?", NULL, 0)));
    SYM_HIERARCHY_STELLA_VARIABLE_PUBLICp = ((Symbol*)(intern_rigid_symbol_wrt_module("VARIABLE-PUBLIC?", NULL, 0)));
    SYM_HIERARCHY_STELLA_VARIABLE_AUXILIARYp = ((Symbol*)(intern_rigid_symbol_wrt_module("VARIABLE-AUXILIARY?", NULL, 0)));
    SYM_HIERARCHY_STELLA_VARIABLE_GET_VALUE_CODE = ((Symbol*)(intern_rigid_symbol_wrt_module("VARIABLE-GET-VALUE-CODE", NULL, 0)));
    SYM_HIERARCHY_STELLA_VARIABLE_SET_VALUE_CODE = ((Symbol*)(intern_rigid_symbol_wrt_module("VARIABLE-SET-VALUE-CODE", NULL, 0)));
    SYM_HIERARCHY_STELLA_VARIABLE_VALUE_STACK = ((Symbol*)(intern_rigid_symbol_wrt_module("VARIABLE-VALUE-STACK", NULL, 0)));
    SYM_HIERARCHY_STELLA_VARIABLE_DOCUMENTATION = ((Symbol*)(intern_rigid_symbol_wrt_module("VARIABLE-DOCUMENTATION", NULL, 0)));
    SYM_HIERARCHY_STELLA_VARIABLE_STRINGIFIED_SOURCE = ((Symbol*)(intern_rigid_symbol_wrt_module("VARIABLE-STRINGIFIED-SOURCE", NULL, 0)));
  }
}

void help_startup_hierarchy5() {
  {
    SGT_HIERARCHY_STELLA_QUOTED_EXPRESSION = ((Surrogate*)(intern_rigid_symbol_wrt_module("QUOTED-EXPRESSION", NULL, 1)));
    SYM_HIERARCHY_STELLA_QUOTATION_TABLE_OFFSET = ((Symbol*)(intern_rigid_symbol_wrt_module("QUOTATION-TABLE-OFFSET", NULL, 0)));
    SYM_HIERARCHY_STELLA_SURROGATE_VALUE_INVERSE = ((Symbol*)(intern_rigid_symbol_wrt_module("SURROGATE-VALUE-INVERSE", NULL, 0)));
    SGT_HIERARCHY_STELLA_MODULE = ((Surrogate*)(intern_rigid_symbol_wrt_module("MODULE", NULL, 1)));
    SGT_HIERARCHY_STELLA_WORLD = ((Surrogate*)(intern_rigid_symbol_wrt_module("WORLD", NULL, 1)));
    SYM_HIERARCHY_STELLA_WORLD_NAME = ((Symbol*)(intern_rigid_symbol_wrt_module("WORLD-NAME", NULL, 0)));
    SYM_HIERARCHY_STELLA_CHILD_CONTEXTS = ((Symbol*)(intern_rigid_symbol_wrt_module("CHILD-CONTEXTS", NULL, 0)));
    SYM_HIERARCHY_STELLA_BASE_MODULE = ((Symbol*)(intern_rigid_symbol_wrt_module("BASE-MODULE", NULL, 0)));
    SYM_HIERARCHY_STELLA_ALL_SUPER_CONTEXTS = ((Symbol*)(intern_rigid_symbol_wrt_module("ALL-SUPER-CONTEXTS", NULL, 0)));
    SYM_HIERARCHY_STELLA_CONTEXT_NUMBER = ((Symbol*)(intern_rigid_symbol_wrt_module("CONTEXT-NUMBER", NULL, 0)));
    SYM_HIERARCHY_STELLA_MODULE_LISP_PACKAGE = ((Symbol*)(intern_rigid_symbol_wrt_module("MODULE-LISP-PACKAGE", NULL, 0)));
    SYM_HIERARCHY_STELLA_MODULE_CPP_PACKAGE = ((Symbol*)(intern_rigid_symbol_wrt_module("MODULE-CPP-PACKAGE", NULL, 0)));
    SYM_HIERARCHY_STELLA_SHADOWED_SURROGATES = ((Symbol*)(intern_rigid_symbol_wrt_module("SHADOWED-SURROGATES", NULL, 0)));
    SYM_HIERARCHY_STELLA_JAVA_PACKAGE = ((Symbol*)(intern_rigid_symbol_wrt_module("JAVA-PACKAGE", NULL, 0)));
    SYM_HIERARCHY_STELLA_CLEARABLEp = ((Symbol*)(intern_rigid_symbol_wrt_module("CLEARABLE?", NULL, 0)));
    SYM_HIERARCHY_STELLA_PARENT_MODULES = ((Symbol*)(intern_rigid_symbol_wrt_module("PARENT-MODULES", NULL, 0)));
    SYM_HIERARCHY_STELLA_NICKNAMES = ((Symbol*)(intern_rigid_symbol_wrt_module("NICKNAMES", NULL, 0)));
    SYM_HIERARCHY_STELLA_USES = ((Symbol*)(intern_rigid_symbol_wrt_module("USES", NULL, 0)));
    SYM_HIERARCHY_STELLA_USED_BY = ((Symbol*)(intern_rigid_symbol_wrt_module("USED-BY", NULL, 0)));
    SYM_HIERARCHY_STELLA_REQUIRES = ((Symbol*)(intern_rigid_symbol_wrt_module("REQUIRES", NULL, 0)));
    SYM_HIERARCHY_STELLA_CASE_SENSITIVEp = ((Symbol*)(intern_rigid_symbol_wrt_module("CASE-SENSITIVE?", NULL, 0)));
    SYM_HIERARCHY_STELLA_MODULE_FULL_NAME = ((Symbol*)(intern_rigid_symbol_wrt_module("MODULE-FULL-NAME", NULL, 0)));
    SYM_HIERARCHY_STELLA_MODULE_NAME = ((Symbol*)(intern_rigid_symbol_wrt_module("MODULE-NAME", NULL, 0)));
    SYM_HIERARCHY_STELLA_MODULE_STRINGIFIED_SOURCE = ((Symbol*)(intern_rigid_symbol_wrt_module("MODULE-STRINGIFIED-SOURCE", NULL, 0)));
    SYM_HIERARCHY_STELLA_STRINGIFIED_OPTIONS = ((Symbol*)(intern_rigid_symbol_wrt_module("STRINGIFIED-OPTIONS", NULL, 0)));
    SYM_HIERARCHY_STELLA_CARDINAL_MODULE = ((Symbol*)(intern_rigid_symbol_wrt_module("CARDINAL-MODULE", NULL, 0)));
    SYM_HIERARCHY_STELLA_SYMBOL_OFFSET_TABLE = ((Symbol*)(intern_rigid_symbol_wrt_module("SYMBOL-OFFSET-TABLE", NULL, 0)));
    SYM_HIERARCHY_STELLA_SURROGATE_OFFSET_TABLE = ((Symbol*)(intern_rigid_symbol_wrt_module("SURROGATE-OFFSET-TABLE", NULL, 0)));
    SYM_HIERARCHY_STELLA_STRICT_INFERENCE_CACHE = ((Symbol*)(intern_rigid_symbol_wrt_module("STRICT-INFERENCE-CACHE", NULL, 0)));
    SYM_HIERARCHY_STELLA_DEFAULT_INFERENCE_CACHE = ((Symbol*)(intern_rigid_symbol_wrt_module("DEFAULT-INFERENCE-CACHE", NULL, 0)));
    SYM_HIERARCHY_STELLA_PROTOTYPE_INFERENCE_CACHE = ((Symbol*)(intern_rigid_symbol_wrt_module("PROTOTYPE-INFERENCE-CACHE", NULL, 0)));
    SYM_HIERARCHY_STELLA_PARENT_CONTEXT = ((Symbol*)(intern_rigid_symbol_wrt_module("PARENT-CONTEXT", NULL, 0)));
    SGT_HIERARCHY_STELLA_CS_VALUE = ((Surrogate*)(intern_rigid_symbol_wrt_module("CS-VALUE", NULL, 1)));
    SGT_HIERARCHY_STELLA_INTERVAL = ((Surrogate*)(intern_rigid_symbol_wrt_module("INTERVAL", NULL, 1)));
    SGT_HIERARCHY_STELLA_TAXONOMY_NODE = ((Surrogate*)(intern_rigid_symbol_wrt_module("TAXONOMY-NODE", NULL, 1)));
    SYM_HIERARCHY_STELLA_NATIVE_OBJECT = ((Symbol*)(intern_rigid_symbol_wrt_module("NATIVE-OBJECT", NULL, 0)));
    SYM_HIERARCHY_STELLA_LABEL = ((Symbol*)(intern_rigid_symbol_wrt_module("LABEL", NULL, 0)));
    SYM_HIERARCHY_STELLA_INTERVALS = ((Symbol*)(intern_rigid_symbol_wrt_module("INTERVALS", NULL, 0)));
    SYM_HIERARCHY_STELLA_INITIAL_INTERVAL = ((Symbol*)(intern_rigid_symbol_wrt_module("INITIAL-INTERVAL", NULL, 0)));
    SYM_HIERARCHY_STELLA_FIRST_INTERVAL_LOWER_BOUND = ((Symbol*)(intern_rigid_symbol_wrt_module("FIRST-INTERVAL-LOWER-BOUND", NULL, 0)));
    SYM_HIERARCHY_STELLA_FIRST_INTERVAL_UPPER_BOUND = ((Symbol*)(intern_rigid_symbol_wrt_module("FIRST-INTERVAL-UPPER-BOUND", NULL, 0)));
    SYM_HIERARCHY_STELLA_PARENTS = ((Symbol*)(intern_rigid_symbol_wrt_module("PARENTS", NULL, 0)));
    SYM_HIERARCHY_STELLA_CHILDREN = ((Symbol*)(intern_rigid_symbol_wrt_module("CHILDREN", NULL, 0)));
    SYM_HIERARCHY_STELLA_TOTAL_ANCESTORS = ((Symbol*)(intern_rigid_symbol_wrt_module("TOTAL-ANCESTORS", NULL, 0)));
    SYM_HIERARCHY_STELLA_TREE_CHILDREN = ((Symbol*)(intern_rigid_symbol_wrt_module("TREE-CHILDREN", NULL, 0)));
    SYM_HIERARCHY_STELLA_MARKER = ((Symbol*)(intern_rigid_symbol_wrt_module("MARKER", NULL, 0)));
    SGT_HIERARCHY_STELLA_TAXONOMY_GRAPH = ((Surrogate*)(intern_rigid_symbol_wrt_module("TAXONOMY-GRAPH", NULL, 1)));
    SYM_HIERARCHY_STELLA_RENUMBER_IF_OUT_OF_NUMBERSp = ((Symbol*)(intern_rigid_symbol_wrt_module("RENUMBER-IF-OUT-OF-NUMBERS?", NULL, 0)));
    SYM_HIERARCHY_STELLA_RENUMBER_RATIO = ((Symbol*)(intern_rigid_symbol_wrt_module("RENUMBER-RATIO", NULL, 0)));
    SYM_HIERARCHY_STELLA_INCREMENTAL_MODEp = ((Symbol*)(intern_rigid_symbol_wrt_module("INCREMENTAL-MODE?", NULL, 0)));
    SYM_HIERARCHY_STELLA_LARGEST_POSTORDER_NUMBER = ((Symbol*)(intern_rigid_symbol_wrt_module("LARGEST-POSTORDER-NUMBER", NULL, 0)));
    SYM_HIERARCHY_STELLA_NUMBER_OF_NODES = ((Symbol*)(intern_rigid_symbol_wrt_module("NUMBER-OF-NODES", NULL, 0)));
    SYM_HIERARCHY_STELLA_NUMBER_OF_FOREIGN_INTERVAL_NODES = ((Symbol*)(intern_rigid_symbol_wrt_module("NUMBER-OF-FOREIGN-INTERVAL-NODES", NULL, 0)));
    SYM_HIERARCHY_STELLA_ROOTS = ((Symbol*)(intern_rigid_symbol_wrt_module("ROOTS", NULL, 0)));
    SYM_HIERARCHY_STELLA_BROKEN_LINKS = ((Symbol*)(intern_rigid_symbol_wrt_module("BROKEN-LINKS", NULL, 0)));
    SYM_HIERARCHY_STELLA_ADDED_LINKS = ((Symbol*)(intern_rigid_symbol_wrt_module("ADDED-LINKS", NULL, 0)));
    SYM_HIERARCHY_STELLA_REMOVED_LINKS = ((Symbol*)(intern_rigid_symbol_wrt_module("REMOVED-LINKS", NULL, 0)));
    SYM_HIERARCHY_STELLA_FILE_NAME = ((Symbol*)(intern_rigid_symbol_wrt_module("FILE-NAME", NULL, 0)));
    SGT_HIERARCHY_STELLA_INTEGER_WRAPPER = ((Surrogate*)(intern_rigid_symbol_wrt_module("INTEGER-WRAPPER", NULL, 1)));
    SYM_HIERARCHY_STELLA_WRAPPER_VALUE = ((Symbol*)(intern_rigid_symbol_wrt_module("WRAPPER-VALUE", NULL, 0)));
  }
}

void help_startup_hierarchy6() {
  {
    SGT_HIERARCHY_STELLA_FLOAT_WRAPPER = ((Surrogate*)(intern_rigid_symbol_wrt_module("FLOAT-WRAPPER", NULL, 1)));
    SGT_HIERARCHY_STELLA_STRING_WRAPPER = ((Surrogate*)(intern_rigid_symbol_wrt_module("STRING-WRAPPER", NULL, 1)));
    SGT_HIERARCHY_STELLA_MUTABLE_STRING_WRAPPER = ((Surrogate*)(intern_rigid_symbol_wrt_module("MUTABLE-STRING-WRAPPER", NULL, 1)));
    SGT_HIERARCHY_STELLA_CHARACTER_WRAPPER = ((Surrogate*)(intern_rigid_symbol_wrt_module("CHARACTER-WRAPPER", NULL, 1)));
    SGT_HIERARCHY_STELLA_BOOLEAN_WRAPPER = ((Surrogate*)(intern_rigid_symbol_wrt_module("BOOLEAN-WRAPPER", NULL, 1)));
    SYM_HIERARCHY_STELLA_TRUE = ((Symbol*)(intern_rigid_symbol_wrt_module("TRUE", NULL, 0)));
    SYM_HIERARCHY_STELLA_FALSE = ((Symbol*)(intern_rigid_symbol_wrt_module("FALSE", NULL, 0)));
    SGT_HIERARCHY_STELLA_FUNCTION_CODE_WRAPPER = ((Surrogate*)(intern_rigid_symbol_wrt_module("FUNCTION-CODE-WRAPPER", NULL, 1)));
    SGT_HIERARCHY_STELLA_METHOD_CODE_WRAPPER = ((Surrogate*)(intern_rigid_symbol_wrt_module("METHOD-CODE-WRAPPER", NULL, 1)));
    SGT_HIERARCHY_STELLA_VERBATIM_STRING_WRAPPER = ((Surrogate*)(intern_rigid_symbol_wrt_module("VERBATIM-STRING-WRAPPER", NULL, 1)));
    SYM_HIERARCHY_STELLA_THREE_VALUED_BOOLEAN = ((Symbol*)(intern_rigid_symbol_wrt_module("THREE-VALUED-BOOLEAN", NULL, 0)));
    SGT_HIERARCHY_STELLA_SYSTEM_DEFINITION = ((Surrogate*)(intern_rigid_symbol_wrt_module("SYSTEM-DEFINITION", NULL, 1)));
    SYM_HIERARCHY_STELLA_NAME = ((Symbol*)(intern_rigid_symbol_wrt_module("NAME", NULL, 0)));
    SYM_HIERARCHY_STELLA_DIRECTORY = ((Symbol*)(intern_rigid_symbol_wrt_module("DIRECTORY", NULL, 0)));
    SYM_HIERARCHY_STELLA_FILES = ((Symbol*)(intern_rigid_symbol_wrt_module("FILES", NULL, 0)));
    SYM_HIERARCHY_STELLA_LISP_ONLY_FILES = ((Symbol*)(intern_rigid_symbol_wrt_module("LISP-ONLY-FILES", NULL, 0)));
    SYM_HIERARCHY_STELLA_CPP_ONLY_FILES = ((Symbol*)(intern_rigid_symbol_wrt_module("CPP-ONLY-FILES", NULL, 0)));
    SYM_HIERARCHY_STELLA_JAVA_ONLY_FILES = ((Symbol*)(intern_rigid_symbol_wrt_module("JAVA-ONLY-FILES", NULL, 0)));
    SYM_HIERARCHY_STELLA_PREPROCESSED_FILES = ((Symbol*)(intern_rigid_symbol_wrt_module("PREPROCESSED-FILES", NULL, 0)));
    SYM_HIERARCHY_STELLA_REQUIRED_SYSTEMS = ((Symbol*)(intern_rigid_symbol_wrt_module("REQUIRED-SYSTEMS", NULL, 0)));
    SYM_HIERARCHY_STELLA_LOADEDp = ((Symbol*)(intern_rigid_symbol_wrt_module("LOADED?", NULL, 0)));
    SYM_HIERARCHY_STELLA_UP_TO_DATEp = ((Symbol*)(intern_rigid_symbol_wrt_module("UP-TO-DATE?", NULL, 0)));
    SYM_HIERARCHY_STELLA_SOURCE_ROOT_DIRECTORY = ((Symbol*)(intern_rigid_symbol_wrt_module("SOURCE-ROOT-DIRECTORY", NULL, 0)));
    SYM_HIERARCHY_STELLA_NATIVE_ROOT_DIRECTORY = ((Symbol*)(intern_rigid_symbol_wrt_module("NATIVE-ROOT-DIRECTORY", NULL, 0)));
    SYM_HIERARCHY_STELLA_BINARY_ROOT_DIRECTORY = ((Symbol*)(intern_rigid_symbol_wrt_module("BINARY-ROOT-DIRECTORY", NULL, 0)));
    SYM_HIERARCHY_STELLA_BANNER = ((Symbol*)(intern_rigid_symbol_wrt_module("BANNER", NULL, 0)));
    SYM_HIERARCHY_STELLA_COPYRIGHT_HEADER = ((Symbol*)(intern_rigid_symbol_wrt_module("COPYRIGHT-HEADER", NULL, 0)));
    SYM_HIERARCHY_STELLA_PRODUCTION_SETTINGS = ((Symbol*)(intern_rigid_symbol_wrt_module("PRODUCTION-SETTINGS", NULL, 0)));
    SYM_HIERARCHY_STELLA_DEVELOPMENT_SETTINGS = ((Symbol*)(intern_rigid_symbol_wrt_module("DEVELOPMENT-SETTINGS", NULL, 0)));
    SYM_HIERARCHY_STELLA_FINALIZATION_FUNCTION = ((Symbol*)(intern_rigid_symbol_wrt_module("FINALIZATION-FUNCTION", NULL, 0)));
    SYM_HIERARCHY_STELLA_STATE = ((Symbol*)(intern_rigid_symbol_wrt_module("STATE", NULL, 0)));
    SGT_HIERARCHY_STELLA_OUTPUT_STREAM = ((Surrogate*)(intern_rigid_symbol_wrt_module("OUTPUT-STREAM", NULL, 1)));
    KWD_HIERARCHY_LINE = ((Keyword*)(intern_rigid_symbol_wrt_module("LINE", NULL, 2)));
    SGT_HIERARCHY_STELLA_INPUT_STREAM = ((Surrogate*)(intern_rigid_symbol_wrt_module("INPUT-STREAM", NULL, 1)));
    SYM_HIERARCHY_STELLA_ECHO_STREAM = ((Symbol*)(intern_rigid_symbol_wrt_module("ECHO-STREAM", NULL, 0)));
    SYM_HIERARCHY_STELLA_TOKENIZER_STATE = ((Symbol*)(intern_rigid_symbol_wrt_module("TOKENIZER-STATE", NULL, 0)));
    SYM_HIERARCHY_STELLA_BUFFERING_SCHEME = ((Symbol*)(intern_rigid_symbol_wrt_module("BUFFERING-SCHEME", NULL, 0)));
    KWD_HIERARCHY_SUPERSEDE = ((Keyword*)(intern_rigid_symbol_wrt_module("SUPERSEDE", NULL, 2)));
    SGT_HIERARCHY_STELLA_OUTPUT_FILE_STREAM = ((Surrogate*)(intern_rigid_symbol_wrt_module("OUTPUT-FILE-STREAM", NULL, 1)));
    SYM_HIERARCHY_STELLA_FILENAME = ((Symbol*)(intern_rigid_symbol_wrt_module("FILENAME", NULL, 0)));
    SYM_HIERARCHY_STELLA_IF_EXISTS_ACTION = ((Symbol*)(intern_rigid_symbol_wrt_module("IF-EXISTS-ACTION", NULL, 0)));
    SYM_HIERARCHY_STELLA_FILE_OUTPUT_STREAM = ((Symbol*)(intern_rigid_symbol_wrt_module("FILE-OUTPUT-STREAM", NULL, 0)));
    KWD_HIERARCHY_BLOCK = ((Keyword*)(intern_rigid_symbol_wrt_module("BLOCK", NULL, 2)));
    KWD_HIERARCHY_ERROR = ((Keyword*)(intern_rigid_symbol_wrt_module("ERROR", NULL, 2)));
    SGT_HIERARCHY_STELLA_INPUT_FILE_STREAM = ((Surrogate*)(intern_rigid_symbol_wrt_module("INPUT-FILE-STREAM", NULL, 1)));
    SYM_HIERARCHY_STELLA_IF_NOT_EXISTS_ACTION = ((Symbol*)(intern_rigid_symbol_wrt_module("IF-NOT-EXISTS-ACTION", NULL, 0)));
    SYM_HIERARCHY_STELLA_FILE_INPUT_STREAM = ((Symbol*)(intern_rigid_symbol_wrt_module("FILE-INPUT-STREAM", NULL, 0)));
    SGT_HIERARCHY_STELLA_OUTPUT_STRING_STREAM = ((Surrogate*)(intern_rigid_symbol_wrt_module("OUTPUT-STRING-STREAM", NULL, 1)));
    SYM_HIERARCHY_STELLA_STRING_OUTPUT_STREAM = ((Symbol*)(intern_rigid_symbol_wrt_module("STRING-OUTPUT-STREAM", NULL, 0)));
    SGT_HIERARCHY_STELLA_INPUT_STRING_STREAM = ((Surrogate*)(intern_rigid_symbol_wrt_module("INPUT-STRING-STREAM", NULL, 1)));
    SYM_HIERARCHY_STELLA_STRING_INPUT_STREAM = ((Symbol*)(intern_rigid_symbol_wrt_module("STRING-INPUT-STREAM", NULL, 0)));
    SYM_HIERARCHY_STELLA_STARTUP_HIERARCHY = ((Symbol*)(intern_rigid_symbol_wrt_module("STARTUP-HIERARCHY", NULL, 0)));
    SYM_HIERARCHY_STELLA_METHOD_STARTUP_CLASSNAME = ((Symbol*)(intern_rigid_symbol_wrt_module("METHOD-STARTUP-CLASSNAME", NULL, 0)));
  }
}

void help_startup_hierarchy7() {
  {
    define_class_from_stringified_source("OBJECT", "(DEFCLASS OBJECT () :DOCUMENTATION \"A reflective object that can participate in dynamically-typed\ncontexts.  Either a STANDARD-OBJECT or a WRAPPER or a GENERIC-OBJECT.\nAlso, either an OBJECT-WITH-INHERITED-TYPE or an OBJECT-WITH-OWN-TYPE.\" :PUBLIC-METHODS ((PRIMARY-TYPE ((SELF OBJECT)) :TYPE TYPE :DOCUMENTATION \"Returns the primary type of 'self'.\nGets defined automatically for every non-abstract subclass of OBJECT.\" (RETURN NULL)) (HOME-MODULE ((SELF OBJECT)) :TYPE MODULE :DOCUMENTATION \"Return the home module of 'self'.\" (RETURN NULL)) (DELETED? ((SELF OBJECT)) :TYPE BOOLEAN :DOCUMENTATION \"Default 'deleted?' method which always returns FALSE.  Objects\nthat inherit DYNAMIC-SLOTS-MIXIN also inherit the dynamically-allocated slot\n'deleted-object?' which is read/writable with specializations of this method.\" (RETURN FALSE))) :PRINT-FORM (PRINT-NATIVE-STREAM STREAM \"|i|\" (PRIMARY-TYPE SELF)) :METHODS ((INCREMENT-REFERENCE-COUNT ((SELF OBJECT)) NULL) (DECREMENT-REFERENCE-COUNT ((SELF OBJECT)) NULL)) :ABSTRACT? TRUE)");
    define_class_from_stringified_source("SECOND-CLASS-OBJECT", "(DEFCLASS SECOND-CLASS-OBJECT () :DOCUMENTATION \"A native reference-type object pretending to be second class,\ni.e., it can't/won't participate in dynamically-typed contexts such as (real)\nmethod dispatch.  Since it is a reference-type object, it is always passed via\na pointer reference similar to a real object.\" :ABSTRACT? TRUE)");
    define_class_from_stringified_source("NON-OBJECT", "(DEFCLASS NON-OBJECT () :DOCUMENTATION \"A data structure that cannot be a dispatch argument to\na (real) method, because its type is not accessible at run-time.\" :ABSTRACT? TRUE)");
    define_class_from_stringified_source("STANDARD-OBJECT", "(DEFCLASS STANDARD-OBJECT (OBJECT) :DOCUMENTATION \"Object that used to find its type stored in a :class slot.\nBy reimplementing 'primary-type' as a method, the justification for this\nclass went away, but we'll keep it for now to keep things working.\" :ABSTRACT? TRUE :SYNONYMS (TYPE-SPEC))");
    { Class* renamed_Class = define_class_from_stringified_source("TRANSIENT-MIXIN", "(DEFCLASS TRANSIENT-MIXIN () :DOCUMENTATION \"Transient objects are periodically swept back into a pool\nof free objects.  A transient object should never be pointed at by a\npermanent object.\" :MIXIN? TRUE :PUBLIC-SLOTS ((TRANSIENT? :TYPE BOOLEAN :INITIALLY TRUE :HARDWIRED? TRUE)))");

      oHARDWIRED_TRANSIENTp_ON_TRANSIENT_MIXINo = TRUE;
    }
    { Class* renamed_Class = define_class_from_stringified_source("REFERENCE-COUNT-OBJECT", "(DEFCLASS REFERENCE-COUNT-OBJECT (OBJECT) :DOCUMENTATION \"A reference count objects contains a slot 'reference-count' \nthat is incremented whenever the object is assigned to a slot, and decremented\nwhenever it is deassigned from a slot.\" :ABSTRACT? TRUE :PUBLIC-SLOTS ((REFERENCE-COUNT :TYPE INTEGER :INITIALLY 0)) :METHODS ((INCREMENT-REFERENCE-COUNT ((SELF REFERENCE-COUNT-OBJECT)) (SETF (REFERENCE-COUNT SELF) (+ (REFERENCE-COUNT SELF) 1))) (DECREMENT-REFERENCE-COUNT ((SELF REFERENCE-COUNT-OBJECT)) (SETF (REFERENCE-COUNT SELF) (- (REFERENCE-COUNT SELF) 1)))))");

      renamed_Class->class_slot_accessor_code = ((cpp_function_code)(&access_reference_count_object_slot_value));
    }
    define_class_from_stringified_source("DYNAMIC-SLOTS-MIXIN", "(DEFCLASS DYNAMIC-SLOTS-MIXIN () :DOCUMENTATION \"Enables objects to store slot values in non-preallocated storage.\" :MIXIN? TRUE :PUBLIC-SLOTS ((DYNAMIC-SLOTS :TYPE KEY-VALUE-LIST :ALLOCATION :EMBEDDED :PUBLIC? TRUE) (BAD? :TYPE BOOLEAN-WRAPPER :ALLOCATION :DYNAMIC :DEFAULT FALSE :DOCUMENTATION \"Indicates that an object is in need of repair.\") (DELETED-OBJECT? :TYPE BOOLEAN-WRAPPER :ALLOCATION :DYNAMIC :READER DELETED? :WRITER DELETED?-SETTER :DOCUMENTATION \"Indicates that an object has been \ndestroyed/deleted.  A deleted object is visible only if something broke\nbefore it was fully excised from the network).\")) :PUBLIC-METHODS ((DELETED? ((SELF DYNAMIC-SLOTS-MIXIN)) :TYPE BOOLEAN (LET ((DELETED? (SLOT-VALUE SELF DELETED-OBJECT?))) (IF (DEFINED? DELETED?) (RETURN DELETED?) (RETURN FALSE)))) (DELETED?-SETTER ((SELF DYNAMIC-SLOTS-MIXIN) (VALUE BOOLEAN)) :TYPE BOOLEAN (SETF (SLOT-VALUE SELF DELETED-OBJECT?) VALUE) (RETURN VALUE))))");
    { Class* renamed_Class = define_class_from_stringified_source("CONTEXT-SENSITIVE-OBJECT", "(DEFCLASS CONTEXT-SENSITIVE-OBJECT (STANDARD-OBJECT) :DOCUMENTATION \"Context sensitive objects inherit a slot 'home-context'\nthat enables context-dependent access machinery to determine the\nvisibility of objects from modules.\" :ABSTRACT? TRUE :PUBLIC-SLOTS ((HOME-CONTEXT :TYPE CONTEXT :INITIALLY *CONTEXT*)) :METHODS ((HOME-MODULE ((SELF CONTEXT-SENSITIVE-OBJECT)) :TYPE MODULE :PUBLIC? TRUE (RETURN (BASE-MODULE (HOME-CONTEXT SELF))))))");

      renamed_Class->class_slot_accessor_code = ((cpp_function_code)(&access_context_sensitive_object_slot_value));
    }
    define_class_from_stringified_source("CONTEXT-SENSITIVE-MIXIN", "(DEFCLASS CONTEXT-SENSITIVE-MIXIN () :DOCUMENTATION \"Mixin class that enables the class to support \ncontext sensitive slots.  The mixin allocates and initializes the\nslot 'home-context'.\" :MIXIN? TRUE :PUBLIC-SLOTS ((HOME-CONTEXT :TYPE CONTEXT :INITIALLY *MODULE*)) :PUBLIC-METHODS ((HOME-MODULE ((SELF CONTEXT-SENSITIVE-MIXIN)) :TYPE MODULE (RETURN (BASE-MODULE (HOME-CONTEXT SELF))))))");
    define_class_from_stringified_source("ACTIVE-OBJECT", "(DEFCLASS ACTIVE-OBJECT (CONTEXT-SENSITIVE-OBJECT DYNAMIC-SLOTS-MIXIN) :DOCUMENTATION \"Active objects call class and slot triggers in response\nto instance creation/destruction and slot updates.  They include internal\nstorage slots needed for class extensions, dynamic storage, and home context.\" :ABSTRACT? TRUE)");
    { Class* renamed_Class = define_class_from_stringified_source("ABSTRACT-ITERATOR", "(DEFCLASS ABSTRACT-ITERATOR (STANDARD-OBJECT) :ABSTRACT? TRUE :DOCUMENTATION \"Instances of ABSTRACT-ITERATOR support iteration over collections.\" :PARAMETERS ((ANY-VALUE :TYPE UNKNOWN)) :PUBLIC-SLOTS ((VALUE :TYPE (LIKE (ANY-VALUE SELF)) :DOCUMENTATION \"The current value of this iterator\") (FIRST-ITERATION? :TYPE BOOLEAN :INITIALLY TRUE)) :PUBLIC-METHODS ((NEXT? ((SELF ABSTRACT-ITERATOR)) :TYPE BOOLEAN (RETURN NULL)) (LENGTH ((SELF ABSTRACT-ITERATOR)) :TYPE INTEGER)))");

      renamed_Class->class_slot_accessor_code = ((cpp_function_code)(&access_abstract_iterator_slot_value));
    }
    define_class_from_stringified_source("ITERATOR", "(DEFCLASS ITERATOR (ABSTRACT-ITERATOR) :ABSTRACT? TRUE :DOCUMENTATION \"Instances of ITERATOR support iteration over \ncollections of OBJECTs.\" :PARAMETERS ((ANY-VALUE :TYPE OBJECT)) :SYNONYMS (OBJECT-ITERATOR))");
    { Class* renamed_Class = define_class_from_stringified_source("CONS-ITERATOR", "(DEFCLASS CONS-ITERATOR (ITERATOR) :DOCUMENTATION \"Iterator class for the class CONS.\" :PARAMETERS ((ANY-VALUE :TYPE OBJECT)) :PUBLIC-SLOTS ((CONS-ITERATOR-CURSOR :TYPE CONS)) :TERMINATOR TERMINATE-CONS-ITERATOR?)");

      renamed_Class->class_constructor_code = ((cpp_function_code)(&new_cons_iterator));
      renamed_Class->class_slot_accessor_code = ((cpp_function_code)(&access_cons_iterator_slot_value));
    }
    { Class* renamed_Class = define_class_from_stringified_source("LIST-ITERATOR", "(DEFCLASS LIST-ITERATOR (ITERATOR) :DOCUMENTATION \"Iterator class for the collection LIST.\" :PARAMETERS ((ANY-VALUE :TYPE OBJECT)) :PUBLIC-SLOTS ((LIST-ITERATOR-CURSOR :TYPE CONS) (LIST-ITERATOR-COLLECTION :TYPE LIST)) :TERMINATOR TERMINATE-LIST-ITERATOR?)");

      renamed_Class->class_constructor_code = ((cpp_function_code)(&new_list_iterator));
      renamed_Class->class_slot_accessor_code = ((cpp_function_code)(&access_list_iterator_slot_value));
    }
    { Class* renamed_Class = define_class_from_stringified_source("DESTRUCTIVE-LIST-ITERATOR", "(DEFCLASS DESTRUCTIVE-LIST-ITERATOR (ITERATOR) :DOCUMENTATION \"An iterator that contains a cons list.  Iterates over\nthe stored list, and destroys it when the iterator is free'd.\" :PARAMETERS ((ANY-VALUE :TYPE OBJECT)) :PUBLIC-SLOTS ((THE-CONS-LIST :TYPE CONS :INITIALLY NIL) (LIST-ITERATOR-CURSOR :TYPE CONS)) :TERMINATOR TERMINATE-DESTRUCTIVE-LIST-ITERATOR?)");

      renamed_Class->class_constructor_code = ((cpp_function_code)(&new_destructive_list_iterator));
      renamed_Class->class_slot_accessor_code = ((cpp_function_code)(&access_destructive_list_iterator_slot_value));
    }
    { Class* renamed_Class = define_class_from_stringified_source("SET-ITERATOR", "(DEFCLASS SET-ITERATOR (ITERATOR) :DOCUMENTATION \"Iterator class for the collection SET.\" :PARAMETERS ((ANY-VALUE :TYPE OBJECT)) :PUBLIC-SLOTS ((SET-ITERATOR-CURSOR :TYPE CONS) (SET-ITERATOR-COLLECTION :TYPE SET)) :TERMINATOR TERMINATE-SET-ITERATOR?)");

      renamed_Class->class_constructor_code = ((cpp_function_code)(&new_set_iterator));
      renamed_Class->class_slot_accessor_code = ((cpp_function_code)(&access_set_iterator_slot_value));
    }
    { Class* renamed_Class = define_class_from_stringified_source("ALL-PURPOSE-ITERATOR", "(DEFCLASS ALL-PURPOSE-ITERATOR (ITERATOR) :DOCUMENTATION \"The all-purpose iterator works by storing a 'next?'\nfunction within itself during iterator allocation.  The method\n'ALL-PURPOSE-ITERAOR.next?' funcalls the stored 'next?' function.\nAn assortment of storage slots are provided for constructing different\nkinds of iterators.\" :PUBLIC-SLOTS ((KEY :TYPE OBJECT) (ITERATOR-NEXT-CODE :TYPE FUNCTION-CODE) (ITERATOR-FILTER-CODE :TYPE FUNCTION-CODE) (ITERATOR-EMPTY-CODE :TYPE FUNCTION-CODE) (ITERATOR-NESTED-ITERATOR :TYPE ITERATOR) (ITERATOR-CONS-LIST :TYPE CONS) (ITERATOR-OBJECT :TYPE OBJECT) (ITERATOR-SECOND-OBJECT :TYPE OBJECT) (ITERATOR-INTEGER :TYPE INTEGER) (ITERATOR-SECOND-INTEGER :TYPE INTEGER)) :TERMINATOR TERMINATE-ALL-PURPOSE-ITERATOR?)");

      renamed_Class->class_constructor_code = ((cpp_function_code)(&new_all_purpose_iterator));
      renamed_Class->class_slot_accessor_code = ((cpp_function_code)(&access_all_purpose_iterator_slot_value));
    }
    { Class* renamed_Class = define_class_from_stringified_source("TYPES-TO-CLASSES-ITERATOR", "(DEFCLASS TYPES-TO-CLASSES-ITERATOR (ITERATOR) :DOCUMENTATION \"Iterator that returns a class for each type\nin a cons list of types.\" :PARAMETERS ((ANY-VALUE :TYPE CLASS)) :PUBLIC-SLOTS ((ITERATOR-CURSOR :TYPE (CONS OF TYPE) :REQUIRED? TRUE)) :METHODS ((NEXT? ((SELF TYPES-TO-CLASSES-ITERATOR)) :TYPE BOOLEAN (IF (FIRST-ITERATION? SELF) (SETF (FIRST-ITERATION? SELF) FALSE) (SETF (ITERATOR-CURSOR SELF) (REST (ITERATOR-CURSOR SELF)))) (COND ((NON-EMPTY? (ITERATOR-CURSOR SELF)) (SETF (VALUE SELF) (TYPE-CLASS (FIRST (ITERATOR-CURSOR SELF)))) (RETURN TRUE)) (OTHERWISE (RETURN FALSE))))))");

      renamed_Class->class_constructor_code = ((cpp_function_code)(&new_types_to_classes_iterator));
      renamed_Class->class_slot_accessor_code = ((cpp_function_code)(&access_types_to_classes_iterator_slot_value));
    }
    define_class_from_stringified_source("ABSTRACT-DICTIONARY-ITERATOR", "(DEFCLASS ABSTRACT-DICTIONARY-ITERATOR (ABSTRACT-ITERATOR) :ABSTRACT? TRUE :DOCUMENTATION \"Instances of ABSTRACT-DICTIONARY-ITERATOR support iteration\nover dictionaries.\" :PARAMETERS ((ANY-KEY :TYPE UNKNOWN) (ANY-VALUE :TYPE UNKNOWN)) :PUBLIC-SLOTS ((KEY :TYPE (LIKE (ANY-KEY SELF)))))");
    define_class_from_stringified_source("DICTIONARY-ITERATOR", "(DEFCLASS DICTIONARY-ITERATOR (ABSTRACT-DICTIONARY-ITERATOR) :ABSTRACT? TRUE :DOCUMENTATION \"Instances of DICTIONARY-ITERATOR support iteration\nover dictionaries with keys and values of type OBJECT.\" :PARAMETERS ((ANY-KEY :TYPE OBJECT) (ANY-VALUE :TYPE OBJECT)) :SYNONYMS (OBJECT-DICTIONARY-ITERATOR))");
    { Class* renamed_Class = define_class_from_stringified_source("PROPERTY-LIST-ITERATOR", "(DEFCLASS PROPERTY-LIST-ITERATOR (DICTIONARY-ITERATOR) :DOCUMENTATION \"Iterator class for the collection PROPERTY-LIST.\" :PUBLIC-SLOTS ((PLIST-ITERATOR-CURSOR :TYPE CONS) (PLIST-ITERATOR-COLLECTION :TYPE PROPERTY-LIST)))");

      renamed_Class->class_constructor_code = ((cpp_function_code)(&new_property_list_iterator));
      renamed_Class->class_slot_accessor_code = ((cpp_function_code)(&access_property_list_iterator_slot_value));
    }
    { Class* renamed_Class = define_class_from_stringified_source("KV-LIST-ITERATOR", "(DEFCLASS KV-LIST-ITERATOR (DICTIONARY-ITERATOR) :PUBLIC-SLOTS ((THE-KV-LIST :TYPE KEY-VALUE-LIST :PUBLIC? TRUE) (KV-LIST-ITERATOR-CURSOR :TYPE KV-CONS)))");

      renamed_Class->class_constructor_code = ((cpp_function_code)(&new_kv_list_iterator));
      renamed_Class->class_slot_accessor_code = ((cpp_function_code)(&access_kv_list_iterator_slot_value));
    }
    define_class_from_stringified_source("INTERVAL-ITERATOR", "(DEFCLASS INTERVAL-ITERATOR (ABSTRACT-ITERATOR) :ABSTRACT? TRUE :DOCUMENTATION \"An iterator that specifies a (possibly infinite) range of\nvalues.  Supports 'member?' test as well as iteration methods.\" :PUBLIC-SLOTS ((LOWER-BOUND :TYPE UNKNOWN :REQUIRED? TRUE) (UPPER-BOUND :TYPE UNKNOWN :REQUIRED? TRUE)))");
    { Class* renamed_Class = define_class_from_stringified_source("INTEGER-INTERVAL-ITERATOR", "(DEFCLASS INTEGER-INTERVAL-ITERATOR (INTERVAL-ITERATOR) :PARAMETERS ((ANY-VALUE :TYPE INTEGER)) :PUBLIC-SLOTS ((INTERVAL-CURSOR :TYPE INTEGER) (LOWER-BOUND :TYPE INTEGER :REQUIRED? TRUE) (UPPER-BOUND :TYPE INTEGER :REQUIRED? TRUE)) :INITIALIZER INITIALIZE-INTEGER-INTERVAL-ITERATOR)");

      renamed_Class->class_constructor_code = ((cpp_function_code)(&new_integer_interval_iterator));
      renamed_Class->class_slot_accessor_code = ((cpp_function_code)(&access_integer_interval_iterator_slot_value));
    }
    { Class* renamed_Class = define_class_from_stringified_source("REVERSE-INTEGER-INTERVAL-ITERATOR", "(DEFCLASS REVERSE-INTEGER-INTERVAL-ITERATOR (INTERVAL-ITERATOR) :PARAMETERS ((ANY-VALUE :TYPE INTEGER)) :PUBLIC-SLOTS ((INTERVAL-CURSOR :TYPE INTEGER) (LOWER-BOUND :TYPE INTEGER :REQUIRED? TRUE) (UPPER-BOUND :TYPE INTEGER :REQUIRED? TRUE)) :INITIALIZER INITIALIZE-REVERSE-INTEGER-INTERVAL-ITERATOR)");

      renamed_Class->class_constructor_code = ((cpp_function_code)(&new_reverse_integer_interval_iterator));
      renamed_Class->class_slot_accessor_code = ((cpp_function_code)(&access_reverse_integer_interval_iterator_slot_value));
    }
    { Class* renamed_Class = define_class_from_stringified_source("STRING-ITERATOR", "(DEFCLASS STRING-ITERATOR (ABSTRACT-ITERATOR) :DOCUMENTATION \"Iterator that yields characters from a string.\" :PARAMETERS ((ANY-VALUE :TYPE CHARACTER)) :PUBLIC-SLOTS ((THE-STRING :TYPE STRING) (CURSOR :TYPE INTEGER) (END :TYPE INTEGER)))");

      renamed_Class->class_constructor_code = ((cpp_function_code)(&new_string_iterator));
      renamed_Class->class_slot_accessor_code = ((cpp_function_code)(&access_string_iterator_slot_value));
    }
    define_class_from_stringified_source("ABSTRACT-COLLECTION", "(DEFCLASS ABSTRACT-COLLECTION (STANDARD-OBJECT) :PARAMETERS ((ANY-VALUE :TYPE UNKNOWN)) :PUBLIC-METHODS ((ALLOCATE-ITERATOR ((SELF ABSTRACT-COLLECTION)) :TYPE (ABSTRACT-ITERATOR OF (LIKE (ANY-VALUE SELF))) (RETURN NULL)) (LENGTH ((SELF ABSTRACT-COLLECTION)) :TYPE INTEGER (RETURN NULL)) (INSERT ((SELF ABSTRACT-COLLECTION) (VALUE (LIKE (ANY-VALUE SELF))))) (REMOVE ((SELF ABSTRACT-COLLECTION) (VALUE (LIKE (ANY-VALUE SELF)))) :TYPE ABSTRACT-COLLECTION)) :ABSTRACT? TRUE)");
    define_class_from_stringified_source("COLLECTION", "(DEFCLASS COLLECTION (ABSTRACT-COLLECTION) :PARAMETERS ((ANY-VALUE :TYPE OBJECT)) :PUBLIC-METHODS ((ALLOCATE-ITERATOR ((SELF COLLECTION)) :TYPE (ITERATOR OF (LIKE (ANY-VALUE SELF))) (RETURN NULL)) (LENGTH ((SELF COLLECTION)) :TYPE INTEGER (RETURN NULL)) (INSERT ((SELF COLLECTION) (VALUE (LIKE (ANY-VALUE SELF)))) (RETURN)) (REMOVE ((SELF COLLECTION) (VALUE (LIKE (ANY-VALUE SELF)))) :TYPE COLLECTION (RETURN NULL))) :ABSTRACT? TRUE :SYNONYMS (OBJECT-COLLECTION))");
    define_class_from_stringified_source("SET-MIXIN", "(DEFCLASS SET-MIXIN () :DOCUMENTATION \"Users of this mixin check for duplicates inside of \nthe method 'insert'.\" :PARAMETERS ((ANY-VALUE :TYPE OBJECT)) :MIXIN? TRUE)");
    define_class_from_stringified_source("SEQUENCE-MIXIN", "(DEFCLASS SEQUENCE-MIXIN () :PARAMETERS ((ANY-VALUE :TYPE UNKNOWN)) :PUBLIC-METHODS ((FIRST ((SEQUENCE SEQUENCE-MIXIN)) :TYPE (LIKE (ANY-VALUE SELF))) (NTH ((SEQUENCE SEQUENCE-MIXIN) (POSITION INTEGER)) :TYPE (LIKE (ANY-VALUE SELF)))) :MIXIN? TRUE)");
    define_class_from_stringified_source("SEQUENCE", "(DEFCLASS SEQUENCE (COLLECTION SEQUENCE-MIXIN) :PARAMETERS ((ANY-VALUE :TYPE OBJECT)) :PUBLIC-METHODS ((ALLOCATE-ITERATOR ((SELF SEQUENCE)) :TYPE (ITERATOR OF (LIKE (ANY-VALUE SELF))) (RETURN NULL)) (LENGTH ((SEQUENCE SEQUENCE)) :TYPE INTEGER (RETURN NULL))) :ABSTRACT? TRUE :SYNONYMS (OBJECT-SEQUENCE))");
    { Class* renamed_Class = define_class_from_stringified_source("LIST", "(DEFCLASS LIST (SEQUENCE) :PARAMETERS ((ANY-VALUE :TYPE OBJECT)) :PUBLIC-SLOTS ((THE-CONS-LIST :TYPE (CONS OF (LIKE (ANY-VALUE SELF))) :INITIALLY NIL :PUBLIC? TRUE)) :INITIAL-VALUE NIL-LIST :PRINT-FORM (PRINT-CONS-LIST (THE-CONS-LIST SELF) STREAM))");

      renamed_Class->class_constructor_code = ((cpp_function_code)(&new_list));
      renamed_Class->class_slot_accessor_code = ((cpp_function_code)(&access_list_slot_value));
    }
    { Class* renamed_Class = define_class_from_stringified_source("SET", "(DEFCLASS SET (COLLECTION SET-MIXIN) :PARAMETERS ((ANY-VALUE :TYPE OBJECT)) :PUBLIC-SLOTS ((THE-CONS-LIST :TYPE (CONS OF (LIKE (ANY-VALUE SELF))) :INITIALLY NIL :PUBLIC? TRUE)) :PRINT-FORM (PRINT-CONS-LIST (THE-CONS-LIST SELF) STREAM))");

      renamed_Class->class_constructor_code = ((cpp_function_code)(&new_set));
      renamed_Class->class_slot_accessor_code = ((cpp_function_code)(&access_set_slot_value));
    }
    { Class* renamed_Class = define_class_from_stringified_source("CLASS-EXTENSION", "(DEFCLASS CLASS-EXTENSION (LIST) :PARAMETERS ((ANY-VALUE :TYPE ACTIVE-OBJECT)))");

      renamed_Class->class_constructor_code = ((cpp_function_code)(&new_class_extension));
    }
    { Class* renamed_Class = define_class_from_stringified_source("CONS", "(DEFCLASS CONS (STANDARD-OBJECT) :PARAMETERS ((ANY-VALUE :TYPE OBJECT)) :METHODS ((ALLOCATE-ITERATOR ((SELF CONS)) :TYPE (ITERATOR OF (LIKE (ANY-VALUE SELF))))) :PUBLIC-SLOTS ((VALUE :TYPE (LIKE (ANY-VALUE SELF)) :PUBLIC? TRUE) (REST :TYPE (CONS OF (LIKE (ANY-VALUE SELF))) :PUBLIC? TRUE :INITIALLY NIL)) :INITIAL-VALUE NIL :PRINT-FORM (PRINT-CONS SELF STREAM \"(\" \")\"))");

      renamed_Class->class_constructor_code = ((cpp_function_code)(&new_cons));
      renamed_Class->class_slot_accessor_code = ((cpp_function_code)(&access_cons_slot_value));
    }
    define_class_from_stringified_source("ABSTRACT-DICTIONARY", "(DEFCLASS ABSTRACT-DICTIONARY (ABSTRACT-COLLECTION) :PARAMETERS ((ANY-KEY :TYPE UNKNOWN) (ANY-VALUE :TYPE UNKNOWN)) :PUBLIC-METHODS ((LOOKUP ((SELF ABSTRACT-DICTIONARY) (KEY (LIKE (ANY-KEY SELF)))) :TYPE (LIKE (ANY-VALUE SELF))) (INSERT-AT ((SELF ABSTRACT-DICTIONARY) (KEY (LIKE (ANY-KEY SELF))) (VALUE (LIKE (ANY-VALUE SELF)))))) :METHODS ((ALLOCATE-ITERATOR ((SELF ABSTRACT-DICTIONARY)) :TYPE (ABSTRACT-DICTIONARY-ITERATOR OF (LIKE (ANY-KEY SELF)) (LIKE (ANY-VALUE SELF))) (RETURN NULL))) :ABSTRACT? TRUE)");
    define_class_from_stringified_source("DICTIONARY", "(DEFCLASS DICTIONARY (ABSTRACT-DICTIONARY) :PARAMETERS ((ANY-KEY :TYPE OBJECT) (ANY-VALUE :TYPE OBJECT)) :METHODS ((ALLOCATE-ITERATOR ((SELF DICTIONARY)) :TYPE (DICTIONARY-ITERATOR OF (LIKE (ANY-KEY SELF)) (LIKE (ANY-VALUE SELF))) (RETURN NULL))) :ABSTRACT? TRUE :SYNONYMS (OBJECT-TO-OBJECT-DICTIONARY))");
    { Class* renamed_Class = define_class_from_stringified_source("PROPERTY-LIST", "(DEFCLASS PROPERTY-LIST (DICTIONARY) :PARAMETERS ((ANY-KEY :TYPE OBJECT) (ANY-VALUE :TYPE OBJECT)) :PUBLIC-SLOTS ((THE-PLIST :TYPE CONS :INITIALLY NIL)) :PRINT-FORM (PRINT-CONS-LIST (THE-PLIST SELF) STREAM))");

      renamed_Class->class_constructor_code = ((cpp_function_code)(&new_property_list));
      renamed_Class->class_slot_accessor_code = ((cpp_function_code)(&access_property_list_slot_value));
    }
    { Class* renamed_Class = define_class_from_stringified_source("KV-CONS", "(DEFCLASS KV-CONS (STANDARD-OBJECT) :PARAMETERS ((ANY-KEY :TYPE OBJECT) (ANY-VALUE :TYPE OBJECT)) :PUBLIC-SLOTS ((KEY :TYPE OBJECT) (VALUE :TYPE OBJECT) (REST :TYPE KV-CONS)) :PRINT-FORM (PRINT-NATIVE-STREAM STREAM \"<\" (KEY SELF) \",\" (VALUE SELF) \">\"))");

      renamed_Class->class_constructor_code = ((cpp_function_code)(&new_kv_cons));
      renamed_Class->class_slot_accessor_code = ((cpp_function_code)(&access_kv_cons_slot_value));
    }
    { Class* renamed_Class = define_class_from_stringified_source("KEY-VALUE-LIST", "(DEFCLASS KEY-VALUE-LIST (DICTIONARY) :PARAMETERS ((ANY-KEY :TYPE OBJECT) (ANY-VALUE :TYPE OBJECT)) :PUBLIC-SLOTS ((THE-KV-LIST :TYPE KV-CONS)) :PRINT-FORM (PRINT-KEY-VALUE-LIST SELF STREAM))");

      renamed_Class->class_constructor_code = ((cpp_function_code)(&new_key_value_list));
      renamed_Class->class_slot_accessor_code = ((cpp_function_code)(&access_key_value_list_slot_value));
    }
    { Class* renamed_Class = define_class_from_stringified_source("RECYCLE-LIST", "(DEFCLASS RECYCLE-LIST (SEQUENCE) :PARAMETERS ((ANY-VALUE :TYPE OBJECT)) :PUBLIC-SLOTS ((RECYCLE-LIST-OF :TYPE TYPE) (RECYCLED-ITEMS :TYPE (LIKE (ANY-VALUE SELF))) (LIST-OF-RECYCLED-ITEMS :TYPE (LIST OF (LIKE (ANY-VALUE SELF))) :INITIALLY (MAKE-NON-RECYCLED-LIST)) (ALL-ITEMS :TYPE (LIKE (ANY-VALUE SELF))) (UNUSED-ITEMS :TYPE (LIKE (ANY-VALUE SELF))) (CURRENT-LENGTH :TYPE INTEGER :INITIALLY 0) (ITEM-SIZE :TYPE INTEGER)) :PRINT-FORM (IF (DEFINED? (RECYCLE-LIST-OF SELF)) (PRINT-NATIVE-STREAM STREAM \"|i|[RECYCLE-LIST OF \" (CURRENT-LENGTH SELF) \" \" (RECYCLE-LIST-OF SELF) \"'s]\") (PRINT-NATIVE-STREAM STREAM \"|i|@RECYCLE-LIST\")))");

      renamed_Class->class_constructor_code = ((cpp_function_code)(&new_recycle_list));
      renamed_Class->class_slot_accessor_code = ((cpp_function_code)(&access_recycle_list_slot_value));
    }
    { Class* renamed_Class = define_class_from_stringified_source("ABSTRACT-HASH-TABLE", "(DEFCLASS ABSTRACT-HASH-TABLE (ABSTRACT-DICTIONARY) :PARAMETERS ((ANY-KEY :TYPE UNKNOWN) (ANY-VALUE :TYPE UNKNOWN)) :PUBLIC-SLOTS ((THE-HASH-TABLE :TYPE NATIVE-HASH-TABLE) (THE-STELLA-HASH-TABLE :TYPE STELLA-HASH-TABLE)) :INITIALIZER INITIALIZE-HASH-TABLE :ABSTRACT? TRUE)");

      renamed_Class->class_slot_accessor_code = ((cpp_function_code)(&access_abstract_hash_table_slot_value));
    }
    { Class* renamed_Class = define_class_from_stringified_source("HASH-TABLE", "(DEFCLASS HASH-TABLE (ABSTRACT-HASH-TABLE) :PARAMETERS ((ANY-KEY :TYPE OBJECT) (ANY-VALUE :TYPE OBJECT)) :INITIALIZER INITIALIZE-HASH-TABLE :SYNONYMS (OBJECT-TO-OBJECT-HASH-TABLE))");

      renamed_Class->class_constructor_code = ((cpp_function_code)(&new_hash_table));
    }
    { Class* renamed_Class = define_class_from_stringified_source("INTEGER-HASH-TABLE", "(DEFCLASS INTEGER-HASH-TABLE (ABSTRACT-HASH-TABLE) :PARAMETERS ((ANY-KEY :TYPE INTEGER) (ANY-VALUE :TYPE OBJECT)) :INITIALIZER INITIALIZE-HASH-TABLE)");

      renamed_Class->class_constructor_code = ((cpp_function_code)(&new_integer_hash_table));
    }
    { Class* renamed_Class = define_class_from_stringified_source("FLOAT-HASH-TABLE", "(DEFCLASS FLOAT-HASH-TABLE (ABSTRACT-HASH-TABLE) :PARAMETERS ((ANY-KEY :TYPE FLOAT) (ANY-VALUE :TYPE OBJECT)) :INITIALIZER INITIALIZE-HASH-TABLE)");

      renamed_Class->class_constructor_code = ((cpp_function_code)(&new_float_hash_table));
    }
    { Class* renamed_Class = define_class_from_stringified_source("STRING-HASH-TABLE", "(DEFCLASS STRING-HASH-TABLE (ABSTRACT-HASH-TABLE) :PARAMETERS ((ANY-KEY :TYPE STRING) (ANY-VALUE :TYPE OBJECT)) :INITIALIZER INITIALIZE-HASH-TABLE)");

      renamed_Class->class_constructor_code = ((cpp_function_code)(&new_string_hash_table));
    }
    { Class* renamed_Class = define_class_from_stringified_source("STRING-TO-INTEGER-HASH-TABLE", "(DEFCLASS STRING-TO-INTEGER-HASH-TABLE (ABSTRACT-HASH-TABLE) :PARAMETERS ((ANY-KEY :TYPE STRING) (ANY-VALUE :TYPE INTEGER)) :INITIALIZER INITIALIZE-HASH-TABLE)");

      renamed_Class->class_constructor_code = ((cpp_function_code)(&new_string_to_integer_hash_table));
    }
    { Class* renamed_Class = define_class_from_stringified_source("VECTOR", "(DEFCLASS VECTOR (SEQUENCE) :PUBLIC-SLOTS ((ARRAY-SIZE :TYPE INTEGER :REQUIRED? TRUE) (THE-ARRAY :TYPE (NATIVE-VECTOR OF (LIKE (ANY-VALUE SELF))))) :PARAMETERS ((ANY-VALUE :TYPE OBJECT)) :PRINT-FORM (PRINT-VECTOR SELF STREAM) :INITIALIZER INITIALIZE-VECTOR)");

      renamed_Class->class_constructor_code = ((cpp_function_code)(&new_vector));
      renamed_Class->class_slot_accessor_code = ((cpp_function_code)(&access_vector_slot_value));
    }
    { Class* renamed_Class = define_class_from_stringified_source("EXTENSIBLE-VECTOR", "(DEFCLASS EXTENSIBLE-VECTOR (VECTOR) :INITIALIZER INITIALIZE-VECTOR)");

      renamed_Class->class_constructor_code = ((cpp_function_code)(&new_extensible_vector));
    }
    { Class* renamed_Class = define_class_from_stringified_source("EXTENSIBLE-SYMBOL-ARRAY", "(DEFCLASS EXTENSIBLE-SYMBOL-ARRAY (EXTENSIBLE-VECTOR) :DOCUMENTATION \"Self-extending array with methods for storing and\naccessing symbols within it.\" :PUBLIC-SLOTS ((TOP-SYMBOL-OFFSET :TYPE INTEGER :INITIALLY -1) (POTENTIAL-FREE-SYMBOL-OFFSET :TYPE INTEGER :INITIALLY 0)) :INITIALIZER INITIALIZE-VECTOR)");

      renamed_Class->class_constructor_code = ((cpp_function_code)(&new_extensible_symbol_array));
      renamed_Class->class_slot_accessor_code = ((cpp_function_code)(&access_extensible_symbol_array_slot_value));
    }
    { Class* renamed_Class = define_class_from_stringified_source("VECTOR-SEQUENCE", "(DEFCLASS VECTOR-SEQUENCE (VECTOR) :ABSTRACT? TRUE :PUBLIC-SLOTS ((RESIZE-INCREMENT :TYPE INTEGER :HARDWIRED? TRUE :INITIALLY 100) (SEQUENCE-LENGTH :TYPE INTEGER)) :INITIALIZER INITIALIZE-VECTOR :SYNONYMS (VSEQ))");

      oHARDWIRED_RESIZE_INCREMENT_ON_VECTOR_SEQUENCEo = 100;
      renamed_Class->class_slot_accessor_code = ((cpp_function_code)(&access_vector_sequence_slot_value));
    }
    { Class* renamed_Class = define_class_from_stringified_source("SHORT-VECTOR-SEQUENCE", "(DEFCLASS SHORT-VECTOR-SEQUENCE (VECTOR-SEQUENCE) :PUBLIC-SLOTS ((RESIZE-INCREMENT :TYPE INTEGER :HARDWIRED? TRUE :INITIALLY 4) (SEQUENCE-LENGTH :TYPE INTEGER :INITIALLY 0)) :INITIALIZER INITIALIZE-VECTOR :SYNONYMS (SVSEQ))");

      oHARDWIRED_RESIZE_INCREMENT_ON_VECTOR_SEQUENCEo = 100;
      oHARDWIRED_RESIZE_INCREMENT_ON_SHORT_VECTOR_SEQUENCEo = 4;
      renamed_Class->class_constructor_code = ((cpp_function_code)(&new_short_vector_sequence));
      renamed_Class->class_slot_accessor_code = ((cpp_function_code)(&access_short_vector_sequence_slot_value));
    }
    { Class* renamed_Class = define_class_from_stringified_source("BOOLEAN-VECTOR", "(DEFCLASS BOOLEAN-VECTOR (VECTOR) :PARAMETERS ((ANY-VALUE :TYPE BOOLEAN-WRAPPER)))");

      renamed_Class->class_constructor_code = ((cpp_function_code)(&new_boolean_vector));
    }
    { Class* renamed_Class = define_class_from_stringified_source("INTEGER-VECTOR", "(DEFCLASS INTEGER-VECTOR (VECTOR) :PARAMETERS ((ANY-VALUE :TYPE INTEGER-WRAPPER)))");

      renamed_Class->class_constructor_code = ((cpp_function_code)(&new_integer_vector));
    }
    define_class_from_stringified_source("ARRAY", "(DEFCLASS ARRAY (SECOND-CLASS-OBJECT) :PARAMETERS ((ANY-VALUE :TYPE UNKNOWN)) :SLOTS ((INITIAL-ELEMENT :TYPE (LIKE (ANY-VALUE SELF)) :REQUIRED? TRUE :ABSTRACT? TRUE :DOCUMENTATION \"Initializer pseudo-slot special-cased by the STELLA translator.\") (SIZE :TYPE (LIST OF INTEGER-WRAPPER) :REQUIRED? TRUE :ABSTRACT? TRUE :DOCUMENTATION \"Size pseudo-slot special-cased by the STELLA translator.\")) :METHODS ((AREF ((SELF ARRAY) (DIM0 INTEGER) |&REST| (DIMS INTEGER)) :TYPE (LIKE (ANY-VALUE SELF)) :PUBLIC? TRUE :NATIVE? TRUE) (AREF-SETTER ((SELF ARRAY) (VALUE (LIKE (ANY-VALUE SELF))) (DIM0 INTEGER) |&REST| (DIMS INTEGER)) :TYPE (LIKE (ANY-VALUE SELF)) :PUBLIC? TRUE :NATIVE? TRUE)) :CL-NATIVE-TYPE \"ARRAY\" :CPP-NATIVE-TYPE \"char*\" :JAVA-NATIVE-TYPE \"Object\")");
    define_class_from_stringified_source("NATIVE-HASH-TABLE", "(DEFCLASS NATIVE-HASH-TABLE (SECOND-CLASS-OBJECT) :CPP-NATIVE-TYPE \"cpp_hash_table*\" :CL-NATIVE-TYPE \"HASH-TABLE\" :JAVA-NATIVE-TYPE \"java.util.HashMap\")");
    define_class_from_stringified_source("NATIVE-VECTOR", "(DEFCLASS NATIVE-VECTOR (SECOND-CLASS-OBJECT) :PARAMETERS ((ANY-VALUE :TYPE OBJECT)) :CPP-NATIVE-TYPE \"Object**\" :CL-NATIVE-TYPE \"VECTOR\" :JAVA-NATIVE-TYPE \"#$(STELLAROOT).Stella_Object[]\" :INITIAL-VALUE (VERBATIM :COMMON-LISP \"STELLA::NULL-NATIVE-VECTOR\" :JAVA \"null\" :CPP \"NULL\"))");
    define_class_from_stringified_source("ACTIVE-COLLECTION-MIXIN", "(DEFCLASS ACTIVE-COLLECTION-MIXIN () :DOCUMENTATION \"Mixin class that provides collection instances with a\nbackpointer to the instance slot they belong to.\" :MIXIN? TRUE :PUBLIC-SLOTS ((ACTIVE-SLOT :TYPE STORAGE-SLOT) (OWNER-INSTANCE :TYPE STANDARD-OBJECT)))");
    { Class* renamed_Class = define_class_from_stringified_source("ACTIVE-LIST", "(DEFCLASS ACTIVE-LIST (LIST ACTIVE-COLLECTION-MIXIN))");

      renamed_Class->class_constructor_code = ((cpp_function_code)(&new_active_list));
    }
    { Class* renamed_Class = define_class_from_stringified_source("ACTIVE-SET", "(DEFCLASS ACTIVE-SET (LIST SET-MIXIN ACTIVE-COLLECTION-MIXIN))");

      renamed_Class->class_constructor_code = ((cpp_function_code)(&new_active_set));
    }
  }
}

void help_startup_hierarchy8() {
  {
    { Class* renamed_Class = define_class_from_stringified_source("DEMON", "(DEFCLASS DEMON (STANDARD-OBJECT) :PUBLIC-SLOTS ((DEMON-NAME :TYPE STRING) (DEMON-ACTION :TYPE KEYWORD) (DEMON-CLASS-REFS :TYPE (LIST OF TYPE) :ALLOCATION :EMBEDDED) (DEMON-SLOT-REFS :TYPE (LIST OF SYMBOL) :ALLOCATION :EMBEDDED) (DEMON-CODE :TYPE FUNCTION-CODE) (DEMON-METHOD :TYPE METHOD-SLOT) (DEMON-DOCUMENTATION :TYPE STRING) (DEMON-GUARD? :TYPE BOOLEAN) (DEMON-ALL? :TYPE BOOLEAN) (DEMON-INHERIT? :TYPE BOOLEAN)))");

      renamed_Class->class_constructor_code = ((cpp_function_code)(&new_demon));
      renamed_Class->class_slot_accessor_code = ((cpp_function_code)(&access_demon_slot_value));
    }
    { Class* renamed_Class = define_class_from_stringified_source("KEYWORD-KEY-VALUE-LIST", "(DEFCLASS KEYWORD-KEY-VALUE-LIST (KEY-VALUE-LIST) :PARAMETERS ((ANY-KEY :TYPE KEYWORD) (ANY-VALUE :TYPE OBJECT)))");

      renamed_Class->class_constructor_code = ((cpp_function_code)(&new_keyword_key_value_list));
    }
    { Class* renamed_Class = define_class_from_stringified_source("GENERALIZED-SYMBOL", "(DEFCLASS GENERALIZED-SYMBOL (CONTEXT-SENSITIVE-OBJECT) :PUBLIC-SLOTS ((SYMBOL-NAME :TYPE STRING :REQUIRED? TRUE) (SYMBOL-ID :TYPE INTEGER) (INTERNED-IN :RENAMES HOME-CONTEXT :TYPE MODULE)) :ABSTRACT? TRUE)");

      renamed_Class->class_slot_accessor_code = ((cpp_function_code)(&access_generalized_symbol_slot_value));
    }
    { Class* renamed_Class = define_class_from_stringified_source("SYMBOL", "(DEFCLASS SYMBOL (GENERALIZED-SYMBOL) :PUBLIC-SLOTS ((SYMBOL-SLOT-OFFSET :TYPE INTEGER) (SYMBOL-VALUE-AND-PLIST :TYPE CONS :INITIALLY NIL)) :PRINT-FORM (PRINT-SYMBOL SELF STREAM))");

      renamed_Class->class_constructor_code = ((cpp_function_code)(&new_symbol));
      renamed_Class->class_slot_accessor_code = ((cpp_function_code)(&access_symbol_slot_value));
    }
    { Class* renamed_Class = define_class_from_stringified_source("SURROGATE", "(DEFCLASS SURROGATE (GENERALIZED-SYMBOL) :DOCUMENTATION \"Rigid surrogate.\" :PUBLIC-SLOTS ((SURROGATE-VALUE :TYPE OBJECT) (SURROGATE-NAME :RENAMES SYMBOL-NAME) (TYPE-CLASS :RENAMES SURROGATE-VALUE :TYPE CLASS) (TYPE-NAME :RENAMES SYMBOL-NAME)) :PUBLIC-SLOTS ((SLOTREF-SLOT :RENAMES SURROGATE-VALUE :TYPE SLOT)) :SYNONYMS (TYPE SLOTREF) :PRINT-FORM (PRINT-SURROGATE SELF STREAM))");

      renamed_Class->class_constructor_code = ((cpp_function_code)(&new_surrogate));
      renamed_Class->class_slot_accessor_code = ((cpp_function_code)(&access_surrogate_slot_value));
    }
    { Class* renamed_Class = define_class_from_stringified_source("KEYWORD", "(DEFCLASS KEYWORD (GENERALIZED-SYMBOL) :PUBLIC-SLOTS ((KEYWORD-NAME :RENAMES SYMBOL-NAME)) :PRINT-FORM (PRINT-KEYWORD SELF STREAM))");

      renamed_Class->class_constructor_code = ((cpp_function_code)(&new_keyword));
      renamed_Class->class_slot_accessor_code = ((cpp_function_code)(&access_keyword_slot_value));
    }
    { Class* renamed_Class = define_class_from_stringified_source("TRANSIENT-SYMBOL", "(DEFCLASS TRANSIENT-SYMBOL (SYMBOL TRANSIENT-MIXIN))");

      oHARDWIRED_TRANSIENTp_ON_TRANSIENT_MIXINo = TRUE;
      renamed_Class->class_constructor_code = ((cpp_function_code)(&new_transient_symbol));
    }
    { Class* renamed_Class = define_class_from_stringified_source("MAPPABLE-OBJECT", "(DEFCLASS MAPPABLE-OBJECT (STANDARD-OBJECT DYNAMIC-SLOTS-MIXIN) :DOCUMENTATION \"The class MAPPABLE-OBJECT enables the definition of projections\nfrom a Stella class, slot, global variable, etc. onto a corresponding native\nentity.\" :ABSTRACT? TRUE :PUBLIC-SLOTS ((PROJECTS-ONTO :TYPE (LIST OF SYMBOL) :ALLOCATION :DYNAMIC) (PROJECTED-FROM :TYPE (LIST OF SYMBOL) :ALLOCATION :DYNAMIC) (PROJECTION-TRANSFORM :TYPE SYMBOL :ALLOCATION :DYNAMIC :DOCUMENTATION \"Names a coersion function that translates\nvalues retrieved from the 'from' entity to the 'projecting' entity.\")) :PUBLIC-SLOTS ((NATIVE-NAME :TYPE STRING :ALLOCATION :DYNAMIC :DOCUMENTATION \"Used in cases when the native name cannot be\ncast as a symbol (e.g., because it contains illegal characters).\")))");

      renamed_Class->class_slot_accessor_code = ((cpp_function_code)(&access_mappable_object_slot_value));
    }
    { Class* renamed_Class = define_class_from_stringified_source("RELATION", "(DEFCLASS RELATION (MAPPABLE-OBJECT) :ABSTRACT? TRUE :PUBLIC-SLOTS ((DOCUMENTATION :TYPE STRING :ALLOCATION :DYNAMIC) (ABSTRACT? :TYPE BOOLEAN :OPTION-KEYWORD :ABSTRACT?) (META-ATTRIBUTES :TYPE (KEY-VALUE-LIST OF GENERALIZED-SYMBOL OBJECT) :ALLOCATION :DYNAMIC :OPTION-KEYWORD :META-ATTRIBUTES) (PROPERTIES :TYPE (LIST OF GENERALIZED-SYMBOL) :ALLOCATION :DYNAMIC :OPTION-KEYWORD :PROPERTIES)) :PUBLIC-METHODS ((NAME ((SELF RELATION)) :TYPE STRING (RETURN NULL)) (HOME-MODULE ((SELF RELATION)) :TYPE MODULE (RETURN NULL)) (ARITY ((SELF RELATION)) :TYPE INTEGER (RETURN NULL)) (PUBLIC? ((SELF RELATION)) :TYPE BOOLEAN (RETURN NULL)) (DIRECT-SUPERS ((SELF RELATION)) :TYPE (LIST OF RELATION) (RETURN NULL)) (ALL-SUPERS ((SELF RELATION)) :TYPE (LIST OF RELATION) (RETURN NULL)) (SLOTS ((SELF RELATION)) :TYPE (ITERATOR OF SLOT) (RETURN NULL))) :SLOTS ((STORED-ACTIVE? :TYPE BOOLEAN-WRAPPER :ALLOCATION :DYNAMIC :OPTION-KEYWORD :ACTIVE?)))");

      renamed_Class->class_slot_accessor_code = ((cpp_function_code)(&access_relation_slot_value));
    }
    { Class* renamed_Class = define_class_from_stringified_source("CLASS", string_concatenate("(DEFCLASS CLASS (RELATION) :PUBLIC-SLOTS ((CL-STRUCT? :TYPE BOOLEAN :INITIALLY FALSE) (MIXIN? :TYPE BOOLEAN :INITIALLY FALSE) (PRINT-FORM :TYPE OBJECT :ALLOCATION :DYNAMIC :OPTION-KEYWORD :PRINT-FORM) (CLASS-TYPE :TYPE TYPE :PUBLIC? TRUE) (CLASS-ARITY :TYPE INTEGER :INITIALLY 1 :HARDWIRED? TRUE) (CLASS-DIRECT-SUPERS :TYPE (LIST OF TYPE) :ALLOCATION :EMBEDDED) (CLASS-DIRECT-SUBS :TYPE (LIST OF TYPE) :ALLOCATION :EMBEDDED) (CLASS-ALL-SUPER-CLASSES :TYPE (CONS OF CLASS) :INITIALLY NIL :PUBLIC? TRUE) (CLASS-ALL-SLOTS :TYPE (CONS OF SLOT)) (CLASS-LOCAL-SLOTS :TYPE (LIST OF SLOT)) (CLASS-SLOT-AND-METHOD-CACHE :TYPE (VECTOR OF SLOT)) (CLASS-ABSTRACT? :TYPE BOOLEAN :INITIALLY FALSE :RENAMES ABSTRACT?) (CLASS-MIXIN? :TYPE BOOLEAN :INITIALLY FALSE :RENAMES MIXIN?) (CLASS-COLLECTION? :TYPE BOOLEAN :INITIALLY FALSE) (CLASS-CL-STRUCT? :TYPE BOOLEAN :INITIALLY FALSE :RENAMES CL-STRUCT?) (CLASS-CL-STRUCT-SLOTS :TYPE (LIST OF STORAGE-SLOT) :ALLOCATION :DYNAMIC) (CLASS-PUBLIC? :TYPE BOOLEAN :INITIALLY TRUE :OPTION-KEYWORD :PUBLIC?) (CLASS-RECYCLE-METHOD :TYPE KEYWORD :ALLOCATION :DYNAMIC :DEFAULT :NONE :OPTION-KEYWORD :RECYCLE-METHOD) (CLASS-FINALIZED? :TYPE BOOLEAN :INITIALLY FALSE :PUBLIC? TRUE) (CLASS-SLOTS-FINALIZED? :TYPE BOOLEAN :INITIALLY FALSE) (CLASS-STRINGIFIED-SOURCE :TYPE STRING) (CLASS-PARAMETERS :TYPE (LIST OF SYMBOL) :ALLOCATION :DYNAMIC :DEFAULT NIL-LIST :OPTION-KEYWORD :PARAMETERS) (CLASS-CONSTRUCTOR-CODE :TYPE FUNCTION-CODE) (CLASS-SLOT-ACCESSOR-CODE :TYPE FUNCTION-CODE) (CLASS-CREATOR :TYPE SYMBOL :ALLOCATION :DYNAMIC :OPTION-KEYWORD :CREATOR) (CLASS-INITIALIZER :TYPE SYMBOL :ALLOCATION :DYNAMIC :OPTION-KEYWORD :INITIALIZER) (CLASS-TERMINATOR :TYPE SYMBOL :ALLOCATION :DYNAMIC :OPTION-KEYWORD :TERMINATOR) (CLASS-DESTRUCTOR :TYPE SYMBOL :ALLOCATION :DYNAMIC :OPTION-KEYWORD :DESTRUCTOR) (CLASS-DOCUMENTATION :ALLOCATION :DYNAMIC :RENAMES DOCUMENTATION :OPTION-KEYWORD :DOCUMENTATION) (CLASS-EXTENSION-NAME :TYPE SYMBOL :ALLOCATION :DYNAMIC :OPTION-KEYWORD :EXTENSION) (CLASS-EXTENSION :TYPE CLASS-EXTENSION :ALLOCATION :DYNAMIC) (CLASS-REQUIRED-SLOT-NAMES :TYPE (LIST OF SYMBOL) :ALLOCATION :DYNAMIC :DEFAULT NIL-LIST) (CLASS-GUARD-CONSTRUCTOR-DEMONS :TYPE (LIST OF DEMON) :ALLOCATION :DYNAMIC) (CLASS-CONSTRUCTOR-DEMONS :TYPE (LIST OF DEMON) :ALLOCATION :DYNAMIC) (CLASS-GUARD-DESTRUCTOR-DEMONS :TYPE (LIST OF DEMON) :ALLOCATION :DYNAMIC) (CLASS-DESTRUCTOR-DEMONS :TYPE (LIST OF DEMON) :ALLOCATION :DYNAMIC) (CLASS-INITIAL-VALUE :TYPE OBJECT :ALLOCATION :DYNAMIC :OPTION-KEYWORD :INITIAL-VALUE) (CLASS-PRINT-FORM :TYPE OBJECT :RENAMES PRINT-FORM :OPTION-KEYWORD :PRINT-FORM) (CLASS-KEY :TYPE (LIST OF SLOT) :ALLOCATION :DYNAMIC :DEFAULT NIL-LIST) (CLASS-SYNONYMS :TYPE (LIST OF TYPE) :ALLOCATION :DYNAMIC :DEFAULT NIL-LIST :OPTION-KEYWORD :SYNONYMS) (CLASS-IS-LINK :TYPE TYPE :ALLOCATION :DYNAMIC) (CLASS-INVERSE-IS-LINK :TYPE CLASS :ALLOCATION :DYNAMIC) (CLASS-CL-NATIVE-TYPE :TYPE STRING :ALLOCATION :DYNAMIC :OPTION-KEYWORD :CL-NATIVE-TYPE) (CLASS-CPP-NATIVE-TYPE :TYPE STRING :ALLOCATION :DYNAMIC :OPTION-KEYWORD :CPP-NATIVE-TYPE) (CLASS-JAVA-NATIVE-TYPE :TYPE STRING :ALLOCATION :DYNAMIC :OPTION-KEYWORD :JAVA-NATIVE-TYPE) (CLASS-IDL-NATIVE-TYPE :TYPE STRING :ALLOCATION :DYNAMIC :OPTION-KEYWORD :IDL-NATIVE-TYPE) (CLASS-MARKED? :TYPE BOOLEAN) (CLASS-PROTOTYPE :TYPE OBJECT :ALLOCATION :DYNAMIC) (CLASS-TAXONOMY-NODE :TYPE TAXONOMY-NODE)) :PUBLIC-METHODS ((NAME ((SELF CLASS)) :TYPE STRING (RETURN (SYMBOL-NAME (CLASS-TYPE SELF)))) (HOME-MODULE ((SELF CLASS)) :TYPE MODULE (RETURN (INTERNED-IN (CLASS-TYPE SELF)))) (ARITY ((SELF CLASS)) :TYPE INTEGER (RETURN 1)) (PUBLIC? ((SELF CLASS)) :TYPE BOOLEAN :STORAGE-SLOT CLASS-PUBLIC? (RETURN (CLASS-PUBLIC? SELF))) (ACTIVE? ((SELF CLASS)) :TYPE BOOLEAN :STORAGE-SLOT STORED-ACTIVE? (RETURN (OR (AND (DEFINED? (STORED-ACTIVE? SELF)) (STORED-ACTIVE? SELF)) (SUBTYPE-OF? (CLASS-TYPE SELF) @ACTIVE-OBJECT)))) (CREATOR ((SELF CLASS)) :TYPE SYMBOL :STORAGE-SLOT CLASS-CREATOR (RETURN (CLASS-CREATOR SELF))) (INITIALIZER ((SEL", "F CLASS)) :TYPE SYMBOL :STORAGE-SLOT CLASS-INITIALIZER :INHERITS-THROUGH SUPER-CLASSES) (TERMINATOR ((SELF CLASS)) :TYPE SYMBOL :STORAGE-SLOT CLASS-TERMINATOR :INHERITS-THROUGH SUPER-CLASSES) (DESTRUCTOR ((SELF CLASS)) :TYPE SYMBOL :STORAGE-SLOT CLASS-DESTRUCTOR (RETURN (CLASS-DESTRUCTOR SELF))) (REQUIRED-SLOTS ((SELF CLASS)) :TYPE (LIST OF SYMBOL) :DOCUMENTATION \"Returns a list of names of required slots for 'self'.\" (RETURN (CLASS-REQUIRED-SLOT-NAMES SELF))) (PARAMETERS ((SELF CLASS)) :TYPE (LIST OF SYMBOL) :DOCUMENTATION \"Returns the list of parameters names of 'self'.\" :STORAGE-SLOT CLASS-PARAMETERS :INHERITS-THROUGH SUPER-CLASSES) (INITIAL-VALUE ((SELF CLASS)) :TYPE OBJECT :DOCUMENTATION \"Return an initial value for the class 'self'.\" :STORAGE-SLOT CLASS-INITIAL-VALUE :INHERITS-THROUGH SUPER-CLASSES) (EXTENSION ((SELF CLASS)) :TYPE CLASS-EXTENSION :DOCUMENTATION \"Return the nearest class extension that records instances\nof the class 'self'.\" :STORAGE-SLOT CLASS-EXTENSION :INHERITS-THROUGH SUPER-CLASSES) (CL-NATIVE-TYPE ((SELF CLASS)) :TYPE STRING :STORAGE-SLOT CLASS-CL-NATIVE-TYPE (RETURN (CLASS-CL-NATIVE-TYPE SELF))) (CPP-NATIVE-TYPE ((SELF CLASS)) :TYPE STRING :STORAGE-SLOT CLASS-CPP-NATIVE-TYPE (RETURN (CLASS-CPP-NATIVE-TYPE SELF))) (IDL-NATIVE-TYPE ((SELF CLASS)) :TYPE STRING :STORAGE-SLOT CLASS-IDL-NATIVE-TYPE (RETURN (CLASS-CPP-NATIVE-TYPE SELF))) (JAVA-NATIVE-TYPE ((SELF CLASS)) :TYPE STRING :STORAGE-SLOT CLASS-JAVA-NATIVE-TYPE (LET ((NATIVETYPE (CLASS-JAVA-NATIVE-TYPE SELF))) (IF (DEFINED? NATIVETYPE) (RETURN (SUBSTITUTE-TEMPLATE-VARIABLES-IN-STRING NATIVETYPE *JAVA-STELLA-PACKAGE-MAPPING*)) (RETURN NULL))))) :PRINT-FORM (PRINT-NATIVE-STREAM STREAM \"|C|\" (CHOOSE (DEFINED? (CLASS-TYPE SELF)) (RELATIVE-NAME (CLASS-SYMBOL SELF)) \"??\")))", 0));

      oHARDWIRED_CLASS_ARITY_ON_CLASSo = 1;
      renamed_Class->class_constructor_code = ((cpp_function_code)(&new_class));
      renamed_Class->class_slot_accessor_code = ((cpp_function_code)(&access_class_slot_value));
    }
    { Class* renamed_Class = define_class_from_stringified_source("POLYMORPHIC-RELATION", "(DEFCLASS POLYMORPHIC-RELATION (RELATION) :ABSTRACT? TRUE :PUBLIC-SLOTS ((SLOT-DIRECT-EQUIVALENT :TYPE SLOT)) :PUBLIC-METHODS ((OWNER ((SELF POLYMORPHIC-RELATION)) :TYPE TYPE (RETURN NULL)) (RENAMES ((SELF POLYMORPHIC-RELATION)) :TYPE SYMBOL (RETURN NULL)) (ACTIVE? ((SELF POLYMORPHIC-RELATION)) :TYPE BOOLEAN :STORAGE-SLOT STORED-ACTIVE? :INHERITS-THROUGH EQUIVALENT-SLOT :DOCUMENTATION \"True if 'self' or a superslot of 'self' is marked active.\")))");

      renamed_Class->class_slot_accessor_code = ((cpp_function_code)(&access_polymorphic_relation_slot_value));
    }
    { Class* renamed_Class = define_class_from_stringified_source("SLOT", "(DEFCLASS SLOT (POLYMORPHIC-RELATION) :ABSTRACT? TRUE :PUBLIC-METHODS ((NAME ((SELF SLOT)) :TYPE STRING (RETURN (SYMBOL-NAME (SLOT-NAME SELF)))) (TYPE ((SELF SLOT)) :TYPE TYPE :STORAGE-SLOT SLOT-BASE-TYPE :INHERITS-THROUGH EQUIVALENT-SLOT :DOCUMENTATION \"The type of a storage slot is its base type.\") (TYPE-SPECIFIER ((SELF SLOT)) :TYPE TYPE-SPEC :DOCUMENTATION \"If 'self' has a complex type return its\ntype specifier, otherwise, return 'type' of 'self'.\" (LET ((TSPEC (SLOT-TYPE-SPECIFIER SELF))) (RETURN (CHOOSE (DEFINED? TSPEC) TSPEC (TYPE SELF))))) (OWNER ((SELF SLOT)) :TYPE TYPE :STORAGE-SLOT SLOT-OWNER (RETURN (SLOT-OWNER SELF))) (HOME-MODULE ((SELF SLOT)) :TYPE MODULE (LET ((MODULE (SLOT-HOME-MODULE SELF))) (IF (DEFINED? MODULE) (RETURN MODULE) (RETURN (INTERNED-IN (SLOT-NAME SELF)))))) (RENAMES ((SELF SLOT)) :TYPE SYMBOL :STORAGE-SLOT SLOT-RENAMES (RETURN (SLOT-RENAMES SELF))) (COLLECTION-VALUED? ((SELF SLOT)) :TYPE BOOLEAN :DOCUMENTATION \"True if slot values are collections.\" (RETURN (SUBTYPE-OF? (SLOT-BASE-TYPE SELF) @COLLECTION))) (CLOSURE-ASSUMPTION ((SELF SLOT)) :TYPE KEYWORD :STORAGE-SLOT SLOT-CLOSURE-ASSUMPTION :INHERITS-THROUGH EQUIVALENT-SLOT)) :PUBLIC-SLOTS ((INVERSE :TYPE SLOT :ALLOCATION :DYNAMIC) (SLOT-NAME :TYPE SYMBOL) (SLOT-OWNER :TYPE TYPE) (SLOT-BASE-TYPE :TYPE TYPE) (SLOT-SLOTREF :TYPE SLOTREF) (SLOT-DIRECT-SUPERS :TYPE (LIST OF SLOT) :ALLOCATION :DYNAMIC) (SLOT-DIRECT-SUBS :TYPE (LIST OF SLOT) :ALLOCATION :DYNAMIC) (SLOT-HOME-MODULE :TYPE MODULE :ALLOCATION :DYNAMIC) (SLOT-TYPE-SPECIFIER :TYPE COMPOUND-TYPE-SPECIFIER :ALLOCATION :DYNAMIC) (SLOT-PUBLIC? :TYPE BOOLEAN :INITIALLY TRUE :OPTION-KEYWORD :PUBLIC?) (SLOT-INVERSE :TYPE SYMBOL :ALLOCATION :DYNAMIC :OPTION-KEYWORD :INVERSE) (SLOT-RENAMES :TYPE SYMBOL :ALLOCATION :DYNAMIC :OPTION-KEYWORD :RENAMES) (SLOT-RENAMED? :TYPE BOOLEAN :INITIALLY FALSE) (SLOT-DOCUMENTATION :TYPE STRING :ALLOCATION :DYNAMIC :RENAMES DOCUMENTATION :OPTION-KEYWORD :DOCUMENTATION) (SLOT-EXTERNAL? :TYPE BOOLEAN :INITIALLY FALSE) (SLOT-MARKED? :TYPE BOOLEAN) (SLOT-AUXILIARY? :TYPE BOOLEAN :ALLOCATION :DYNAMIC :OPTION-KEYWORD :AUXILIARY?) (SLOT-CLOSURE-ASSUMPTION :TYPE KEYWORD :ALLOCATION :DYNAMIC)) :PRINT-FORM (PRINT-NATIVE-STREAM STREAM \"|S|\" (SYMBOL-NAME (SLOT-OWNER SELF)) \".\" (SYMBOL-NAME (SLOT-NAME SELF))))");

      renamed_Class->class_slot_accessor_code = ((cpp_function_code)(&access_slot_slot_value));
    }
    { Class* renamed_Class = define_class_from_stringified_source("STORAGE-SLOT", "(DEFCLASS STORAGE-SLOT (SLOT) :PUBLIC-METHODS ((INITIALLY ((SELF STORAGE-SLOT)) :TYPE OBJECT) (ALLOCATION ((SELF STORAGE-SLOT)) :TYPE KEYWORD :STORAGE-SLOT SLOT-ALLOCATION :INHERITS-THROUGH EQUIVALENT-SLOT :DOCUMENTATION \"Return the most specific :allocation facet, or\n:instance if all inherited values are NULL.\") (DEFAULT-FORM ((SELF STORAGE-SLOT)) :TYPE OBJECT :STORAGE-SLOT SLOT-DEFAULT-EXPRESSION :INHERITS-THROUGH EQUIVALENT-SLOT :DOCUMENTATION \"Returns the current value of default expression when the\nslot has not been assigned a value.\") (REQUIRED? ((SELF STORAGE-SLOT)) :TYPE BOOLEAN :STORAGE-SLOT SLOT-REQUIRED? :INHERITS-THROUGH EQUIVALENT-SLOT :DOCUMENTATION \"True if a value must be assigned to this slot at\ncreation time.\") (COMPONENT? ((SELF STORAGE-SLOT)) :TYPE BOOLEAN :STORAGE-SLOT SLOT-COMPONENT? :INHERITS-THROUGH EQUIVALENT-SLOT :DOCUMENTATION \"True if fillers of this slot are components of the\nowner slot, and therefore should be deleted if the owner is deleted.\") (READER ((SELF STORAGE-SLOT)) :TYPE SYMBOL :STORAGE-SLOT SLOT-READER :INHERITS-THROUGH EQUIVALENT-SLOT :DOCUMENTATION \"Name of a method called to read the value of the slot\n'self'.\") (WRITER ((SELF STORAGE-SLOT)) :TYPE SYMBOL :STORAGE-SLOT SLOT-WRITER :INHERITS-THROUGH EQUIVALENT-SLOT :DOCUMENTATION \"Name of a method called to write the value of the slot\n'self'.\")) :PUBLIC-SLOTS ((SLOT-INITIAL-VALUE :TYPE OBJECT :ALLOCATION :DYNAMIC :OPTION-KEYWORD :INITIALLY) (SLOT-ALLOCATION :TYPE KEYWORD :ALLOCATION :DYNAMIC :DEFAULT :INSTANCE :OPTION-KEYWORD :ALLOCATION) (SLOT-DEFAULT-EXPRESSION :TYPE OBJECT :ALLOCATION :DYNAMIC :OPTION-KEYWORD :DEFAULT) (SLOT-GUARD-DEMONS :TYPE (LIST OF DEMON) :ALLOCATION :DYNAMIC) (SLOT-DEMONS :TYPE (LIST OF DEMON) :ALLOCATION :DYNAMIC) (SLOT-REQUIRED? :TYPE BOOLEAN :OPTION-KEYWORD :REQUIRED?) (SLOT-COMPONENT? :TYPE BOOLEAN :OPTION-KEYWORD :COMPONENT?) (SLOT-READ-ONLY? :TYPE BOOLEAN :OPTION-KEYWORD :READ-ONLY?) (SLOT-HARDWIRED? :TYPE BOOLEAN :OPTION-KEYWORD :HARDWIRED?) (SLOT-CONTEXT-SENSITIVE? :TYPE BOOLEAN :OPTION-KEYWORD :CONTEXT-SENSITIVE?) (SLOT-READER :TYPE SYMBOL :ALLOCATION :DYNAMIC :OPTION-KEYWORD :READER) (SLOT-WRITER :TYPE SYMBOL :ALLOCATION :DYNAMIC :OPTION-KEYWORD :WRITER) (SLOT-OPTION-KEYWORD :TYPE KEYWORD :ALLOCATION :DYNAMIC :OPTION-KEYWORD :OPTION-KEYWORD) (SLOT-OPTION-HANDLER :TYPE SYMBOL :ALLOCATION :DYNAMIC :OPTION-KEYWORD :OPTION-HANDLER)))");

      renamed_Class->class_constructor_code = ((cpp_function_code)(&new_storage_slot));
      renamed_Class->class_slot_accessor_code = ((cpp_function_code)(&access_storage_slot_slot_value));
    }
    { Class* renamed_Class = define_class_from_stringified_source("METHOD-SLOT", "(DEFCLASS METHOD-SLOT (SLOT) :SYNONYMS (FUNCTION) :PUBLIC-SLOTS ((STORAGE-SLOT :TYPE SYMBOL :ALLOCATION :DYNAMIC) (SLOT-REQUIRED? :TYPE BOOLEAN :ALLOCATION :DYNAMIC :DEFAULT FALSE) (METHOD-SETTER? :TYPE BOOLEAN) (METHOD-DOCUMENTATION :TYPE STRING :ALLOCATION :DYNAMIC :RENAMES DOCUMENTATION) (METHOD-PARAMETER-NAMES :TYPE (LIST OF SYMBOL) :DEFAULT NIL-LIST :COMPONENT? TRUE) (METHOD-PARAMETER-TYPE-SPECIFIERS :TYPE (LIST OF TYPE-SPEC) :DEFAULT NIL-LIST :COMPONENT? TRUE) (METHOD-PARAMETER-DIRECTIONS :TYPE (LIST OF SYMBOL) :DEFAULT NIL-LIST :COMPONENT? TRUE :ALLOCATION :DYNAMIC) (METHOD-RETURN-TYPE-SPECIFIERS :TYPE (LIST OF TYPE-SPEC) :DEFAULT NIL-LIST :COMPONENT? TRUE) (METHOD-STRINGIFIED-SOURCE :TYPE STRING) (METHOD-CODE :TYPE METHOD-CODE) (FUNCTION-CODE :TYPE FUNCTION-CODE) (METHOD-FUNCTION? :TYPE BOOLEAN) (METHOD-VARIABLE-ARGUMENTS? :TYPE BOOLEAN :ALLOCATION :DYNAMIC :DEFAULT FALSE) (METHOD-BODY-ARGUMENT? :TYPE BOOLEAN :ALLOCATION :DYNAMIC :DEFAULT FALSE) (METHOD-AUXILIARY? :RENAMES SLOT-AUXILIARY?) (METHOD-NATIVE? :TYPE BOOLEAN :ALLOCATION :DYNAMIC :DEFAULT FALSE) (METHOD-CONSTRUCTOR? :TYPE BOOLEAN :ALLOCATION :DYNAMIC :DEFAULT FALSE) (METHOD-GLOBALLY-INLINE? :TYPE BOOLEAN :ALLOCATION :DYNAMIC :DEFAULT FALSE) (METHOD-INLINED-FUNCTIONS :TYPE (LIST OF SYMBOL) :ALLOCATION :DYNAMIC :DEFAULT NIL-LIST) (METHOD-STORAGE-SLOT :TYPE SYMBOL :ALLOCATION :DYNAMIC :RENAMES STORAGE-SLOT) (METHOD-INHERITS-THROUGH :TYPE SYMBOL :ALLOCATION :DYNAMIC) (METHOD-COMMAND? :TYPE BOOLEAN :ALLOCATION :DYNAMIC :DEFAULT FALSE) (METHOD-EVALUATE-ARGUMENTS? :TYPE BOOLEAN :ALLOCATION :DYNAMIC :DEFAULT TRUE) (METHOD-LISP-MACRO? :TYPE BOOLEAN-WRAPPER :ALLOCATION :DYNAMIC) (EVALUATOR-WRAPPER-CODE :TYPE FUNCTION-CODE :ALLOCATION :DYNAMIC)) :PUBLIC-METHODS ((SETTER? ((SELF METHOD-SLOT)) :TYPE BOOLEAN :STORAGE-SLOT METHOD-SETTER? (RETURN (METHOD-SETTER? SELF)))) :PRINT-FORM (IF (METHOD-FUNCTION? SELF) (PRINT-NATIVE-STREAM STREAM \"|F|\" (SYMBOL-NAME (SLOT-NAME SELF))) (PRINT-NATIVE-STREAM STREAM \"|M|\" (SYMBOL-NAME (SLOT-OWNER SELF)) \".\" (SYMBOL-NAME (SLOT-NAME SELF)))))");

      renamed_Class->class_constructor_code = ((cpp_function_code)(&new_method_slot));
      renamed_Class->class_slot_accessor_code = ((cpp_function_code)(&access_method_slot_slot_value));
    }
    define_class_from_stringified_source("COMPOUND-TYPE-SPECIFIER", "(DEFCLASS COMPOUND-TYPE-SPECIFIER (STANDARD-OBJECT) :ABSTRACT? TRUE :PUBLIC? FALSE :PRINT-FORM (PRINT-NATIVE-STREAM STREAM \"|TS|\" (YIELD-TYPE-SPEC-TREE SELF)))");
    { Class* renamed_Class = define_class_from_stringified_source("PARAMETRIC-TYPE-SPECIFIER", "(DEFCLASS PARAMETRIC-TYPE-SPECIFIER (COMPOUND-TYPE-SPECIFIER) :PUBLIC-SLOTS ((SPECIFIER-BASE-TYPE :TYPE TYPE) (SPECIFIER-PARAMETER-TYPES :TYPE (LIST OF TYPE-SPEC) :ALLOCATION :EMBEDDED) (SPECIFIER-DIMENSIONS :TYPE (LIST OF OBJECT))))");

      renamed_Class->class_constructor_code = ((cpp_function_code)(&new_parametric_type_specifier));
      renamed_Class->class_slot_accessor_code = ((cpp_function_code)(&access_parametric_type_specifier_slot_value));
    }
    { Class* renamed_Class = define_class_from_stringified_source("ANCHORED-TYPE-SPECIFIER", "(DEFCLASS ANCHORED-TYPE-SPECIFIER (COMPOUND-TYPE-SPECIFIER) :PUBLIC-SLOTS ((SPECIFIER-PARAMETER-NAME :TYPE SYMBOL)))");

      renamed_Class->class_constructor_code = ((cpp_function_code)(&new_anchored_type_specifier));
      renamed_Class->class_slot_accessor_code = ((cpp_function_code)(&access_anchored_type_specifier_slot_value));
    }
    { Class* renamed_Class = define_class_from_stringified_source("TABLE", "(DEFCLASS TABLE (SLOT) :PARAMETERS ((ANY-VALUE :TYPE BOOLEAN)) :PUBLIC-SLOTS ((TUPLE-DOMAINS :TYPE (LIST OF TYPE-SPEC)) (VARIABLE-ARITY-TABLE? :TYPE BOOLEAN :ALLOCATION :DYNAMIC)) :PRINT-FORM (PRINT-NATIVE-STREAM STREAM \"|T|\" (SYMBOL-NAME (SLOT-NAME SELF))))");

      renamed_Class->class_constructor_code = ((cpp_function_code)(&new_table));
      renamed_Class->class_slot_accessor_code = ((cpp_function_code)(&access_table_slot_value));
    }
    { Class* renamed_Class = define_class_from_stringified_source("GLOBAL-VARIABLE", "(DEFCLASS GLOBAL-VARIABLE (MAPPABLE-OBJECT) :PUBLIC-SLOTS ((DOCUMENTATION :TYPE STRING)) :PUBLIC-METHODS ((NAME ((SELF GLOBAL-VARIABLE)) :TYPE STRING (RETURN (SYMBOL-NAME (VARIABLE-NAME SELF)))) (TYPE ((SELF GLOBAL-VARIABLE)) :TYPE TYPE :STORAGE-SLOT VARIABLE-TYPE (RETURN (VARIABLE-TYPE SELF))) (HOME-MODULE ((SELF GLOBAL-VARIABLE)) :TYPE MODULE (LET ((MODULE (VARIABLE-HOME-MODULE SELF))) (IF (DEFINED? MODULE) (RETURN MODULE) (RETURN (INTERNED-IN (VARIABLE-NAME SELF))))))) :PUBLIC-SLOTS ((VARIABLE-NAME :TYPE SYMBOL) (VARIABLE-TYPE :TYPE TYPE) (VARIABLE-TYPE-SPECIFIER :TYPE TYPE-SPEC :ALLOCATION :DYNAMIC) (VARIABLE-HOME-MODULE :TYPE MODULE :ALLOCATION :DYNAMIC) (VARIABLE-SPECIAL? :TYPE BOOLEAN) (VARIABLE-CONSTANT? :TYPE BOOLEAN) (VARIABLE-PUBLIC? :TYPE BOOLEAN :INITIALLY TRUE) (VARIABLE-AUXILIARY? :TYPE BOOLEAN) (VARIABLE-GET-VALUE-CODE :TYPE FUNCTION-CODE) (VARIABLE-SET-VALUE-CODE :TYPE FUNCTION-CODE) (VARIABLE-VALUE-STACK :TYPE LIST) (VARIABLE-DOCUMENTATION :TYPE STRING :RENAMES DOCUMENTATION) (VARIABLE-STRINGIFIED-SOURCE :TYPE STRING)))");

      renamed_Class->class_constructor_code = ((cpp_function_code)(&new_global_variable));
      renamed_Class->class_slot_accessor_code = ((cpp_function_code)(&access_global_variable_slot_value));
    }
    { Class* renamed_Class = define_class_from_stringified_source("QUOTED-EXPRESSION", "(DEFCLASS QUOTED-EXPRESSION (STANDARD-OBJECT) :PUBLIC-SLOTS ((QUOTATION-TABLE-OFFSET :TYPE INTEGER)))");

      renamed_Class->class_constructor_code = ((cpp_function_code)(&new_quoted_expression));
      renamed_Class->class_slot_accessor_code = ((cpp_function_code)(&access_quoted_expression_slot_value));
    }
    define_class_from_stringified_source("VOID", "(DEFCLASS VOID () :ABSTRACT? TRUE :JAVA-NATIVE-TYPE \"void\" :IDL-NATIVE-TYPE \"void\" :CPP-NATIVE-TYPE \"void\")");
    define_class_from_stringified_source("UNKNOWN", "(DEFCLASS UNKNOWN () :ABSTRACT? TRUE :CPP-NATIVE-TYPE \"unknown\" :IDL-NATIVE-TYPE \"unknown\" :JAVA-NATIVE-TYPE \"unknown\" :SLOTS ((UNKNOWN-SLOT :TYPE UNKNOWN)))");
    { Class* renamed_Class = define_class_from_stringified_source("THING", "(DEFCLASS THING (STANDARD-OBJECT DYNAMIC-SLOTS-MIXIN) :ABSTRACT? TRUE :DOCUMENTATION \"Defines a class that must be inherited by any\nclass that participates in the PowerLoom side of things.\" :PUBLIC-SLOTS ((SURROGATE-VALUE-INVERSE :TYPE SURROGATE)) :PRINT-FORM (IF (DEFINED? (SURROGATE-VALUE-INVERSE SELF)) (PRINT-NATIVE-STREAM STREAM \"|i|\" (SYMBOL-NAME (SURROGATE-VALUE-INVERSE SELF))) (PRINT-NATIVE-STREAM STREAM \"|i|\" (PRIMARY-TYPE SELF))))");

      renamed_Class->class_slot_accessor_code = ((cpp_function_code)(&access_thing_slot_value));
    }
    { Class* renamed_Class = define_class_from_stringified_source("CONTEXT", "(DEFCLASS CONTEXT (THING) :ABSTRACT? TRUE :PUBLIC-SLOTS ((CHILD-CONTEXTS :TYPE (LIST OF CONTEXT) :ALLOCATION :EMBEDDED) (BASE-MODULE :TYPE MODULE) (ALL-SUPER-CONTEXTS :TYPE (CONS OF CONTEXT)) (CONTEXT-NUMBER :TYPE INTEGER)) :METHODS ((CONTEXT-NAME ((SELF CONTEXT)) :TYPE STRING (LET ((AUX SELF)) (TYPECASE AUX (MODULE (RETURN (MODULE-NAME AUX))) (WORLD (RETURN (WORLD-NAME AUX)))))) (PARENT-CONTEXTS ((SELF CONTEXT)) :TYPE (ITERATOR OF CONTEXT)) (HOME-MODULE ((SELF CONTEXT)) :TYPE MODULE :PUBLIC? TRUE (RETURN (BASE-MODULE SELF)))) :PRINT-FORM (PRINT-CONTEXT SELF STREAM))");

      renamed_Class->class_slot_accessor_code = ((cpp_function_code)(&access_context_slot_value));
    }
    { Class* renamed_Class = define_class_from_stringified_source("MODULE", "(DEFCLASS MODULE (CONTEXT) :PUBLIC-SLOTS ((PARENT-MODULES :TYPE (LIST OF MODULE) :ALLOCATION :EMBEDDED) (DOCUMENTATION :TYPE STRING) (NICKNAMES :TYPE (LIST OF STRING-WRAPPER)) (USES :TYPE (LIST OF MODULE) :ALLOCATION :EMBEDDED) (USED-BY :TYPE (LIST OF MODULE) :ALLOCATION :EMBEDDED) (SHADOWED-SURROGATES :TYPE (LIST OF SURROGATE) :ALLOCATION :DYNAMIC) (REQUIRES :TYPE (LIST OF MODULE)) (MODULE-LISP-PACKAGE :TYPE STRING :DEFAULT \"STELLA\" :ALLOCATION :DYNAMIC) (MODULE-CPP-PACKAGE :TYPE STRING :DEFAULT \"stella\" :ALLOCATION :DYNAMIC) (JAVA-PACKAGE :TYPE STRING :DEFAULT NULL :ALLOCATION :DYNAMIC) (CASE-SENSITIVE? :TYPE BOOLEAN :INITIALLY FALSE) (CLEARABLE? :TYPE BOOLEAN :ALLOCATION :DYNAMIC :DEFAULT TRUE :OPTION-KEYWORD :CLEARABLE?) (CODE-ONLY? :TYPE BOOLEAN :ALLOCATION :DYNAMIC) (API? :TYPE BOOLEAN :ALLOCATION :DYNAMIC) (PROTECT-SURROGATES? :TYPE BOOLEAN :ALLOCATION :DYNAMIC :OPTION-KEYWORD :PROTECT-SURROGATES?) (JAVA-FLOTSAM-CLASS :TYPE STRING :ALLOCATION :DYNAMIC :OPTION-KEYWORD :JAVA-CATCHALL-CLASS) (MODULE-FULL-NAME :TYPE STRING) (MODULE-NAME :TYPE STRING :ALLOCATION :INSTANCE) (MODULE-STRINGIFIED-SOURCE :TYPE STRING) (STRINGIFIED-OPTIONS :TYPE STRING) (CARDINAL-MODULE :TYPE MODULE) (SYMBOL-OFFSET-TABLE :TYPE STRING-TO-INTEGER-HASH-TABLE) (SURROGATE-OFFSET-TABLE :TYPE STRING-TO-INTEGER-HASH-TABLE) (STRICT-INFERENCE-CACHE :TYPE WORLD) (DEFAULT-INFERENCE-CACHE :TYPE WORLD) (PROTOTYPE-INFERENCE-CACHE :TYPE WORLD)) :PUBLIC-METHODS ((NAME ((SELF MODULE)) :TYPE STRING (RETURN (MODULE-NAME SELF))) (PARENT-MODULE ((SELF MODULE)) :TYPE MODULE (RETURN (POP (PARENT-CONTEXTS SELF)))) (LISP-PACKAGE ((SELF MODULE)) :TYPE STRING :STORAGE-SLOT MODULE-LISP-PACKAGE :INHERITS-THROUGH PARENT-MODULES) (CPP-PACKAGE ((SELF MODULE)) :TYPE STRING :STORAGE-SLOT MODULE-CPP-PACKAGE :INHERITS-THROUGH PARENT-MODULES)))");

      renamed_Class->class_constructor_code = ((cpp_function_code)(&new_module));
      renamed_Class->class_slot_accessor_code = ((cpp_function_code)(&access_module_slot_value));
    }
    { Class* renamed_Class = define_class_from_stringified_source("WORLD", "(DEFCLASS WORLD (CONTEXT) :PUBLIC-SLOTS ((PARENT-CONTEXT :TYPE CONTEXT) (CHILD-CONTEXTS :TYPE (LIST OF WORLD) :ALLOCATION :EMBEDDED) (WORLD-NAME :TYPE STRING :ALLOCATION :DYNAMIC)))");

      renamed_Class->class_constructor_code = ((cpp_function_code)(&new_world));
      renamed_Class->class_slot_accessor_code = ((cpp_function_code)(&access_world_slot_value));
    }
    { Class* renamed_Class = define_class_from_stringified_source("CS-VALUE", "(DEFCLASS CS-VALUE (KEY-VALUE-LIST) :DOCUMENTATION \"Contextualized value.  Contains a sorted kv-cons list indexed\nby context. The kv-cons list is never null.  Newer (higher numbered) contexts\nappear first.\" :PARAMETERS ((ANY-KEY :TYPE CONTEXT) (ANY-VALUE :TYPE OBJECT)))");

      renamed_Class->class_constructor_code = ((cpp_function_code)(&new_cs_value));
    }
    { Class* renamed_Class = define_class_from_stringified_source("INTERVAL", "(DEFCLASS INTERVAL (STANDARD-OBJECT) :PUBLIC-SLOTS ((LOWER-BOUND :TYPE INTEGER) (UPPER-BOUND :TYPE INTEGER)) :PRINT-FORM (PRINT-NATIVE-STREAM STREAM \"[\" (LOWER-BOUND SELF) \",\" (UPPER-BOUND SELF) \"]\"))");

      renamed_Class->class_constructor_code = ((cpp_function_code)(&new_interval));
      renamed_Class->class_slot_accessor_code = ((cpp_function_code)(&access_interval_slot_value));
    }
    { Class* renamed_Class = define_class_from_stringified_source("TAXONOMY-NODE", "(DEFCLASS TAXONOMY-NODE (STANDARD-OBJECT) :PUBLIC-SLOTS ((NATIVE-OBJECT :TYPE OBJECT) (LABEL :TYPE INTEGER) (INTERVALS :TYPE (CONS OF INTERVAL) :INITIALLY NIL) (INITIAL-INTERVAL :TYPE INTERVAL) (FIRST-INTERVAL-LOWER-BOUND :TYPE INTEGER :INITIALLY -1) (FIRST-INTERVAL-UPPER-BOUND :TYPE INTEGER :INITIALLY -1) (PARENTS :TYPE (CONS OF TAXONOMY-NODE) :INITIALLY NIL) (CHILDREN :TYPE (CONS OF TAXONOMY-NODE) :INITIALLY NIL) (TOTAL-ANCESTORS :TYPE INTEGER :INITIALLY 1) (TREE-CHILDREN :TYPE (CONS OF TAXONOMY-NODE) :INITIALLY NIL) (MARKER :TYPE OBJECT)) :PRINT-FORM (PRINT-NATIVE-STREAM STREAM \"|TN|\" (LABEL SELF) \" \" (INTERVALS SELF) \" \" (NATIVE-OBJECT SELF)))");

      renamed_Class->class_constructor_code = ((cpp_function_code)(&new_taxonomy_node));
      renamed_Class->class_slot_accessor_code = ((cpp_function_code)(&access_taxonomy_node_slot_value));
    }
    { Class* renamed_Class = define_class_from_stringified_source("TAXONOMY-GRAPH", "(DEFCLASS TAXONOMY-GRAPH (STANDARD-OBJECT) :PUBLIC-SLOTS ((RENUMBER-IF-OUT-OF-NUMBERS? :DOCUMENTATION \"If TRUE, relabel the entire graph when a new interval\nfor an incrementally inserted node can't be allocated due to lack of\nsplittable interval space.  Otherwise, create a `foreign' interval and\npropagate it.\" :TYPE BOOLEAN :INITIALLY FALSE) (RENUMBER-RATIO :DOCUMENTATION \"Whenever the ratio between foreign intervals and\nnon-foreign intervals becomes greater than `renumber-ratio', relabel\nthe whole graph non-incrementally.\" :TYPE FLOAT :INITIALLY 0.1)) :SLOTS ((INCREMENTAL-MODE? :TYPE BOOLEAN) (LARGEST-POSTORDER-NUMBER :TYPE INTEGER) (NUMBER-OF-NODES :TYPE INTEGER) (NUMBER-OF-FOREIGN-INTERVAL-NODES :TYPE INTEGER) (ROOTS :TYPE (LIST OF TAXONOMY-NODE)) (BROKEN-LINKS :TYPE (LIST OF (CONS OF TAXONOMY-NODE))) (ADDED-LINKS :TYPE (LIST OF (CONS OF TAXONOMY-NODE))) (REMOVED-LINKS :TYPE (LIST OF (CONS OF TAXONOMY-NODE)))) :INITIALIZER INITIALIZE-TAXONOMY-GRAPH :PRINT-FORM (PRINT-NATIVE-STREAM STREAM \"|TG|\" (ROOTS SELF)))");

      renamed_Class->class_constructor_code = ((cpp_function_code)(&new_taxonomy_graph));
      renamed_Class->class_slot_accessor_code = ((cpp_function_code)(&access_taxonomy_graph_slot_value));
    }
    define_class_from_stringified_source("NATIVE-EXCEPTION", "(DEFCLASS NATIVE-EXCEPTION () :JAVA-NATIVE-TYPE \"java.lang.Exception\" :CL-NATIVE-TYPE \"CONDITION\" :CPP-NATIVE-TYPE \"exception\")");
    define_class_from_stringified_source("STELLA-ROOT-EXCEPTION", "(DEFCLASS STELLA-ROOT-EXCEPTION (NATIVE-EXCEPTION) :PUBLIC-SLOTS ((MESSAGE :TYPE STRING :READER EXCEPTION-MESSAGE :REQUIRED? TRUE :ABSTRACT? TRUE)) :ABSTRACT? TRUE :JAVA-NATIVE-TYPE \"java.lang.RuntimeException\" :CL-NATIVE-TYPE \"SIMPLE-ERROR\" :CPP-NATIVE-TYPE \"runtime_error\")");
    { Class* renamed_Class = define_class_from_stringified_source("STELLA-EXCEPTION", "(DEFCLASS STELLA-EXCEPTION (STELLA-ROOT-EXCEPTION))");

      renamed_Class->class_constructor_code = ((cpp_function_code)(&new_stella_exception));
    }
    { Class* renamed_Class = define_class_from_stringified_source("READ-EXCEPTION", "(DEFCLASS READ-EXCEPTION (STELLA-EXCEPTION))");

      renamed_Class->class_constructor_code = ((cpp_function_code)(&new_read_exception));
    }
    { Class* renamed_Class = define_class_from_stringified_source("UNHANDLED-EXCEPTION", "(DEFCLASS UNHANDLED-EXCEPTION (STELLA-EXCEPTION))");

      renamed_Class->class_constructor_code = ((cpp_function_code)(&new_unhandled_exception));
    }
    { Class* renamed_Class = define_class_from_stringified_source("EVALUATION-EXCEPTION", "(DEFCLASS EVALUATION-EXCEPTION (STELLA-EXCEPTION))");

      renamed_Class->class_constructor_code = ((cpp_function_code)(&new_evaluation_exception));
    }
    { Class* renamed_Class = define_class_from_stringified_source("NO-SUCH-OBJECT-EXCEPTION", "(DEFCLASS NO-SUCH-OBJECT-EXCEPTION (STELLA-EXCEPTION))");

      renamed_Class->class_constructor_code = ((cpp_function_code)(&new_no_such_object_exception));
    }
    { Class* renamed_Class = define_class_from_stringified_source("NO-SUCH-CONTEXT-EXCEPTION", "(DEFCLASS NO-SUCH-CONTEXT-EXCEPTION (NO-SUCH-OBJECT-EXCEPTION))");

      renamed_Class->class_constructor_code = ((cpp_function_code)(&new_no_such_context_exception));
    }
    define_class_from_stringified_source("LITERAL", "(DEFCLASS LITERAL (NON-OBJECT) :ABSTRACT? TRUE)");
    define_class_from_stringified_source("NUMBER", "(DEFCLASS NUMBER (LITERAL) :ABSTRACT? TRUE)");
    define_class_from_stringified_source("INTEGER", "(DEFCLASS INTEGER (NUMBER) :CL-NATIVE-TYPE \"INTEGER\" :CPP-NATIVE-TYPE \"int\" :IDL-NATIVE-TYPE \"long\" :JAVA-NATIVE-TYPE \"int\" :INITIAL-VALUE NULL-INTEGER :EQUALITY-TEST EQL?)");
    define_class_from_stringified_source("FLOAT", "(DEFCLASS FLOAT (NUMBER) :CL-NATIVE-TYPE \"FLOAT\" :CPP-NATIVE-TYPE \"double\" :IDL-NATIVE-TYPE \"double\" :JAVA-NATIVE-TYPE \"double\" :INITIAL-VALUE NULL-FLOAT :EQUALITY-TEST EQL?)");
    define_class_from_stringified_source("MUTABLE-STRING", "(DEFCLASS MUTABLE-STRING (LITERAL) :CL-NATIVE-TYPE \"STRING\" :IDL-NATIVE-TYPE \"string\" :JAVA-NATIVE-TYPE \"StringBuffer\" :CPP-NATIVE-TYPE \"char*\" :EQUALITY-TEST STRING-EQL? :INITIAL-VALUE (VERBATIM :COMMON-LISP \"STELLA::NULL-STRING\" :JAVA \"null\" :CPP \"NULL\"))");
    define_class_from_stringified_source("STRING", "(DEFCLASS STRING (LITERAL) :SYNONYMS (FILE-NAME) :CL-NATIVE-TYPE \"STRING\" :IDL-NATIVE-TYPE \"string\" :JAVA-NATIVE-TYPE \"String\" :CPP-NATIVE-TYPE \"char*\" :EQUALITY-TEST STRING-EQL? :INITIAL-VALUE (VERBATIM :COMMON-LISP \"STELLA::NULL-STRING\" :JAVA \"null\" :CPP \"NULL\"))");
    define_class_from_stringified_source("CHARACTER", "(DEFCLASS CHARACTER (LITERAL) :CL-NATIVE-TYPE \"CHARACTER\" :CPP-NATIVE-TYPE \"char\" :IDL-NATIVE-TYPE \"char\" :JAVA-NATIVE-TYPE \"char\" :INITIAL-VALUE NULL-CHARACTER)");
    define_class_from_stringified_source("SHORT-INTEGER", "(DEFCLASS SHORT-INTEGER (INTEGER) :CL-NATIVE-TYPE \"INTEGER\" :CPP-NATIVE-TYPE \"short int\" :IDL-NATIVE-TYPE \"short int\" :JAVA-NATIVE-TYPE \"short\" :INITIAL-VALUE NULL-SHORT-INTEGER :EQUALITY-TEST EQL?)");
    define_class_from_stringified_source("LONG-INTEGER", "(DEFCLASS LONG-INTEGER (INTEGER) :CL-NATIVE-TYPE \"INTEGER\" :CPP-NATIVE-TYPE \"long int\" :IDL-NATIVE-TYPE \"long int\" :JAVA-NATIVE-TYPE \"long\" :INITIAL-VALUE NULL-LONG-INTEGER :EQUALITY-TEST EQL?)");
    define_class_from_stringified_source("UNSIGNED-SHORT-INTEGER", "(DEFCLASS UNSIGNED-SHORT-INTEGER (INTEGER) :CL-NATIVE-TYPE \"INTEGER\" :CPP-NATIVE-TYPE \"unsigned short int\" :IDL-NATIVE-TYPE \"unsigned short int\" :JAVA-NATIVE-TYPE \"short\" :INITIAL-VALUE NULL-UNSIGNED-SHORT-INTEGER :EQUALITY-TEST EQL?)");
    define_class_from_stringified_source("UNSIGNED-LONG-INTEGER", "(DEFCLASS UNSIGNED-LONG-INTEGER (INTEGER) :CL-NATIVE-TYPE \"INTEGER\" :IDL-NATIVE-TYPE \"unsigned long int\" :CPP-NATIVE-TYPE \"unsigned long int\" :JAVA-NATIVE-TYPE \"long\" :INITIAL-VALUE NULL-UNSIGNED-LONG-INTEGER :EQUALITY-TEST EQL?)");
    define_class_from_stringified_source("SINGLE-FLOAT", "(DEFCLASS SINGLE-FLOAT (FLOAT) :CL-NATIVE-TYPE \"FLOAT\" :JAVA-NATIVE-TYPE \"float\" :IDL-NATIVE-TYPE \"float\" :CPP-NATIVE-TYPE \"float\" :INITIAL-VALUE NULL-SINGLE-FLOAT :EQUALITY-TEST EQL?)");
    define_class_from_stringified_source("DOUBLE-FLOAT", "(DEFCLASS DOUBLE-FLOAT (FLOAT) :CL-NATIVE-TYPE \"FLOAT\" :CPP-NATIVE-TYPE \"double\" :JAVA-NATIVE-TYPE \"double\" :IDL-NATIVE-TYPE \"double\" :INITIAL-VALUE NULL-DOUBLE-FLOAT :EQUALITY-TEST EQL?)");
    define_class_from_stringified_source("BYTE", "(DEFCLASS BYTE (LITERAL) :CL-NATIVE-TYPE \"FIXNUM\" :CPP-NATIVE-TYPE \"char\" :IDL-NATIVE-TYPE \"char\" :JAVA-NATIVE-TYPE \"byte\" :INITIAL-VALUE NULL-BYTE :EQUALITY-TEST EQL?)");
    define_class_from_stringified_source("OCTET", "(DEFCLASS OCTET (BYTE) :CL-NATIVE-TYPE \"FIXNUM\" :CPP-NATIVE-TYPE \"char\" :IDL-NATIVE-TYPE \"octet\" :JAVA-NATIVE-TYPE \"byte\" :INITIAL-VALUE NULL-OCTET :EQUALITY-TEST EQL?)");
    define_class_from_stringified_source("WRAPPER", "(DEFCLASS WRAPPER (OBJECT) :PUBLIC-SLOTS ((WRAPPER-VALUE :TYPE UNKNOWN :REQUIRED? TRUE)) :KEY (WRAPPER-VALUE) :PUBLIC? TRUE :ABSTRACT? TRUE)");
    define_class_from_stringified_source("LITERAL-WRAPPER", "(DEFCLASS LITERAL-WRAPPER (WRAPPER) :ABSTRACT? TRUE)");
    define_class_from_stringified_source("NUMBER-WRAPPER", "(DEFCLASS NUMBER-WRAPPER (LITERAL-WRAPPER) :ABSTRACT? TRUE)");
    { Class* renamed_Class = define_class_from_stringified_source("INTEGER-WRAPPER", "(DEFCLASS INTEGER-WRAPPER (NUMBER-WRAPPER) :PUBLIC-SLOTS ((WRAPPER-VALUE :TYPE INTEGER)) :PRINT-FORM (IF *PRINTREADABLY?* (PRINT-NATIVE-STREAM STREAM (WRAPPER-VALUE SELF)) (PRINT-NATIVE-STREAM STREAM \"|L|\" (WRAPPER-VALUE SELF))))");

      renamed_Class->class_constructor_code = ((cpp_function_code)(&new_integer_wrapper));
      renamed_Class->class_slot_accessor_code = ((cpp_function_code)(&access_integer_wrapper_slot_value));
    }
    { Class* renamed_Class = define_class_from_stringified_source("FLOAT-WRAPPER", "(DEFCLASS FLOAT-WRAPPER (NUMBER-WRAPPER) :PUBLIC-SLOTS ((WRAPPER-VALUE :TYPE FLOAT)) :PRINT-FORM (IF *PRINTREADABLY?* (PRINT-NATIVE-STREAM STREAM (WRAPPER-VALUE SELF)) (PRINT-NATIVE-STREAM STREAM \"|L|\" (WRAPPER-VALUE SELF))))");

      renamed_Class->class_constructor_code = ((cpp_function_code)(&new_float_wrapper));
      renamed_Class->class_slot_accessor_code = ((cpp_function_code)(&access_float_wrapper_slot_value));
    }
    { Class* renamed_Class = define_class_from_stringified_source("STRING-WRAPPER", "(DEFCLASS STRING-WRAPPER (LITERAL-WRAPPER) :PUBLIC-SLOTS ((WRAPPER-VALUE :TYPE STRING)) :PRINT-FORM (LET ((VALUE (WRAPPER-VALUE SELF))) (IF *PRINTREADABLY?* (PRINT-STRING-READABLY VALUE STREAM) (IF (DEFINED? VALUE) (PRINT-NATIVE-STREAM STREAM \"|L|\" #\\\" VALUE #\\\") (PRINT-NATIVE-STREAM STREAM \"|L|\" \"NULL-STRING\")))))");

      renamed_Class->class_constructor_code = ((cpp_function_code)(&new_string_wrapper));
      renamed_Class->class_slot_accessor_code = ((cpp_function_code)(&access_string_wrapper_slot_value));
    }
    { Class* renamed_Class = define_class_from_stringified_source("MUTABLE-STRING-WRAPPER", "(DEFCLASS MUTABLE-STRING-WRAPPER (LITERAL-WRAPPER) :PUBLIC-SLOTS ((WRAPPER-VALUE :TYPE MUTABLE-STRING)) :PRINT-FORM (LET ((VALUE (WRAPPER-VALUE SELF))) (IF *PRINTREADABLY?* (PRINT-STRING-READABLY VALUE STREAM) (IF (DEFINED? VALUE) (PRINT-NATIVE-STREAM STREAM \"|L|\" #\\\" (MUTABLE-STRING-TO-STRING VALUE) #\\\") (PRINT-NATIVE-STREAM STREAM \"|L|\" \"NULL-MUTABLE-STRING\")))))");

      renamed_Class->class_constructor_code = ((cpp_function_code)(&new_mutable_string_wrapper));
    }
  }
}

void startup_hierarchy() {
  { 
    BIND_STELLA_SPECIAL(oMODULEo, Module*, oSTELLA_MODULEo);
    BIND_STELLA_SPECIAL(oCONTEXTo, Context*, oMODULEo.get());
    if (current_startup_time_phaseP(2)) {
      help_startup_hierarchy1();
      help_startup_hierarchy2();
      help_startup_hierarchy3();
      help_startup_hierarchy4();
      help_startup_hierarchy5();
      help_startup_hierarchy6();
    }
    if (current_startup_time_phaseP(5)) {
      help_startup_hierarchy7();
      help_startup_hierarchy8();
      { Class* renamed_Class = define_class_from_stringified_source("CHARACTER-WRAPPER", "(DEFCLASS CHARACTER-WRAPPER (LITERAL-WRAPPER) :PUBLIC-SLOTS ((WRAPPER-VALUE :TYPE CHARACTER)) :PRINT-FORM (IF *PRINTREADABLY?* (PRINT-CHARACTER (WRAPPER-VALUE SELF) STREAM) (PROGN (PRINT-NATIVE-STREAM STREAM \"|L|\") (PRINT-CHARACTER (WRAPPER-VALUE SELF) STREAM))))");

        renamed_Class->class_constructor_code = ((cpp_function_code)(&new_character_wrapper));
        renamed_Class->class_slot_accessor_code = ((cpp_function_code)(&access_character_wrapper_slot_value));
      }
      { Class* renamed_Class = define_class_from_stringified_source("BOOLEAN-WRAPPER", "(DEFCLASS BOOLEAN-WRAPPER (LITERAL-WRAPPER) :DOCUMENTATION \"Objectified version of the BOOLEAN data type.  The NULL\nvalue can be used for a 3-valued semantics (see THREE-VALUED-BOOLEAN).\" :PUBLIC-SLOTS ((WRAPPER-VALUE :TYPE BOOLEAN :ALLOCATION :INSTANCE)) :PRINT-FORM (PRINT-NATIVE-STREAM STREAM (CHOOSE *PRINTREADABLY?* \"\" \"|L|\") (CHOOSE (WRAPPER-VALUE SELF) (QUOTE TRUE) (QUOTE FALSE))))");

        renamed_Class->class_constructor_code = ((cpp_function_code)(&new_boolean_wrapper));
        renamed_Class->class_slot_accessor_code = ((cpp_function_code)(&access_boolean_wrapper_slot_value));
      }
      define_class_from_stringified_source("CODE-WRAPPER", "(DEFCLASS CODE-WRAPPER (WRAPPER) :ABSTRACT? TRUE)");
      { Class* renamed_Class = define_class_from_stringified_source("FUNCTION-CODE-WRAPPER", "(DEFCLASS FUNCTION-CODE-WRAPPER (CODE-WRAPPER) :PUBLIC-SLOTS ((WRAPPER-VALUE :TYPE FUNCTION-CODE)))");

        renamed_Class->class_constructor_code = ((cpp_function_code)(&new_function_code_wrapper));
        renamed_Class->class_slot_accessor_code = ((cpp_function_code)(&access_function_code_wrapper_slot_value));
      }
      { Class* renamed_Class = define_class_from_stringified_source("METHOD-CODE-WRAPPER", "(DEFCLASS METHOD-CODE-WRAPPER (CODE-WRAPPER) :PUBLIC-SLOTS ((WRAPPER-VALUE :TYPE METHOD-CODE)))");

        renamed_Class->class_constructor_code = ((cpp_function_code)(&new_method_code_wrapper));
        renamed_Class->class_slot_accessor_code = ((cpp_function_code)(&access_method_code_wrapper_slot_value));
      }
      { Class* renamed_Class = define_class_from_stringified_source("VERBATIM-STRING-WRAPPER", "(DEFCLASS VERBATIM-STRING-WRAPPER (STRING-WRAPPER) :DOCUMENTATION \"Wrapper class used to hold verbatim native code strings.\" :PRINT-FORM (PRINT-NATIVE-STREAM STREAM (CHOOSE *PRINTREADABLY?* \"\" \"|V|\") (WRAPPER-VALUE SELF)))");

        renamed_Class->class_constructor_code = ((cpp_function_code)(&new_verbatim_string_wrapper));
      }
      define_class_from_stringified_source("BOOLEAN", "(DEFCLASS BOOLEAN (LITERAL) :DOCUMENTATION \"Boolean type with values TRUE and FALSE.\" :INITIAL-VALUE FALSE :CL-NATIVE-TYPE \"FIXNUM\" :CPP-NATIVE-TYPE \"boolean\" :IDL-NATIVE-TYPE \"boolean\" :JAVA-NATIVE-TYPE \"boolean\")");
      define_class_from_stringified_source("ONE-BIT-BOOLEAN", "(DEFCLASS ONE-BIT-BOOLEAN (BOOLEAN) :DOCUMENTATION \"Space-saving representation of BOOLEAN which takes longer\nto read and write than BOOLEAN.  Not yet implemented.\" :CPP-NATIVE-TYPE \"one_bit_boolean\")");
      define_stella_type_from_stringified_source("(DEFTYPE THREE-VALUED-BOOLEAN BOOLEAN-WRAPPER)");
      define_class_from_stringified_source("CODE", "(DEFCLASS CODE (LITERAL) :ABSTRACT? TRUE)");
      define_class_from_stringified_source("FUNCTION-CODE", "(DEFCLASS FUNCTION-CODE (CODE) :CL-NATIVE-TYPE \"FUNCTION\" :JAVA-NATIVE-TYPE \"java.lang.reflect.Method\" :CPP-NATIVE-TYPE \"cpp_function_code\")");
      define_class_from_stringified_source("METHOD-CODE", "(DEFCLASS METHOD-CODE (CODE) :CL-NATIVE-TYPE \"STANDARD-GENERIC-FUNCTION\" :JAVA-NATIVE-TYPE \"java.lang.reflect.Method\" :CPP-NATIVE-TYPE \"cpp_method_code\" :INITIAL-VALUE (SAFE-CAST NULL @METHOD-CODE))");
      define_class_from_stringified_source("LISP-CODE", "(DEFCLASS LISP-CODE () :ABSTRACT? TRUE :DOCUMENTATION \"Used to indicate variables that input or output Common Lisp~\n      structures.\")");
      define_class_from_stringified_source("THIRTY-TWO-BIT-VECTOR", "(DEFCLASS THIRTY-TWO-BIT-VECTOR (INTEGER) :CL-NATIVE-TYPE \"FIXNUM\" :CPP-NATIVE-TYPE \"long\" :JAVA-NATIVE-TYPE \"int\" :DOCUMENTATION \"Bit vector used to implement BOOLEAN slots.\" :INITIAL-VALUE NULL-INTEGER)");
      define_class_from_stringified_source("TICKTOCK", "(DEFCLASS TICKTOCK () :DOCUMENTATION \"A data type that is used to hold information for computing\ntiming of code.  It is intentionally opaque to the user.\nIt should only be used as an argument to the TICKTOCK-DIFFERENCE\nfunction.  Values are obtained with the GET-TICKTOCK function.\" :CPP-NATIVE-TYPE \"clock_t\" :CL-NATIVE-TYPE \"FIXNUM\" :JAVA-NATIVE-TYPE \"long\")");
      define_class_from_stringified_source("NATIVE-OBJECT-POINTER", "(DEFCLASS NATIVE-OBJECT-POINTER (SECOND-CLASS-OBJECT) :DOCUMENTATION \"A pointer type that can point at an arbitrary native\nnon-literal object.  Usable to store native objects that fall outside the\nSTELLA OBJECT hierarchy (e.g., native arrays) without having to use their\nexact type.\" :CL-NATIVE-TYPE \"T\" :CPP-NATIVE-TYPE \"char*\" :JAVA-NATIVE-TYPE \"Object\")");
      define_class_from_stringified_source("PROCESS-LOCK-OBJECT", "(DEFCLASS PROCESS-LOCK-OBJECT () :DOCUMENTATION \"A process lock object for synchronizing in a multi-threaded environment.\nFully supported in Java; supported in some Common Lisp systems; not supported\nin C++.\" :CL-NATIVE-TYPE \"T\" :CPP-NATIVE-TYPE \"char*\" :JAVA-NATIVE-TYPE \"Object\")");
      { Class* renamed_Class = define_class_from_stringified_source("SYSTEM-DEFINITION", "(DEFCLASS SYSTEM-DEFINITION (STANDARD-OBJECT) :DOCUMENTATION \"The System-Definition class is used to define systems of files\nthat constitute Stella applications.\" :SLOTS ((NAME :TYPE STRING) (DIRECTORY :TYPE FILE-NAME) (FILES :TYPE (CONS OF STRING-WRAPPER)) (LISP-ONLY-FILES :TYPE (CONS OF STRING-WRAPPER)) (CPP-ONLY-FILES :TYPE (CONS OF STRING-WRAPPER)) (JAVA-ONLY-FILES :TYPE (CONS OF STRING-WRAPPER)) (PREPROCESSED-FILES :TYPE (CONS OF STRING-WRAPPER)) (REQUIRED-SYSTEMS :TYPE (CONS OF STRING-WRAPPER)) (LOADED? :TYPE BOOLEAN) (UP-TO-DATE? :TYPE BOOLEAN) (CARDINAL-MODULE :TYPE STRING) (SOURCE-ROOT-DIRECTORY :TYPE FILE-NAME) (NATIVE-ROOT-DIRECTORY :TYPE FILE-NAME) (BINARY-ROOT-DIRECTORY :TYPE FILE-NAME) (BANNER :TYPE STRING) (COPYRIGHT-HEADER :RENAMES BANNER) (PRODUCTION-SETTINGS :TYPE (CONS OF INTEGER-WRAPPER)) (DEVELOPMENT-SETTINGS :TYPE (CONS OF INTEGER-WRAPPER)) (FINALIZATION-FUNCTION :TYPE SYMBOL)) :PRINT-FORM (PRINT-NATIVE-STREAM STREAM \"|SYSTEM|\" (NAME SELF)))");

        renamed_Class->class_constructor_code = ((cpp_function_code)(&new_system_definition));
        renamed_Class->class_slot_accessor_code = ((cpp_function_code)(&access_system_definition_slot_value));
      }
      define_class_from_stringified_source("ARGUMENT-LIST", "(DEFCLASS ARGUMENT-LIST () :PARAMETERS ((ANY-VALUE :TYPE UNKNOWN)) :PUBLIC-METHODS ((LENGTH ((SELF ARGUMENT-LIST)) :TYPE INTEGER :NATIVE? TRUE)) :METHODS ((ALLOCATE-ITERATOR ((SELF ARGUMENT-LIST)) :TYPE (ARGUMENT-LIST-ITERATOR OF (LIKE (ANY-VALUE SELF))) :NATIVE? TRUE)) :CL-NATIVE-TYPE \"CONS\" :CPP-NATIVE-TYPE \"void\" :JAVA-NATIVE-TYPE \"void\")");
      define_class_from_stringified_source("ARGUMENT-LIST-ITERATOR", "(DEFCLASS ARGUMENT-LIST-ITERATOR (ABSTRACT-ITERATOR) :PUBLIC-SLOTS ((VALUE :READER ARGUMENT)) :METHODS ((ARGUMENT ((SELF ARGUMENT-LIST-ITERATOR)) :TYPE (LIKE (ANY-VALUE SELF)) :NATIVE? TRUE) (NEXT? ((SELF ARGUMENT-LIST-ITERATOR)) :TYPE BOOLEAN :NATIVE? TRUE)) :CL-NATIVE-TYPE \"CONS\" :CPP-NATIVE-TYPE \"va_list\" :JAVA-NATIVE-TYPE \"void\")");
      define_class_from_stringified_source("NATIVE-STREAM", "(DEFCLASS NATIVE-STREAM () :ABSTRACT? TRUE :CPP-NATIVE-TYPE \"ios*\" :JAVA-NATIVE-TYPE \"Object\" :CL-NATIVE-TYPE \"STREAM\")");
      define_class_from_stringified_source("NATIVE-OUTPUT-STREAM", "(DEFCLASS NATIVE-OUTPUT-STREAM (NATIVE-STREAM) :ABSTRACT? TRUE :CPP-NATIVE-TYPE \"ostream*\" :JAVA-NATIVE-TYPE \"java.io.PrintStream\" :CL-NATIVE-TYPE \"STREAM\")");
      define_class_from_stringified_source("NATIVE-INPUT-STREAM", "(DEFCLASS NATIVE-INPUT-STREAM (NATIVE-STREAM) :ABSTRACT? TRUE :CPP-NATIVE-TYPE \"istream*\" :JAVA-NATIVE-TYPE \"java.io.PushbackInputStream\" :CL-NATIVE-TYPE \"STREAM\")");
      { Class* renamed_Class = define_class_from_stringified_source("STREAM", "(DEFCLASS STREAM (STANDARD-OBJECT) :ABSTRACT? TRUE :PUBLIC-SLOTS ((STATE :TYPE KEYWORD)))");

        renamed_Class->class_slot_accessor_code = ((cpp_function_code)(&access_stream_slot_value));
      }
      { Class* renamed_Class = define_class_from_stringified_source("OUTPUT-STREAM", "(DEFCLASS OUTPUT-STREAM (STREAM) :PUBLIC-SLOTS ((NATIVE-STREAM :TYPE NATIVE-OUTPUT-STREAM :PUBLIC? TRUE) (NATIVE-OUTPUT-STREAM :RENAMES NATIVE-STREAM)) :TERMINATOR TERMINATE-OUTPUT-STREAM?)");

        renamed_Class->class_constructor_code = ((cpp_function_code)(&new_output_stream));
      }
      { Class* renamed_Class = define_class_from_stringified_source("INPUT-STREAM", "(DEFCLASS INPUT-STREAM (STREAM) :PUBLIC-SLOTS ((NATIVE-STREAM :TYPE NATIVE-INPUT-STREAM :PUBLIC? TRUE) (NATIVE-INPUT-STREAM :RENAMES NATIVE-STREAM) (ECHO-STREAM :TYPE OUTPUT-STREAM) (TOKENIZER-STATE :TYPE TOKENIZER-STREAM-STATE) (BUFFERING-SCHEME :TYPE KEYWORD :INITIALLY :LINE :DOCUMENTATION \"One of :CHARACTER, :LINE or :BLOCK indicating what kind of\ninput buffer we are reading from.  This determines what read function to use\nfor tokenization.\")) :TERMINATOR TERMINATE-INPUT-STREAM?)");

        renamed_Class->class_constructor_code = ((cpp_function_code)(&new_input_stream));
        renamed_Class->class_slot_accessor_code = ((cpp_function_code)(&access_input_stream_slot_value));
      }
      { Class* renamed_Class = define_class_from_stringified_source("OUTPUT-FILE-STREAM", "(DEFCLASS OUTPUT-FILE-STREAM (OUTPUT-STREAM) :SYNONYMS (FILE-OUTPUT-STREAM) :PUBLIC-SLOTS ((FILENAME :TYPE STRING :REQUIRED? TRUE :PUBLIC? TRUE) (IF-EXISTS-ACTION :TYPE KEYWORD :INITIALLY :SUPERSEDE :PUBLIC? TRUE)) :INITIALIZER INITIALIZE-FILE-OUTPUT-STREAM :TERMINATOR TERMINATE-FILE-OUTPUT-STREAM? :PRINT-FORM (PRINT-NATIVE-STREAM STREAM \"|FOS|'\" (FILENAME SELF) \"'\"))");

        renamed_Class->class_constructor_code = ((cpp_function_code)(&new_output_file_stream));
        renamed_Class->class_slot_accessor_code = ((cpp_function_code)(&access_output_file_stream_slot_value));
      }
      { Class* renamed_Class = define_class_from_stringified_source("INPUT-FILE-STREAM", "(DEFCLASS INPUT-FILE-STREAM (INPUT-STREAM) :SYNONYMS (FILE-INPUT-STREAM) :PUBLIC-SLOTS ((FILENAME :TYPE STRING :REQUIRED? TRUE :PUBLIC? TRUE) (IF-NOT-EXISTS-ACTION :TYPE KEYWORD :INITIALLY :ERROR :PUBLIC? TRUE) (BUFFERING-SCHEME :INITIALLY :BLOCK)) :INITIALIZER INITIALIZE-FILE-INPUT-STREAM :TERMINATOR TERMINATE-FILE-INPUT-STREAM? :PRINT-FORM (PRINT-NATIVE-STREAM STREAM \"|FIS|'\" (FILENAME SELF) \"'\"))");

        renamed_Class->class_constructor_code = ((cpp_function_code)(&new_input_file_stream));
        renamed_Class->class_slot_accessor_code = ((cpp_function_code)(&access_input_file_stream_slot_value));
      }
      { Class* renamed_Class = define_class_from_stringified_source("OUTPUT-STRING-STREAM", "(DEFCLASS OUTPUT-STRING-STREAM (OUTPUT-STREAM) :SYNONYMS (STRING-OUTPUT-STREAM) :PUBLIC-SLOTS ((THE-STRING :TYPE STRING :PUBLIC? TRUE :READER THE-STRING-READER)) :INITIALIZER INITIALIZE-STRING-OUTPUT-STREAM)");

        renamed_Class->class_constructor_code = ((cpp_function_code)(&new_output_string_stream));
        renamed_Class->class_slot_accessor_code = ((cpp_function_code)(&access_output_string_stream_slot_value));
      }
      { Class* renamed_Class = define_class_from_stringified_source("INPUT-STRING-STREAM", "(DEFCLASS INPUT-STRING-STREAM (INPUT-STREAM) :SYNONYMS (STRING-INPUT-STREAM) :PUBLIC-SLOTS ((THE-STRING :TYPE STRING :REQUIRED? TRUE :PUBLIC? TRUE) (BUFFERING-SCHEME :INITIALLY :BLOCK)) :INITIALIZER INITIALIZE-STRING-INPUT-STREAM)");

        renamed_Class->class_constructor_code = ((cpp_function_code)(&new_input_string_stream));
        renamed_Class->class_slot_accessor_code = ((cpp_function_code)(&access_input_string_stream_slot_value));
      }
    }
    if (current_startup_time_phaseP(6)) {
      finalize_classes();
    }
    if (current_startup_time_phaseP(7)) {
      define_function_object("STARTUP-HIERARCHY", "(DEFUN STARTUP-HIERARCHY () :PUBLIC? TRUE)", ((cpp_function_code)(&startup_hierarchy)), NULL);
      { Method_Slot* function = lookup_function(SYM_HIERARCHY_STELLA_STARTUP_HIERARCHY);

        set_dynamic_slot_value(function->dynamic_slots, SYM_HIERARCHY_STELLA_METHOD_STARTUP_CLASSNAME, wrap_string("Startup-Hierarchy"), NULL_STRING_WRAPPER);
      }
    }
    if (current_startup_time_phaseP(8)) {
      finalize_slots();
      cleanup_unfinalized_classes();
    }
  }
}

Symbol* SYM_HIERARCHY_STELLA_TYPE_SPEC = NULL;

boolean oHARDWIRED_TRANSIENTp_ON_TRANSIENT_MIXINo = FALSE;

Symbol* SYM_HIERARCHY_STELLA_REFERENCE_COUNT = NULL;

Symbol* SYM_HIERARCHY_STELLA_HOME_CONTEXT = NULL;

Symbol* SYM_HIERARCHY_STELLA_BADp = NULL;

Symbol* SYM_HIERARCHY_STELLA_DELETED_OBJECTp = NULL;

Symbol* SYM_HIERARCHY_STELLA_FIRST_ITERATIONp = NULL;

Symbol* SYM_HIERARCHY_STELLA_OBJECT_ITERATOR = NULL;

Surrogate* SGT_HIERARCHY_STELLA_CONS_ITERATOR = NULL;

Symbol* SYM_HIERARCHY_STELLA_CONS_ITERATOR_CURSOR = NULL;

Surrogate* SGT_HIERARCHY_STELLA_LIST_ITERATOR = NULL;

Symbol* SYM_HIERARCHY_STELLA_LIST_ITERATOR_CURSOR = NULL;

Symbol* SYM_HIERARCHY_STELLA_LIST_ITERATOR_COLLECTION = NULL;

Surrogate* SGT_HIERARCHY_STELLA_DESTRUCTIVE_LIST_ITERATOR = NULL;

Symbol* SYM_HIERARCHY_STELLA_THE_CONS_LIST = NULL;

Surrogate* SGT_HIERARCHY_STELLA_SET_ITERATOR = NULL;

Symbol* SYM_HIERARCHY_STELLA_SET_ITERATOR_CURSOR = NULL;

Symbol* SYM_HIERARCHY_STELLA_SET_ITERATOR_COLLECTION = NULL;

Surrogate* SGT_HIERARCHY_STELLA_ALL_PURPOSE_ITERATOR = NULL;

Symbol* SYM_HIERARCHY_STELLA_KEY = NULL;

Symbol* SYM_HIERARCHY_STELLA_ITERATOR_NEXT_CODE = NULL;

Symbol* SYM_HIERARCHY_STELLA_ITERATOR_FILTER_CODE = NULL;

Symbol* SYM_HIERARCHY_STELLA_ITERATOR_EMPTY_CODE = NULL;

Symbol* SYM_HIERARCHY_STELLA_ITERATOR_NESTED_ITERATOR = NULL;

Symbol* SYM_HIERARCHY_STELLA_ITERATOR_CONS_LIST = NULL;

Symbol* SYM_HIERARCHY_STELLA_ITERATOR_OBJECT = NULL;

Symbol* SYM_HIERARCHY_STELLA_ITERATOR_SECOND_OBJECT = NULL;

Symbol* SYM_HIERARCHY_STELLA_ITERATOR_INTEGER = NULL;

Symbol* SYM_HIERARCHY_STELLA_ITERATOR_SECOND_INTEGER = NULL;

Surrogate* SGT_HIERARCHY_STELLA_TYPES_TO_CLASSES_ITERATOR = NULL;

Symbol* SYM_HIERARCHY_STELLA_ITERATOR_CURSOR = NULL;

Symbol* SYM_HIERARCHY_STELLA_OBJECT_DICTIONARY_ITERATOR = NULL;

Surrogate* SGT_HIERARCHY_STELLA_PROPERTY_LIST_ITERATOR = NULL;

Symbol* SYM_HIERARCHY_STELLA_PLIST_ITERATOR_CURSOR = NULL;

Symbol* SYM_HIERARCHY_STELLA_PLIST_ITERATOR_COLLECTION = NULL;

Surrogate* SGT_HIERARCHY_STELLA_KV_LIST_ITERATOR = NULL;

Symbol* SYM_HIERARCHY_STELLA_THE_KV_LIST = NULL;

Symbol* SYM_HIERARCHY_STELLA_KV_LIST_ITERATOR_CURSOR = NULL;

Surrogate* SGT_HIERARCHY_STELLA_INTEGER_INTERVAL_ITERATOR = NULL;

Symbol* SYM_HIERARCHY_STELLA_INTERVAL_CURSOR = NULL;

Symbol* SYM_HIERARCHY_STELLA_LOWER_BOUND = NULL;

Symbol* SYM_HIERARCHY_STELLA_UPPER_BOUND = NULL;

Surrogate* SGT_HIERARCHY_STELLA_REVERSE_INTEGER_INTERVAL_ITERATOR = NULL;

Surrogate* SGT_HIERARCHY_STELLA_STRING_ITERATOR = NULL;

Symbol* SYM_HIERARCHY_STELLA_THE_STRING = NULL;

Symbol* SYM_HIERARCHY_STELLA_CURSOR = NULL;

Symbol* SYM_HIERARCHY_STELLA_END = NULL;

Symbol* SYM_HIERARCHY_STELLA_OBJECT_COLLECTION = NULL;

Symbol* SYM_HIERARCHY_STELLA_OBJECT_SEQUENCE = NULL;

Surrogate* SGT_HIERARCHY_STELLA_LIST = NULL;

Surrogate* SGT_HIERARCHY_STELLA_SET = NULL;

Surrogate* SGT_HIERARCHY_STELLA_CLASS_EXTENSION = NULL;

Surrogate* SGT_HIERARCHY_STELLA_CONS = NULL;

Symbol* SYM_HIERARCHY_STELLA_VALUE = NULL;

Symbol* SYM_HIERARCHY_STELLA_REST = NULL;

Symbol* SYM_HIERARCHY_STELLA_OBJECT_TO_OBJECT_DICTIONARY = NULL;

Surrogate* SGT_HIERARCHY_STELLA_PROPERTY_LIST = NULL;

Symbol* SYM_HIERARCHY_STELLA_THE_PLIST = NULL;

Surrogate* SGT_HIERARCHY_STELLA_KV_CONS = NULL;

Surrogate* SGT_HIERARCHY_STELLA_KEY_VALUE_LIST = NULL;

Surrogate* SGT_HIERARCHY_STELLA_RECYCLE_LIST = NULL;

Symbol* SYM_HIERARCHY_STELLA_RECYCLE_LIST_OF = NULL;

Symbol* SYM_HIERARCHY_STELLA_RECYCLED_ITEMS = NULL;

Symbol* SYM_HIERARCHY_STELLA_LIST_OF_RECYCLED_ITEMS = NULL;

Symbol* SYM_HIERARCHY_STELLA_ALL_ITEMS = NULL;

Symbol* SYM_HIERARCHY_STELLA_UNUSED_ITEMS = NULL;

Symbol* SYM_HIERARCHY_STELLA_CURRENT_LENGTH = NULL;

Symbol* SYM_HIERARCHY_STELLA_ITEM_SIZE = NULL;

Symbol* SYM_HIERARCHY_STELLA_THE_STELLA_HASH_TABLE = NULL;

Surrogate* SGT_HIERARCHY_STELLA_HASH_TABLE = NULL;

Symbol* SYM_HIERARCHY_STELLA_OBJECT_TO_OBJECT_HASH_TABLE = NULL;

Surrogate* SGT_HIERARCHY_STELLA_INTEGER_HASH_TABLE = NULL;

Surrogate* SGT_HIERARCHY_STELLA_FLOAT_HASH_TABLE = NULL;

Surrogate* SGT_HIERARCHY_STELLA_STRING_HASH_TABLE = NULL;

Surrogate* SGT_HIERARCHY_STELLA_STRING_TO_INTEGER_HASH_TABLE = NULL;

Surrogate* SGT_HIERARCHY_STELLA_VECTOR = NULL;

Symbol* SYM_HIERARCHY_STELLA_ARRAY_SIZE = NULL;

Surrogate* SGT_HIERARCHY_STELLA_EXTENSIBLE_VECTOR = NULL;

Surrogate* SGT_HIERARCHY_STELLA_EXTENSIBLE_SYMBOL_ARRAY = NULL;

Symbol* SYM_HIERARCHY_STELLA_TOP_SYMBOL_OFFSET = NULL;

Symbol* SYM_HIERARCHY_STELLA_POTENTIAL_FREE_SYMBOL_OFFSET = NULL;

int oHARDWIRED_RESIZE_INCREMENT_ON_VECTOR_SEQUENCEo = NULL_INTEGER;

Symbol* SYM_HIERARCHY_STELLA_RESIZE_INCREMENT = NULL;

Symbol* SYM_HIERARCHY_STELLA_SEQUENCE_LENGTH = NULL;

Symbol* SYM_HIERARCHY_STELLA_VSEQ = NULL;

Surrogate* SGT_HIERARCHY_STELLA_SHORT_VECTOR_SEQUENCE = NULL;

int oHARDWIRED_RESIZE_INCREMENT_ON_SHORT_VECTOR_SEQUENCEo = NULL_INTEGER;

Symbol* SYM_HIERARCHY_STELLA_SVSEQ = NULL;

Surrogate* SGT_HIERARCHY_STELLA_BOOLEAN_VECTOR = NULL;

Surrogate* SGT_HIERARCHY_STELLA_INTEGER_VECTOR = NULL;

Surrogate* SGT_HIERARCHY_STELLA_ACTIVE_LIST = NULL;

Surrogate* SGT_HIERARCHY_STELLA_ACTIVE_SET = NULL;

Surrogate* SGT_HIERARCHY_STELLA_DEMON = NULL;

Symbol* SYM_HIERARCHY_STELLA_DEMON_NAME = NULL;

Symbol* SYM_HIERARCHY_STELLA_DEMON_ACTION = NULL;

Symbol* SYM_HIERARCHY_STELLA_DEMON_CLASS_REFS = NULL;

Symbol* SYM_HIERARCHY_STELLA_DEMON_SLOT_REFS = NULL;

Symbol* SYM_HIERARCHY_STELLA_DEMON_CODE = NULL;

Symbol* SYM_HIERARCHY_STELLA_DEMON_METHOD = NULL;

Symbol* SYM_HIERARCHY_STELLA_DEMON_DOCUMENTATION = NULL;

Symbol* SYM_HIERARCHY_STELLA_DEMON_GUARDp = NULL;

Symbol* SYM_HIERARCHY_STELLA_DEMON_ALLp = NULL;

Symbol* SYM_HIERARCHY_STELLA_DEMON_INHERITp = NULL;

Surrogate* SGT_HIERARCHY_STELLA_KEYWORD_KEY_VALUE_LIST = NULL;

Symbol* SYM_HIERARCHY_STELLA_SYMBOL_NAME = NULL;

Symbol* SYM_HIERARCHY_STELLA_SYMBOL_ID = NULL;

Symbol* SYM_HIERARCHY_STELLA_INTERNED_IN = NULL;

Surrogate* SGT_HIERARCHY_STELLA_SYMBOL = NULL;

Symbol* SYM_HIERARCHY_STELLA_SYMBOL_SLOT_OFFSET = NULL;

Symbol* SYM_HIERARCHY_STELLA_SYMBOL_VALUE_AND_PLIST = NULL;

Surrogate* SGT_HIERARCHY_STELLA_SURROGATE = NULL;

Symbol* SYM_HIERARCHY_STELLA_SURROGATE_VALUE = NULL;

Symbol* SYM_HIERARCHY_STELLA_SURROGATE_NAME = NULL;

Symbol* SYM_HIERARCHY_STELLA_TYPE_CLASS = NULL;

Symbol* SYM_HIERARCHY_STELLA_TYPE_NAME = NULL;

Symbol* SYM_HIERARCHY_STELLA_SLOTREF_SLOT = NULL;

Symbol* SYM_HIERARCHY_STELLA_TYPE = NULL;

Symbol* SYM_HIERARCHY_STELLA_SLOTREF = NULL;

Surrogate* SGT_HIERARCHY_STELLA_KEYWORD = NULL;

Symbol* SYM_HIERARCHY_STELLA_KEYWORD_NAME = NULL;

Surrogate* SGT_HIERARCHY_STELLA_TRANSIENT_SYMBOL = NULL;

Symbol* SYM_HIERARCHY_STELLA_PROJECTS_ONTO = NULL;

Symbol* SYM_HIERARCHY_STELLA_PROJECTED_FROM = NULL;

Symbol* SYM_HIERARCHY_STELLA_PROPERTIES = NULL;

Symbol* SYM_HIERARCHY_STELLA_ABSTRACTp = NULL;

Surrogate* SGT_HIERARCHY_STELLA_CLASS = NULL;

Symbol* SYM_HIERARCHY_STELLA_STORED_ACTIVEp = NULL;

Surrogate* SGT_HIERARCHY_STELLA_ACTIVE_OBJECT = NULL;

Symbol* SYM_HIERARCHY_STELLA_CLASS_CREATOR = NULL;

Symbol* SYM_HIERARCHY_STELLA_CLASS_INITIALIZER = NULL;

Symbol* SYM_HIERARCHY_STELLA_CLASS_TERMINATOR = NULL;

Symbol* SYM_HIERARCHY_STELLA_CLASS_DESTRUCTOR = NULL;

Symbol* SYM_HIERARCHY_STELLA_CLASS_PARAMETERS = NULL;

Symbol* SYM_HIERARCHY_STELLA_CLASS_INITIAL_VALUE = NULL;

Symbol* SYM_HIERARCHY_STELLA_CLASS_EXTENSION = NULL;

Symbol* SYM_HIERARCHY_STELLA_CLASS_CL_NATIVE_TYPE = NULL;

Symbol* SYM_HIERARCHY_STELLA_CLASS_CPP_NATIVE_TYPE = NULL;

Symbol* SYM_HIERARCHY_STELLA_CLASS_JAVA_NATIVE_TYPE = NULL;

int oHARDWIRED_CLASS_ARITY_ON_CLASSo = NULL_INTEGER;

Symbol* SYM_HIERARCHY_STELLA_CLASS_CL_STRUCT_SLOTS = NULL;

Symbol* SYM_HIERARCHY_STELLA_CLASS_RECYCLE_METHOD = NULL;

Keyword* KWD_HIERARCHY_NONE = NULL;

Symbol* SYM_HIERARCHY_STELLA_CLASS_REQUIRED_SLOT_NAMES = NULL;

Symbol* SYM_HIERARCHY_STELLA_CLASS_GUARD_CONSTRUCTOR_DEMONS = NULL;

Symbol* SYM_HIERARCHY_STELLA_CLASS_CONSTRUCTOR_DEMONS = NULL;

Symbol* SYM_HIERARCHY_STELLA_CLASS_GUARD_DESTRUCTOR_DEMONS = NULL;

Symbol* SYM_HIERARCHY_STELLA_CLASS_DESTRUCTOR_DEMONS = NULL;

Symbol* SYM_HIERARCHY_STELLA_CLASS_KEY = NULL;

Symbol* SYM_HIERARCHY_STELLA_CLASS_SYNONYMS = NULL;

Symbol* SYM_HIERARCHY_STELLA_CL_STRUCTp = NULL;

Symbol* SYM_HIERARCHY_STELLA_MIXINp = NULL;

Symbol* SYM_HIERARCHY_STELLA_CLASS_TYPE = NULL;

Symbol* SYM_HIERARCHY_STELLA_CLASS_ARITY = NULL;

Symbol* SYM_HIERARCHY_STELLA_CLASS_DIRECT_SUPERS = NULL;

Symbol* SYM_HIERARCHY_STELLA_CLASS_DIRECT_SUBS = NULL;

Symbol* SYM_HIERARCHY_STELLA_CLASS_ALL_SUPER_CLASSES = NULL;

Symbol* SYM_HIERARCHY_STELLA_CLASS_ALL_SLOTS = NULL;

Symbol* SYM_HIERARCHY_STELLA_CLASS_LOCAL_SLOTS = NULL;

Symbol* SYM_HIERARCHY_STELLA_CLASS_SLOT_AND_METHOD_CACHE = NULL;

Symbol* SYM_HIERARCHY_STELLA_CLASS_ABSTRACTp = NULL;

Symbol* SYM_HIERARCHY_STELLA_CLASS_MIXINp = NULL;

Symbol* SYM_HIERARCHY_STELLA_CLASS_COLLECTIONp = NULL;

Symbol* SYM_HIERARCHY_STELLA_CLASS_CL_STRUCTp = NULL;

Symbol* SYM_HIERARCHY_STELLA_CLASS_PUBLICp = NULL;

Symbol* SYM_HIERARCHY_STELLA_CLASS_FINALIZEDp = NULL;

Symbol* SYM_HIERARCHY_STELLA_CLASS_SLOTS_FINALIZEDp = NULL;

Symbol* SYM_HIERARCHY_STELLA_CLASS_STRINGIFIED_SOURCE = NULL;

Symbol* SYM_HIERARCHY_STELLA_CLASS_CONSTRUCTOR_CODE = NULL;

Symbol* SYM_HIERARCHY_STELLA_CLASS_SLOT_ACCESSOR_CODE = NULL;

Symbol* SYM_HIERARCHY_STELLA_CLASS_MARKEDp = NULL;

Symbol* SYM_HIERARCHY_STELLA_CLASS_TAXONOMY_NODE = NULL;

Symbol* SYM_HIERARCHY_STELLA_CLASS_DOCUMENTATION = NULL;

Symbol* SYM_HIERARCHY_STELLA_CLASS_PRINT_FORM = NULL;

Symbol* SYM_HIERARCHY_STELLA_DOCUMENTATION = NULL;

Symbol* SYM_HIERARCHY_STELLA_PRINT_FORM = NULL;

Symbol* SYM_HIERARCHY_STELLA_SLOT_DIRECT_EQUIVALENT = NULL;

Symbol* SYM_HIERARCHY_STELLA_SLOT_TYPE_SPECIFIER = NULL;

Symbol* SYM_HIERARCHY_STELLA_SLOT_HOME_MODULE = NULL;

Symbol* SYM_HIERARCHY_STELLA_SLOT_RENAMES = NULL;

Surrogate* SGT_HIERARCHY_STELLA_COLLECTION = NULL;

Symbol* SYM_HIERARCHY_STELLA_SLOT_CLOSURE_ASSUMPTION = NULL;

Symbol* SYM_HIERARCHY_STELLA_SLOT_DIRECT_SUPERS = NULL;

Symbol* SYM_HIERARCHY_STELLA_SLOT_DIRECT_SUBS = NULL;

Symbol* SYM_HIERARCHY_STELLA_SLOT_NAME = NULL;

Symbol* SYM_HIERARCHY_STELLA_SLOT_OWNER = NULL;

Symbol* SYM_HIERARCHY_STELLA_SLOT_BASE_TYPE = NULL;

Symbol* SYM_HIERARCHY_STELLA_SLOT_SLOTREF = NULL;

Symbol* SYM_HIERARCHY_STELLA_SLOT_PUBLICp = NULL;

Symbol* SYM_HIERARCHY_STELLA_SLOT_RENAMEDp = NULL;

Symbol* SYM_HIERARCHY_STELLA_SLOT_EXTERNALp = NULL;

Symbol* SYM_HIERARCHY_STELLA_SLOT_MARKEDp = NULL;

Symbol* SYM_HIERARCHY_STELLA_SLOT_DOCUMENTATION = NULL;

Surrogate* SGT_HIERARCHY_STELLA_STORAGE_SLOT = NULL;

Symbol* SYM_HIERARCHY_STELLA_SLOT_ALLOCATION = NULL;

Keyword* KWD_HIERARCHY_INSTANCE = NULL;

Symbol* SYM_HIERARCHY_STELLA_SLOT_DEFAULT_EXPRESSION = NULL;

Symbol* SYM_HIERARCHY_STELLA_SLOT_READER = NULL;

Symbol* SYM_HIERARCHY_STELLA_SLOT_WRITER = NULL;

Symbol* SYM_HIERARCHY_STELLA_SLOT_GUARD_DEMONS = NULL;

Symbol* SYM_HIERARCHY_STELLA_SLOT_DEMONS = NULL;

Symbol* SYM_HIERARCHY_STELLA_SLOT_REQUIREDp = NULL;

Symbol* SYM_HIERARCHY_STELLA_SLOT_COMPONENTp = NULL;

Symbol* SYM_HIERARCHY_STELLA_SLOT_READ_ONLYp = NULL;

Symbol* SYM_HIERARCHY_STELLA_SLOT_HARDWIREDp = NULL;

Symbol* SYM_HIERARCHY_STELLA_SLOT_CONTEXT_SENSITIVEp = NULL;

Surrogate* SGT_HIERARCHY_STELLA_METHOD_SLOT = NULL;

Symbol* SYM_HIERARCHY_STELLA_METHOD_PARAMETER_DIRECTIONS = NULL;

Symbol* SYM_HIERARCHY_STELLA_METHOD_INLINED_FUNCTIONS = NULL;

Symbol* SYM_HIERARCHY_STELLA_METHOD_EVALUATE_ARGUMENTSp = NULL;

Symbol* SYM_HIERARCHY_STELLA_METHOD_SETTERp = NULL;

Symbol* SYM_HIERARCHY_STELLA_METHOD_PARAMETER_NAMES = NULL;

Symbol* SYM_HIERARCHY_STELLA_METHOD_PARAMETER_TYPE_SPECIFIERS = NULL;

Symbol* SYM_HIERARCHY_STELLA_METHOD_RETURN_TYPE_SPECIFIERS = NULL;

Symbol* SYM_HIERARCHY_STELLA_METHOD_STRINGIFIED_SOURCE = NULL;

Symbol* SYM_HIERARCHY_STELLA_METHOD_CODE = NULL;

Symbol* SYM_HIERARCHY_STELLA_FUNCTION_CODE = NULL;

Symbol* SYM_HIERARCHY_STELLA_METHOD_FUNCTIONp = NULL;

Symbol* SYM_HIERARCHY_STELLA_METHOD_DOCUMENTATION = NULL;

Symbol* SYM_HIERARCHY_STELLA_METHOD_AUXILIARYp = NULL;

Symbol* SYM_HIERARCHY_STELLA_METHOD_STORAGE_SLOT = NULL;

Symbol* SYM_HIERARCHY_STELLA_SLOT_AUXILIARYp = NULL;

Symbol* SYM_HIERARCHY_STELLA_STORAGE_SLOT = NULL;

Symbol* SYM_HIERARCHY_STELLA_FUNCTION = NULL;

Surrogate* SGT_HIERARCHY_STELLA_PARAMETRIC_TYPE_SPECIFIER = NULL;

Symbol* SYM_HIERARCHY_STELLA_SPECIFIER_BASE_TYPE = NULL;

Symbol* SYM_HIERARCHY_STELLA_SPECIFIER_PARAMETER_TYPES = NULL;

Symbol* SYM_HIERARCHY_STELLA_SPECIFIER_DIMENSIONS = NULL;

Surrogate* SGT_HIERARCHY_STELLA_ANCHORED_TYPE_SPECIFIER = NULL;

Symbol* SYM_HIERARCHY_STELLA_SPECIFIER_PARAMETER_NAME = NULL;

Surrogate* SGT_HIERARCHY_STELLA_TABLE = NULL;

Symbol* SYM_HIERARCHY_STELLA_TUPLE_DOMAINS = NULL;

Surrogate* SGT_HIERARCHY_STELLA_GLOBAL_VARIABLE = NULL;

Symbol* SYM_HIERARCHY_STELLA_VARIABLE_HOME_MODULE = NULL;

Symbol* SYM_HIERARCHY_STELLA_VARIABLE_NAME = NULL;

Symbol* SYM_HIERARCHY_STELLA_VARIABLE_TYPE = NULL;

Symbol* SYM_HIERARCHY_STELLA_VARIABLE_SPECIALp = NULL;

Symbol* SYM_HIERARCHY_STELLA_VARIABLE_CONSTANTp = NULL;

Symbol* SYM_HIERARCHY_STELLA_VARIABLE_PUBLICp = NULL;

Symbol* SYM_HIERARCHY_STELLA_VARIABLE_AUXILIARYp = NULL;

Symbol* SYM_HIERARCHY_STELLA_VARIABLE_GET_VALUE_CODE = NULL;

Symbol* SYM_HIERARCHY_STELLA_VARIABLE_SET_VALUE_CODE = NULL;

Symbol* SYM_HIERARCHY_STELLA_VARIABLE_VALUE_STACK = NULL;

Symbol* SYM_HIERARCHY_STELLA_VARIABLE_DOCUMENTATION = NULL;

Symbol* SYM_HIERARCHY_STELLA_VARIABLE_STRINGIFIED_SOURCE = NULL;

Surrogate* SGT_HIERARCHY_STELLA_QUOTED_EXPRESSION = NULL;

Symbol* SYM_HIERARCHY_STELLA_QUOTATION_TABLE_OFFSET = NULL;

Symbol* SYM_HIERARCHY_STELLA_SURROGATE_VALUE_INVERSE = NULL;

Surrogate* SGT_HIERARCHY_STELLA_MODULE = NULL;

Surrogate* SGT_HIERARCHY_STELLA_WORLD = NULL;

Symbol* SYM_HIERARCHY_STELLA_WORLD_NAME = NULL;

Symbol* SYM_HIERARCHY_STELLA_CHILD_CONTEXTS = NULL;

Symbol* SYM_HIERARCHY_STELLA_BASE_MODULE = NULL;

Symbol* SYM_HIERARCHY_STELLA_ALL_SUPER_CONTEXTS = NULL;

Symbol* SYM_HIERARCHY_STELLA_CONTEXT_NUMBER = NULL;

Symbol* SYM_HIERARCHY_STELLA_MODULE_LISP_PACKAGE = NULL;

Symbol* SYM_HIERARCHY_STELLA_MODULE_CPP_PACKAGE = NULL;

Symbol* SYM_HIERARCHY_STELLA_SHADOWED_SURROGATES = NULL;

Symbol* SYM_HIERARCHY_STELLA_JAVA_PACKAGE = NULL;

Symbol* SYM_HIERARCHY_STELLA_CLEARABLEp = NULL;

Symbol* SYM_HIERARCHY_STELLA_PARENT_MODULES = NULL;

Symbol* SYM_HIERARCHY_STELLA_NICKNAMES = NULL;

Symbol* SYM_HIERARCHY_STELLA_USES = NULL;

Symbol* SYM_HIERARCHY_STELLA_USED_BY = NULL;

Symbol* SYM_HIERARCHY_STELLA_REQUIRES = NULL;

Symbol* SYM_HIERARCHY_STELLA_CASE_SENSITIVEp = NULL;

Symbol* SYM_HIERARCHY_STELLA_MODULE_FULL_NAME = NULL;

Symbol* SYM_HIERARCHY_STELLA_MODULE_NAME = NULL;

Symbol* SYM_HIERARCHY_STELLA_MODULE_STRINGIFIED_SOURCE = NULL;

Symbol* SYM_HIERARCHY_STELLA_STRINGIFIED_OPTIONS = NULL;

Symbol* SYM_HIERARCHY_STELLA_CARDINAL_MODULE = NULL;

Symbol* SYM_HIERARCHY_STELLA_SYMBOL_OFFSET_TABLE = NULL;

Symbol* SYM_HIERARCHY_STELLA_SURROGATE_OFFSET_TABLE = NULL;

Symbol* SYM_HIERARCHY_STELLA_STRICT_INFERENCE_CACHE = NULL;

Symbol* SYM_HIERARCHY_STELLA_DEFAULT_INFERENCE_CACHE = NULL;

Symbol* SYM_HIERARCHY_STELLA_PROTOTYPE_INFERENCE_CACHE = NULL;

Symbol* SYM_HIERARCHY_STELLA_PARENT_CONTEXT = NULL;

Surrogate* SGT_HIERARCHY_STELLA_CS_VALUE = NULL;

Surrogate* SGT_HIERARCHY_STELLA_INTERVAL = NULL;

Surrogate* SGT_HIERARCHY_STELLA_TAXONOMY_NODE = NULL;

Symbol* SYM_HIERARCHY_STELLA_NATIVE_OBJECT = NULL;

Symbol* SYM_HIERARCHY_STELLA_LABEL = NULL;

Symbol* SYM_HIERARCHY_STELLA_INTERVALS = NULL;

Symbol* SYM_HIERARCHY_STELLA_INITIAL_INTERVAL = NULL;

Symbol* SYM_HIERARCHY_STELLA_FIRST_INTERVAL_LOWER_BOUND = NULL;

Symbol* SYM_HIERARCHY_STELLA_FIRST_INTERVAL_UPPER_BOUND = NULL;

Symbol* SYM_HIERARCHY_STELLA_PARENTS = NULL;

Symbol* SYM_HIERARCHY_STELLA_CHILDREN = NULL;

Symbol* SYM_HIERARCHY_STELLA_TOTAL_ANCESTORS = NULL;

Symbol* SYM_HIERARCHY_STELLA_TREE_CHILDREN = NULL;

Symbol* SYM_HIERARCHY_STELLA_MARKER = NULL;

Surrogate* SGT_HIERARCHY_STELLA_TAXONOMY_GRAPH = NULL;

Symbol* SYM_HIERARCHY_STELLA_RENUMBER_IF_OUT_OF_NUMBERSp = NULL;

Symbol* SYM_HIERARCHY_STELLA_RENUMBER_RATIO = NULL;

Symbol* SYM_HIERARCHY_STELLA_INCREMENTAL_MODEp = NULL;

Symbol* SYM_HIERARCHY_STELLA_LARGEST_POSTORDER_NUMBER = NULL;

Symbol* SYM_HIERARCHY_STELLA_NUMBER_OF_NODES = NULL;

Symbol* SYM_HIERARCHY_STELLA_NUMBER_OF_FOREIGN_INTERVAL_NODES = NULL;

Symbol* SYM_HIERARCHY_STELLA_ROOTS = NULL;

Symbol* SYM_HIERARCHY_STELLA_BROKEN_LINKS = NULL;

Symbol* SYM_HIERARCHY_STELLA_ADDED_LINKS = NULL;

Symbol* SYM_HIERARCHY_STELLA_REMOVED_LINKS = NULL;

Symbol* SYM_HIERARCHY_STELLA_FILE_NAME = NULL;

Surrogate* SGT_HIERARCHY_STELLA_INTEGER_WRAPPER = NULL;

Symbol* SYM_HIERARCHY_STELLA_WRAPPER_VALUE = NULL;

Surrogate* SGT_HIERARCHY_STELLA_FLOAT_WRAPPER = NULL;

Surrogate* SGT_HIERARCHY_STELLA_STRING_WRAPPER = NULL;

Surrogate* SGT_HIERARCHY_STELLA_MUTABLE_STRING_WRAPPER = NULL;

Surrogate* SGT_HIERARCHY_STELLA_CHARACTER_WRAPPER = NULL;

Surrogate* SGT_HIERARCHY_STELLA_BOOLEAN_WRAPPER = NULL;

Symbol* SYM_HIERARCHY_STELLA_TRUE = NULL;

Symbol* SYM_HIERARCHY_STELLA_FALSE = NULL;

Surrogate* SGT_HIERARCHY_STELLA_FUNCTION_CODE_WRAPPER = NULL;

Surrogate* SGT_HIERARCHY_STELLA_METHOD_CODE_WRAPPER = NULL;

Surrogate* SGT_HIERARCHY_STELLA_VERBATIM_STRING_WRAPPER = NULL;

Symbol* SYM_HIERARCHY_STELLA_THREE_VALUED_BOOLEAN = NULL;

Surrogate* SGT_HIERARCHY_STELLA_SYSTEM_DEFINITION = NULL;

Symbol* SYM_HIERARCHY_STELLA_NAME = NULL;

Symbol* SYM_HIERARCHY_STELLA_DIRECTORY = NULL;

Symbol* SYM_HIERARCHY_STELLA_FILES = NULL;

Symbol* SYM_HIERARCHY_STELLA_LISP_ONLY_FILES = NULL;

Symbol* SYM_HIERARCHY_STELLA_CPP_ONLY_FILES = NULL;

Symbol* SYM_HIERARCHY_STELLA_JAVA_ONLY_FILES = NULL;

Symbol* SYM_HIERARCHY_STELLA_PREPROCESSED_FILES = NULL;

Symbol* SYM_HIERARCHY_STELLA_REQUIRED_SYSTEMS = NULL;

Symbol* SYM_HIERARCHY_STELLA_LOADEDp = NULL;

Symbol* SYM_HIERARCHY_STELLA_UP_TO_DATEp = NULL;

Symbol* SYM_HIERARCHY_STELLA_SOURCE_ROOT_DIRECTORY = NULL;

Symbol* SYM_HIERARCHY_STELLA_NATIVE_ROOT_DIRECTORY = NULL;

Symbol* SYM_HIERARCHY_STELLA_BINARY_ROOT_DIRECTORY = NULL;

Symbol* SYM_HIERARCHY_STELLA_BANNER = NULL;

Symbol* SYM_HIERARCHY_STELLA_COPYRIGHT_HEADER = NULL;

Symbol* SYM_HIERARCHY_STELLA_PRODUCTION_SETTINGS = NULL;

Symbol* SYM_HIERARCHY_STELLA_DEVELOPMENT_SETTINGS = NULL;

Symbol* SYM_HIERARCHY_STELLA_FINALIZATION_FUNCTION = NULL;

Symbol* SYM_HIERARCHY_STELLA_STATE = NULL;

Surrogate* SGT_HIERARCHY_STELLA_OUTPUT_STREAM = NULL;

Keyword* KWD_HIERARCHY_LINE = NULL;

Surrogate* SGT_HIERARCHY_STELLA_INPUT_STREAM = NULL;

Symbol* SYM_HIERARCHY_STELLA_ECHO_STREAM = NULL;

Symbol* SYM_HIERARCHY_STELLA_TOKENIZER_STATE = NULL;

Symbol* SYM_HIERARCHY_STELLA_BUFFERING_SCHEME = NULL;

Keyword* KWD_HIERARCHY_SUPERSEDE = NULL;

Surrogate* SGT_HIERARCHY_STELLA_OUTPUT_FILE_STREAM = NULL;

Symbol* SYM_HIERARCHY_STELLA_FILENAME = NULL;

Symbol* SYM_HIERARCHY_STELLA_IF_EXISTS_ACTION = NULL;

Symbol* SYM_HIERARCHY_STELLA_FILE_OUTPUT_STREAM = NULL;

Keyword* KWD_HIERARCHY_BLOCK = NULL;

Keyword* KWD_HIERARCHY_ERROR = NULL;

Surrogate* SGT_HIERARCHY_STELLA_INPUT_FILE_STREAM = NULL;

Symbol* SYM_HIERARCHY_STELLA_IF_NOT_EXISTS_ACTION = NULL;

Symbol* SYM_HIERARCHY_STELLA_FILE_INPUT_STREAM = NULL;

Surrogate* SGT_HIERARCHY_STELLA_OUTPUT_STRING_STREAM = NULL;

Symbol* SYM_HIERARCHY_STELLA_STRING_OUTPUT_STREAM = NULL;

Surrogate* SGT_HIERARCHY_STELLA_INPUT_STRING_STREAM = NULL;

Symbol* SYM_HIERARCHY_STELLA_STRING_INPUT_STREAM = NULL;

Symbol* SYM_HIERARCHY_STELLA_STARTUP_HIERARCHY = NULL;

Symbol* SYM_HIERARCHY_STELLA_METHOD_STARTUP_CLASSNAME = NULL;

} // end of namespace stella
